@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "alist.h"
 #include "astutil.h"
@@ -33,26 +47,33 @@ Expr* AList::last(void) {
 Expr* AList::only(void) {
   if (!head)
     INT_FATAL("only() called on empty list");
+
   if (head == tail)
     return first();
   else
     INT_FATAL("only() called on list with more than one element");
+
   return NULL;
 }
 
 
-Expr* AList::get(int index) {
+Expr* AList::get(int index) const {
   if (index <=0) {
     INT_FATAL("Indexing list must use positive integer");
   }
+
   int i = 0;
+
   for_alist(node, *this) {
     i++;
+
     if (i == index) {
       return node;
     }
   }
+
   INT_FATAL("Indexing list out of bounds");
+
   return NULL;
 }
 
