@@ -1,16 +1,41 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "astutil.h"
 #include "baseAST.h"
+#include "CForLoop.h"
+#include "ForLoop.h"
 #include "expr.h"
 #include "passes.h"
+#include "ParamForLoop.h"
+#include "stlUtil.h"
 #include "stmt.h"
 #include "symbol.h"
 #include "type.h"
+#include "WhileStmt.h"
 
+#include "oldCollectors.h" // Deprecated. To be removed.
+// After #include "oldCollectors.h" has been removed from all other source
+// files, the corresponding collector functions can be removed from this
+// implementation file, then this #include, and finally the .h file itself.
+
+#include <vector>
 
 static void pruneUnusedAggregateTypes(Vec<TypeSymbol*>& types);
 static void pruneUnusedRefs(Vec<TypeSymbol*>& types);
@@ -25,8 +50,8 @@ void collectFnCalls(BaseAST* ast, Vec<CallExpr*>& calls) {
 }
 
 
-void collectFnCallsSTL(BaseAST* ast, std::vector<CallExpr*>& calls) {
-  AST_CHILDREN_CALL(ast, collectFnCallsSTL, calls);
+void collectFnCalls(BaseAST* ast, std::vector<CallExpr*>& calls) {
+  AST_CHILDREN_CALL(ast, collectFnCalls, calls);
   if (CallExpr* call = toCallExpr(ast))
     if (call->isResolved())
       calls.push_back(call);
@@ -47,11 +72,11 @@ void collect_stmts(BaseAST* ast, Vec<Expr*>& stmts) {
   }
 }
 
-void collect_stmts_STL(BaseAST* ast, std::vector<Expr*>& stmts) {
+void collect_stmts(BaseAST* ast, std::vector<Expr*>& stmts) {
   if (Expr* expr = toExpr(ast)) {
     stmts.push_back(expr);
     if (isBlockStmt(expr) || isCondStmt(expr)) {
-      AST_CHILDREN_CALL(ast, collect_stmts_STL, stmts);
+      AST_CHILDREN_CALL(ast, collect_stmts, stmts);
     }
   }
 }
@@ -62,8 +87,8 @@ void collectDefExprs(BaseAST* ast, Vec<DefExpr*>& defExprs) {
     defExprs.add(defExpr);
 }
 
-void collectDefExprsSTL(BaseAST* ast, std::vector<DefExpr*>& defExprs) {
-  AST_CHILDREN_CALL(ast, collectDefExprsSTL, defExprs);
+void collectDefExprs(BaseAST* ast, std::vector<DefExpr*>& defExprs) {
+  AST_CHILDREN_CALL(ast, collectDefExprs, defExprs);
   if (DefExpr* defExpr = toDefExpr(ast))
     defExprs.push_back(defExpr);
 }
@@ -74,8 +99,8 @@ void collectCallExprs(BaseAST* ast, Vec<CallExpr*>& callExprs) {
     callExprs.add(callExpr);
 }
 
-void collectCallExprsSTL(BaseAST* ast, std::vector<CallExpr*>& callExprs) {
-  AST_CHILDREN_CALL(ast, collectCallExprsSTL, callExprs);
+void collectCallExprs(BaseAST* ast, std::vector<CallExpr*>& callExprs) {
+  AST_CHILDREN_CALL(ast, collectCallExprs, callExprs);
   if (CallExpr* callExpr = toCallExpr(ast))
     callExprs.push_back(callExpr);
 }
@@ -88,9 +113,9 @@ void collectMyCallExprs(BaseAST* ast, Vec<CallExpr*>& callExprs,
       callExprs.add(callExpr);
 }
 
-void collectMyCallExprsSTL(BaseAST* ast, std::vector<CallExpr*>& callExprs,
+void collectMyCallExprs(BaseAST* ast, std::vector<CallExpr*>& callExprs,
                            FnSymbol* parent_fn) {
-  AST_CHILDREN_CALL(ast, collectMyCallExprsSTL, callExprs, parent_fn);
+  AST_CHILDREN_CALL(ast, collectMyCallExprs, callExprs, parent_fn);
   if (CallExpr* callExpr = toCallExpr(ast))
     if (callExpr->parentSymbol == parent_fn)
       callExprs.push_back(callExpr);
@@ -102,8 +127,8 @@ void collectGotoStmts(BaseAST* ast, Vec<GotoStmt*>& gotoStmts) {
     gotoStmts.add(gotoStmt);
 }
 
-void collectGotoStmtsSTL(BaseAST* ast, std::vector<GotoStmt*>& gotoStmts) {
-  AST_CHILDREN_CALL(ast, collectGotoStmtsSTL, gotoStmts);
+void collectGotoStmts(BaseAST* ast, std::vector<GotoStmt*>& gotoStmts) {
+  AST_CHILDREN_CALL(ast, collectGotoStmts, gotoStmts);
   if (GotoStmt* gotoStmt = toGotoStmt(ast))
     gotoStmts.push_back(gotoStmt);
 }
@@ -114,8 +139,8 @@ void collectSymExprs(BaseAST* ast, Vec<SymExpr*>& symExprs) {
     symExprs.add(symExpr);
 }
 
-void collectSymExprsSTL(BaseAST* ast, std::vector<SymExpr*>& symExprs) {
-  AST_CHILDREN_CALL(ast, collectSymExprsSTL, symExprs);
+void collectSymExprs(BaseAST* ast, std::vector<SymExpr*>& symExprs) {
+  AST_CHILDREN_CALL(ast, collectSymExprs, symExprs);
   if (SymExpr* symExpr = toSymExpr(ast))
     symExprs.push_back(symExpr);
 }
@@ -153,8 +178,8 @@ void collectSymbols(BaseAST* ast, Vec<Symbol*>& symbols) {
     symbols.add(symbol);
 }
 
-void collectSymbolsSTL(BaseAST* ast, std::vector<Symbol*>& symbols) {
-  AST_CHILDREN_CALL(ast, collectSymbolsSTL, symbols);
+void collectSymbols(BaseAST* ast, std::vector<Symbol*>& symbols) {
+  AST_CHILDREN_CALL(ast, collectSymbols, symbols);
   if (Symbol* symbol = toSymbol(ast))
     symbols.push_back(symbol);
 }
@@ -164,9 +189,9 @@ void collect_asts(BaseAST* ast, Vec<BaseAST*>& asts) {
   AST_CHILDREN_CALL(ast, collect_asts, asts);
 }
 
-void collect_asts_STL(BaseAST* ast, std::vector<BaseAST*>& asts) {
+void collect_asts(BaseAST* ast, std::vector<BaseAST*>& asts) {
   asts.push_back(ast);
-  AST_CHILDREN_CALL(ast, collect_asts_STL, asts);
+  AST_CHILDREN_CALL(ast, collect_asts, asts);
 }
 
 void collect_asts_postorder(BaseAST* ast, Vec<BaseAST*>& asts) {
@@ -174,8 +199,8 @@ void collect_asts_postorder(BaseAST* ast, Vec<BaseAST*>& asts) {
   asts.add(ast);
 }
 
-void collect_asts_postorder_STL(BaseAST* ast, std::vector<BaseAST*>& asts) {
-  AST_CHILDREN_CALL(ast, collect_asts_postorder_STL, asts);
+void collect_asts_postorder(BaseAST* ast, std::vector<BaseAST*>& asts) {
+  AST_CHILDREN_CALL(ast, collect_asts_postorder, asts);
   asts.push_back(ast);
 }
 
@@ -190,10 +215,10 @@ void collect_top_asts(BaseAST* ast, Vec<BaseAST*>& asts) {
   AST_CHILDREN_CALL(ast, collect_top_asts_internal, asts);
   asts.add(ast);
 }
-              
-static void collect_top_asts_internal_STL(BaseAST* ast, std::vector<BaseAST*>& asts) {
+
+static void collect_top_asts_internal(BaseAST* ast, std::vector<BaseAST*>& asts) {
   if (!isSymbol(ast) || isArgSymbol(ast)) {
-    AST_CHILDREN_CALL(ast, collect_top_asts_internal_STL, asts);
+    AST_CHILDREN_CALL(ast, collect_top_asts_internal, asts);
     asts.push_back(ast);
   }
 }
@@ -205,11 +230,11 @@ static void collect_top_asts_internal_STL(BaseAST* ast, std::vector<BaseAST*>& a
 // replaced by more specific traversals implemented in this file.
 // Something to check out another day.
 //
-void collect_top_asts_STL(BaseAST* ast, std::vector<BaseAST*>& asts) {
-  AST_CHILDREN_CALL(ast, collect_top_asts_internal_STL, asts);
+void collect_top_asts(BaseAST* ast, std::vector<BaseAST*>& asts) {
+  AST_CHILDREN_CALL(ast, collect_top_asts_internal, asts);
   asts.push_back(ast);
 }
-              
+
 void reset_ast_loc(BaseAST* destNode, BaseAST* sourceNode) {
   reset_ast_loc(destNode, sourceNode->astloc);
 }
@@ -219,7 +244,6 @@ void reset_ast_loc(BaseAST* destNode, astlocT astlocArg) {
   AST_CHILDREN_CALL(destNode, reset_ast_loc, astlocArg);
 }
 
-
 void compute_call_sites() {
   forv_Vec(FnSymbol, fn, gFnSymbols) {
     if (fn->calledBy)
@@ -227,16 +251,23 @@ void compute_call_sites() {
     else
       fn->calledBy = new Vec<CallExpr*>();
   }
+
   forv_Vec(CallExpr, call, gCallExprs) {
-    if (FnSymbol* fn = call->isResolved()) {
+    if (call->isEmpty() == true) {
+
+    } else if (FnSymbol* fn = call->isResolved()) {
       fn->calledBy->add(call);
+
     } else if (call->isPrimitive(PRIM_FTABLE_CALL)) {
       // sjd: do we have to do anything special here?
       //      should this call be added to some function's calledBy list?
-    } else if (call->isPrimitive(PRIM_VMT_CALL)) {
-      FnSymbol* fn = toFnSymbol(toSymExpr(call->get(1))->var);
+
+    } else if (call->isPrimitive(PRIM_VIRTUAL_METHOD_CALL)) {
+      FnSymbol*       fn       = toFnSymbol(toSymExpr(call->get(1))->var);
       Vec<FnSymbol*>* children = virtualChildrenMap.get(fn);
+
       fn->calledBy->add(call);
+
       forv_Vec(FnSymbol, child, *children)
         child->calledBy->add(call);
     }
@@ -252,7 +283,7 @@ void buildDefUseMaps(Map<Symbol*,Vec<SymExpr*>*>& defMap,
   Vec<Symbol*> symSet;
   forv_Vec(DefExpr, def, gDefExprs) {
     if (def->parentSymbol) {
-      if (isVarSymbol(def->sym) || isArgSymbol(def->sym)) {
+      if (isLcnSymbol(def->sym)) {
         symSet.set_add(def->sym);
       }
     }
@@ -270,7 +301,7 @@ void collectSymbolSetSymExprVec(BaseAST* ast,
                                 Vec<Symbol*>& symSet,
                                 Vec<SymExpr*>& symExprs) {
   if (DefExpr* def = toDefExpr(ast)) {
-    if (isVarSymbol(def->sym) || isArgSymbol(def->sym)) {
+    if (isLcnSymbol(def->sym)) {
       symSet.set_add(def->sym);
     }
   } else if (SymExpr* se = toSymExpr(ast)) {
@@ -327,24 +358,42 @@ void addUse(Map<Symbol*,Vec<SymExpr*>*>& useMap, SymExpr* use) {
 
 //
 // Checks if a callExpr is one of the op= primitives
-// Note, this does not check if a callExpr is an 
+// Note, this does not check if a callExpr is an
 // op= function call (such as before inlining)
 //
 bool isOpEqualPrim(CallExpr* call) {
-  if (call->isPrimitive(PRIM_ADD_ASSIGN) ||
+  if (call->isPrimitive(PRIM_ADD_ASSIGN)      ||
       call->isPrimitive(PRIM_SUBTRACT_ASSIGN) ||
-      call->isPrimitive(PRIM_MULT_ASSIGN) ||
-      call->isPrimitive(PRIM_DIV_ASSIGN) ||
-      call->isPrimitive(PRIM_MOD_ASSIGN) ||
-      call->isPrimitive(PRIM_LSH_ASSIGN) ||
-      call->isPrimitive(PRIM_RSH_ASSIGN) ||
-      call->isPrimitive(PRIM_AND_ASSIGN) ||
-      call->isPrimitive(PRIM_OR_ASSIGN) ||
+      call->isPrimitive(PRIM_MULT_ASSIGN)     ||
+      call->isPrimitive(PRIM_DIV_ASSIGN)      ||
+      call->isPrimitive(PRIM_MOD_ASSIGN)      ||
+      call->isPrimitive(PRIM_LSH_ASSIGN)      ||
+      call->isPrimitive(PRIM_RSH_ASSIGN)      ||
+      call->isPrimitive(PRIM_AND_ASSIGN)      ||
+      call->isPrimitive(PRIM_OR_ASSIGN)       ||
       call->isPrimitive(PRIM_XOR_ASSIGN)) {
-    return true;      
-    }
-    //otherwise false
-    return false;
+    return true;
+  }
+  //otherwise false
+  return false;
+}
+
+
+//
+// Check if a callExpr is a relational operator primitive (==, !=, <=, >=, <, >)
+//
+bool isRelationalOperator(CallExpr* call) {
+  if (call->isPrimitive(PRIM_EQUAL)          ||
+      call->isPrimitive(PRIM_NOTEQUAL)       ||
+      call->isPrimitive(PRIM_LESSOREQUAL)    ||
+      call->isPrimitive(PRIM_GREATEROREQUAL) ||
+      call->isPrimitive(PRIM_LESS)           ||
+      call->isPrimitive(PRIM_GREATER)) {
+    return true;
+  }
+  //otherwise false
+  return false;
+
 }
 
 
@@ -352,7 +401,7 @@ bool isOpEqualPrim(CallExpr* call) {
 // TODO this should be fixed to include PRIM_SET_MEMBER
 // See notes in iterator.cpp and/or loopInvariantCodeMotion.cpp
 // TODO this should also be fixed to include the PRIM_SVEC_SET_MEMBER
-// which gets inserted from the returnStartTuplesByRefArgs pass 
+// which gets inserted from the returnStartTuplesByRefArgs pass
 // return & 1 is true if se is a def
 // return & 2 is true if se is a use
 //
@@ -517,7 +566,7 @@ actual_to_formal(Expr *a) {
         if (a == actual)
           return formal;
       }
-    } else if (call->isPrimitive(PRIM_VMT_CALL)) {
+    } else if (call->isPrimitive(PRIM_VIRTUAL_METHOD_CALL)) {
       FnSymbol* fn = toFnSymbol(toSymExpr(call->get(1))->var);
       int i = 0;
       for_actuals(actual, call) {
@@ -544,15 +593,75 @@ Expr* formal_to_actual(CallExpr* call, Symbol* arg) {
 }
 
 
+bool isTypeExpr(Expr* expr)
+{
+  if (SymExpr* sym = toSymExpr(expr))
+  {
+    if (isTypeSymbol(sym->var))
+      return true;
+
+    if (sym->var->hasFlag(FLAG_TYPE_VARIABLE))
+      return true;
+
+    if (FnSymbol* fn = toFnSymbol(sym->var))
+      if (fn->retTag == RET_TYPE)
+        return true;
+  }
+
+  if (CallExpr* call = toCallExpr(expr))
+  {
+    if (call->isPrimitive(PRIM_TYPEOF))
+      return true;
+
+    if (call->isPrimitive(PRIM_GET_MEMBER_VALUE) ||
+        call->isPrimitive(PRIM_GET_MEMBER))
+    {
+      AggregateType* ct = toAggregateType(call->get(1)->typeInfo());
+      INT_ASSERT(ct);
+
+      if (ct->symbol->hasFlag(FLAG_REF))
+        ct = toAggregateType(ct->getValType());
+
+      SymExpr* left = toSymExpr(call->get(1));
+
+      if (left->var->type->symbol->hasFlag(FLAG_TUPLE) &&
+          left->var->hasFlag(FLAG_TYPE_VARIABLE))
+        return true;
+
+      SymExpr* right = toSymExpr(call->get(2));
+      VarSymbol* var = toVarSymbol(right->var);
+
+      if (var->isType())
+        return true;
+
+      if (var->immediate)
+      {
+        const char* name = var->immediate->v_string;
+        for_fields(field, ct) {
+          if (!strcmp(field->name, name))
+            if (field->hasFlag(FLAG_TYPE_VARIABLE))
+              return true;
+        }
+      }
+    }
+
+    if (FnSymbol* fn = call->isResolved())
+      if (fn->retTag == RET_TYPE)
+        return true;
+  }
+
+  return false;
+}
+
+
 static void
 pruneVisit(TypeSymbol* ts, Vec<FnSymbol*>& fns, Vec<TypeSymbol*>& types) {
   types.set_add(ts);
-  Vec<BaseAST*> asts;
-  collect_asts(ts, asts);
-  forv_Vec(BaseAST, ast, asts) {
-    if (DefExpr* def = toDefExpr(ast))
-      if (def->sym->type && !types.set_in(def->sym->type->symbol))
-        pruneVisit(def->sym->type->symbol, fns, types);
+  std::vector<DefExpr*> defExprs;
+  collectDefExprs(ts, defExprs);
+  for_vector(DefExpr, def, defExprs) {
+    if (def->sym->type && !types.set_in(def->sym->type->symbol))
+      pruneVisit(def->sym->type->symbol, fns, types);
   }
   if (ts->hasFlag(FLAG_DATA_CLASS))
     pruneVisit(getDataClassType(ts), fns, types);
@@ -562,16 +671,14 @@ pruneVisit(TypeSymbol* ts, Vec<FnSymbol*>& fns, Vec<TypeSymbol*>& types) {
 static void
 pruneVisit(FnSymbol* fn, Vec<FnSymbol*>& fns, Vec<TypeSymbol*>& types) {
   fns.set_add(fn);
-  Vec<BaseAST*> asts;
-  collect_asts(fn, asts);
-  forv_Vec(BaseAST, ast, asts) {
-    if (SymExpr* se = toSymExpr(ast)) {
-      if (FnSymbol* next = toFnSymbol(se->var))
-        if (!fns.set_in(next))
-          pruneVisit(next, fns, types);
-      if (se->var && se->var->type && !types.set_in(se->var->type->symbol))
-        pruneVisit(se->var->type->symbol, fns, types);
-    }
+  std::vector<SymExpr*> symExprs;
+  collectSymExprs(fn, symExprs);
+  for_vector(SymExpr, se, symExprs) {
+    if (FnSymbol* next = toFnSymbol(se->var))
+      if (!fns.set_in(next))
+        pruneVisit(next, fns, types);
+    if (se->var && se->var->type && !types.set_in(se->var->type->symbol))
+      pruneVisit(se->var->type->symbol, fns, types);
   }
   for_formals(formal, fn) {
     if (!types.set_in(formal->type->symbol))
@@ -586,7 +693,9 @@ static void
 visitVisibleFunctions(Vec<FnSymbol*>& fns, Vec<TypeSymbol*>& types)
 {
   // chpl_gen_main is always visible (if it exists).
-  pruneVisit(chpl_gen_main, fns, types);
+  // --ipe does not build chpl_gen_main
+  if (chpl_gen_main)
+    pruneVisit(chpl_gen_main, fns, types);
 
   // When present, the printModuleInitOrder function is always visible;
   // it will be NULL for --minimal-modules compilations
@@ -717,9 +826,21 @@ static void changeDeadTypesToVoid(Vec<TypeSymbol*>& types)
 }
 
 
-// Done this way because the log letter and hence the pass name for
-// each pass must be unique.  See initLogFlags() in runpasses.cpp.
-void prune2() { prune(); } // Synonym for prune.
+static void removeVoidMoves()
+{
+  forv_Vec(CallExpr, call, gCallExprs)
+  {
+    if (! call->isPrimitive(PRIM_MOVE))
+      continue;
+
+    SymExpr* se = toSymExpr(call->get(1));
+    if (se->var->type != dtVoid)
+      continue;
+
+    call->remove();
+  }
+}
+
 
 // Determine sets of used functions and types, and then delete
 // functions which are not visible and classes which are not used.
@@ -739,6 +860,71 @@ prune() {
   }
 
   pruneUnusedTypes(types);
+
+  removeVoidMoves();
 }
 
 
+// Done this way because the log letter and hence the pass name for
+// each pass must be unique.  See initLogFlags() in runpasses.cpp.
+void prune2() { prune(); } // Synonym for prune.
+
+/*
+ * Takes a call that is a PRIM_SVEC_GET_MEMBER* and returns the symbol of the
+ * field. Normally the call is something of the form PRIM_SVEC_GET_MEMBER(p, 1)
+ * and what this function gets out is the symbol that is the first field
+ * instead of just the number 1.
+ */
+Symbol* getSvecSymbol(CallExpr* call) {
+  INT_ASSERT(call->isPrimitive(PRIM_GET_SVEC_MEMBER)       ||
+             call->isPrimitive(PRIM_GET_SVEC_MEMBER_VALUE) ||
+             call->isPrimitive(PRIM_SET_SVEC_MEMBER));
+  Type* type = call->get(1)->getValType();
+  AggregateType* tuple = toAggregateType(type);
+  SymExpr* fieldVal = toSymExpr(call->get(2));
+  VarSymbol* fieldSym = toVarSymbol(fieldVal->var);
+  if (fieldSym) {
+    int immediateVal = fieldSym->immediate->int_value();
+
+    INT_ASSERT(immediateVal >= 1 && immediateVal <= tuple->fields.length);
+    return tuple->getField(immediateVal);
+  } else {
+    // GET_SVEC_MEMBER(p, i), where p is a star tuple
+    return NULL;
+  }
+}
+
+
+static void addToUsedFnSymbols(std::set<FnSymbol*>& fnSymbols, FnSymbol* newFn) {
+  if(fnSymbols.count(newFn) == 0) {
+    fnSymbols.insert(newFn);
+    AST_CHILDREN_CALL(newFn->body, collectUsedFnSymbols, fnSymbols);
+  }
+}
+
+/*
+* Collect all of the functions in the call graph at and below the function
+* call.
+*/
+void collectUsedFnSymbols(BaseAST* ast, std::set<FnSymbol*>& fnSymbols) {
+  AST_CHILDREN_CALL(ast, collectUsedFnSymbols, fnSymbols);
+  //if there is a function call, get the FnSymbol associated with it
+  //and look through that FnSymbol for other function calls. Do not
+  //look through an already visited FnSymbol, or you'll have an infinite
+  //loop in the case of recursion.
+  if (CallExpr* call = toCallExpr(ast)) {
+    if (FnSymbol* fn = call->isResolved()) {
+      addToUsedFnSymbols(fnSymbols, fn);
+    } else if (call->isPrimitive(PRIM_FTABLE_CALL)) {
+      //
+      // TODO: We'd like a way to accurately find the set of functions that
+      // might be called from a PRIM_FTABLE_CALL. The complicated nature
+      // of recursive iterator lowering seems to be too big of an obstacle
+      // right now.
+      //
+      forv_Vec(FnSymbol, fn, ftableVec) {
+        addToUsedFnSymbols(fnSymbols, fn);
+      }
+    }
+  }
+}
