@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 // checkAST.cpp
 //
@@ -76,11 +90,9 @@ void checkPrimitives()
      case PRIM_TYPE_TO_STRING:
      case PRIM_TO_LEADER:
      case PRIM_TO_FOLLOWER:
-     case PRIM_NEXT_UINT32:
      case PRIM_FIELD_NUM_TO_NAME:
+     case PRIM_FIELD_NAME_TO_NUM:
      case PRIM_FIELD_VALUE_BY_NUM:
-     case PRIM_FIELD_ID_BY_NUM:
-     case PRIM_FIELD_VALUE_BY_NAME:
      case PRIM_ENUM_MIN_BITS:
      case PRIM_ENUM_IS_SIGNED:
      case PRIM_IS_UNION_TYPE:
@@ -174,9 +186,6 @@ void checkPrimitives()
      case PRIM_SINGLE_IS_FULL:
      case PRIM_GET_END_COUNT:
      case PRIM_SET_END_COUNT:
-     case PRIM_PROCESS_TASK_LIST:
-     case PRIM_EXECUTE_TASKS_IN_LIST:
-     case PRIM_FREE_TASK_LIST:
      case PRIM_GET_SERIAL:              // get serial state
      case PRIM_SET_SERIAL:              // set serial state to true or false
      case PRIM_SIZEOF:
@@ -187,12 +196,14 @@ void checkPrimitives()
      case PRIM_CAST:
      case PRIM_DYNAMIC_CAST:
      case PRIM_TYPEOF:
-     case PRIM_USE:
      case PRIM_USED_MODULES_LIST:       // used modules in BlockStmt::modUses
      case PRIM_TUPLE_EXPAND:
      case PRIM_TUPLE_AND_EXPAND:
      case PRIM_CHPL_COMM_GET:           // Direct calls to the Chapel comm layer
      case PRIM_CHPL_COMM_PUT:           // may eventually add others (e.g.: non-blocking)
+     case PRIM_CHPL_COMM_ARRAY_GET:
+     case PRIM_CHPL_COMM_ARRAY_PUT:
+     case PRIM_CHPL_COMM_REMOTE_PREFETCH:
      case PRIM_CHPL_COMM_GET_STRD:      // Direct calls to the Chapel comm layer for strided comm
      case PRIM_CHPL_COMM_PUT_STRD:      //  may eventually add others (e.g.: non-blocking)
      case PRIM_ARRAY_ALLOC:
@@ -220,11 +231,6 @@ void checkPrimitives()
      case PRIM_BLOCK_LOCAL:             // BlockStmt::blockInfo - local block
      case PRIM_BLOCK_UNLOCAL:           // BlockStmt::blockInfo - unlocal local block
      case PRIM_DELETE:
-     case PRIM_GC_CC_INIT:              // Initialize heap for copy-collecting
-     case PRIM_GC_ADD_ROOT:             // Add a root variable for garbage collection
-     case PRIM_GC_ADD_NULL_ROOT:        // Add a root and point it to NULL
-     case PRIM_GC_DELETE_ROOT:          // Remove a root variable for garbage collection
-     case PRIM_GC_CLEANUP:              // Free GC heaps
      case PRIM_CALL_DESTRUCTOR:         // call destructor on type (do not free)
      case PRIM_LOGICAL_FOLDER:          // Help fold logical && and ||
      case PRIM_WIDE_GET_LOCALE:         // Returns the "locale" portion of a wide pointer.
@@ -238,12 +244,10 @@ void checkPrimitives()
      case PRIM_CAPTURE_FN:
      case PRIM_CREATE_FN_TYPE:
      case PRIM_STRING_COPY:
-     case PRIM_STRING_NORMALIZE:        // Set the size field in a (wide) string.
      case PRIM_CAST_TO_VOID_STAR:       // Cast the object argument to void*.
      case PRIM_RT_ERROR:
      case PRIM_RT_WARNING:
      case PRIM_NEW_PRIV_CLASS:
-     case PRIM_NUM_PRIV_CLASSES:
      case PRIM_GET_PRIV_CLASS:
      case PRIM_GET_USER_LINE:
      case PRIM_GET_USER_FILE:
@@ -251,8 +255,9 @@ void checkPrimitives()
      case PRIM_SET_SVEC_MEMBER:
      case PRIM_GET_SVEC_MEMBER:
      case PRIM_GET_SVEC_MEMBER_VALUE:
-     case PRIM_VMT_CALL:
+     case PRIM_VIRTUAL_METHOD_CALL:
      case PRIM_NUM_FIELDS:
+     case PRIM_IS_POD:
       break;
     }
   }
