@@ -1,13 +1,27 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
-
-// primitive.cpp
+#include "primitive.h"
 
 #include "expr.h"
 #include "iterator.h"
-#include "primitive.h"
+#include "stringutil.h"
 #include "type.h"
 
 static Type*
@@ -21,8 +35,8 @@ returnInfoVoid(CallExpr* call) {
 }
 
 static Type*
-returnInfoOpaque(CallExpr* call) {
-  return dtOpaque;
+returnInfoCVoidPtr(CallExpr* call) {
+  return dtCVoidPtr;
 }
 
 static Type*
@@ -40,6 +54,10 @@ returnInfoStringC(CallExpr* call) {
   return dtStringC;
 }
 
+static Type*
+returnInfoStringCopy(CallExpr* call) {
+  return dtStringCopy;
+}
 
 static Type*
 returnInfoLocaleID(CallExpr* call) {
@@ -61,6 +79,11 @@ returnInfoInt64(CallExpr* call) {
   return dtInt[INT_SIZE_64];
 }
 
+static Type*
+returnInfoUInt64(CallExpr* call) {
+  return dtUInt[INT_SIZE_64];
+}
+
 static Type*
 returnInfoSizeType(CallExpr* call) {
   return SIZE_TYPE;
@@ -78,17 +101,12 @@ returnInfoDefaultInt(CallExpr* call) {
   return returnInfoInt64(call);
 }
 
+/*
 static Type*
 returnInfoUInt32(CallExpr* call) { // unexecuted none/gasnet on 4/25/08
   return dtUInt[INT_SIZE_32];
 }
 
-/*
-static Type*
-returnInfoUInt64(CallExpr* call) {
-  return dtUInt[INT_SIZE_64];
-}
-
 static Type*
 returnInfoReal32(CallExpr* call) {
   return dtReal[FLOAT_SIZE_32];
@@ -130,7 +148,7 @@ returnInfoCast(CallExpr* call) {
   if (t2->symbol->hasFlag(FLAG_WIDE_CLASS))
     if (wideClassMap.get(t1))
       t1 = wideClassMap.get(t1);
-  if (t2->symbol->hasFlag(FLAG_WIDE))
+  if (t2->symbol->hasFlag(FLAG_WIDE_REF))
     if (wideRefMap.get(t1))
       t1 = wideRefMap.get(t1);
   return t1;
@@ -257,17 +275,17 @@ returnInfoGetMemberRef(CallExpr* call) {
     return field->type->refType ? field->type->refType : field->type;
   } else
     return var->type->refType ? var->type->refType : var->type;
-  INT_FATAL(call, "bad member primitive");
-  return NULL;
 }
 
 static Type*
 returnInfoEndCount(CallExpr* call) {
   static Type* endCountType = NULL;
   if (endCountType == NULL) {
-    forv_Vec(TypeSymbol, ts, gTypeSymbols) {
-      if (!strcmp(ts->name, "_EndCount")) {
-        endCountType = ts->type;
+    // Look for the type var `_remoteEndCountType` in ChapelBase.
+    forv_Vec(VarSymbol, var, gVarSymbols) {
+      const char* searchStr = "_remoteEndCountType";
+      if (strcmp(var->cname, searchStr) == 0) {
+        endCountType = var->type;
         break;
       }
     }
@@ -284,6 +302,13 @@ returnInfoVirtualMethodCall(CallExpr* call) {
   return fn->retType;
 }
 
+static Type*
+returnInfoSecondType(CallExpr* call) {
+  Type* t = call->get(2)->typeInfo();
+  return t;
+}
+
+
 // print the number of each type of primitive present in the AST
 void printPrimitiveCounts(const char* passName) {
   int primCounts[NUM_KNOWN_PRIMS];
@@ -365,12 +390,14 @@ void
 initPrimitive() {
   primitives[PRIM_UNKNOWN] = NULL;
 
+
   prim_def(PRIM_ACTUALS_LIST, "actuals list", returnInfoVoid);
   prim_def(PRIM_NOOP, "noop", returnInfoVoid);
   prim_def(PRIM_MOVE, "move", returnInfoVoid, false, true);
   prim_def(PRIM_INIT, "init", returnInfoFirstDeref);
   prim_def(PRIM_NO_INIT, "no init", returnInfoFirstDeref);
-  prim_def(PRIM_REF_TO_STRING, "ref to string", returnInfoString);
+  prim_def(PRIM_TYPE_INIT, "type init", returnInfoFirstDeref);
+  prim_def(PRIM_REF_TO_STRING, "ref to string", returnInfoStringC);
   prim_def(PRIM_RETURN, "return", returnInfoFirst, true);
   prim_def(PRIM_YIELD, "yield", returnInfoFirst, true);
   prim_def(PRIM_UNARY_MINUS, "u-", returnInfoFirst);
@@ -423,6 +450,8 @@ initPrimitive() {
   prim_def(PRIM_GET_REAL, "complex_get_real", returnInfoComplexField);
   prim_def(PRIM_GET_IMAG, "complex_get_imag", returnInfoComplexField);
   prim_def(PRIM_QUERY, "query", returnInfoUnknown);
+  prim_def(PRIM_QUERY_PARAM_FIELD, "query param field", returnInfoGetMemberRef);
+  prim_def(PRIM_QUERY_TYPE_FIELD, "query type field", returnInfoGetMember);
 
   prim_def(PRIM_ADDR_OF, "addr of", returnInfoRef);
   prim_def(PRIM_DEREF,   "deref",   returnInfoVal, false, true);
@@ -462,10 +491,6 @@ initPrimitive() {
   prim_def(PRIM_GET_END_COUNT, "get end count", returnInfoEndCount);
   prim_def(PRIM_SET_END_COUNT, "set end count", returnInfoVoid, true);
 
-  prim_def(PRIM_PROCESS_TASK_LIST, "process task list", returnInfoVoid, true, true);
-  prim_def(PRIM_EXECUTE_TASKS_IN_LIST, "execute tasks in list", returnInfoVoid, true, true);
-  prim_def(PRIM_FREE_TASK_LIST, "free task list", returnInfoVoid, true, true);
-
   // task primitives
   prim_def(PRIM_GET_SERIAL, "task_get_serial", returnInfoBool);
   prim_def(PRIM_SET_SERIAL, "task_set_serial", returnInfoVoid, true);
@@ -476,17 +501,22 @@ initPrimitive() {
   prim_def(PRIM_INIT_FIELDS, "chpl_init_record", returnInfoVoid, true);
   prim_def(PRIM_PTR_EQUAL, "ptr_eq", returnInfoBool);
   prim_def(PRIM_PTR_NOTEQUAL, "ptr_neq", returnInfoBool);
+  // PRIM_IS_SUBTYPE arguments are (parent, sub) and it checks
+  // if sub is a sub-type of parent.
   prim_def(PRIM_IS_SUBTYPE, "is_subtype", returnInfoBool);
+  // PRIM_CAST arguments are (type to cast to, value to cast)
   prim_def(PRIM_CAST, "cast", returnInfoCast, false, true);
   prim_def(PRIM_DYNAMIC_CAST, "dynamic_cast", returnInfoCast, false, true);
   prim_def(PRIM_TYPEOF, "typeof", returnInfoFirstDeref);
-  prim_def(PRIM_USE, "use", returnInfoVoid, true);
   prim_def(PRIM_USED_MODULES_LIST, "used modules list", returnInfoVoid);
   prim_def(PRIM_TUPLE_EXPAND, "expand_tuple", returnInfoVoid);
   prim_def(PRIM_TUPLE_AND_EXPAND, "and_expand_tuple", returnInfoVoid);
 
   prim_def(PRIM_CHPL_COMM_GET, "chpl_comm_get", returnInfoVoid, true, true);
   prim_def(PRIM_CHPL_COMM_PUT, "chpl_comm_put", returnInfoVoid, true, true);
+  prim_def(PRIM_CHPL_COMM_ARRAY_GET, "chpl_comm_array_get", returnInfoVoid, true, true);
+  prim_def(PRIM_CHPL_COMM_ARRAY_PUT, "chpl_comm_array_put", returnInfoVoid, true, true);
+  prim_def(PRIM_CHPL_COMM_REMOTE_PREFETCH, "chpl_comm_remote_prefetch", returnInfoVoid, true, true);
   prim_def(PRIM_CHPL_COMM_GET_STRD, "chpl_comm_get_strd", returnInfoVoid, true, true);
   prim_def(PRIM_CHPL_COMM_PUT_STRD, "chpl_comm_put_strd", returnInfoVoid, true, true);
 
@@ -510,6 +540,7 @@ initPrimitive() {
   prim_def(PRIM_BLOCK_WHILEDO_LOOP, "while...do loop", returnInfoVoid);
   prim_def(PRIM_BLOCK_DOWHILE_LOOP, "do...while loop", returnInfoVoid);
   prim_def(PRIM_BLOCK_FOR_LOOP, "for loop", returnInfoVoid);
+  prim_def(PRIM_BLOCK_C_FOR_LOOP, "C for loop", returnInfoVoid);
   prim_def(PRIM_BLOCK_BEGIN, "begin block", returnInfoVoid);
   prim_def(PRIM_BLOCK_COBEGIN, "cobegin block", returnInfoVoid);
   prim_def(PRIM_BLOCK_COFORALL, "coforall loop", returnInfoVoid);
@@ -520,26 +551,25 @@ initPrimitive() {
   prim_def(PRIM_BLOCK_LOCAL, "local block", returnInfoVoid);
   prim_def(PRIM_BLOCK_UNLOCAL, "unlocal block", returnInfoVoid);
 
+  prim_def(PRIM_FORALL_LOOP, "forall loop", returnInfoVoid);
   prim_def(PRIM_TO_LEADER, "to leader", returnInfoVoid);
   prim_def(PRIM_TO_FOLLOWER, "to follower", returnInfoVoid);
+  prim_def(PRIM_TO_STANDALONE, "to standalone", returnInfoVoid);
 
   prim_def(PRIM_DELETE, "delete", returnInfoVoid);
 
-  prim_def(PRIM_GC_CC_INIT, "_chpl_gc_init", returnInfoVoid);
-  prim_def(PRIM_GC_ADD_ROOT, "_addRoot", returnInfoVoid);
-  prim_def(PRIM_GC_ADD_NULL_ROOT, "_addNullRoot", returnInfoVoid);
-  prim_def(PRIM_GC_DELETE_ROOT, "_deleteRoot", returnInfoVoid);
-  prim_def(PRIM_GC_CLEANUP, "_chpl_gc_cleanup", returnInfoVoid);
-
   prim_def(PRIM_CALL_DESTRUCTOR, "call destructor", returnInfoVoid, true);
 
   prim_def(PRIM_LOGICAL_FOLDER, "_paramFoldLogical", returnInfoBool);
 
   prim_def(PRIM_WIDE_GET_LOCALE, "_wide_get_locale", returnInfoLocaleID, false, true);
-  // This will be unnecessary once the module code calls the corresponding
-  // function directly.
+  // MPF - 10/9/2015 - neither _wide_get_node nor _wide_get_addr
+  // is used in the module or test code. insertWideReferences uses
+  // PRIM_WIDE_GET_NODE. It might make sense to keep both of these
+  // functions for debugging.
   prim_def(PRIM_WIDE_GET_NODE, "_wide_get_node", returnInfoNodeID, false, true);
-  prim_def(PRIM_WIDE_GET_ADDR, "_wide_get_addr", returnInfoInt64, false, true);
+  prim_def(PRIM_WIDE_GET_ADDR, "_wide_get_addr", returnInfoUInt64, false, true);
+  prim_def(PRIM_IS_WIDE_PTR, "is wide pointer", returnInfoBool);
 
   prim_def(PRIM_ON_LOCALE_NUM, "chpl_on_locale_num", returnInfoLocaleID);
 
@@ -552,18 +582,15 @@ initPrimitive() {
   prim_def(PRIM_CAPTURE_FN, "capture fn", returnInfoVoid);
   prim_def(PRIM_CREATE_FN_TYPE, "create fn type", returnInfoVoid);
 
-  prim_def("chpl_string_compare", returnInfoDefaultInt, true);
+  prim_def("string_compare", returnInfoDefaultInt, true);
   prim_def("string_contains", returnInfoBool, true);
-  prim_def("string_concat", returnInfoString, true, true);
+  prim_def("string_concat", returnInfoStringCopy, true, true);
   prim_def("string_length", returnInfoDefaultInt);
   prim_def("ascii", returnInfoInt32);
-  prim_def("string_index", returnInfoString, true, true);
-  prim_def(PRIM_STRING_COPY, "string_copy", returnInfoString, false, true);
-  prim_def(PRIM_STRING_NORMALIZE, "string_normalize", returnInfoVoid, true, false);
-  prim_def(PRIM_STRING_FROM_C_STRING, "string_from_c_string", returnInfoString, false, true);
-  prim_def(PRIM_C_STRING_FROM_STRING, "c_string_from_string", returnInfoStringC, false, true);
-  prim_def(PRIM_CAST_TO_VOID_STAR, "cast_to_void_star", returnInfoOpaque, true, false);
-  prim_def("string_select", returnInfoString, true, true);
+  prim_def("string_index", returnInfoStringCopy, true, true);
+  prim_def(PRIM_STRING_COPY, "string_copy", returnInfoStringCopy, false, true);
+  prim_def(PRIM_CAST_TO_VOID_STAR, "cast_to_void_star", returnInfoCVoidPtr, true, false);
+  prim_def("string_select", returnInfoStringCopy, true, true);
   prim_def("sleep", returnInfoVoid, true);
   prim_def("real2int", returnInfoDefaultInt);
   prim_def("object2int", returnInfoDefaultInt);
@@ -573,12 +600,10 @@ initPrimitive() {
   prim_def(PRIM_RT_WARNING, "chpl_warning", returnInfoVoid, true, true);
 
   prim_def(PRIM_NEW_PRIV_CLASS, "chpl_newPrivatizedClass", returnInfoVoid, true);
-  prim_def(PRIM_NUM_PRIV_CLASSES, "chpl_numPrivatizedClasses", returnInfoDefaultInt);
   prim_def(PRIM_GET_PRIV_CLASS, "chpl_getPrivatizedClass",  returnInfoFirst);
   
-  prim_def(PRIM_NEXT_UINT32, "_next_uint32", returnInfoUInt32);
   prim_def(PRIM_GET_USER_LINE, "_get_user_line", returnInfoDefaultInt, true, true);
-  prim_def(PRIM_GET_USER_FILE, "_get_user_file", returnInfoString, true, true);
+  prim_def(PRIM_GET_USER_FILE, "_get_user_file", returnInfoInt32, true, true);
 
   prim_def(PRIM_FTABLE_CALL, "call ftable function", returnInfoVoid, true);
 
@@ -590,18 +615,37 @@ initPrimitive() {
   prim_def(PRIM_GET_SVEC_MEMBER, "get svec member", returnInfoGetTupleMemberRef);
   prim_def(PRIM_GET_SVEC_MEMBER_VALUE, "get svec member value", returnInfoGetTupleMember, false, true);
 
-  prim_def(PRIM_VMT_CALL, "virtual method call", returnInfoVirtualMethodCall, true, true);
+  prim_def(PRIM_VIRTUAL_METHOD_CALL, "virtual method call", returnInfoVirtualMethodCall, true, true);
 
   prim_def(PRIM_NUM_FIELDS, "num fields", returnInfoInt32);
   prim_def(PRIM_FIELD_NUM_TO_NAME, "field num to name", returnInfoString);
+  prim_def(PRIM_FIELD_NAME_TO_NUM, "field name to num", returnInfoInt32);
   prim_def(PRIM_FIELD_VALUE_BY_NUM, "field value by num", returnInfoUnknown);
-  prim_def(PRIM_FIELD_ID_BY_NUM, "field id by num", returnInfoInt32);
-  prim_def(PRIM_FIELD_VALUE_BY_NAME, "field value by name", returnInfoUnknown);
   prim_def(PRIM_IS_UNION_TYPE, "is union type", returnInfoBool);
   prim_def(PRIM_IS_ATOMIC_TYPE, "is atomic type", returnInfoBool);
+  prim_def(PRIM_IS_REF_ITER_TYPE, "is ref iter type", returnInfoBool);
+  
+  prim_def(PRIM_IS_POD, "is pod type", returnInfoBool);
+
+  // This primitive allows normalize to request function resolution
+  // coerce a return value to the declared return type, even though
+  // the declared return type is not really known until function
+  // resolution.
+  // It coerces its first argument to the type stored in the second argument.
+  prim_def(PRIM_COERCE, "coerce", returnInfoSecondType);
+
+  prim_def(PRIM_CALL_RESOLVES, "call resolves", returnInfoBool);
+  prim_def(PRIM_METHOD_CALL_RESOLVES, "method call resolves", returnInfoBool);
 
   prim_def(PRIM_ENUM_MIN_BITS, "enum min bits", returnInfoInt32);
   prim_def(PRIM_ENUM_IS_SIGNED, "enum is signed", returnInfoBool);
+
+  prim_def(PRIM_START_RMEM_FENCE, "chpl_rmem_consist_acquire", returnInfoVoid, true, true);
+  prim_def(PRIM_FINISH_RMEM_FENCE, "chpl_rmem_consist_release", returnInfoVoid, true, true);
+
+  prim_def(PRIM_LOOKUP_FILENAME, "chpl_lookupFilename", returnInfoStringC, false, false);
+
+  prim_def(PRIM_GET_COMPILER_VAR, "get compiler variable", returnInfoString);
 }
 
 Map<const char*, VarSymbol*> memDescsMap;
