@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 /*****************************************************************************
 
@@ -201,7 +215,12 @@ void beautify(fileinfo* origfile) {
         sscanf(cp, ZLINEINPUTFORMAT, &zline, zname);
         znptr = strrchr(zname,'/');
         if (znptr != NULL) {
-          strcpy(zname,znptr+1);
+          // We can't use strcpy here because the source
+          // and destination strings can overlap.
+          char *src = znptr+1;
+          char *dst = zname;
+          size_t len = strlen(src) + 1; // also copy null
+          memmove(dst, src, len);
         }
         continue;
       }
@@ -236,13 +255,8 @@ void beautify(fileinfo* origfile) {
       fprintf(outputfile, "%s", cp);    /* output line */
       break;
     default:
-      if (strncmp(cp, "case", 4) == 0)
-        i = 1;
-      else
-        i = 0;
-
       if ((indent == TRUE) && (cp[0] != '#'))
-        for (i = 0; i < 2*depth+justify; i++)
+        for (i = (strncmp(cp, "case", 4) == 0) ? 1 : 0; i < 2*depth+justify; i++)
           fprintf(outputfile, " ");
       fprintf(outputfile, "%s", cp);  /* output line */
 
