@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef __STDC_FORMAT_MACROS
 #define __STDC_FORMAT_MACROS
@@ -42,10 +56,11 @@ snprint_imm(char *str, size_t max, char *control_string, Immediate &imm) {
   switch (imm.const_kind) {
     case NUM_KIND_NONE:
       break;
+    case NUM_KIND_BOOL: {
+      res = snprintf(str, max, control_string, imm.bool_value()); break;
+    }
     case NUM_KIND_UINT: {
       switch (imm.num_index) {
-        case INT_SIZE_1: 
-          res = snprintf(str, max, control_string, imm.v_bool); break;
         case INT_SIZE_8: 
           res = snprintf(str, max, control_string, imm.v_uint8); break;
         case INT_SIZE_16:
@@ -60,8 +75,6 @@ snprint_imm(char *str, size_t max, char *control_string, Immediate &imm) {
     }
     case NUM_KIND_INT: {
       switch (imm.num_index) {
-        case INT_SIZE_1: 
-          res = snprintf(str, max, control_string, imm.v_bool); break;
         case INT_SIZE_8: 
           res = snprintf(str, max, control_string, imm.v_int8); break;
         case INT_SIZE_16:
@@ -109,10 +122,13 @@ snprint_imm(char *str, size_t max, Immediate &imm) {
   switch (imm.const_kind) {
     case NUM_KIND_NONE:
       break;
+    case NUM_KIND_BOOL: {
+      // Since I'm using uints to store the bools, should cast to the
+      // same to be consistent.
+      res = snprintf(str, max, "%u", (unsigned)imm.bool_value()); break;
+    }
     case NUM_KIND_UINT: {
       switch (imm.num_index) {
-        case INT_SIZE_1: 
-          res = snprintf(str, max, "%u", (unsigned)imm.v_bool); break;
         case INT_SIZE_8: 
           res = snprintf(str, max, "%u", (unsigned)imm.v_uint8); break;
         case INT_SIZE_16:
@@ -120,23 +136,21 @@ snprint_imm(char *str, size_t max, Immediate &imm) {
         case INT_SIZE_32:
           res = snprintf(str, max, "%u", (unsigned)imm.v_uint32); break;
         case INT_SIZE_64:
-          res = snprintf(str, max, "%"PRIu64, imm.v_uint64); break;
+          res = snprintf(str, max, "%" PRIu64, imm.v_uint64); break;
         default: INT_FATAL("Unhandled case in switch statement");
       }
       break;
     }
     case NUM_KIND_INT: {
       switch (imm.num_index) {
-        case INT_SIZE_1: 
-          res = snprintf(str, max, "%d", imm.v_bool); break;
         case INT_SIZE_8: 
           res = snprintf(str, max, "%d", imm.v_int8); break;
         case INT_SIZE_16:
           res = snprintf(str, max, "%d", imm.v_int16); break;
         case INT_SIZE_32:
-          res = snprintf(str, max, "%"PRId32, imm.v_int32); break;
+          res = snprintf(str, max, "%" PRId32, imm.v_int32); break;
         case INT_SIZE_64:
-          res = snprintf(str, max, "%"PRId64, imm.v_int64); break;
+          res = snprintf(str, max, "%" PRId64, imm.v_int64); break;
         default: INT_FATAL("Unhandled case in switch statement");
       }
       break;
@@ -161,71 +175,106 @@ snprint_imm(char *str, size_t max, Immediate &imm) {
         default: INT_FATAL("Unhandled case in switch statement");
       }
       break;
-    case CONST_KIND_STRING:
-      res = snprintf(str, max, "\"%s\"", imm.v_string);
+    case CONST_KIND_STRING: {
+      char const * fmt = NULL;
+      if (imm.string_kind == STRING_KIND_C_STRING)
+        fmt = "c\"%s\"";
+      else
+        fmt = "\"%s\"";
+      res = snprintf(str, max, fmt, imm.v_string);
       break;
+    }
   }
   return res;
 }
 
 int 
-fprint_imm(FILE *fp, Immediate &imm) {
+fprint_imm(FILE *fp, Immediate &imm, bool showType) {
   int res = -1;
   switch (imm.const_kind) {
     case NUM_KIND_NONE:
+      if (showType) res = fputs(":NONE", fp);
       break;
+    case NUM_KIND_BOOL: {
+      // Since I'm using uints to store the bools, should cast to the
+      // same to be consistent.
+      res = fprintf(fp, "%" PRIu64, imm.bool_value());
+      if (showType) res += fputs(" :bool", fp);
+      break;
+    }
     case NUM_KIND_UINT: {
       switch (imm.num_index) {
-        case INT_SIZE_1: 
-          res = fprintf(fp, "%u", (unsigned)imm.v_bool); break;
         case INT_SIZE_8: 
-          res = fprintf(fp, "%u", (unsigned)imm.v_uint8); break;
+          res = fprintf(fp, "%u", (unsigned)imm.v_uint8);
+          if (showType) res += fputs(" :uint(8)", fp);
+          break;
         case INT_SIZE_16:
-          res = fprintf(fp, "%u", (unsigned)imm.v_uint16); break;
+          res = fprintf(fp, "%u", (unsigned)imm.v_uint16);
+          if (showType) res += fputs(" :uint(16)", fp);
+          break;
         case INT_SIZE_32:
-          res = fprintf(fp, "%u", (unsigned)imm.v_uint32); break;
+          res = fprintf(fp, "%u", (unsigned)imm.v_uint32);
+          if (showType) res += fputs(" :uint(32)", fp);
+          break;
         case INT_SIZE_64:
-          res = fprintf(fp, "%"PRIu64, imm.v_uint64); break;
+          res = fprintf(fp, "%" PRIu64, imm.v_uint64);
+          if (showType) res += fputs(" :uint(64)", fp);
+          break;
         default: INT_FATAL("Unhandled case in switch statement");
       }
       break;
     }
     case NUM_KIND_INT: {
       switch (imm.num_index) {
-        case INT_SIZE_1: 
-          res = fprintf(fp, "%d", imm.v_bool); break;
         case INT_SIZE_8: 
-          res = fprintf(fp, "%d", imm.v_int8); break;
+          res = fprintf(fp, "%d", imm.v_int8);
+          if (showType) res += fputs(" :int(8)", fp);
+          break;
         case INT_SIZE_16:
-          res = fprintf(fp, "%d", imm.v_int16); break;
+          res = fprintf(fp, "%d", imm.v_int16);
+          if (showType) res += fputs(" :int(16)", fp);
+          break;
         case INT_SIZE_32:
-          res = fprintf(fp, "%"PRId32, imm.v_int32); break;
+          res = fprintf(fp, "%" PRId32, imm.v_int32);
+          if (showType) res += fputs(" :int(32)", fp);
+          break;
         case INT_SIZE_64:
-          res = fprintf(fp, "%"PRId64, imm.v_int64); break;
+          res = fprintf(fp, "%" PRId64, imm.v_int64);
+          if (showType) res += fputs(" :int(64)", fp);
+          break;
         default: INT_FATAL("Unhandled case in switch statement");
       }
       break;
     }
-    case NUM_KIND_REAL: case NUM_KIND_IMAG:
+    case NUM_KIND_REAL: case NUM_KIND_IMAG: {
       char str[80];
+      const char* size = NULL;
       switch (imm.num_index) {
         case FLOAT_SIZE_32:  
-          res = sprint_float_val(str, imm.v_float32); 
+          res = sprint_float_val(str, imm.v_float32);
+          size = "(32)";
           break;
         case FLOAT_SIZE_64: {
           res = sprint_float_val(str, imm.v_float64); 
+          size = "(64)";
           break;
         }
         default: INT_FATAL("Unhandled case in switch statement");
       }
       fputs(str, fp);
+      if (showType) {
+        res += fputs(imm.const_kind == NUM_KIND_IMAG?" :imag":" :real", fp);
+        res += fputs(size, fp);
+      }
       break;
+    }
     case NUM_KIND_COMPLEX:
       switch (imm.num_index) {
         case COMPLEX_SIZE_64: {
           char str[80];
           res = sprint_complex_val(str, imm.v_complex64.r, imm.v_complex64.i); 
           fputs(str, fp);
+          if (showType) res += fputs(" :complex(64)", fp);
           break;
         }
         case COMPLEX_SIZE_128: {
@@ -233,14 +282,23 @@ fprint_imm(FILE *fp, Immediate &imm) {
           res = sprint_complex_val(str, 
                                    imm.v_complex128.r, imm.v_complex128.i); 
           fputs(str, fp);
+          if (showType) res += fputs(" :complex(128)", fp);
           break;
         }
         default: INT_FATAL("Unhandled case in switch statement");
       }
       break;
-    case CONST_KIND_STRING:
-      res = fprintf(fp, "\"%s\"", imm.v_string);
+    case CONST_KIND_STRING: {
+      char const * fmt = NULL;
+      if (imm.string_kind == STRING_KIND_C_STRING)
+        fmt = "c\"%s\"";
+      else
+        fmt = "\"%s\"";
+      res = fprintf(fp, fmt, imm.v_string);
+      // obvious, skip: if (showType) res += fputs(" :string", fp);
       break;
+    }
+    default: INT_FATAL("Unhandled case in switch statement");
   }
   return res;
 }
@@ -254,10 +312,11 @@ coerce_immediate(Immediate *from, Immediate *to) {
       switch (imm->const_kind) { \
         case NUM_KIND_NONE: \
           break; \
+        case NUM_KIND_BOOL: { \
+          imm->v_bool = im1.bool_value() _op im2.bool_value(); break; \
+        } \
         case NUM_KIND_UINT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = im1.v_bool _op im2.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_uint8 = im1.v_uint8 _op im2.v_uint8; break; \
             case INT_SIZE_16: \
@@ -265,15 +324,13 @@ coerce_immediate(Immediate *from, Immediate *to) {
             case INT_SIZE_32: \
               imm->v_uint32 = im1.v_uint32 _op im2.v_uint32; break; \
             case INT_SIZE_64: \
-              imm->v_uint64 = im1.uint_value() _op im2.uint_value(); break; \
+              imm->v_uint64 = im1.v_uint64 _op im2.v_uint64; break; \
             default: INT_FATAL("Unhandled case in switch statement"); \
           } \
           break; \
         } \
         case NUM_KIND_INT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = im1.v_bool _op im2.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_int8 = im1.v_int8 _op im2.v_int8; break; \
             case INT_SIZE_16: \
@@ -281,7 +338,7 @@ coerce_immediate(Immediate *from, Immediate *to) {
             case INT_SIZE_32: \
               imm->v_int32 = im1.v_int32 _op im2.v_int32; break; \
           case INT_SIZE_64: {                                            \
-              imm->v_int64 = im1.int_value() _op im2.int_value(); \
+              imm->v_int64 = im1.v_int64 _op im2.v_int64; \
               break; \
           } \
             default: INT_FATAL("Unhandled case in switch statement"); \
@@ -335,14 +392,13 @@ coerce_immediate(Immediate *from, Immediate *to) {
       switch (imm->const_kind) { \
         case NUM_KIND_NONE: \
           break; \
+        case NUM_KIND_BOOL: { \
+          COMPUTE_UINT_POW(uint64_t, im1.bool_value(), im2.bool_value()); \
+          imm->v_bool = res;                                     \
+          break;                                                  \
+        } \
         case NUM_KIND_UINT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              {                                                         \
-                COMPUTE_UINT_POW(uint8_t, im1.v_bool, im2.v_bool);       \
-                imm->v_bool = res;                                     \
-                break;                                                  \
-              }                                                         \
             case INT_SIZE_8:  \
               {                                                         \
                 COMPUTE_UINT_POW(uint8_t, im1.v_uint8, im2.v_uint8);       \
@@ -363,7 +419,7 @@ coerce_immediate(Immediate *from, Immediate *to) {
               }                                                         \
             case INT_SIZE_64: \
               {                                                         \
-                COMPUTE_UINT_POW(uint64_t, im1.uint_value(), im2.uint_value()); \
+                COMPUTE_UINT_POW(uint64_t, im1.v_uint64, im2.v_uint64); \
                 imm->v_uint64 = res;                                     \
                 break;                                                  \
               }                                                         \
@@ -373,12 +429,6 @@ coerce_immediate(Immediate *from, Immediate *to) {
         } \
         case NUM_KIND_INT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              {                                                         \
-                COMPUTE_INT_POW(int8_t, im1.v_bool, im2.v_bool);       \
-                imm->v_bool = res;                                     \
-                break;                                                  \
-              }                                                         \
             case INT_SIZE_8: \
               {                                                         \
                 COMPUTE_INT_POW(int8_t, im1.v_int8, im2.v_int8);       \
@@ -399,7 +449,7 @@ coerce_immediate(Immediate *from, Immediate *to) {
               }                                                         \
             case INT_SIZE_64: \
               {                                                         \
-                COMPUTE_INT_POW(int64_t, im1.int_value(), im2.int_value()); \
+                COMPUTE_INT_POW(int64_t, im1.v_int64, im2.v_int64); \
                 imm->v_int64 = res;                                     \
                 break;                                                  \
               }                                                         \
@@ -416,10 +466,11 @@ coerce_immediate(Immediate *from, Immediate *to) {
       switch (im1.const_kind) { \
         case NUM_KIND_NONE: \
           break; \
+        case NUM_KIND_BOOL: { \
+          imm->v_bool = im1.bool_value() _op im2.bool_value(); break; \
+        } \
         case NUM_KIND_UINT: { \
           switch (im1.num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = im1.v_bool _op im2.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_bool = im1.v_uint8 _op im2.v_uint8; break; \
             case INT_SIZE_16: \
@@ -427,15 +478,13 @@ coerce_immediate(Immediate *from, Immediate *to) {
             case INT_SIZE_32: \
               imm->v_bool = im1.v_uint32 _op im2.v_uint32; break; \
             case INT_SIZE_64: \
-              imm->v_bool = im1.uint_value() _op im2.uint_value(); break; \
+              imm->v_bool = im1.v_uint64 _op im2.v_uint64; break; \
             default: INT_FATAL("Unhandled case in switch statement"); \
           } \
           break; \
         } \
         case NUM_KIND_INT: { \
           switch (im1.num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = im1.v_bool _op im2.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_bool = im1.v_int8 _op im2.v_int8; break; \
             case INT_SIZE_16: \
@@ -443,7 +492,7 @@ coerce_immediate(Immediate *from, Immediate *to) {
             case INT_SIZE_32: \
               imm->v_bool = im1.v_int32 _op im2.v_int32; break; \
             case INT_SIZE_64: \
-              imm->v_bool = im1.int_value() _op im2.int_value(); break; \
+              imm->v_bool = im1.v_int64 _op im2.v_int64; break; \
             default: INT_FATAL("Unhandled case in switch statement"); \
           } \
           break; \
@@ -463,10 +512,11 @@ coerce_immediate(Immediate *from, Immediate *to) {
       switch (imm->const_kind) { \
         case NUM_KIND_NONE: \
           break; \
+        case NUM_KIND_BOOL: { \
+          imm->v_bool = im1.bool_value() _op im2.bool_value(); break; \
+        } \
         case NUM_KIND_UINT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = im1.v_bool _op im2.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_uint8 = im1.v_uint8 _op im2.v_uint8; break; \
             case INT_SIZE_16: \
@@ -474,15 +524,13 @@ coerce_immediate(Immediate *from, Immediate *to) {
             case INT_SIZE_32: \
               imm->v_uint32 = im1.v_uint32 _op im2.v_uint32; break; \
             case INT_SIZE_64: \
-              imm->v_uint64 = im1.uint_value() _op im2.uint_value(); break; \
+              imm->v_uint64 = im1.v_uint64 _op im2.v_uint64; break; \
             default: INT_FATAL("Unhandled case in switch statement"); \
           } \
           break; \
         } \
         case NUM_KIND_INT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = im1.v_bool _op im2.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_int8 = im1.v_int8 _op im2.v_int8; break; \
             case INT_SIZE_16: \
@@ -490,7 +538,7 @@ coerce_immediate(Immediate *from, Immediate *to) {
             case INT_SIZE_32: \
               imm->v_int32 = im1.v_int32 _op im2.v_int32; break; \
             case INT_SIZE_64: \
-              imm->v_int64 = im1.int_value() _op im2.int_value(); break; \
+              imm->v_int64 = im1.v_int64 _op im2.v_int64; break; \
             default: INT_FATAL("Unhandled case in switch statement"); \
           } \
           break; \
@@ -506,10 +554,11 @@ coerce_immediate(Immediate *from, Immediate *to) {
       switch (imm->const_kind) { \
         case NUM_KIND_NONE: \
           break; \
+        case NUM_KIND_BOOL: { \
+          imm->v_bool = _op im1.bool_value(); break; \
+        } \
         case NUM_KIND_UINT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = _op im1.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_uint8 = _op im1.v_uint8; break; \
             case INT_SIZE_16: \
@@ -524,8 +573,6 @@ coerce_immediate(Immediate *from, Immediate *to) {
         } \
         case NUM_KIND_INT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = _op im1.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_int8 = _op im1.v_int8; break; \
             case INT_SIZE_16: \
@@ -533,7 +580,7 @@ coerce_immediate(Immediate *from, Immediate *to) {
             case INT_SIZE_32: \
               imm->v_int32 = _op im1.v_int32; break; \
             case INT_SIZE_64: \
-              imm->v_int64 = _op im1.int_value(); break;        \
+              imm->v_int64 = _op im1.v_int64; break;        \
             default: INT_FATAL("Unhandled case in switch statement"); \
           } \
           break; \
@@ -553,10 +600,11 @@ coerce_immediate(Immediate *from, Immediate *to) {
       switch (imm->const_kind) { \
         case NUM_KIND_NONE: \
           break; \
+        case NUM_KIND_BOOL: { \
+          imm->v_bool = _op im1.bool_value(); break; \
+        } \
         case NUM_KIND_UINT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = _op im1.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_uint8 = _op im1.v_uint8; break; \
             case INT_SIZE_16: \
@@ -571,8 +619,6 @@ coerce_immediate(Immediate *from, Immediate *to) {
         } \
         case NUM_KIND_INT: { \
           switch (imm->num_index) { \
-            case INT_SIZE_1:  \
-              imm->v_bool = _op im1.v_bool; break; \
             case INT_SIZE_8:  \
               imm->v_int8 = _op im1.v_int8; break; \
             case INT_SIZE_16: \
@@ -580,7 +626,7 @@ coerce_immediate(Immediate *from, Immediate *to) {
             case INT_SIZE_32: \
               imm->v_int32 = _op im1.v_int32; break; \
             case INT_SIZE_64: \
-              imm->v_int64 = _op im1.int_value(); break;        \
+              imm->v_int64 = _op im1.v_int64; break;        \
             default: INT_FATAL("Unhandled case in switch statement"); \
           } \
           break; \
@@ -647,27 +693,53 @@ fold_result(Immediate *im1, Immediate *im2, Immediate *imm) {
     imm->num_index = FLOAT_SIZE_64;
     return;
   }
-  // mixed signed and unsigned
-  if (im1->num_index >= INT_SIZE_64 || im2->num_index >= INT_SIZE_64) {
-    imm->const_kind = NUM_KIND_INT;
+  if (im1->const_kind != NUM_KIND_BOOL && im2->const_kind != NUM_KIND_BOOL) {
+    // mixed signed and unsigned
+    if (im1->num_index >= INT_SIZE_64 || im2->num_index >= INT_SIZE_64) {
+      imm->const_kind = NUM_KIND_INT;
+      imm->num_index = INT_SIZE_64;
+      return;
+    } else if (im1->num_index >= INT_SIZE_32 || im2->num_index >= INT_SIZE_32) {
+      imm->const_kind = NUM_KIND_INT;
+      imm->num_index = INT_SIZE_32;
+      return;
+    } else if (im1->num_index >= INT_SIZE_16 || im2->num_index >= INT_SIZE_16) {
+      imm->const_kind = NUM_KIND_INT;
+      imm->num_index = INT_SIZE_16;
+      return;
+    } else {
+      imm->const_kind = NUM_KIND_INT;
+      imm->num_index = INT_SIZE_8;
+      return;
+    }
+  }
+  // At this point, we can assume we are mixing a bool and either a uint or an
+  // int.
+  if (im2->const_kind == NUM_KIND_BOOL) {
+    // swap im1 to the bool to simplify the code
+    Immediate *t = im1;
+    im1 = im2;
+    im2 = t;
+  }
+  // The const_kind will be either a uint or an int, depending on what the
+  // non-bool argument is.  Keep the larger size
+  if (im1->num_index >= BOOL_SIZE_64 || im2->num_index >= INT_SIZE_64) {
+    imm->const_kind = im2->const_kind;
     imm->num_index = INT_SIZE_64;
     return;
-  } else if (im1->num_index >= INT_SIZE_32 || im2->num_index >= INT_SIZE_32) {
-    imm->const_kind = NUM_KIND_INT;
+  } else if (im1->num_index >= BOOL_SIZE_32 || im2->num_index >= INT_SIZE_32) {
+    imm->const_kind = im2->const_kind;
     imm->num_index = INT_SIZE_32;
     return;
-  } else if (im1->num_index >= INT_SIZE_16 || im2->num_index >= INT_SIZE_16) {
-    imm->const_kind = NUM_KIND_INT;
+  } else if (im1->num_index >= BOOL_SIZE_16 || im2->num_index >= INT_SIZE_16) {
+    imm->const_kind = im2->const_kind;
     imm->num_index = INT_SIZE_16;
     return;
-  } else if (im1->num_index >= INT_SIZE_8 || im2->num_index >= INT_SIZE_8) {
-    imm->const_kind = NUM_KIND_INT;
+  } else {
+    imm->const_kind = im2->const_kind;
     imm->num_index = INT_SIZE_8;
     return;
   }
-  imm->const_kind = NUM_KIND_UINT;
-  imm->num_index = INT_SIZE_1;
-  return;
 }
 
 void
@@ -702,14 +774,14 @@ fold_constant(int op, Immediate *aim1, Immediate *aim2, Immediate *imm) {
     case P_prim_equal:
     case P_prim_notequal:
       fold_result(&im1, &im2, &coerce);
-      imm->const_kind = NUM_KIND_UINT;
-      imm->num_index = INT_SIZE_1;
+      imm->const_kind = NUM_KIND_BOOL;
+      imm->num_index = BOOL_SIZE_SYS;
       break;
     case P_prim_land:
     case P_prim_lor:
     case P_prim_lnot:
-      coerce.const_kind = imm->const_kind = NUM_KIND_UINT;
-      coerce.num_index = imm->num_index = INT_SIZE_1;
+      coerce.const_kind = imm->const_kind = NUM_KIND_BOOL;
+      coerce.num_index = imm->num_index = BOOL_SIZE_SYS;
       break;
     case P_prim_plus:
     case P_prim_minus:
@@ -762,19 +834,20 @@ convert_string_to_immediate(const char *str, Immediate *imm) {
   switch (imm->const_kind) {
     case NUM_KIND_NONE:
       break;
+    case NUM_KIND_BOOL: {
+      if (!strcmp(str, "false")) {
+        imm->v_bool = false;
+      } else if (!strcmp(str, "true")) {
+        imm->v_bool = true;
+      } else if (str[0] == '\0') {
+        imm->v_bool = !imm->v_bool;
+      } else {
+        INT_FATAL("Bad bool value");
+      }
+      break;
+    }
     case NUM_KIND_UINT: {
       switch (imm->num_index) {
-        case INT_SIZE_1:
-          if (!strcmp(str, "false")) {
-            imm->v_bool = false;
-          } else if (!strcmp(str, "true")) {
-            imm->v_bool = true;
-          } else if (str[0] == '\0') {
-            imm->v_bool = !imm->v_bool;
-          } else {
-            INT_FATAL("Bad bool value");
-          }
-          break;
         case INT_SIZE_8: 
           if (str[0] != '\'')
             imm->v_uint8 = str2uint8(str);
@@ -828,3 +901,17 @@ convert_string_to_immediate(const char *str, Immediate *imm) {
       break;
   }
 }
+
+const char* istrFromUserUint(long long unsigned int i) {
+  char s[64];
+  if (sprintf(s, "%llu", i) > 63)
+    INT_FATAL("istr buffer overflow");
+  return astr(s);
+}
+
+const char* istrFromUserInt(long long int i) {
+  char s[64];
+  if (sprintf(s, "%lld", i) > 63)
+    INT_FATAL("istr buffer overflow");
+  return astr(s);
+}
