@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _ASTUTIL_H_
 #define _ASTUTIL_H_
@@ -10,6 +24,7 @@
 #include "alist.h"
 
 #include <vector>
+#include <set>
 
 class Type;
 class FnSymbol;
@@ -23,34 +38,22 @@ class Expr;
 void normalize(BaseAST* base);
 
 // return vec of CallExprs of FnSymbols (no primitives)
-void collectFnCalls(BaseAST* ast, Vec<CallExpr*>& calls);
-void collectFnCallsSTL(BaseAST* ast, std::vector<CallExpr*>& calls);
+void collectFnCalls(BaseAST* ast, std::vector<CallExpr*>& calls);
 
 // collect Stmts and Exprs in the AST and return them in vectors
 // Versions ending in 'STL' use the C++ std::vector class
-void collect_asts(BaseAST* ast, Vec<BaseAST*>& asts);
-void collect_asts_STL(BaseAST* ast, std::vector<BaseAST*>& asts);
-void collect_asts_postorder(BaseAST*, Vec<BaseAST*>& asts);
-void collect_asts_postorder_STL(BaseAST*, std::vector<BaseAST*>& asts);
-void collect_top_asts(BaseAST* ast, Vec<BaseAST*>& asts);
-void collect_top_asts_STL(BaseAST* ast, std::vector<BaseAST*>& asts);
+void collect_asts(BaseAST* ast, std::vector<BaseAST*>& asts);
+void collect_asts_postorder(BaseAST*, std::vector<BaseAST*>& asts);
+void collect_top_asts(BaseAST* ast, std::vector<BaseAST*>& asts);
 void collectExprs(BaseAST* ast, std::vector<Expr*>& exprs);
-void collect_stmts(BaseAST* ast, Vec<Expr*>& stmts);
-void collect_stmts_STL(BaseAST* ast, std::vector<Expr*>& stmts);
-void collectDefExprs(BaseAST* ast, Vec<DefExpr*>& defExprs);
-void collectDefExprsSTL(BaseAST* ast, std::vector<DefExpr*>& defExprs);
-void collectCallExprs(BaseAST* ast, Vec<CallExpr*>& callExprs);
-void collectCallExprsSTL(BaseAST* ast, std::vector<CallExpr*>& callExprs);
-void collectMyCallExprs(BaseAST* ast, Vec<CallExpr*>& callExprs, FnSymbol* fn);
-void collectMyCallExprsSTL(BaseAST* ast, std::vector<CallExpr*>& callExprs, FnSymbol* fn);
-void collectGotoStmts(BaseAST* ast, Vec<GotoStmt*>& gotoStmts);
-void collectGotoStmtsSTL(BaseAST* ast, std::vector<GotoStmt*>& gotoStmts);
-void collectSymExprs(BaseAST* ast, Vec<SymExpr*>& symExprs);
-void collectSymExprsSTL(BaseAST* ast, std::vector<SymExpr*>& symExprs);
-void collectMySymExprs(Symbol* me, Vec<SymExpr*>& symExprs);
+void collect_stmts(BaseAST* ast, std::vector<Expr*>& stmts);
+void collectDefExprs(BaseAST* ast, std::vector<DefExpr*>& defExprs);
+void collectCallExprs(BaseAST* ast, std::vector<CallExpr*>& callExprs);
+void collectMyCallExprs(BaseAST* ast, std::vector<CallExpr*>& callExprs, FnSymbol* fn);
+void collectGotoStmts(BaseAST* ast, std::vector<GotoStmt*>& gotoStmts);
+void collectSymExprs(BaseAST* ast, std::vector<SymExpr*>& symExprs);
 void collectMySymExprs(Symbol* me, std::vector<SymExpr*>& symExprs);
-void collectSymbols(BaseAST* ast, Vec<Symbol*>& symbols);
-void collectSymbolsSTL(BaseAST* ast, std::vector<Symbol*>& symbols);
+void collectSymbols(BaseAST* ast, std::vector<Symbol*>& symbols);
 
 // utility routines for clearing and resetting lineno and filename
 void reset_ast_loc(BaseAST* destNode, astlocT astloc);
@@ -71,11 +74,16 @@ void collectSymbolSetSymExprVec(BaseAST* ast,
 
 //
 // Checks if a callExpr is one of the op= primitives
-// Note, this does not check if a callExpr is an 
+// Note, this does not check if a callExpr is an
 // op= function call (such as before inlining)
 //
 bool isOpEqualPrim(CallExpr* call);
 
+//
+// Checks if a callExpr is a relational operator (<, <=, >, >=, ==, !=)
+//
+bool isRelationalOperator(CallExpr* call);
+
 //
 // Return value & 1 is true if se is a def
 // Return value & 2 is true if se is a use
@@ -169,8 +177,12 @@ void insert_help(BaseAST* ast, Expr* parentExpr, Symbol* parentSymbol);
 ArgSymbol* actual_to_formal( Expr *a);
 Expr* formal_to_actual(CallExpr* call, Symbol* formal);
 
+bool isTypeExpr(Expr* expr);
+
+Symbol* getSvecSymbol(CallExpr* call);
+void collectUsedFnSymbols(BaseAST* ast, std::set<FnSymbol*>& fnSymbols);
+
 // move to resolve when scope resolution is put in resolution directory
 BlockStmt* getVisibilityBlock(Expr* expr);
-void flattenNestedFunctions(Vec<FnSymbol*>& nestedFunctions);
 
 #endif
