@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 //
 // The BaseAST class is the parent class of all AST node types.
@@ -19,10 +33,11 @@
 #ifndef _BASEAST_H_
 #define _BASEAST_H_
 
-#include "chpl.h"
-#include "stringutil.h"
+#include <ostream>
+#include <string>
 
-#include "genret.h"
+#include "map.h"
+#include "vec.h"
 
 //
 // foreach_ast_sep: invoke a 'macro' for every AST node type,
@@ -34,18 +49,23 @@
   macro(PrimitiveType) sep                         \
   macro(EnumType) sep                              \
   macro(AggregateType) sep                         \
+                                                   \
   macro(ModuleSymbol) sep                          \
-  macro(VarSymbol) sep                             \
-  macro(ArgSymbol) sep                             \
-  macro(TypeSymbol) sep                            \
-  macro(FnSymbol) sep                              \
-  macro(EnumSymbol) sep                            \
-  macro(LabelSymbol) sep                           \
+  macro(VarSymbol)    sep                          \
+  macro(ArgSymbol)    sep                          \
+  macro(TypeSymbol)   sep                          \
+  macro(FnSymbol)     sep                          \
+  macro(EnumSymbol)   sep                          \
+  macro(LabelSymbol)  sep                          \
+                                                   \
   macro(SymExpr) sep                               \
   macro(UnresolvedSymExpr) sep                     \
   macro(DefExpr) sep                               \
   macro(CallExpr) sep                              \
+  macro(ContextCallExpr) sep                       \
   macro(NamedExpr) sep                             \
+                                                   \
+  macro(UseStmt) sep                               \
   macro(BlockStmt) sep                             \
   macro(CondStmt) sep                              \
   macro(GotoStmt) sep                              \
@@ -54,12 +74,22 @@
 #define foreach_ast(macro)                         \
   foreach_ast_sep(macro, ;)
 
-//
-// prototype Symbol, Type, Expr, and all AST node types
-//
+class AstVisitor;
+class Expr;
+class GenRet;
+class LcnSymbol;
 class Symbol;
 class Type;
-class Expr;
+
+class BlockStmt;
+class LoopStmt;
+class WhileStmt;
+class WhileDoStmt;
+class DoWhileStmt;
+class ForLoop;
+class CForLoop;
+class ParamForLoop;
+
 #define proto_classes(type) class type
 foreach_ast(proto_classes);
 #undef proto_classes
@@ -79,39 +109,36 @@ foreach_ast(decl_gvecs);
 //
 // type definitions for common maps
 //
-typedef Map<Symbol*,Symbol*> SymbolMap;
+typedef Map<Symbol*,Symbol*>     SymbolMap;
 typedef MapElem<Symbol*,Symbol*> SymbolMapElem;
 
-// get the current AST node id
-extern int lastNodeIDUsed();
-
-// trace various AST node removals
-extern void trace_remove(BaseAST* ast, char flag);
-
 // how an AST node knows its location in the source code
 // (assumed to get copied upon assignment and parameter passing)
-struct astlocT {
-  const char* filename;  // filename of location
-  int lineno;    // line number of location
-  astlocT(int linenoArg, const char* filenameArg):
+class astlocT {
+public:
+  astlocT(int linenoArg, const char* filenameArg) :
     filename(filenameArg), lineno(linenoArg)
     {}
+
+  const char* filename;  // filename of location
+  int         lineno;    // line number of location
 };
 
 //
-// enumerated type of all AST node types (and superclass types)
+// enumerated type of all AST node types
 //
 enum AstTag {
   E_SymExpr,
   E_UnresolvedSymExpr,
   E_DefExpr,
   E_CallExpr,
+  E_ContextCallExpr,
   E_NamedExpr,
+  E_UseStmt,
   E_BlockStmt,
   E_CondStmt,
   E_GotoStmt,
   E_ExternBlockStmt,
-  E_Expr,
 
   E_ModuleSymbol,
   E_VarSymbol,
@@ -120,20 +147,21 @@ enum AstTag {
   E_FnSymbol,
   E_EnumSymbol,
   E_LabelSymbol,
-  E_Symbol,
 
   E_PrimitiveType,
   E_EnumType,
-  E_AggregateType,
-  E_Type,
-
-  E_BaseAST
+  E_AggregateType
 };
 
-//
-// string names of all AST node types (used for debugging)
-//
-extern const char* astTagName[];
+static inline bool isExpr(AstTag tag)
+{ return tag >= E_SymExpr        && tag <= E_ExternBlockStmt; }
+
+static inline bool isSymbol(AstTag tag)
+{ return tag >= E_ModuleSymbol   && tag <= E_LabelSymbol; }
+
+static inline bool isType(AstTag tag)
+{ return tag >= E_PrimitiveType  && tag <= E_AggregateType; }
+
 
 //
 // macros used to define the copy method on all AST node types, and to
@@ -156,6 +184,9 @@ extern const char* astTagName[];
   }                                                                     \
   virtual type* copyInner(SymbolMap* map)
 
+// This should be expanded verbatim and overloaded, so we don't create a map if
+// internal is false.
+// copyInner must now copy flags.
 #define DECLARE_SYMBOL_COPY(type)                                       \
   type* copy(SymbolMap* map = NULL, bool internal = false) {            \
     SymbolMap localMap;                                                 \
@@ -163,7 +194,6 @@ extern const char* astTagName[];
       map = &localMap;                                                  \
     type* _this = copyInner(map);                                       \
     _this->astloc = astloc;                                             \
-    _this->copyFlags(this);                                             \
     map->put(this, _this);                                              \
     if (!internal)                                                      \
       update_symbols(_this, map);                                       \
@@ -180,38 +210,58 @@ extern const char* astTagName[];
 // abstract parent of all AST node types
 //
 class BaseAST {
- public:
-  AstTag astTag; // BaseAST subclass
-  int id;        // Unique ID
-  astlocT astloc; // Location of this node in the source code
+public:
+  virtual GenRet    codegen()                                          = 0;
+  virtual bool      inTree()                                           = 0;
+  virtual Type*     typeInfo()                                         = 0;
+  virtual void      verify()                                           = 0;
+  virtual void      accept(AstVisitor* visitor)                        = 0;
 
-  BaseAST(AstTag type);
-  virtual ~BaseAST() { }
-  virtual void verify() = 0;
-  virtual BaseAST* copy(SymbolMap* map = NULL, bool internal = false) = 0;
-  virtual BaseAST* copyInner(SymbolMap* map) = 0;
-  virtual bool inTree(void) = 0;
-  virtual GenRet codegen() = 0;
+  const char*       fname()                                      const;
+  int               linenum()                                    const;
+  const char*       stringLoc()                                  const;
 
-  const char* stringLoc(void);
-  ModuleSymbol* getModule();
-  FnSymbol* getFunction();
-  int linenum() { return astloc.lineno; }
-  const char* fname() { return astloc.filename; }
+  FnSymbol*         getFunction();
+  ModuleSymbol*     getModule();
+  Type*             getValType();
+  Type*             getRefType();
 
-  virtual Type* typeInfo(void) = 0;
-  Type* getValType();
-  Type* getRefType();
-  Type* getWideRefType();
+  const char*       astTagAsString()                             const;
+
+  AstTag            astTag;     // BaseAST subclass
+  int               id;         // Unique ID
+  astlocT           astloc;     // Location of this node in the source code
+
+  void                printTabs(std::ostream *file, unsigned int tabs);
+  virtual void        printDocsDescription(const char *doc, std::ostream *file, unsigned int tabs);
+
+  static  const       std::string tabText;
+
+protected:
+                    BaseAST(AstTag type);
+  virtual          ~BaseAST();
+
+private:
+                    BaseAST();
+
+  Type*             getWideRefType();
 };
 
 GenRet baseASTCodegen(BaseAST* ast);
 GenRet baseASTCodegenInt(int x);
 GenRet baseASTCodegenString(const char* str);
 
+// get the current AST node id
+int    lastNodeIDUsed();
+
+// trace various AST node removals
+void   trace_remove(BaseAST* ast, char flag);
+
+
 //
 // macro to update the global line number used to set the line number
-// of an AST node when it is constructed
+// of an AST node when it is constructed - or to print out the line
+// number of code related to a core dump.
 //
 // This should be used before constructing new nodes to make sure the
 // line number is correctly set. The global line number reverts to
@@ -221,55 +271,55 @@ GenRet baseASTCodegenString(const char* str);
 // todo - should we add it to DECLARE_COPY/DECLARE_SYMBOL_COPY ?
 //
 #define SET_LINENO(ast) astlocMarker markAstLoc(ast->astloc)
+
 extern astlocT currentAstLoc;
 
-struct astlocMarker {
-  astlocT previousAstLoc;
+class astlocMarker {
+public:
+  astlocMarker(astlocT newAstLoc);
+  astlocMarker(int lineno, const char* filename);
+  ~astlocMarker();
 
-  // constructor, invoked upon SET_LINENO
-  astlocMarker(astlocT newAstLoc)
-    : previousAstLoc(currentAstLoc)
-  {
-    //previousAstLoc = currentAstLoc;
-    currentAstLoc = newAstLoc;
-  }
-  // constructor, for special occasions
-  astlocMarker(int lineno, const char* filename)
-    : previousAstLoc(currentAstLoc)
-  {
-    currentAstLoc.lineno = lineno;
-    currentAstLoc.filename = astr(filename);
-  }
-  // destructor, invoked upon leaving SET_LINENO's scope
-  ~astlocMarker() {
-    currentAstLoc = previousAstLoc;
-  }
+  astlocT previousAstLoc;
 };
 
 //
 // vectors of modules
 //
 extern Vec<ModuleSymbol*> allModules;  // contains all modules
-extern Vec<ModuleSymbol*> userModules; // contains main + user modules
-extern Vec<ModuleSymbol*> mainModules; // contains main modules
+extern Vec<ModuleSymbol*> userModules; // contains user modules
 
 //
 // class test inlines: determine the dynamic type of a BaseAST*
 //
-static inline bool isExpr(BaseAST* a)
-{ return a && a->astTag < E_Expr; }
-static inline bool isSymbol(BaseAST* a)
-{ return a && a->astTag > E_Expr && a->astTag < E_Symbol; }
-static inline bool isType(BaseAST* a)
-{ return (a && a->astTag > E_Symbol && a->astTag < E_Type); }
+static inline bool isExpr(const BaseAST* a)
+{ return a && isExpr(a->astTag); }
+
+static inline bool isSymbol(const BaseAST* a)
+{ return a && isSymbol(a->astTag); }
+
+static inline bool isType(const BaseAST* a)
+{ return a && isType(a->astTag); }
+
+static inline bool isLcnSymbol(const BaseAST* a)
+{ return a && (a->astTag == E_ArgSymbol || a->astTag == E_VarSymbol); }
+
+static inline bool isCallExpr(const BaseAST* a)
+{ return a && (a->astTag == E_CallExpr || a->astTag == E_ContextCallExpr); }
+
+
+#define def_is_ast(Type)                          \
+  static inline bool is##Type(const BaseAST* a)   \
+  {                                               \
+    return a && a->astTag == E_##Type;            \
+  }
 
-#define def_is_ast(Type) \
-  static inline bool is##Type(BaseAST* a) { return a && a->astTag == E_##Type; }
 def_is_ast(SymExpr)
 def_is_ast(UnresolvedSymExpr)
 def_is_ast(DefExpr)
-def_is_ast(CallExpr)
+def_is_ast(ContextCallExpr)
 def_is_ast(NamedExpr)
+def_is_ast(UseStmt)
 def_is_ast(BlockStmt)
 def_is_ast(CondStmt)
 def_is_ast(GotoStmt)
@@ -286,17 +336,30 @@ def_is_ast(EnumType)
 def_is_ast(AggregateType)
 #undef def_is_ast
 
+bool isLoopStmt(const BaseAST* a);
+bool isWhileStmt(const BaseAST* a);
+bool isWhileDoStmt(const BaseAST* a);
+bool isDoWhileStmt(const BaseAST* a);
+bool isParamForLoop(const BaseAST* a);
+bool isForLoop(const BaseAST* a);
+bool isCoforallLoop(const BaseAST* a);
+bool isCForLoop(const BaseAST* a);
+
 //
 // safe downcast inlines: downcast BaseAST*, Expr*, Symbol*, or Type*
 //   note: toDerivedClass is equivalent to dynamic_cast<DerivedClass*>
 //
 #define def_to_ast(Type) \
-  static inline Type * to##Type(BaseAST* a) { return is##Type(a) ? (Type*)a : NULL; }
+  static inline Type * to##Type(BaseAST* a) { return is##Type(a) ? (Type*)a : NULL; } \
+  static inline const Type * toConst##Type(const BaseAST* a) \
+    { return is##Type(a) ? (const Type*)a : NULL; }
+
 def_to_ast(SymExpr)
 def_to_ast(UnresolvedSymExpr)
 def_to_ast(DefExpr)
-def_to_ast(CallExpr)
+def_to_ast(ContextCallExpr)
 def_to_ast(NamedExpr)
+def_to_ast(UseStmt)
 def_to_ast(BlockStmt)
 def_to_ast(CondStmt)
 def_to_ast(GotoStmt)
@@ -314,7 +377,46 @@ def_to_ast(PrimitiveType)
 def_to_ast(EnumType)
 def_to_ast(AggregateType)
 def_to_ast(Type)
+
+def_to_ast(LoopStmt);
+def_to_ast(WhileStmt);
+def_to_ast(WhileDoStmt);
+def_to_ast(DoWhileStmt);
+def_to_ast(ForLoop);
+def_to_ast(CForLoop);
+def_to_ast(ParamForLoop);
+
 #undef def_to_ast
+
+static inline LcnSymbol* toLcnSymbol(BaseAST* a)
+{
+  return isLcnSymbol(a) ? (LcnSymbol*) a : NULL;
+}
+
+static inline const LcnSymbol* toConstLcnSymbol(const BaseAST* a)
+{
+  return isLcnSymbol(a) ? (const LcnSymbol*) a : NULL;
+}
+
+CallExpr* getDesignatedCall(const ContextCallExpr* a);
+
+static inline CallExpr* toCallExpr(BaseAST* a)
+{
+  if (!a) return NULL;
+  if (a->astTag == E_CallExpr) return (CallExpr*) a;
+  if (a->astTag == E_ContextCallExpr) return getDesignatedCall((ContextCallExpr*)a);
+  return NULL;
+}
+
+static inline const CallExpr* toConstCallExpr(const BaseAST* a)
+{
+  if (!a) return NULL;
+  if (a->astTag == E_CallExpr) return (const CallExpr*) a;
+  if (a->astTag == E_ContextCallExpr) return getDesignatedCall((const ContextCallExpr*)a);
+  return NULL;
+}
+
+
 //
 // traversal macros
 //
@@ -340,6 +442,9 @@ def_to_ast(Type)
     AST_CALL_CHILD(_a, CallExpr, baseExpr, call, __VA_ARGS__);          \
     AST_CALL_LIST(_a, CallExpr, argList, call, __VA_ARGS__);            \
     break;                                                              \
+  case E_ContextCallExpr:                                               \
+    AST_CALL_LIST(_a, ContextCallExpr, options, call, __VA_ARGS__);     \
+    break;                                                              \
   case E_NamedExpr:                                                     \
     AST_CALL_CHILD(_a, NamedExpr, actual, call, __VA_ARGS__);           \
     break;                                                              \
@@ -348,12 +453,50 @@ def_to_ast(Type)
     AST_CALL_CHILD(_a, DefExpr, exprType, call, __VA_ARGS__);           \
     AST_CALL_CHILD(_a, DefExpr, sym, call, __VA_ARGS__);                \
     break;                                                              \
-  case E_BlockStmt:                                                     \
-    AST_CALL_LIST(_a, BlockStmt, body, call, __VA_ARGS__);              \
-    AST_CALL_CHILD(_a, BlockStmt, blockInfo, call, __VA_ARGS__);        \
-    AST_CALL_CHILD(_a, BlockStmt, modUses, call, __VA_ARGS__);          \
-    AST_CALL_CHILD(_a, BlockStmt, byrefVars, call, __VA_ARGS__);        \
+  case E_UseStmt:                                                       \
+    AST_CALL_CHILD(_a, UseStmt, src, call, __VA_ARGS__);                \
     break;                                                              \
+                                                                               \
+  case E_BlockStmt: {                                                          \
+    BlockStmt* stmt = toBlockStmt(_a);                                         \
+                                                                               \
+    if (stmt->isWhileDoStmt() == true) {                                       \
+      AST_CALL_LIST (stmt, WhileStmt,    body,           call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, WhileStmt,    condExprGet(),  call, __VA_ARGS__);   \
+                                                                               \
+    } else if (stmt->isDoWhileStmt()  == true) {                               \
+      AST_CALL_LIST (stmt, WhileStmt,    body,           call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, WhileStmt,    condExprGet(),  call, __VA_ARGS__);   \
+                                                                               \
+    } else if (stmt->isForLoop()      == true) {                               \
+      AST_CALL_LIST (stmt, ForLoop,      body,           call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, ForLoop,      indexGet(),     call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, ForLoop,      iteratorGet(),  call, __VA_ARGS__);   \
+                                                                               \
+    } else if (stmt->isCoforallLoop() == true) {                               \
+      AST_CALL_LIST (stmt, ForLoop,      body,           call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, ForLoop,      indexGet(),     call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, ForLoop,      iteratorGet(),  call, __VA_ARGS__);   \
+                                                                               \
+    } else if (stmt->isCForLoop()     == true) {                               \
+      AST_CALL_LIST (stmt, CForLoop,     body,           call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, CForLoop,     initBlockGet(), call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, CForLoop,     testBlockGet(), call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, CForLoop,     incrBlockGet(), call, __VA_ARGS__);   \
+                                                                               \
+    } else if (stmt->isParamForLoop() == true) {                               \
+      AST_CALL_LIST (stmt, ParamForLoop, body,           call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, ParamForLoop, resolveInfo(),  call, __VA_ARGS__);   \
+                                                                               \
+    } else  {                                                                  \
+      AST_CALL_LIST (stmt, BlockStmt,    body,           call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, BlockStmt,    blockInfoGet(), call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, BlockStmt,    modUses,        call, __VA_ARGS__);   \
+      AST_CALL_CHILD(stmt, BlockStmt,    byrefVars,      call, __VA_ARGS__);   \
+    }                                                                          \
+    break;                                                                     \
+  }                                                                            \
+                                                                               \
   case E_CondStmt:                                                      \
     AST_CALL_CHILD(_a, CondStmt, condExpr, call, __VA_ARGS__);          \
     AST_CALL_CHILD(_a, CondStmt, thenStmt, call, __VA_ARGS__);          \
@@ -375,7 +518,6 @@ def_to_ast(Type)
     break;                                                              \
   case E_FnSymbol:                                                      \
     AST_CALL_LIST(_a, FnSymbol, formals, call, __VA_ARGS__);            \
-    AST_CALL_CHILD(_a, FnSymbol, setter, call, __VA_ARGS__);            \
     AST_CALL_CHILD(_a, FnSymbol, body, call, __VA_ARGS__);              \
     AST_CALL_CHILD(_a, FnSymbol, where, call, __VA_ARGS__);             \
     AST_CALL_CHILD(_a, FnSymbol, retExprType, call, __VA_ARGS__);       \
