@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 // checks.h
 #ifndef CHECKS_H
@@ -11,6 +25,7 @@
 void check_parse();
 void check_checkParsed();
 void check_readExternC();
+void check_expandExternArrayCalls();
 void check_cleanup();
 void check_scopeResolve();
 void check_flattenClasses();
@@ -29,7 +44,6 @@ void check_callDestructors();
 void check_lowerIterators();
 void check_parallel();
 void check_prune();
-void check_complex2record();
 void check_bulkCopyRecords();
 void check_removeUnnecessaryAutoCopyCalls();
 void check_inlineFunctions();
