@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef CLANGUTIL_H
 #define CLANGUTIL_H
@@ -18,6 +32,8 @@
 #include "clangSupport.h"
 #endif
 
+void cleanupExternC();
+
 #include "files.h"
 #include "genret.h"
 
@@ -41,8 +57,9 @@ class LayeredValueTable
 {
   private:
     struct Storage {
-      // note: this following nested struct should be a union
-      // since only one of the following is necessary?
+      // We use both the cTypeDecl and cValueDecl fields
+      // in some situations. (e.g. struct stat vs fn stat).
+      // Generally only one field will be set.
       struct s_u {
         llvm::Value *value;
         llvm::BasicBlock *block;
@@ -50,7 +67,8 @@ class LayeredValueTable
         // Note that clang will cache clang->llvm for types, at least
         // It would be possible for us to cache everything here
         // but that is probably redundant.
-        clang::NamedDecl *cdecl;
+        clang::TypeDecl *cTypeDecl;
+        clang::ValueDecl *cValueDecl;
         // Macros get stored here.
         VarSymbol* chplVar;
       } u;
@@ -65,7 +83,8 @@ class LayeredValueTable
         u.value = NULL;
         u.block = NULL;
         u.type = NULL;
-        u.cdecl = NULL;
+        u.cTypeDecl = NULL;
+        u.cValueDecl = NULL;
         u.chplVar = NULL;
         isLVPtr = GEN_VAL;
         isUnsigned = false;
@@ -73,8 +92,8 @@ class LayeredValueTable
       }
     };
    
-    typedef llvm::StringMap<Storage> map_type;
-    typedef std::list<map_type> layers_type;
+    typedef llvm::StringMap<Storage> map_type;//just map, key is string, value is Storage
+    typedef std::list<map_type> layers_type;// each element of the list is a map
     typedef layers_type::iterator layer_iterator;
     typedef map_type::iterator value_iterator;
     
@@ -89,12 +108,13 @@ class LayeredValueTable
     void addGlobalValue(llvm::StringRef name, GenRet gend);
     void addGlobalType(llvm::StringRef name, llvm::Type *type);
     void addGlobalCDecl(clang::NamedDecl* cdecl);
+    void addGlobalCDecl(llvm::StringRef name, clang::NamedDecl* cdecl);
     void addGlobalVarSymbol(llvm::StringRef name, VarSymbol* var);
     void addBlock(llvm::StringRef name, llvm::BasicBlock *block);
     GenRet getValue(llvm::StringRef name);
     llvm::BasicBlock *getBlock(llvm::StringRef name);
     llvm::Type *getType(llvm::StringRef name);
-    clang::NamedDecl* getCDecl(llvm::StringRef name);
+    void getCDecl(llvm::StringRef name, clang::TypeDecl** cTypeOut, clang::ValueDecl** cValueOut);
     VarSymbol* getVarSymbol(llvm::StringRef name);
  
     bool isAlreadyInChapelAST(llvm::StringRef name);
@@ -115,11 +135,15 @@ void finishCodegenLLVM();
 void runClang(const char* just_parse_filename);
 
 bool lookupInExternBlock(ModuleSymbol* module, const char* name,
-                             clang::NamedDecl** cDecl, Type** chplType);
+                         clang::TypeDecl** cTypeOut,
+                         clang::ValueDecl** cValueOut,
+                         Type** chplTypeOut);
 bool alreadyConvertedExtern(ModuleSymbol* module, const char* name);
 bool setAlreadyConvertedExtern(ModuleSymbol* module, const char* name);
-#endif // HAVE_LLVM
 
 extern fileinfo gAllExternCode;
+extern fileinfo gChplCompilationConfig;
+
+#endif // HAVE_LLVM
 
 #endif //CLANGUTIL_H
