@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef CODEGEN_H
 #define CODEGEN_H
@@ -111,7 +125,7 @@ struct GenInfo {
   GlobalToWideInfo globalToWideInfo;
 
   // Optimizations to apply immediately after code-generating a fn
-  llvm::FunctionPassManager* FPM_postgen;
+  LEGACY_FUNCTION_PASS_MANAGER* FPM_postgen;
 
   // When using a function, just use cgModule->GetAddrOfFunction,
   // which will cause cgModule to emit it on Builder->Release.
@@ -131,8 +145,8 @@ struct GenInfo {
 
 
 extern GenInfo* gGenInfo;
-extern int gMaxVMT;
-extern int stmtCount;
+extern int      gMaxVMT;
+extern int      gStmtCount;
 
 #ifdef HAVE_LLVM
 void setupClang(GenInfo* info, std::string rtmain);
