@@ -1,55 +1,39 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _driver_H_
 #define _driver_H_
 
-#include <cstdio>
-#include "timer.h"
-#include "map.h"
 #include "chpl.h"
+#include "map.h"
 
-// For versions of bison beyond 2.3, this struct and the following
-// two defines should be moved back into chapel.ypp into a %code requires
-// block so that YYLTYPE is near the YYLLOC_DEFAULT definition.
-typedef struct YYLTYPE {    
-       int first_line;      
-       int first_column;    
-       int last_line;       
-       int last_column;     
-       char *comment;       
-} YYLTYPE;
-#define YYLTYPE_IS_DECLARED 1
-#define YYLTYPE_IS_TRIVIAL 1
+#include <cstdio>
+#include <map>
+#include <string>
 
-extern const char *chplBinaryName;
+class Timer;
 
-extern FILE* html_index_file;
-extern FILE* deletedIdHandle;
-extern char deletedIdFilename[FILENAME_MAX+1];
-#define deletedIdON (deletedIdFilename[0] != '\0')
-
-extern char log_dir[FILENAME_MAX+1];
-extern char log_module[FILENAME_MAX+1];
-extern char log_symbol[FILENAME_MAX+1];
-extern bool fLogIds;
-
-extern int instantiation_limit;
-
-extern int fdump_html;
-extern char fdump_html_chpl_home[FILENAME_MAX+1];
-extern bool fdump_html_incude_system_modules;
-extern bool fdump_html_wrap_lines;
-extern bool fdump_html_print_block_IDs;
-
-extern int currentPassNo;
-extern const char* currentPassName;
+extern int  instantiation_limit;
 
 // optimization control flags
 extern bool fFastFlag;
-extern int fConditionalDynamicDispatchLimit;
+extern int  fConditionalDynamicDispatchLimit;
 extern bool fNoBoundsChecks;
 extern bool fNoCopyPropagation;
 extern bool fNoDeadCodeElimination;
@@ -59,8 +43,13 @@ extern bool fNoInlineIterators;
 extern bool fNoloopInvariantCodeMotion;
 extern bool fNoInline;
 extern bool fNoLiveAnalysis;
+extern bool fNoFormalDomainChecks;
 extern bool fNoLocalChecks;
 extern bool fNoNilChecks;
+extern bool fNoStackChecks;
+extern bool fNoCastChecks;
+extern bool fMungeUserIdents;
+extern bool fEnableTaskTracking;
 extern bool fLLVMWideOpt;
 
 extern bool fNoRemoteValueForwarding;
@@ -68,26 +57,35 @@ extern bool fNoRemoveCopyCalls;
 extern bool fNoScalarReplacement;
 extern bool fNoTupleCopyOpt;
 extern bool fNoOptimizeLoopIterators;
+extern bool fNoVectorize;
 extern bool fNoPrivatization;
 extern bool fNoOptimizeOnClauses;
 extern bool fNoRemoveEmptyRecords;
-extern int optimize_on_clause_limit;
-extern int scalar_replace_limit;
-extern int tuple_copy_limit;
+extern bool fRemoveUnreachableBlocks;
+extern int  optimize_on_clause_limit;
+extern int  scalar_replace_limit;
+extern int  tuple_copy_limit;
+
 
 extern bool report_inlining;
-extern char CHPL_HOME[FILENAME_MAX];
+
+// Chapel Envs
+bool useDefaultEnv(std::string key);
+
+extern std::map<std::string, const char*> envMap;
+
+extern char CHPL_HOME[FILENAME_MAX+1];
 
 extern const char* CHPL_HOST_PLATFORM;
 extern const char* CHPL_HOST_COMPILER;
 extern const char* CHPL_TARGET_PLATFORM;
 extern const char* CHPL_TARGET_COMPILER;
+extern const char* CHPL_TARGET_ARCH;
 extern const char* CHPL_LOCALE_MODEL;
 extern const char* CHPL_COMM;
 extern const char* CHPL_COMM_SUBSTRATE;
 extern const char* CHPL_GASNET_SEGMENT;
 extern const char* CHPL_TASKS;
-extern const char* CHPL_THREADS;
 extern const char* CHPL_LAUNCHER;
 extern const char* CHPL_TIMERS;
 extern const char* CHPL_MEM;
@@ -101,6 +99,9 @@ extern const char* CHPL_WIDE_POINTERS;
 extern const char* CHPL_LLVM;
 extern const char* CHPL_AUX_FILESYS;
 
+extern bool  printPasses;
+extern FILE* printPassesFile;
+
 // Set true if CHPL_WIDE_POINTERS==struct.
 // In that case, the code generator emits structures
 // for wide pointers. Otherwise, wide pointers are
@@ -109,47 +110,49 @@ extern bool widePointersStruct;
 
 extern char fExplainCall[256];
 extern int  explainCallID;
+extern int  breakOnResolveID;
 extern char fExplainInstantiation[256];
 /// If true, then print additional (disambiguation) information about
 /// resolution.
 extern bool fExplainVerbose;
 extern bool fPrintCallStackOnError;
 extern bool fPrintIDonError;
-extern bool fCLineNumbers;
+extern bool fPrintModuleResolution;
 extern bool fPrintEmittedCodeSize;
 extern char fPrintStatistics[256];
 extern bool fPrintDispatch;
 extern bool fGenIDS;
 extern bool fLocal;
+extern bool fIgnoreLocalClasses;
 extern bool fHeterogeneous;
-extern bool fieeefloat;
+extern int  ffloatOpt;
 extern int  fMaxCIdentLen;
 
 extern bool llvmCodegen;
 
+// Is the cache for remote data enabled?
+extern bool fCacheRemote;
+
 // externC allows blocks like extern { } to be parsed
 // with clang and then added to the enclosing module's scope
 extern bool externC;
 extern char breakOnCodegenCname[256];
+extern int breakOnCodegenID;
 
 enum { LS_DEFAULT=0, LS_STATIC, LS_DYNAMIC };
-extern int fLinkStyle;
 
-extern int debugParserLevel;
-extern int debugShortLoc;
+extern int  fLinkStyle;
+
+extern int  debugParserLevel;
+extern int  debugShortLoc;
 extern bool fLibraryCompile;
+extern bool fUseNoinit;
 extern bool no_codegen;
-extern bool genExternPrototypes;
 extern bool developer;
 extern bool fVerify;
-extern int num_constants_per_variable;
+extern int  num_constants_per_variable;
 extern bool printCppLineno;
 
-extern bool fDocs;
-extern bool fDocsAlphabetize;
-extern char fDocsCommentLabel[256];
-extern char fDocsFolder[256];
-extern bool fDocsTextOnly;
 extern char defaultDist[256];
 extern char mainModuleName[256];
 extern bool printSearchDirs;
@@ -157,26 +160,23 @@ extern bool printModuleFiles;
 extern bool ignore_warnings;
 extern bool ignore_errors;
 extern bool ignore_errors_for_pass;
-extern int squelch_header_errors;
+extern int  squelch_header_errors;
 extern bool fWarnConstLoops;
 
 extern bool fReportOptimizedLoopIterators;
+extern bool fReportOrderIndependentLoops;
 extern bool fReportOptimizedOn;
 extern bool fReportPromotion;
 extern bool fReportScalarReplace;
 extern bool fReportDeadBlocks;
+extern bool fReportDeadModules;
 
-extern bool debugCCode, optimizeCCode;
-
-extern bool fEnableTimers;
-extern Timer timer1;
-extern Timer timer2;
-extern Timer timer3;
-extern Timer timer4;
-extern Timer timer5;
+extern bool debugCCode;
+extern bool optimizeCCode;
+extern bool specializeCCode;
 
 extern bool fNoMemoryFrees;
-extern int numGlobalsOnHeap;
+extern int  numGlobalsOnHeap;
 extern bool preserveInlinedLineNumbers;
 
 extern int breakOnID;
@@ -188,7 +188,10 @@ extern char compileVersion[64];
 
 // This flag is useful for testing
 // the compiler but breaks the language!
-extern bool fNoInternalModules;
 extern bool fMinimalModules;
 
+// Set to true if we want to use the experimental
+// Interactive Programming Environmment (IPE) mode.
+extern bool fUseIPE;
+
 #endif
