@@ -1,20 +1,35 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _files_H_
 #define _files_H_
 
 #include <cstdio>
-#include <vector>
+#include <map>
 #include <string>
+#include <vector>
 #include "vec.h"
 
 extern char executableFilename[FILENAME_MAX+1];
 extern char saveCDir[FILENAME_MAX+1];
-extern char ccflags[256];
-extern char ldflags[256];
+extern std::string ccflags;
+extern std::string ldflags;
 extern bool ccwarnings;
 extern Vec<const char*> incDirs;
 extern int numLibFlags;
@@ -29,13 +44,15 @@ struct fileinfo {
 void codegen_makefile(fileinfo* mainfile, const char** tmpbinname=NULL, bool skip_compile_link=false);
 
 void ensureDirExists(const char* /* dirname */, const char* /* explanation */);
+const char* getCwd();
+const char* makeTempDir(const char* dirPrefix);
+void deleteDir(const char* dirname);
 void deleteTmpDir(void);
 const char* objectFileForCFile(const char* cfile);
 
 const char* genIntermediateFilename(const char* filename);
 
 void openCFile(fileinfo* fi, const char* name, const char* ext = NULL);
-void appendCFile(fileinfo* fi, const char* name, const char* ext = NULL);
 void closeCFile(fileinfo* fi, bool beautifyIt=true);
 
 fileinfo* openTmpFile(const char* tmpfilename, const char* mode = "w");
@@ -49,26 +66,34 @@ bool isChplSource(const char* filename);
 bool isCHeader(const char* filename);
 bool isCSource(const char* filename);
 bool isObjFile(const char* filename);
-void testInputFiles(int numFilenames, char* filename[]);
+void addSourceFiles(int numFilenames, const char* filename[]);
+void addSourceFile(const char* filename);
 const char* nthFilename(int i);
 void addLibInfo(const char* filename);
 void addIncInfo(const char* incDir);
 
 void genIncludeCommandLineHeaders(FILE* outfile);
 
-const char* createGDBFile(int argc, char* argv[]);
+const char* createDebuggerFile(const char* debugger, int argc, char* argv[]);
 
-const char* runUtilScript(const char* script);
+std::string runPrintChplEnv(std::map<std::string, const char*> varMap);
+std::string runCommand(std::string& command);
 
 void setupModulePaths(void);
 void addFlagModulePath(const char* newpath);
 void addDashMsToUserPath(void);
 void addModulePathFromFilename(const char* filename);
 
-const char* modNameToFilename(const char* modName, bool isInternal, 
-                              bool* isStandard);
+const char* modNameToFilename(const char* modName,
+                              bool        isInternal,
+                              bool*       isStandard);
 const char* stdModNameToFilename(const char* modName);
 
+const char* filenameToModulename(const char* filename);
+
+const char* pathNameForInternalFile(const char* baseName);
+const char* pathNameForStandardFile(const char* baseName);
+
 void printModuleSearchPath(void);
 
 const char* getIntermediateDirName();
