@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef CHPL_FLAGS_H_
 #define CHPL_FLAGS_H_
@@ -29,6 +43,8 @@
 
 
 class BaseAST;
+class Symbol;
+class TypeSymbol;
 
 
 enum Flag {
@@ -36,17 +52,20 @@ enum Flag {
 # define symbolFlag(NAME,PRAGMA,MAPNAME,COMMENT) NAME,
 # include "flags_list.h"
 # undef symbolFlag
-  NUM_FLAGS
+  NUM_FLAGS,
+  FLAG_FIRST = FLAG_UNKNOWN + 1, // index of the first flag
+  FLAG_LAST  = NUM_FLAGS - 1     // index of the last flag
 };
 
 // only meaningful flags are allowed
 #define CHECK_FLAG(FLAG) \
-  INT_ASSERT(FLAG_UNKNOWN < (FLAG) && (FLAG) < NUM_FLAGS)
+  INT_ASSERT(FLAG_FIRST <= (FLAG) && (FLAG) <= FLAG_LAST)
 
 
 Flag pragma2flag(const char* str);
 void initFlags();
 void viewFlags(BaseAST* sym);
+void writeFlags(FILE* fp, Symbol* sym);
 TypeSymbol* getDataClassType(TypeSymbol* ts);
 void setDataClassType(TypeSymbol* ts, TypeSymbol* ets);
 
