@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef GENRET_H
 #define GENRET_H
@@ -11,13 +25,13 @@
 // need llvm::Value, Type
 #include "llvmUtil.h"
 
-#define GEN_VAL 0
-#define GEN_PTR 1
+#define GEN_VAL      0
+#define GEN_PTR      1
 #define GEN_WIDE_PTR 2
 
 class BaseAST;
 class Type;
-struct GenRet;
+class GenRet;
 
 extern GenRet baseASTCodegen(BaseAST* ast);
 extern GenRet baseASTCodegenInt(int x);
@@ -68,7 +82,8 @@ extern GenRet baseASTCodegenString(const char* str);
    int (generate an int)
 
  */
-struct GenRet {
+class GenRet {
+public:
   // When generating c, this should be set.
   // expression or type goes here
   std::string c;
