@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 /*****************************************************************************
 
@@ -47,21 +61,22 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #define _list_H_
 
 #include <cstddef>
-#if defined(__FreeBSD__) && (__FreeBSD_version < 500000)
+
 #include <stdint.h>
-#endif
 
 template <class C> class SLink {
  public:
   C *next;
   SLink() : next(NULL) {};
 };
+
 #define GetSLinkNext(_c, _e, _o) (((SLink<_c> *)(void*)(((intptr_t)(void*)_e) + _o))->next)
 
 template <class C> struct Link : SLink<C> {
   C *prev;
   Link() : prev(NULL) {}
 };
+
 #define GetLinkNext(_c, _e, _o) (((Link<_c> *)(void*)(((intptr_t)(void*)_e) + _o))->next)
 #define GetLinkPrev(_c, _e, _o) (((Link<_c> *)(void*)(((intptr_t)(void*)_e) + _o))->prev)
 
@@ -75,6 +90,7 @@ template <class C, int o> class SLL {
 
   SLL() : head(NULL) {}
 };
+
 #define SList(_c, _f)  SLL<_c, offsetof(_c, _f)>
 #define forl_LL(_c, _p, _l) for (_c *_p = (_l).head; _p; _p = (_l).next_link(_p))
 
@@ -92,6 +108,7 @@ template <class C, int o> class DLL {
 
   DLL() : head(NULL) {}
 };
+
 #define DList(_c, _f)  DLL<_c, offsetof(_c, _f)>
 
 template <class C, int o> class Queue : public DLL<C, o> {
@@ -110,6 +127,7 @@ template <class C, int o> class Queue : public DLL<C, o> {
   
   Queue() : tail(NULL) {}
 };
+
 #define Que(_c, _f) Queue<_c, offsetof(_c, _f)>
 
 template <class C, int o>
@@ -126,6 +144,7 @@ class CountQueue : public Queue<C, o> {
   inline void append(CountQueue<C, o> &q);
   inline void append_clear(CountQueue<C, o> &q);
 };
+
 #define CountQue(_c, _f) CountQueue<_c, offsetof(_c, _f)>
 
 template <class C>
