@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 /*****************************************************************************
 
@@ -60,11 +74,13 @@ template <class K, class C> class MapElem {
   K     key;
   C     value;
   bool operator==(MapElem &e) { return e.key == key; }
-  operator unsigned int(void) { return (unsigned int)(uintptr_t)key; }
-  MapElem(unsigned int x) { assert(!x); key = 0; }
-  MapElem(K akey, C avalue) : key(akey), value(avalue) {}
-  MapElem(MapElem &e) : key(e.key), value(e.value) {}
-  MapElem() : key(0) {}
+
+  operator uintptr_t()     { return (uintptr_t) key; }
+
+  MapElem()                 : key(0)                     { }
+  MapElem(K akey, C avalue) : key(akey),  value(avalue)  { }
+  MapElem(MapElem &e)       : key(e.key), value(e.value) { }
+  MapElem(unsigned long x)                               { assert(!x); key = 0; }
 };
 
 template <class K, class C> class Map : public Vec<MapElem<K,C> > {
