@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _OPTIMIZATIONS_H_
 #define _OPTIMIZATIONS_H_
@@ -15,11 +29,19 @@
 // was hoisted into this common include file for convenience.
 #define DEBUG_CP 0
 
+#include "map.h"
+#include "vec.h"
+
 #include <map>
+#include <vector>
 
+class BaseAST;
 class BitVec;
+class BlockStmt;
+class FnSymbol;
+class Symbol;
+class SymExpr;
 
-void collapseBlocks(BlockStmt* block);
 void removeUnnecessaryGotos(FnSymbol* fn);
 void removeUnusedLabels(FnSymbol* fn);
 size_t localCopyPropagation(FnSymbol* fn);
@@ -51,11 +73,5 @@ freeDefUseChains(std::map<SymExpr*,Vec<SymExpr*>*>& DU,
 void
 remoteValueForwarding(Vec<FnSymbol*>& fns);
 
-void
-narrowWideReferences();
-
-// Debug functions from narrowWideReferences.cpp
-void printNode(BaseAST*);
-void printNode(int id);
 
 #endif
