@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _PASSES_H_
 #define _PASSES_H_
@@ -26,12 +40,12 @@ void checkParsed();
 void checkResolved();
 void cleanup();
 void codegen();
-void complex2record();
-void createTaskFunctions();
 void copyPropagation();
+void createTaskFunctions();
 void cullOverReferences();
 void deadCodeElimination();
 void docs();
+void expandExternArrayCalls();
 void flattenClasses();
 void flattenFunctions();
 void inlineFunctions();
@@ -53,9 +67,9 @@ void refPropagation();
 void removeEmptyRecords();
 void removeUnnecessaryAutoCopyCalls();
 void removeWrapRecords();
-void returnStarTuplesByRefArgs();
 void resolve();
 void resolveIntents();
+void returnStarTuplesByRefArgs();
 void scalarReplace();
 void scopeResolve();
 void verify();
@@ -75,10 +89,30 @@ void checkReturnTypesHaveRefTypes();
 // utility functions in pass-containing code files
 //
 
+// buildDefaultFunctions.cpp
+void buildDefaultDestructor(AggregateType* ct);
+
+// createTaskFunctions.cpp -> implementForallIntents.cpp
+extern Symbol* markPruned;
+extern Symbol* markUnspecified;
+void markOuterVarsWithIntents(CallExpr* byrefVars, SymbolMap& uses);
+void replaceVarUses(Expr* topAst, SymbolMap& vars);
+void pruneThisArg(Symbol* parent, SymbolMap& uses);
+
+// deadCodeElimination.cpp
+void deadBlockElimination();
+
+// flattenFunctions.cpp
+void flattenNestedFunctions(Vec<FnSymbol*>& nestedFunctions);
+
+// callDestructors.cpp
+void insertReferenceTemps(CallExpr* call);
+
 // parallel.cpp
-bool isRefWideString(Type* t);
-bool isWideString(Type* t);
 Type* getOrMakeRefTypeDuringCodegen(Type* type);
 Type* getOrMakeWideTypeDuringCodegen(Type* refType);
 
+// type.cpp
+void initForTaskIntents();
+
 #endif
