@@ -1,13 +1,31 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _PRIM_H_
 #define _PRIM_H_
 
 #include "chpl.h"
 
+class CallExpr;
+class Type;
+class VarSymbol;
+
 enum PrimitiveTag {
   PRIM_UNKNOWN = 0,    // use for any primitives not in this list
 
@@ -16,6 +34,9 @@ enum PrimitiveTag {
   PRIM_MOVE,
   PRIM_INIT,
   PRIM_NO_INIT,
+  PRIM_TYPE_INIT,       // Used in a context where only a type is needed.
+                        // Establishes the type of the result without
+                        // generating code.
   PRIM_REF_TO_STRING,
   PRIM_RETURN,
   PRIM_YIELD,
@@ -69,6 +90,8 @@ enum PrimitiveTag {
   PRIM_GET_REAL,            // get complex real component
   PRIM_GET_IMAG,            // get complex imag component
   PRIM_QUERY,               // query expression primitive
+  PRIM_QUERY_PARAM_FIELD,
+  PRIM_QUERY_TYPE_FIELD,
 
   PRIM_ADDR_OF,             // set a reference to a value
   PRIM_DEREF,               // dereference a reference
@@ -105,10 +128,6 @@ enum PrimitiveTag {
   PRIM_GET_END_COUNT,
   PRIM_SET_END_COUNT,
 
-  PRIM_PROCESS_TASK_LIST,
-  PRIM_EXECUTE_TASKS_IN_LIST,
-  PRIM_FREE_TASK_LIST,
-
   PRIM_GET_SERIAL,              // get serial state
   PRIM_SET_SERIAL,              // set serial state to true or false
 
@@ -121,13 +140,15 @@ enum PrimitiveTag {
   PRIM_CAST,
   PRIM_DYNAMIC_CAST,
   PRIM_TYPEOF,
-  PRIM_USE,
   PRIM_USED_MODULES_LIST,       // used modules in BlockStmt::modUses
   PRIM_TUPLE_EXPAND,
   PRIM_TUPLE_AND_EXPAND,
 
   PRIM_CHPL_COMM_GET,           // Direct calls to the Chapel comm layer
   PRIM_CHPL_COMM_PUT,           // may eventually add others (e.g., non-blocking)
+  PRIM_CHPL_COMM_ARRAY_GET,
+  PRIM_CHPL_COMM_ARRAY_PUT,
+  PRIM_CHPL_COMM_REMOTE_PREFETCH,
   PRIM_CHPL_COMM_GET_STRD,      // Direct calls to the Chapel comm layer for strided comm
   PRIM_CHPL_COMM_PUT_STRD,      //  may eventually add others (e.g., non-blocking)
 
@@ -151,6 +172,9 @@ enum PrimitiveTag {
   PRIM_BLOCK_WHILEDO_LOOP,      // BlockStmt::blockInfo - while do loop (cond)
   PRIM_BLOCK_DOWHILE_LOOP,      // BlockStmt::blockInfo - do while loop (cond)
   PRIM_BLOCK_FOR_LOOP,          // BlockStmt::blockInfo - for loop (index, iterator)
+  PRIM_BLOCK_C_FOR_LOOP,        // BlockStmt::blockInfo -
+                                //   C for loop (initExpr, testExpr, incrExpr)
+
   PRIM_BLOCK_BEGIN,             // BlockStmt::blockInfo - begin block
   PRIM_BLOCK_COBEGIN,           // BlockStmt::blockInfo - cobegin block
   PRIM_BLOCK_COFORALL,          // BlockStmt::blockInfo - coforall block
@@ -161,17 +185,13 @@ enum PrimitiveTag {
   PRIM_BLOCK_LOCAL,             // BlockStmt::blockInfo - local block
   PRIM_BLOCK_UNLOCAL,           // BlockStmt::blockInfo - unlocal local block
 
+  PRIM_FORALL_LOOP,             // BlockStmt::byrefVars - forall loop body
   PRIM_TO_LEADER,
   PRIM_TO_FOLLOWER,
+  PRIM_TO_STANDALONE,
 
   PRIM_DELETE,
 
-  PRIM_GC_CC_INIT,              // Initialize heap for copy-collecting
-  PRIM_GC_ADD_ROOT,             // Add a root variable for garbage collection
-  PRIM_GC_ADD_NULL_ROOT,        // Add a root and point it to NULL
-  PRIM_GC_DELETE_ROOT,          // Remove a root variable for garbage collection
-  PRIM_GC_CLEANUP,              // Free GC heaps
-
   PRIM_CALL_DESTRUCTOR,         // call destructor on type (do not free)
 
   PRIM_LOGICAL_FOLDER,          // Help fold logical && and ||
@@ -180,6 +200,7 @@ enum PrimitiveTag {
 
   PRIM_WIDE_GET_NODE,           // Get just the node portion of a wide pointer.
   PRIM_WIDE_GET_ADDR,           // Get just the address portion of a wide pointer.
+  PRIM_IS_WIDE_PTR,             // Returns true if the symbol is represented by a wide poitner.
 
   PRIM_ON_LOCALE_NUM,           // specify a particular localeID for an on clause.
 
@@ -196,19 +217,14 @@ enum PrimitiveTag {
   PRIM_CREATE_FN_TYPE,
 
   PRIM_STRING_COPY,
-  PRIM_STRING_NORMALIZE,        // Set the size field in a (wide) string.
-  PRIM_STRING_FROM_C_STRING,
-  PRIM_C_STRING_FROM_STRING,
   PRIM_CAST_TO_VOID_STAR,       // Cast the object argument to void*.
 
   PRIM_RT_ERROR,
   PRIM_RT_WARNING,
 
   PRIM_NEW_PRIV_CLASS,
-  PRIM_NUM_PRIV_CLASSES,
   PRIM_GET_PRIV_CLASS,
 
-  PRIM_NEXT_UINT32,
   PRIM_GET_USER_LINE,
   PRIM_GET_USER_FILE,
 
@@ -222,19 +238,33 @@ enum PrimitiveTag {
   PRIM_GET_SVEC_MEMBER,
   PRIM_GET_SVEC_MEMBER_VALUE,
 
-  PRIM_VMT_CALL,        // virtual method call
+  PRIM_VIRTUAL_METHOD_CALL,
 
   PRIM_NUM_FIELDS,
   PRIM_FIELD_NUM_TO_NAME,
+  PRIM_FIELD_NAME_TO_NUM,
   PRIM_FIELD_VALUE_BY_NUM,
-  PRIM_FIELD_ID_BY_NUM,
-  PRIM_FIELD_VALUE_BY_NAME,
   PRIM_IS_UNION_TYPE,
   PRIM_IS_ATOMIC_TYPE,
+  PRIM_IS_REF_ITER_TYPE,
+
+  PRIM_IS_POD,
+
+  PRIM_COERCE,
+
+  PRIM_CALL_RESOLVES,
+  PRIM_METHOD_CALL_RESOLVES,
 
   PRIM_ENUM_MIN_BITS,
   PRIM_ENUM_IS_SIGNED,
 
+  PRIM_START_RMEM_FENCE,
+  PRIM_FINISH_RMEM_FENCE,
+
+  PRIM_LOOKUP_FILENAME,   // Given an index, get a given filename (c_string)
+
+  PRIM_GET_COMPILER_VAR,
+
   NUM_KNOWN_PRIMS
 };
 
