@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 // checks.cpp
 
@@ -10,7 +24,7 @@
 #include "expr.h"
 #include "passes.h"
 #include "primitive.h"
-#include "../resolution/resolution.h"
+#include "resolution.h"
 
 //
 // Static function declarations.
@@ -19,21 +33,29 @@
 //
 
 static void check_afterEveryPass(); // Checks to be performed after every pass.
+static void check_afterScopeResolve(); // Checks to be performeed after the
+                                       // scopeResolve pass.
 static void check_afterNormalization(); // Checks to be performed after
                                         // normalization.
 static void check_afterResolution(); // Checks to be performed after every pass
                                      // following resolution.
 static void check_afterCallDestructors(); // Checks to be performed after every
                                           // pass following callDestructors.
+static void check_afterLowerIterators();
+static void checkAggregateTypes(); // Checks that class and record types have
+                                   // default initializers and default type
+                                   // constructors.
 static void checkResolveRemovedPrims(void); // Checks that certain primitives
                                             // are removed after resolution
 static void checkTaskRemovedPrims(); // Checks that certain primitives are
                                      // removed after task functions are
                                      // created.
+static void checkLowerIteratorsRemovedPrims();
 static void checkFlagRelationships(); // Checks expected relationships between
                                       // flags.
 static void checkAutoCopyMap();
 static void checkFormalActualBaseTypesMatch();
+static void checkRetTypeMatchesRetVarType();
 static void checkFormalActualTypesMatch();
 
 
@@ -57,6 +79,11 @@ void check_readExternC()
   // Suggestion: Ensure extern C types defined.
 }
 
+void check_expandExternArrayCalls()
+{
+  check_afterEveryPass();
+}
+
 void check_cleanup()
 {
   check_afterEveryPass();
@@ -66,6 +93,7 @@ void check_cleanup()
 void check_scopeResolve()
 {
   check_afterEveryPass();
+  check_afterScopeResolve();
   // Suggestion: Ensure identifiers resolved to scope.
 }
 
@@ -145,6 +173,11 @@ void check_cullOverReferences()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterResolution();
+
+  // No ContextCallExprs should remain in the tree.
+  forv_Vec(ContextCallExpr, cc, gContextCallExprs) {
+    INT_FATAL("ContextCallExpr should no longer be in AST");
+  }
 }
 
 void check_callDestructors()
@@ -160,6 +193,7 @@ void check_lowerIterators()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 //  check_afterResolution(); // Oho! Iterator functions do not obey the invariant
   // checked in checkReturnPaths() [semanticChecks.cpp:250].
   // So check_afterResolution has been disabled in this and all subsequent post-pass
@@ -173,6 +207,7 @@ void check_parallel()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
   // Suggestion: Ensure parallelization applied (if not --local).
 }
 
@@ -181,22 +216,16 @@ void check_prune()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
   // Suggestion: Ensure no dead classes or functions.
 }
 
-void check_complex2record()
-{
-  check_afterEveryPass();
-  check_afterNormalization();
-  check_afterCallDestructors();
-  // Suggestion: Ensure no more constants or variables of complex type.
-}
-
 void check_bulkCopyRecords()
 {
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_removeUnnecessaryAutoCopyCalls()
@@ -204,6 +233,7 @@ void check_removeUnnecessaryAutoCopyCalls()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
   // Suggestion: Ensure no unnecessary autoCopy calls.
 }
 
@@ -212,6 +242,7 @@ void check_inlineFunctions()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_scalarReplace()
@@ -219,6 +250,7 @@ void check_scalarReplace()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
   // Suggestion: Ensure no constant expresions.
 }
 
@@ -227,6 +259,7 @@ void check_refPropagation()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_copyPropagation()
@@ -234,13 +267,15 @@ void check_copyPropagation()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
-}
+  check_afterLowerIterators();}
+
 
 void check_deadCodeElimination()
 {
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators(); 
   // Suggestion: Ensure no dead code.
 }
 
@@ -249,6 +284,7 @@ void check_removeWrapRecords()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
   // Suggestion: Ensure no more wrap records.
 }
 
@@ -257,6 +293,7 @@ void check_removeEmptyRecords()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
   // Suggestion: Ensure no empty records.
 }
 
@@ -265,6 +302,7 @@ void check_localizeGlobals()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_loopInvariantCodeMotion()
@@ -272,6 +310,7 @@ void check_loopInvariantCodeMotion()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_prune2()
@@ -279,6 +318,7 @@ void check_prune2()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
   // Suggestion: Ensure no dead classes or functions.
 }
 
@@ -287,6 +327,7 @@ void check_returnStarTuplesByRefArgs()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_insertWideReferences()
@@ -294,6 +335,7 @@ void check_insertWideReferences()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_optimizeOnClauses()
@@ -301,6 +343,7 @@ void check_optimizeOnClauses()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_addInitCalls()
@@ -308,6 +351,7 @@ void check_addInitCalls()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_insertLineNumbers()
@@ -315,6 +359,7 @@ void check_insertLineNumbers()
   check_afterEveryPass();
   check_afterNormalization();
   check_afterCallDestructors();
+  check_afterLowerIterators();
 }
 
 void check_codegen()
@@ -342,6 +387,14 @@ void check_afterEveryPass()
   }
 }
 
+static void check_afterScopeResolve()
+{
+  if (fVerify)
+  {
+    checkAggregateTypes();
+  }
+}
+
 // Checks that should remain true after the normalization pass is complete.
 static void check_afterNormalization()
 {
@@ -365,6 +418,7 @@ static void check_afterResolution()
 // See test/extern/hilde/namedExtern.chpl.
 //    checkNoUnresolveds();
     checkFormalActualBaseTypesMatch();
+    checkRetTypeMatchesRetVarType();
     checkAutoCopyMap();
   }
 }
@@ -387,6 +441,28 @@ static void check_afterCallDestructors()
 }
 
 
+static void check_afterLowerIterators()
+{
+  checkLowerIteratorsRemovedPrims();
+}
+
+
+//
+// Checks that class and record types have a default initializer and a default
+// type constructor.
+//
+static void checkAggregateTypes()
+{
+  forv_Vec(AggregateType, at, gAggregateTypes)
+  {
+    if (! at->defaultInitializer)
+      INT_FATAL(at, "aggregate type has no initializer");
+    if (! at->defaultTypeConstructor)
+      INT_FATAL(at, "aggregate type has no default type constructor");
+  }
+}
+
+
 //
 // Checks that certain primitives are removed after function resolution
 //
@@ -398,6 +474,7 @@ checkResolveRemovedPrims(void) {
         case PRIM_BLOCK_PARAM_LOOP:
         case PRIM_INIT:
         case PRIM_NO_INIT:
+        case PRIM_TYPE_INIT:
         case PRIM_LOGICAL_FOLDER:
         case PRIM_TYPEOF:
         case PRIM_TYPE_TO_STRING:
@@ -408,7 +485,11 @@ checkResolveRemovedPrims(void) {
         case PRIM_IS_SUBTYPE:
         case PRIM_TUPLE_EXPAND:
         case PRIM_QUERY:
+        case PRIM_QUERY_PARAM_FIELD:
+        case PRIM_QUERY_TYPE_FIELD:
         case PRIM_ERROR:
+        case PRIM_FORALL_LOOP:
+        case PRIM_COERCE:
           if (call->parentSymbol)
             INT_FATAL("Primitive should no longer be in AST");
           break;
@@ -441,6 +522,22 @@ checkTaskRemovedPrims()
       }
 }
 
+static void 
+checkLowerIteratorsRemovedPrims()
+{
+  forv_Vec(CallExpr, call, gCallExprs)
+    if (call->primitive)
+      switch(call->primitive->tag)
+      {
+       case PRIM_YIELD:
+        if (call->parentSymbol)
+          INT_FATAL("Primitive should no longer be in AST");
+        break;
+       default:
+        break;
+      }
+}
+
 // Some flags imply other flags.
 // Make sure that these relations always hold.
 static void
@@ -473,22 +570,43 @@ checkAutoCopyMap()
   }
 }
 
+
+// TODO: Can this be merged with checkFormalActualTypesMatch()?
 static void
 checkFormalActualBaseTypesMatch()
 {
   forv_Vec(CallExpr, call, gCallExprs)
   {
+    if (! call->parentSymbol)
+      // Call is not in tree
+      continue;
+
+    // Only look at calls in functions that have been resolved.
+    if (! call->parentSymbol->hasFlag(FLAG_RESOLVED))
+      continue;
+
     if (FnSymbol* fn = call->isResolved())
     {
       if (fn->hasFlag(FLAG_EXTERN))
         continue;
+
+      if (! fn->hasFlag(FLAG_RESOLVED))
+        continue;
+
       for_formals_actuals(formal, actual, call)
       {
         if (actual->typeInfo() == dtNil) {
-          if (!isClass(formal->type))
-            INT_FATAL(call, "nil is passed to the formal %s of a non-class type",
-                      formal->name);
-          continue;
+          if (formal->type == dtNil)
+            // Exact match, so OK.
+            continue;
+
+          if (isClass(formal->type))
+            // dtNil can be converted to any class type, so OK.
+            continue;
+
+          // All other cases == error.
+          INT_FATAL(call, "nil is passed to the formal %s of a non-class type",
+                    formal->name);
         }
 
         if (formal->type->getValType() != actual->typeInfo()->getValType())
@@ -502,6 +620,26 @@ checkFormalActualBaseTypesMatch()
   }
 }
 
+// After resolution the retType field is just a cached version of the type of
+// the return value variable.
+static void
+checkRetTypeMatchesRetVarType()
+{
+  forv_Vec(FnSymbol, fn, gFnSymbols)
+  {
+    if (fn->isIterator())
+      // Iterators break this rule.
+      // retType is the type of the iterator record
+      // The return value type is the type of the index the iterator returns.
+      continue;
+    if (fn->hasFlag(FLAG_AUTO_II))
+      // auto ii functions break this rule, but only during the time that
+      // they are prototypes.  After the body is filled in, they should obey it.
+      continue;
+    INT_ASSERT(fn->retType == fn->getReturnSymbol()->type);
+  }
+}
+
 static void
 checkFormalActualTypesMatch()
 {
@@ -511,13 +649,21 @@ checkFormalActualTypesMatch()
     {
       if (fn->hasFlag(FLAG_EXTERN))
         continue;
+
       for_formals_actuals(formal, actual, call)
       {
         if (actual->typeInfo() == dtNil) {
-          if (!isClass(formal->type))
-            INT_FATAL(call, "nil is passed to the formal %s of a non-class type",
-                      formal->name);
-          continue;
+          if (formal->type == dtNil)
+            // Exact match, so OK.
+            continue;
+
+          if (isClass(formal->type))
+            // dtNil can be converted to any class type, so OK.
+            continue;
+
+          // All other cases == error.
+          INT_FATAL(call, "nil is passed to the formal %s of a non-class type",
+                    formal->name);
         }
 
         if (formal->type != actual->typeInfo())
@@ -531,4 +677,3 @@ checkFormalActualTypesMatch()
   }
 }
 
-
