@@ -1,18 +1,33 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 // copyPropagation.cpp
 //
+#include "optimizations.h"
+
 #include "astutil.h"
 #include "bb.h"
 #include "bitVec.h"
 #include "expr.h"
-#include "optimizations.h"
 #include "passes.h"
+#include "stlUtil.h"
 #include "stmt.h"
-#include "view.h"
 
 //#############################################################################
 //# COPY PROPAGATION
@@ -24,7 +39,7 @@
 //#  (move y x)
 //#  ("foo" ... x ... )
 //# That is, cut out the middle man.  If it then happens that y is unused, its
-//# declaration and the moved that defines it can be removed (by
+//# declaration and the move that defines it can be removed (by
 //# deadVariableElimination).
 //#
 //# Clearly, we can only make the substitution if neither x nor y is reassigned
@@ -88,15 +103,30 @@
 //# the entire function.
 //#############################################################################
 
+//#############################################################################
+//# CAVEAT:
+//#
+//# Reference tracking currently does not traverse record field assignments, so
+//# given:
+//#  (move rx (addr_of x))
+//#  (.= record field rx)
+//#  (move rx2 (.v record field))
+//#  (move x 3)                    // Creates pair (x, 3)
+//#  (move (deref rx2) 7)          // Should kill (x, 3) but doesn't
+//# the current implementation does not discover that x has been overwritten
+//# through rx2.  This can lead to the generation of incorrect code.
+//#
+//# Because scalar replacement unwraps records, it eliminates many (if not all)
+//# of the cases that could cause this error to occur.  But the fact that some
+//# test cases fail (e.g. test_nested_var_iterator3) when scalar propagation is
+//# turned off means this is a known weakness that should be addressed.
+//#############################################################################
+
 
 static size_t s_repl_count; ///< The number of pairs replaced by GCP this pass.
 static size_t s_ref_repl_count; ///< The number of references replaced this pass.
 
 
-#define USE_NEW_IMPLEMENTATION 1
-
-#if USE_NEW_IMPLEMENTATION
-
 //#############################################################################
 //# LOCAL COPY PROPAGATION
 //#############################################################################
@@ -143,7 +173,14 @@ static void extractReferences(Expr* expr,
   // We're only interested in call expressions.
   if (CallExpr* call = toCallExpr(expr))
   {
-    // Only the move primitive creates an available pair.
+    // Consider primitives that can create aliases:
+    // 1. An assign or move primitive that has ref variables on both sides.
+    // 2. An assign or move that has an 'addr of' primitive on its rhs.
+    // 3. A field assignment or extraction that has ref variables on both
+    //    sides. (not implemented)
+    // 4. A field assignment that has an 'addr of' primitive on its rhs. (not
+    //    implemented)
+    // 5. An assign or move that has a PRIM_GET_MEMBER on the rhs. (not implemented)
     if (call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN))
     {
       SymExpr* lhe = toSymExpr(call->get(1)); // Left-Hand Expression
@@ -224,6 +261,8 @@ static bool isDef(SymExpr* se)
     return false;
   if (toBlockStmt(se->parentExpr))
     return false;
+  if (isDefExpr(se->parentExpr))
+    return false;
 
   CallExpr* call = toCallExpr(se->parentExpr);
   if (FnSymbol* fn = call->isResolved())
@@ -315,6 +354,8 @@ static bool isUse(SymExpr* se)
     return true;
   if (toBlockStmt(se->parentExpr))
     return true;
+  if (isDefExpr(se->parentExpr))
+    return false;
 
   CallExpr* call = toCallExpr(se->parentExpr);
   if (FnSymbol* fn = call->isResolved())
@@ -330,12 +371,23 @@ static bool isUse(SymExpr* se)
   }
   else
   {
+    INT_ASSERT(call->primitive);
     switch(call->primitive->tag)
     {
      default:
       return true;
      case PRIM_MOVE:
      case PRIM_ASSIGN:
+     case PRIM_ADD_ASSIGN:
+     case PRIM_SUBTRACT_ASSIGN:
+     case PRIM_MULT_ASSIGN:
+     case PRIM_DIV_ASSIGN:
+     case PRIM_MOD_ASSIGN:
+     case PRIM_LSH_ASSIGN:
+     case PRIM_RSH_ASSIGN:
+     case PRIM_AND_ASSIGN:
+     case PRIM_OR_ASSIGN:
+     case PRIM_XOR_ASSIGN:
       if (se == call->get(1))
         return false;
       return true;
@@ -347,6 +399,8 @@ static bool isUse(SymExpr* se)
       return false;
      case PRIM_CHPL_COMM_GET:
      case PRIM_CHPL_COMM_PUT:
+     case PRIM_CHPL_COMM_ARRAY_GET:
+     case PRIM_CHPL_COMM_ARRAY_PUT:
      case PRIM_CHPL_COMM_GET_STRD:
      case PRIM_CHPL_COMM_PUT_STRD:
       // ('comm_get/put' locAddr locale widePtr len)
@@ -355,6 +409,13 @@ static bool isUse(SymExpr* se)
       if (se == call->get(2) || se == call->get(4))
         return true;
       return false;
+     case PRIM_CHPL_COMM_REMOTE_PREFETCH:
+      // comm prefetch locale widePtr len
+      // second argument is an address
+      // first and third are values.
+      if (se == call->get(1) || se == call->get(3))
+        return true;
+      return false;
      case PRIM_SET_MEMBER:
       // The first operand works like a reference, and the second is a field
       // name.  Only the third is a replaceable use.
@@ -401,6 +462,8 @@ static bool isRefUse(SymExpr* se)
     return false;
   if (toBlockStmt(se->parentExpr))
     return false;
+  if (isDefExpr(se->parentExpr))
+    return false;
 
   CallExpr* call = toCallExpr(se->parentExpr);
   if (FnSymbol* fn = call->isResolved())
@@ -491,6 +554,8 @@ static CallExpr* derefUse(SymExpr* se)
     return NULL;
   if (toBlockStmt(se->parentExpr))
     return NULL;
+  if (isDefExpr(se->parentExpr))
+    return NULL;
 
   CallExpr* call = toCallExpr(se->parentExpr);
   if (call->isPrimitive(PRIM_DEREF))
@@ -507,6 +572,7 @@ static void propagateCopies(std::vector<SymExpr*>& symExprs,
   // Scan the SymExprs in the current block, looking for ones that can be replaced.
   for_vector(SymExpr, se, symExprs)
   {
+    INT_ASSERT(! isUnresolvedSymExpr(se));
     // Replace an alias with its definition, using the current set of
     // available pairs.
     if (isUse(se))
@@ -516,19 +582,6 @@ static void propagateCopies(std::vector<SymExpr*>& symExprs,
       // If so, replace the alias with its definition.
       if (alias_def_pair != available.end())
       {
-        // This special case is needed to prevent propagating a string literal
-        // into the return statement of a function.  Right now,
-        // insertWideReferences does not automatically widen the result of a
-        // function returning a narrow string.  Instead, it relies on there
-        // being a return value variable that is widened before it is
-        // returned.  If we return a literal, that variable doesn't exist, so
-        // the return type ends up being narrow (which is not correct).
-        // After chapel strings become records, this special-case code can be
-        // removed.
-        if (CallExpr* call = toCallExpr(se->parentExpr))
-          if (call->isPrimitive(PRIM_RETURN) &&
-              se->var->typeInfo() == dtString)
-            continue;
 #if DEBUG_CP
         if (debug > 0)
           printf("Replacing %s[%d] with %s[%d]\n",
@@ -710,6 +763,21 @@ static void extractCopies(Expr* expr,
             !rhs->type->symbol->hasFlag(FLAG_REF))
           return; // Not a pair.
 
+        // Assignment between two references "peels" the RHS, so performs a
+        // copy of the *value* of the RHS into the lvalue pointed to by the
+        // LHS.  In contrast a move between two refs just copies one ref into
+        // the other.
+        // In the PRIM_ASSIGN case, the two references may point to different
+        // objects, so the primitive does not create a pair as far as CP is
+        // concerned.
+        // TODO: Based on the semantics of PRIM_ASSIGN, if both LHS and RHS are
+        // references, we *could* create a pair between the symbols they refer
+        // to.  There might be a neat recursive way to do this....
+        if (lhs->type->symbol->hasFlag(FLAG_REF) &&
+            rhs->type->symbol->hasFlag(FLAG_REF))
+          if (call->isPrimitive(PRIM_ASSIGN))
+            return;
+
         // We can't make substitutions if the lhs or rhs may change at any
         // time.
         if (maybeVolatile(lhe) || maybeVolatile(rhe))
@@ -740,10 +808,10 @@ static void extractCopies(Expr* expr,
 // if a variable can be accessed concurrently, so we assume that a separate
 // pass has already marked such symbols.
 static void
-localCopyPropagationCore(BasicBlock* bb,
-                         AvailableMap& available,
+localCopyPropagationCore(BasicBlock*          bb,
+                         AvailableMap&        available,
                          ReverseAvailableMap& ravailable,
-                         RefMap& refs)
+                         RefMap&              refs)
 {
   for_vector(Expr, expr, bb->exprs)
   {
@@ -753,7 +821,8 @@ localCopyPropagationCore(BasicBlock* bb,
 #endif
 
     std::vector<SymExpr*> symExprs;
-    collectSymExprsSTL(expr, symExprs);
+
+    collectSymExprs(expr, symExprs);
 
     propagateCopies(symExprs, available, refs);
 
@@ -769,19 +838,22 @@ localCopyPropagationCore(BasicBlock* bb,
 //
 size_t localCopyPropagation(FnSymbol* fn)
 {
-  buildBasicBlocks(fn);
-
   RefMap refs;
+
+  BasicBlock::buildBasicBlocks(fn);
+
   computeRefMap(fn, refs);
 
-  s_repl_count = s_ref_repl_count = 0;
+  s_repl_count     = 0;
+  s_ref_repl_count = 0;
+
   for_vector(BasicBlock, bb1, *fn->basicBlocks)
   {
     AvailableMap available;
     ReverseAvailableMap ravailable;
     localCopyPropagationCore(bb1, available, ravailable, refs);
   }
-  
+
   return s_repl_count + s_ref_repl_count;
 }
 
@@ -792,10 +864,11 @@ size_t localCopyPropagation(FnSymbol* fn)
 
 
 static void createPairSet(std::vector<BitVec*>& set,
-                          size_t nbbs, size_t size)
+                          size_t                nbbs,
+                          size_t                size)
 {
   // Create a BitVec of length size for each block.
-  for (size_t i = 0; i < nbbs; ++i) 
+  for (size_t i = 0; i < nbbs; ++i)
     set.push_back(new BitVec(size));
 }
 
@@ -853,14 +926,13 @@ static void computeKillSets(FnSymbol* fn,
     BasicBlock* bb2 = (*fn->basicBlocks)[i];
 
     // Collect up the set of symbols killed in this block in killSet.
-    // You were here!!!
     std::set<Symbol*> killSet;
     for_vector(Expr, expr, bb2->exprs)
     {
-      Vec<SymExpr*> symExprs;
+      std::vector<SymExpr*> symExprs;
       collectSymExprs(expr, symExprs);
 
-      forv_Vec(SymExpr, se, symExprs)
+      for_vector(SymExpr, se, symExprs)
       {
         // Invalidate a symbol if it is redefined.
         if (isDef(se))
@@ -886,7 +958,7 @@ static void computeKillSets(FnSymbol* fn,
     // Use killSet to initialize the KILL set for this block.
     // It's OK if we include the pairs from this block in KILL[i] because we
     // put them back when we add in the COPY set.
-    for (int j = 0; j < KILL[i]->size(); ++j)
+    for (size_t j = 0; j < KILL[i]->size(); ++j)
       if (killSet.find(availablePairs[j].first) != killSet.end() ||
           killSet.find(availablePairs[j].second) != killSet.end())
         KILL[i]->set(j);
@@ -924,8 +996,10 @@ static void initCopySets(std::vector<BitVec*>& COPY, std::vector<size_t>& ends,
 // When these are corrected and the test becomes true, then we can drop back
 // to the simpler form given here:
 #ifdef INLINING_DOES_NOT_LEAVE_INTERNAL_BASIC_BLOCKS_WITHOUT_PREDECESSORS
-static void initInSets(std::vector<BitVec*>& IN, size_t nbbs)
+static void initInSets(std::vector<BitVec*>& IN, FnSymbol* fn)
 {
+  size_t nbbs = fn->basicBlocks->size();
+
   // Note that we start with i = 1, so that IN[0] is left as all zeroes.
   for (size_t i = 1; i < nbbs; i++)
     IN[i]->set();
@@ -968,16 +1042,19 @@ static void initInSets(std::vector<BitVec*>& IN, FnSymbol* fn)
 // immediately after it would be redundant.
 //
 size_t globalCopyPropagation(FnSymbol* fn) {
-  buildBasicBlocks(fn);
-
   RefMap refs;
+
+  BasicBlock::buildBasicBlocks(fn);
+
   computeRefMap(fn, refs);
 
-  size_t nbbs = fn->basicBlocks->size();
+  size_t                     nbbs = fn->basicBlocks->size();
 
   std::vector<AvailablePair> availablePairs;
-  std::vector<size_t> ends;
+  std::vector<size_t>        ends;
+
   extractAvailablePairs(fn, refs, availablePairs, ends);
+
   size_t size = availablePairs.size();
 
 #if DEBUG_CP
@@ -986,7 +1063,7 @@ size_t globalCopyPropagation(FnSymbol* fn) {
     printf("\n");
     list_view(fn);
 
-    printBasicBlocks(fn);
+    BasicBlock::printBasicBlocks(fn);
   }
 #endif
 
@@ -1001,31 +1078,33 @@ size_t globalCopyPropagation(FnSymbol* fn) {
   createPairSet(OUT,  nbbs, size);
 
   initCopySets(COPY, ends, nbbs);
+
   computeKillSets(fn, refs, availablePairs, KILL);
+
   initInSets(IN, fn);
 
 #if DEBUG_CP
   if (debug > 0)
   {
-    printf("COPY:\n"); printBitVectorSets(COPY);
-    printf("KILL:\n"); printBitVectorSets(KILL);
+    printf("COPY:\n"); BasicBlock::printBitVectorSets(COPY);
+    printf("KILL:\n"); BasicBlock::printBitVectorSets(KILL);
   }
 #endif
 
-  forwardFlowAnalysis(fn, COPY, KILL, IN, OUT, true);
+  BasicBlock::forwardFlowAnalysis(fn, COPY, KILL, IN, OUT, true);
 
 #if DEBUG_CP
   if (debug > 0)
   {
-    printf("IN:\n"); printBitVectorSets(IN);
-    printf("OUT:\n"); printBitVectorSets(OUT);
+    printf("IN:\n");  BasicBlock::printBitVectorSets(IN);
+    printf("OUT:\n"); BasicBlock::printBitVectorSets(OUT);
   }
 #endif
 
   // This is the main loop.
   // Use the set IN[i] to initialize the available pairs at the top of a
   // block.  Then call local copy propagation to perform legal substitutions
-  // within a block.  
+  // within a block.
   // The local copy propagation algorithm will invalidate pairs when their LHS
   // or RHS is overwritten, so the available set remains correct as the block
   // is traversed.  As a check, the contents of the available set following
@@ -1066,422 +1145,6 @@ size_t globalCopyPropagation(FnSymbol* fn) {
 
   return s_repl_count + s_ref_repl_count;
 }
-#else // ! USE_NEW_IMPLEMENTATION
-
-static bool isCandidateForCopyPropagation(FnSymbol* fn, VarSymbol* var) {
-  // Note: iterator yield symbols are distinct from the return symbol
-  // and so are not ruled out by the first test below.
-  return
-    var != fn->getReturnSymbol() &&
-    var->type->refType && // hilde sez: This makes no sense!
-    // It appears that this test is intended to ensure that the selected var has value
-    // semantics, but there can be refs of refs, so it doesn't really.
-    // TODO: Try removing the above line or replacing the test with what is really meant.
-    !var->hasFlag(FLAG_CONCURRENTLY_ACCESSED);
-}
-
-// 
-static bool invalidateCopies(SymExpr* se, Vec<SymExpr*>& defSet, Vec<SymExpr*>& useSet) {
-  if (defSet.set_in(se))
-    return true;
-  if (useSet.set_in(se)) {
-    if (CallExpr* parent = toCallExpr(se->parentExpr)) {
-      if (parent->isPrimitive(PRIM_ADDR_OF))
-        return true;
-      if (isRecord(se->var->type)) {
-        if (parent->isPrimitive(PRIM_GET_MEMBER))
-          return true;
-        if (parent->isPrimitive(PRIM_SET_MEMBER) && parent->get(1) == se)
-          return true;
-      }
-      if (is_complex_type(se->var->type)) {
-        if (parent->isPrimitive(PRIM_GET_REAL) ||
-            parent->isPrimitive(PRIM_GET_IMAG))
-          return true;
-      }
-    }
-  }
-  return false;
-}
-
-//
-// The keys in the ReverseAvailableMap are the values in the
-// AvailableMap.  Its values are the keys in the AvailableMap so there
-// may be more than one.
-//
-typedef Map<Symbol*,Symbol*> AvailableMap;
-typedef MapElem<Symbol*,Symbol*> AvailableMapElem;
-typedef Map<Symbol*,Vec<Symbol*>*> ReverseAvailableMap;
-
-
-static void
-makeAvailable(AvailableMap& available,
-              ReverseAvailableMap& reverseAvailable,
-              Symbol* key,
-              Symbol* value) {
-  Vec<Symbol*>* keys = reverseAvailable.get(value);
-  if (!keys)
-    keys = new Vec<Symbol*>();
-  keys->add(key);
-  reverseAvailable.put(value, keys);
-  available.put(key, value);
-}
-
-
-static void
-removeAvailable(AvailableMap& available,
-                ReverseAvailableMap& reverseAvailable,
-                Symbol* sym) {
-  Vec<Symbol*>* keys = reverseAvailable.get(sym);
-  if (keys) {
-    forv_Vec(Symbol, key, *keys) {
-      if (sym == available.get(key))
-        available.put(key, NULL);
-    }
-  }
-  available.put(sym, NULL);
-}
-
-
-static void
-freeReverseAvailable(ReverseAvailableMap& reverseAvailable) {
-  Vec<Symbol*> values;
-  reverseAvailable.get_keys(values);
-  forv_Vec(Symbol, value, values) {
-    delete reverseAvailable.get(value);
-  }
-}
-
-
-/// \param useSet A SymExpr appearing in useSet is read (used) by the call in
-/// which it appears as determined by isDefAndOrUse.
-/// \param defSet A SymExpr appearing in defSet is written (defined) by the call in
-/// which it appears as determined by isDefAndOrUse.
-static void
-localCopyPropagationCore(BasicBlock* bb,
-                         AvailableMap& available,
-                         ReverseAvailableMap& reverseAvailable,
-                         Vec<SymExpr*>& useSet,
-                         Vec<SymExpr*>& defSet) {
-  for_vector(Expr, expr, bb->exprs) {
-    Vec<SymExpr*> symExprs;
-    collectSymExprs(expr, symExprs);
-
-    //
-    // replace uses with available copies
-    //
-    forv_Vec(SymExpr, se, symExprs) {
-      if (useSet.set_in(se)) {
-        if (Symbol* sym = available.get(se->var)) {
-          if (!invalidateCopies(se, defSet, useSet))
-          {
-            se->var = sym;
-            ++s_repl_count;
-          }
-        }
-      }
-    }
-
-    //
-    // invalidate available copies based on defs
-    //  also if a reference to a variable is taken since the reference
-    //  can be assigned
-    //
-    forv_Vec(SymExpr, se, symExprs) {
-      if (invalidateCopies(se, defSet, useSet)) {
-        removeAvailable(available, reverseAvailable, se->var);
-      }
-    }
-
-    //
-    // insert pairs into available copies map
-    //
-    if (CallExpr* call = toCallExpr(expr))
-      if (call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN))
-        if (SymExpr* rhs = toSymExpr(call->get(2)))
-          if (SymExpr* lhs = toSymExpr(call->get(1)))
-            if (lhs->var != rhs->var)
-              if (defSet.set_in(lhs))
-                if (useSet.set_in(rhs) || rhs->var->isConstant() || rhs->var->isImmediate())
-                  makeAvailable(available, reverseAvailable, lhs->var, rhs->var);
-  }
-}
-
-
-//
-// Apply local copy propagation to basic blocks of function
-//
-size_t localCopyPropagation(FnSymbol* fn) {
-  buildBasicBlocks(fn);
-
-  //
-  // locals: a vector of local variables in function fn that are
-  // candidates for copy propagation
-  //
-  Vec<Symbol*> locals;
-  s_repl_count = s_ref_repl_count = 0;
-  for_vector(BasicBlock, bb, *fn->basicBlocks) {
-    for_vector(Expr, expr, bb->exprs) {
-      if (DefExpr* def = toDefExpr(expr))
-        if (VarSymbol* var = toVarSymbol(def->sym))
-          if (isCandidateForCopyPropagation(fn, var))
-            locals.add(def->sym);
-    }
-  }
-
-  //
-  // defSet: a set of SymExprs that are defs of local variables
-  // useSet: a set of SymExprs that are uses of local variables
-  //
-  Vec<SymExpr*> defSet;
-  Vec<SymExpr*> useSet;
-  buildDefUseSets(locals, fn, defSet, useSet);
-
-  for_vector(BasicBlock, bb1, *fn->basicBlocks) {
-    AvailableMap available;
-    ReverseAvailableMap reverseAvailable;
-    localCopyPropagationCore(bb1, available, reverseAvailable, useSet, defSet);
-    freeReverseAvailable(reverseAvailable);
-  }
-
-  return s_repl_count + s_ref_repl_count;
-}
-
-
-//
-// Apply global copy propagation to basic blocks of function
-//
-size_t globalCopyPropagation(FnSymbol* fn) 
-{
-  buildBasicBlocks(fn);
-
-  // global copy propagation will have no effect
-  if (fn->basicBlocks->size() <= 1)
-    return 0;
-
-  //
-  // locals: a vector of local variables in function fn that are
-  // candidates for copy propagation
-  //
-  Vec<Symbol*> locals;
-  for_vector(BasicBlock, bb, *fn->basicBlocks) {
-    for_vector(Expr, expr, bb->exprs) {
-      if (DefExpr* def = toDefExpr(expr))
-        if (VarSymbol* var = toVarSymbol(def->sym))
-          if (isCandidateForCopyPropagation(fn, var))
-            locals.add(def->sym);
-    }
-  }
-
-  //
-  // defSet: a set of SymExprs that are defs of local variables
-  // useSet: a set of SymExprs that are uses of local variables
-  //
-  Vec<SymExpr*> defSet;
-  Vec<SymExpr*> useSet;
-  buildDefUseSets(locals, fn, defSet, useSet);
-
-  //
-  // spsLHS: left-hand side of copy
-  // spsRHS: right-hand side of copy
-  // spsN: basic block locations; basic block i contains the copies from
-  //     spsN(i-1) to spsN(i)
-  //
-  // Compute sparse versions of these vectors; convert to dense and
-  // drop sps; these are sparse to zero out invalidated copies
-  //
-  Vec<int> spsN;
-  Vec<SymExpr*> spsLHS;
-  Vec<SymExpr*> spsRHS;
-  int start = 0;
-  for_vector(BasicBlock, bb1, *fn->basicBlocks) {
-    for_vector(Expr, expr, bb1->exprs) {
-
-      Vec<SymExpr*> symExprs;
-      collectSymExprs(expr, symExprs);
-
-      //
-      // invalidate available copies based on defs
-      //
-      forv_Vec(SymExpr, se, symExprs) {
-        if (invalidateCopies(se, defSet, useSet)) {
-          for (int i = start; i < spsLHS.n; i++) {
-            if (spsLHS.v[i]) {
-              if (spsLHS.v[i]->var == se->var || spsRHS.v[i]->var == se->var) {
-                spsLHS.v[i] = 0;
-                spsRHS.v[i] = 0;
-              }
-            }
-          }
-        }
-      }
-
-      //
-      // insert pairs into available copies map
-      //
-      if (CallExpr* call = toCallExpr(expr))
-        if (call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN))
-          if (SymExpr* lhs = toSymExpr(call->get(1)))
-            if (SymExpr* rhs = toSymExpr(call->get(2)))
-              if (lhs->var != rhs->var &&
-                  defSet.set_in(lhs) &&
-                  (useSet.set_in(rhs) || rhs->var->isConstant() || rhs->var->isImmediate())) {
-                spsLHS.add(lhs);
-                spsRHS.add(rhs);
-              }
-    }
-    spsN.add(spsLHS.n);
-    start = spsLHS.n;
-  }
-
-#ifdef DEBUG_CP
-  printf("\n");
-  list_view(fn);
-
-  printBasicBlocks(fn);
-#endif
-
-  //
-  // Turn sparse spsLHS, spsRHS, spsN into dense LHS, RHS, N
-  //
-  Vec<int> N;
-  Vec<SymExpr*> LHS;
-  Vec<SymExpr*> RHS;
-
-  size_t nbbs = fn->basicBlocks->size();
-  int j = 0;
-  s_repl_count = s_ref_repl_count = 0; // Zero out counters.
-  for(size_t i = 0; i < nbbs; ++i) {
-#ifdef DEBUG_CP
-    printf("%d:\n", (int) i);
-#endif
-    while (j < spsN.v[i]) {
-      if (spsLHS.v[j]) {
-#ifdef DEBUG_CP
-        list_view(spsLHS.v[j]->parentExpr);
-#endif
-        LHS.add(spsLHS.v[j]);
-        RHS.add(spsRHS.v[j]);
-      }
-      j++;
-    }
-    N.add(LHS.n);
-  }
-#ifdef DEBUG_CP
-  printf("\n");
-#endif
-
-  std::vector<BitVec*> COPY;
-  std::vector<BitVec*> KILL;
-  std::vector<BitVec*> IN;
-  std::vector<BitVec*> OUT;
-  j = 0;
-  for(size_t i = 0; i < nbbs; ++i) {
-    BitVec* copy = new BitVec(N.v[nbbs-1]);
-    BitVec* kill = new BitVec(N.v[nbbs-1]);
-    BitVec* in = new BitVec(N.v[nbbs-1]);
-    BitVec* out = new BitVec(N.v[nbbs-1]);
-    while (j < N.v[i]) {
-      copy->set(j);
-      j++;
-    }
-    COPY.push_back(copy);
-    KILL.push_back(kill);
-    IN.push_back(in);
-    OUT.push_back(out);
-  }
-
-#ifdef DEBUG_CP
-  printf("COPY:\n"); printBitVectorSets(COPY);
-#endif
-
-  //
-  // compute kill set
-  //
-  size_t i = 0;
-  start = 0;
-  for_vector(BasicBlock, bb2, *fn->basicBlocks) {
-    int stop = N.v[i];
-
-    Vec<Symbol*> killSet;
-    for_vector(Expr, expr, bb2->exprs) {
-
-      Vec<SymExpr*> symExprs;
-      collectSymExprs(expr, symExprs);
-
-      //
-      // invalidate available copies based on defs
-      //
-      forv_Vec(SymExpr, se, symExprs) {
-        if (invalidateCopies(se, defSet, useSet)) {
-          if (!killSet.set_in(se->var)) {
-            killSet.set_add(se->var);
-          }
-        }
-      }
-    }
-    for (int j = 0; j < start; j++) {
-      if (killSet.set_in(LHS.v[j]->var) || killSet.set_in(RHS.v[j]->var)) {
-        KILL[i]->set(j);
-      }
-    }
-    for (int j = stop; j < LHS.n; j++) {
-      if (killSet.set_in(LHS.v[j]->var) || killSet.set_in(RHS.v[j]->var)) {
-        KILL[i]->set(j);
-      }
-    }
-    start = stop;
-    i++;
-  }
-
-#ifdef DEBUG_CP
-  printf("KILL:\n"); printBitVectorSets(KILL);
-#endif
-
-  // initialize IN set
-  for (size_t i = 1; i < nbbs; i++) {
-    for (int j = 0; j < LHS.n; j++) {
-      IN[i]->set(j);
-    }
-  }
-
-#ifdef DEBUG_CP
-  printf("IN:\n"); printBitVectorSets(IN);
-#endif
-
-  forwardFlowAnalysis(fn, COPY, KILL, IN, OUT, true);
-
-  for (size_t i = 0; i < nbbs; i++) {
-    BasicBlock* bb = (*fn->basicBlocks)[i];
-    AvailableMap available;
-    ReverseAvailableMap reverseAvailable;
-    bool proceed = false;
-    for (int j = 0; j < LHS.n; j++) {
-      if (IN[i]->get(j)) {
-        makeAvailable(available, reverseAvailable, LHS.v[j]->var, RHS.v[j]->var);
-        proceed = true;
-      }
-    }
-    if (proceed)
-      localCopyPropagationCore(bb, available, reverseAvailable, useSet, defSet);
-    freeReverseAvailable(reverseAvailable);
-  }
-
-  for_vector(BitVec, copy, COPY)
-    delete copy, copy = 0;
-
-  for_vector(BitVec, kill, KILL)
-    delete kill, kill = 0;
-
-  for_vector(BitVec, in, IN)
-    delete in, in = 0;
-
-  for_vector(BitVec, out, OUT)
-    delete out, out = 0;
-
-  return s_repl_count + s_ref_repl_count;
-}
-#endif
 
 
 // If there is exactly one definition of var by something of reference type, 
@@ -1561,7 +1224,12 @@ eliminateSingleAssignmentReference(Map<Symbol*,Vec<SymExpr*>*>& defMap,
         }
         if (!stillAlive) {
           var->defPoint->remove();
-          defMap.get(var)->v[0]->getStmtExpr()->remove();
+          Vec<SymExpr*>* defs = defMap.get(var);
+          if (defs == NULL) {
+            INT_FATAL(var, "Expected var to be defined");
+          }
+          // Remove the first definition from the AST.
+          defs->v[0]->getStmtExpr()->remove();
         }
       } else if (rhs->isPrimitive(PRIM_GET_MEMBER) ||
                  rhs->isPrimitive(PRIM_GET_SVEC_MEMBER)) {
@@ -1650,14 +1318,14 @@ eliminateSingleAssignmentReference(Map<Symbol*,Vec<SymExpr*>*>& defMap,
 
 
 size_t singleAssignmentRefPropagation(FnSymbol* fn) {
-  Vec<BaseAST*> asts;
+  std::vector<BaseAST*> asts;
   collect_asts(fn, asts);
 
   Vec<Symbol*> refSet;
   Vec<Symbol*> refVec;
   Vec<SymExpr*> symExprs;
   // Walk the asts in this function, and build lists of reference variables and sym exprs.
-  forv_Vec(BaseAST, ast, asts) {
+  for_vector(BaseAST, ast, asts) {
     if (VarSymbol* var = toVarSymbol(ast)) {
       if (isReferenceType(var->type)) {
         refVec.add(var);
