@@ -1,13 +1,31 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
+#include "passes.h"
+
+#include "astutil.h"
+#include "stlUtil.h"
+#include "expr.h"
+#include "stmt.h"
+#include "stringutil.h"
 
 #include <cstdio>
-#include "astutil.h"
-#include "expr.h"
-#include "passes.h"
-#include "stmt.h"
 
 // This pass pulls out any global constants to the top of the given
 // function (with the motivation to avoid loads during loops).
@@ -21,20 +39,26 @@ void localizeGlobals() {
   if (fNoGlobalConstOpt) return;
   forv_Vec(FnSymbol, fn, gFnSymbols) {
     Map<Symbol*,VarSymbol*> globals;
-    Vec<BaseAST*> asts;
+    std::vector<BaseAST*> asts;
     collect_asts(fn->body, asts);
-    forv_Vec(BaseAST, ast, asts) {
+    for_vector(BaseAST, ast, asts) {
       if (SymExpr* se = toSymExpr(ast)) {
         Symbol* var = se->var;
         ModuleSymbol* parentmod = toModuleSymbol(var->defPoint->parentSymbol);
+        CallExpr* parentExpr = toCallExpr(se->parentExpr);
+        bool inAddrOf = parentExpr && parentExpr->isPrimitive(PRIM_ADDR_OF);
 
         // Is var a global constant?
         // Don't replace the var name in its init function since that's
-        //      where we're setting the value.
+        //      where we're setting the value. Similarly, dont replace them
+        //      inside initStringLiterals
         // If the parentSymbol is the rootModule, the var is 'void,'
         //      'false,' '0,' ...
+        // Also don't replace it when it's in an addr of primitive.
         if (parentmod &&
             fn != parentmod->initFn &&
+            fn != initStringLiterals &&
+            !inAddrOf &&
             var->hasFlag(FLAG_CONST) &&
             var->defPoint->parentSymbol != rootModule) {
           VarSymbol* local_global = globals.get(var);
