@@ -1,40 +1,62 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
+#include "passes.h"
 
 #include "astutil.h"
 #include "bb.h"
-#include "passes.h"
-#include "stmt.h"
-#include "expr.h"
-#include "symbol.h"
 #include "bitVec.h"
+#include "CForLoop.h"
 #include "dominator.h"
+#include "expr.h"
+#include "ForLoop.h"
+#include "ParamForLoop.h"
+#include "stlUtil.h"
+#include "stmt.h"
+#include "stringutil.h"
+#include "symbol.h"
+#include "timer.h"
+#include "WhileStmt.h"
 
-#include <stack>
-#include <set>
 #include <algorithm>
+#include <set>
+#include <stack>
 
 
-#include "view.h"
 
 //#define debugHoisting
 #ifdef debugHoisting
   #define printDebug(string) printf string
-#else 
+#else
   #define printDebug(string) //do nothing
 #endif
 
 
-//#define detailedTiming
+// #define detailedTiming
+
 #ifdef detailedTiming
   #define startTimer(timer) timer.start()
-  #define stopTimer(timer) timer.stop()
+  #define stopTimer(timer)  timer.stop()
 #else
-  #define startTimer(timer) //do nothing 
-  #define stopTimer(timer) //do nothing 
-#endif 
+  #define startTimer(timer) // do nothing
+  #define stopTimer(timer)  // do nothing
+#endif
 
 
 Timer allOperandsAreLoopInvariantTimer;
@@ -51,111 +73,115 @@ Timer overallTimer;
 
 #define MAX_NUM_ALIASES 200000
 
-//TODO The alias analysis is extremely conservative. Beyond possibly not hoisting 
-//things that can be, it is also a performance issue because you have a lot more 
-//definitions to consider before declaration something invariant. 
+//TODO The alias analysis is extremely conservative. Beyond possibly not hoisting
+//things that can be, it is also a performance issue because you have a lot more
+//definitions to consider before declaration something invariant.
 
-//TODO some other possible optimizations are if you're looking at an outer 
-//loop in a nest you can ignore everything inside the inner loop(s) since you 
+//TODO some other possible optimizations are if you're looking at an outer
+//loop in a nest you can ignore everything inside the inner loop(s) since you
 //already know they can't be hoisted
 
- 
+
 /*
 * This is really just a wrapper for the collection of basic blocks that
-* make up a loop. However, it also stores the header, and builds the bit 
+* make up a loop. However, it also stores the header, and builds the bit
 * representation as the loop is built to save time. The bit representation
-* and the bit representation of the exits can be gotten, putting them 
-* in a centralized location. 
+* and the bit representation of the exits can be gotten, putting them
+* in a centralized location.
 */
 class Loop {
-
-  private:
+private:
     std::vector<BasicBlock*>* loopBlocks;
-    BasicBlock* header;
-    BitVec* bitBlocks;
-    BitVec* bitExits;
+    BasicBlock*               header;
+    BitVec*                   bitBlocks;
+    BitVec*                   bitExits;
 
-  public: 
+public:
     Loop(int nBlocks) {
       loopBlocks = new std::vector<BasicBlock*>;
-      bitBlocks = new BitVec(nBlocks); 
-      bitExits = new BitVec(nBlocks);
+      bitBlocks  = new BitVec(nBlocks);
+      bitExits   = new BitVec(nBlocks);
     }
-    
+
     ~Loop() {
       delete loopBlocks;
-      loopBlocks = 0;
       delete bitBlocks;
-      bitBlocks = 0;
       delete bitExits;
-      bitExits = 0;      
     }
-    
-    // This function exists to place an expr in the 
-    // "preheader" of the loop, 
+
+    // This function exists to place an expr in the
+    // "preheader" of the loop,
     void insertBefore(Expr* expr) {
-      if(header->exprs.size() != 0) {
-        if(BlockStmt* blockStmt = toBlockStmt(header->exprs.at(0)->parentExpr)) {
-          if(blockStmt->isLoop())
+      if (header->exprs.size() != 0) {
+        // find the first expr in the header, and get it's parent expr (for
+        // most cases it will be the surrounding block statement of the loop)
+        if (BlockStmt* blockStmt = toBlockStmt(header->exprs.at(0)->parentExpr)) {
+          if (blockStmt->isLoopStmt()) {
             blockStmt->insertBefore(expr->remove());
+
+          } else if (blockStmt->blockTag == BLOCK_C_FOR_LOOP) {
+            CForLoop* cforLoop = CForLoop::loopForClause(blockStmt);
+
+            cforLoop->insertBefore(expr->remove());
+          }
         }
       }
     }
-    
+
     //Set the header, and insert the header into the loop blocks
     void setHeader(BasicBlock* setHeader) {
       header = setHeader;
       insertBlock(setHeader);
     }
-    
+
     //add all the blocks from other loop to this loop
     void combine(Loop* otherLoop) {
       for_vector(BasicBlock, block, *otherLoop->getBlocks()) {
         insertBlock(block);
       }
     }
-    
+
     //insert a block and update the bit representation
     void insertBlock(BasicBlock* block) {
       //if this block is already in the loop, do nothing
       if(bitBlocks->test(block->id)) {
         return;
       }
-    
+
       //add the block to the list of blocks and to the bit representation
       loopBlocks->push_back(block);
       bitBlocks->set(block->id);
     }
-    
+
     //check if a block is in the loop based on block id
     bool contains(int i) {
       return bitBlocks->test(i);
     }
-    
-    //check if a block is in the loop 
+
+    //check if a block is in the loop
     bool contains(BasicBlock* block) {
       return bitBlocks->test(block->id);
     }
-    
-    //get the header block 
+
+    //get the header block
     BasicBlock* getHeader() {
       return header;
     }
-    
-    //get the actual blocks in the loop 
+
+    //get the actual blocks in the loop
     std::vector<BasicBlock*>* getBlocks() {
       return loopBlocks;
     }
-    
+
     //get the bitvector that represents the blocks in the loop
     BitVec* getBitBlocks() {
       return bitBlocks;
     }
-    
+
     //get the bitvector that represents the exit blocks
     //the exit blocks are the blocks that have a next basic
-    //block that is outside of the loop. 
-    BitVec* getBitExits() {    
+    //block that is outside of the loop.
+    BitVec* getBitExits() {
       bitExits->reset();
       for_vector(BasicBlock, block, *loopBlocks) {
         for_vector(BasicBlock, out, block->outs) {
@@ -166,7 +192,7 @@ class Loop {
       }
       return bitExits;
     }
-    
+
     //return the number of blocks in the loop
     unsigned size() {
       return loopBlocks->size();
@@ -273,27 +299,6 @@ void collectNaturalLoopForEdge(Loop* loop, BasicBlock* header, BasicBlock* tail)
   } 
 }
 
-/*
- * Takes a call that is a PRIM_SVEC_GET_MEMBER* and returns the symbol of the
- * field. Normally the call is something of the form PRIM_SVEC_GET_MEMBER(p, 1) 
- * and what this function gets out is the symbol that is the first field
- * instead of just the number 1. 
- */
-static Symbol* getSvecSymbol(CallExpr* call) {
-  INT_ASSERT(call->isPrimitive(PRIM_GET_SVEC_MEMBER)       ||
-             call->isPrimitive(PRIM_GET_SVEC_MEMBER_VALUE) ||
-             call->isPrimitive(PRIM_SET_SVEC_MEMBER));
-  
-  Type* type = call->get(1)->getValType();
-  AggregateType* tuple = toAggregateType(type);
-  SymExpr* fieldVal = toSymExpr(call->get(2));
-  VarSymbol* fieldSym = toVarSymbol(fieldVal->var);
-  int immediateVal = fieldSym->immediate->int_value();
-
-  INT_ASSERT(immediateVal >= 1 && immediateVal <= tuple->fields.length);
-  return tuple->getField(immediateVal);
-}
-
 // Returns rhs var if lhs aliases rhs
 //
 //
@@ -304,11 +309,7 @@ rhsAlias(CallExpr* call) {
   for_alist(expr, call->argList) {
     if(SymExpr* symExpr = toSymExpr(expr)) {
       Type* symType = symExpr->var->type->symbol->type;
-      bool isWideClass = symExpr->var->type->symbol->hasFlag(FLAG_WIDE_CLASS);
-      bool isWideRef = symExpr->var->type->symbol->hasFlag(FLAG_WIDE);
-      bool isWideStr = isWideString(symExpr->var->type);
-      if(isReferenceType(symType) || isRecordWrappedType(symType) ||
-        isWideClass || isWideRef || isWideStr) {
+      if(isReferenceType(symType) || isRecordWrappedType(symType)) {
         hasRef = true;
       }
     }
@@ -352,6 +353,11 @@ rhsAlias(CallExpr* call) {
           SymExpr* rhs = toSymExpr(rhsCall->get(1));
           INT_ASSERT(rhs);
           return rhs->var;
+        } else if (rhsCall->isPrimitive(PRIM_GET_REAL) ||
+                   rhsCall->isPrimitive(PRIM_GET_IMAG)) {
+          SymExpr* rhs = toSymExpr(rhsCall->get(1));
+          INT_ASSERT(rhs);
+          return rhs->var;
         }
       } else {
         // alias via autocopy
@@ -514,22 +520,22 @@ static void buildLocalDefUseMaps(Loop* loop, symToVecSymExprMap& localDefMap, sy
               }
             }
           }
-        } 
+        }
       }
-    
+
       //Check each symExpr to see if its a use and or def and add to the appropriate lists
       std::vector<SymExpr*> symExprs;
-      collectSymExprsSTL(expr, symExprs);
+      collectSymExprs(expr, symExprs);
       for_vector(SymExpr, symExpr, symExprs) {
         if(symExpr->parentSymbol) {
-          if(isVarSymbol(symExpr->var) || isArgSymbol(symExpr->var)) {
+          if(isLcnSymbol(symExpr->var)) {
             localMap[symExpr] = block->id;
             int result = isDefAndOrUse(symExpr);
-            //Add defs 
+            //Add defs
             if(result & 1) {
               addDefOrUse(localDefMap, symExpr->var, symExpr);
             }
-            //add uses 
+            //add uses
             if(result & 2) {
               addDefOrUse(localUseMap, symExpr->var, symExpr);
             }
@@ -551,7 +557,7 @@ static void buildLocalDefUseMaps(Loop* loop, symToVecSymExprMap& localDefMap, sy
         }
       }
     }
-  }   
+  }
 }
 
 
@@ -597,14 +603,6 @@ static bool allOperandsAreLoopInvariant(Expr* expr, std::set<SymExpr*>& loopInva
     return false;
   } else if(SymExpr* symExpr = toSymExpr(expr)) {
   
-    //do not hoist things that are wide 
-    bool isWideObj = symExpr->var->type->symbol->hasFlag(FLAG_WIDE_CLASS);
-    bool isWideRef = symExpr->var->type->symbol->hasFlag(FLAG_WIDE);
-    bool isWideStr = isWideString(symExpr->var->type);
-    if(isWideObj || isWideRef || isWideStr) {
-      return false;
-    }
-  
     //If the operand is invariant (0 defs in the loop, or constant)
     //it is invariant 
     if(loopInvariants.count(symExpr) == 1) {
@@ -642,7 +640,7 @@ static bool allOperandsAreLoopInvariant(Expr* expr, std::set<SymExpr*>& loopInva
       for_vector(BasicBlock, block, *loop->getBlocks()) {
         for_vector(Expr, expr, block->exprs) {
           std::vector<SymExpr*> symExprs;
-          collectSymExprsSTL(expr, symExprs);
+          collectSymExprs(expr, symExprs);
           for_vector(SymExpr, symExpr2, symExprs) {
 
             //mark that we have seen the definition of the operand 
@@ -682,13 +680,15 @@ static bool allOperandsAreLoopInvariant(Expr* expr, std::set<SymExpr*>& loopInva
 static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
     loop, symToVecSymExprMap& localDefMap, FnSymbol* fn) {
  
-  //collect all of the symExpr and defExpr in the loop 
+  // collect all of the symExprs, defExprs, and callExprs in the loop
   startTimer(collectSymExprAndDefTimer);
   std::vector<SymExpr*> loopSymExprs;
   std::set<Symbol*> defsInLoop;
+  std::vector<CallExpr*> callsInLoop;
   for_vector(BasicBlock, block, *loop->getBlocks()) {
     for_vector(Expr, expr, block->exprs) {
-      collectSymExprsSTL(expr, loopSymExprs);
+      collectFnCalls(expr, callsInLoop);
+      collectSymExprs(expr, loopSymExprs);
       if (DefExpr* defExpr = toDefExpr(expr)) {
         if (toVarSymbol(defExpr->sym)) {
           defsInLoop.insert(defExpr->sym);
@@ -708,17 +708,17 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
   //as it seems unlikely that anybody would be willing to wait the 2+ hours it
   //takes for a program to actual compile. However, if compilation times become
   //unpredictable a compiler flag can be added. With less conservative analysis
-  //though, this threshold should be eliminated. 
+  //though, this threshold should be eliminated.
   int numAliases = 0;
-  //compute the map of aliases for each symbol 
+  //compute the map of aliases for each symbol
   startTimer(computeAliasTimer);
   std::map<Symbol*, std::set<Symbol*> > aliases;
 
   //Compute the aliases for the function's parameters. Any args passed by ref
-  //can potentially alias each other. 
+  //can potentially alias each other.
   for_alist(formal1, fn->formals) {
     for_alist(formal2, fn->formals) {
-      if(formal1 == formal2) 
+      if(formal1 == formal2)
         continue;
       if(ArgSymbol* arg1 = toArgSymbol(toDefExpr(formal1)->sym)) {
         if(ArgSymbol* arg2 = toArgSymbol(toDefExpr(formal2)->sym)) {
@@ -735,10 +735,10 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
     //if there are too many aliases, just return. Since nothing has been added
     //to the list of invariants, nothing will be hoisted from the current fn
     if(numAliases > MAX_NUM_ALIASES) {
-#ifdef detailedTiming 
+#ifdef detailedTiming
        FILE* tooManyAliasesFile = fopen(astr(CHPL_HOME,"/LICMaliases.txt"), "a");
        fprintf(tooManyAliasesFile, "Skipping fn %s %d of module %s %d "
-           "because there were too many aliases\n", fn->name, fn->id, 
+           "because there were too many aliases\n", fn->name, fn->id,
            fn->getModule()->name, fn->getModule()->id);
 
        fclose(tooManyAliasesFile);
@@ -746,29 +746,29 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
       stopTimer(computeAliasTimer);
       return;
     }
- 
+
     for_vector(Expr, expr, block2->exprs) {
       if(CallExpr* call = toCallExpr(expr)) {
         Symbol* rhs = rhsAlias(call);
         if(rhs != NULL) {
           Symbol* lhs = NULL;
-          if(call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN)) 
+          if(call->isPrimitive(PRIM_MOVE) || call->isPrimitive(PRIM_ASSIGN))
             lhs = toSymExpr(call->get(1))->var;
           else
             lhs = toSymExpr(call->get(2))->var;
-        
+
           for_set(Symbol, rhsAlias, aliases[rhs]) {
             printDebug(("%s %d aliases %s %d\n", lhs->name, lhs->id,
               rhsAlias->name, rhsAlias->id));
-            
+
             aliases[rhsAlias].insert(lhs);
             aliases[lhs].insert(rhsAlias);
             numAliases += 2;
           }
-          printDebug(("%s %d aliases %s %d\n", lhs->name, lhs->id, rhs->name, 
+          printDebug(("%s %d aliases %s %d\n", lhs->name, lhs->id, rhs->name,
             rhs->id));
-          
-          aliases[rhs].insert(lhs); 
+
+          aliases[rhs].insert(lhs);
           aliases[lhs].insert(rhs);
           numAliases += 2;
         }
@@ -776,26 +776,26 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
     }
   }
   stopTimer(computeAliasTimer);
-  
-  //calculate the actual defs of a symbol including the defs of 
-  //its aliases. If there are no defs or we have a constant, 
+
+  //calculate the actual defs of a symbol including the defs of
+  //its aliases. If there are no defs or we have a constant,
   //add it to the list of invariants
   startTimer(calculateActualDefsTimer);
-  std::set<SymExpr*> loopInvariantOperands;  
+  std::set<SymExpr*> loopInvariantOperands;
   std::set<SymExpr*> loopInvariantInstructions;
   std::map<SymExpr*, std::set<SymExpr*> > actualDefs;
   for_vector(SymExpr, symExpr, loopSymExprs) {
-  
+
     //skip already known invariants
     if(loopInvariantOperands.count(symExpr) == 1) {
       continue;
     }
-    
-    //mark all the const operands 
+
+    //mark all the const operands
     if(isConst(symExpr)) {
       loopInvariantOperands.insert(symExpr);
-    }    
-        
+    }
+
     //calculate defs of the aliases
     if(aliases.count(symExpr->var) == 1) {
       for_set(Symbol, symbol, aliases[symExpr->var]) {
@@ -804,22 +804,22 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
           for_vector(SymExpr, aliasSymExpr, *localDefMap[symbol]) {
             if(CallExpr* call = toCallExpr(aliasSymExpr->parentExpr)) {
               if(symExpr->var == rhsAlias(call)) {
-                //do nothing 
+                //do nothing
               }
               else if(aliases[symExpr->var].count(rhsAlias(call)) == 0) {
                 actualDefs[symExpr].insert(aliasSymExpr);
-              }  
+              }
             }
           }
         }
       }
-    }    
-   
+    }
+
     // We have to note that for records if a field is deffed then the record
     // itself is also deffed. This is handled in buildDefUseMaps for normal
     // variables but we have to handle when a variable aliases a record's field
     // and the alias is deffed.
-    if(isVarSymbol(symExpr->var) || isArgSymbol(symExpr->var)) {
+    if(isLcnSymbol(symExpr->var)) {
       // if the current variable is a record
       if(AggregateType* ct = toAggregateType(symExpr->var->type->symbol->type)) {
         if(isRecord(symExpr->var->type->symbol->type)) {
@@ -854,19 +854,43 @@ static void computeLoopInvariants(std::vector<SymExpr*>& loopInvariants, Loop*
     }
 
     bool mightHaveBeenDeffedElseWhere = false;
-    //assume that anything passed in by ref has been changed elsewhere 
+    // assume that anything passed in by ref has been changed elsewhere
+    // Note that not all things that are passed by ref will have the ref intent
+    // flag, and may just be ref variables. This is a known bug, see comments
+    // in addVarsToFormals(): flattenFunctions.cpp.
     if(ArgSymbol* argSymbol = toArgSymbol(symExpr->var)) {
-      if(argSymbol->intent == INTENT_REF) {
+      if(argSymbol->intent == INTENT_REF ||
+         argSymbol->intent == INTENT_CONST_REF ||
+         isReferenceType(argSymbol->type)) {
         mightHaveBeenDeffedElseWhere = true;
       }
     }
     for_set(Symbol, aliasSym, aliases[symExpr->var]) {
       if(ArgSymbol* argSymbol = toArgSymbol(aliasSym)) {
-        if(argSymbol->intent == INTENT_REF) {
+        if(argSymbol->intent == INTENT_REF ||
+           argSymbol->intent == INTENT_CONST_REF ||
+           isReferenceType(argSymbol->type)) {
           mightHaveBeenDeffedElseWhere = true;
         }
       }
     }
+    // Find where the variable is defined.
+    Symbol* defScope = symExpr->var->defPoint->parentSymbol;
+    // if the variable is a module level (global) variable
+    if (isModuleSymbol(defScope)) {
+      // if there are any function calls inside the loop, assume that one of
+      // the functions may have changed the value of the global. Note that we
+      // don't have to worry about a different task updating the global since
+      // that would have to be protected with a sync or be atomic in which case
+      // no hoisting will occur in the function at all. Any defs to the global
+      // inside of this loop will be detected just like any other variable
+      // definitions.
+      // TODO this could be improved to check which functions modify the global
+      // and see if any of those functions are being called in this loop.
+      if (callsInLoop.size() != 0) {
+        mightHaveBeenDeffedElseWhere = true;
+      }
+    }
     //if there were no defs of the symbol, it is invariant 
     if(actualDefs.count(symExpr) == 0 && !mightHaveBeenDeffedElseWhere) {
       loopInvariantOperands.insert(symExpr);
@@ -984,7 +1008,7 @@ static bool defDominatesAllExits(Loop* loop, SymExpr* def, std::vector<BitVec*>&
   
   BitVec* bitExits = loop->getBitExits();
    
-  for(int i = 0; i < bitExits->size(); i++) {
+  for(size_t i = 0; i < bitExits->size(); i++) {
     if(bitExits->test(i)) {
       if(dominates(defBlock, i, dominators) == false) {
         return false;
@@ -995,47 +1019,25 @@ static bool defDominatesAllExits(Loop* loop, SymExpr* def, std::vector<BitVec*>&
 }
 
 
-/*
- * Collect all of the function symbols that belong to function calls 
- * and nested function calls that occur from baseAST. In other words
- * look through the baseAST and find all the function and nested function
- * calls and collect their fnsymbols. 
- */
-static void collectUsedFnSymbolsSTL(BaseAST* ast, std::set<FnSymbol*>& fnSymbols) {
-  AST_CHILDREN_CALL(ast, collectUsedFnSymbolsSTL, fnSymbols);
-  //if there is a function call, get the FnSymbol associated with it 
-  //and look through that FnSymbol for other function calls. Do not 
-  //look through an already visited FnSymbol, or you'll have an infinite
-  //loop in the case of recursion. 
-  if (CallExpr* call = toCallExpr(ast)) {
-    if (FnSymbol* fnSymbol = call->isResolved()) {
-      if(fnSymbols.count(fnSymbol) == 0) {
-        fnSymbols.insert(fnSymbol);
-        for_alist(expr, fnSymbol->body->body) {
-          AST_CHILDREN_CALL(expr, collectUsedFnSymbolsSTL, fnSymbols);
-        }
-      }
-    }
-  }
-}
-
 
 /*
- * Collects the uses and defs of symbols the baseAST 
- * and checks for any synchronization variables such as 
- * atomics, syncs, and singles. 
+ * Collects the uses and defs of symbols the baseAST
+ * and checks for any synchronization variables such as
+ * atomics, syncs, and singles.
  */
 static bool containsSynchronizationVar(BaseAST* ast) {
   std::vector<SymExpr*> symExprs;
-  collectSymExprsSTL(ast, symExprs);
+  collectSymExprs(ast, symExprs);
   for_vector(SymExpr, symExpr, symExprs) {
-    
-    if(isVarSymbol(symExpr->var) || isArgSymbol(symExpr->var)) {
+
+    if(isLcnSymbol(symExpr->var)) {
       Type* symType = symExpr->var->type;
-      if (isSyncType(symType) || isAtomicType(symType)) {
+      Type* valType = symType->getValType();
+      if (isSyncType(symType) || isAtomicType(symType) ||
+          isSyncType(valType) || isAtomicType(valType)) {
         return true;
       }
-    }  
+    }
   }
   return false;
 }
@@ -1063,7 +1065,7 @@ static bool canPerformCodeMotion(Loop* loop) {
       //Check for nested function calls containing 
       //synchronization variables 
       std::set<FnSymbol*> fnSymbols;
-      collectUsedFnSymbolsSTL(expr, fnSymbols);
+      collectUsedFnSymbols(expr, fnSymbols);
       for_set(FnSymbol, fnSymbol2, fnSymbols) {
         if(containsSynchronizationVar(fnSymbol2)) {
           return false;
@@ -1107,13 +1109,18 @@ void loopInvariantCodeMotion(void) {
     
   //TODO use stl routine here
   forv_Vec(FnSymbol, fn, gFnSymbols) {
-     
+
     //build the basic blocks, where the first bb is the entry block 
     startTimer(buildBBTimer);
-    buildBasicBlocks(fn);
+
+    BasicBlock::buildBasicBlocks(fn);
+
     std::vector<BasicBlock*> basicBlocks = *fn->basicBlocks;
+
     BasicBlock* entryBlock = basicBlocks[0];
+
     unsigned nBlocks = basicBlocks.size();
+
     stopTimer(buildBBTimer);
     
     //compute the dominators 
@@ -1192,36 +1199,36 @@ void loopInvariantCodeMotion(void) {
   maxTimeFile = fopen(astr(CHPL_HOME,"/LICMmaxTime.txt"), "a");
 
   fprintf(timingFile, "For compilation of %s:                         \n", compileCommand );
-  fprintf(timingFile, "Spent %2.3f seconds building basic blocks      \n", buildBBTimer.elapsed()); 
-  fprintf(timingFile, "Spent %2.3f seconds computing dominators       \n", computeDominatorTimer.elapsed()); 
-  fprintf(timingFile, "Spent %2.3f seconds collecting natural loops   \n", collectNaturalLoopsTimer.elapsed()); 
-  fprintf(timingFile, "Spent %2.3f seconds building local def maps    \n", buildLocalDefMapsTimer.elapsed()); 
-  fprintf(timingFile, "Spent %2.3f seconds on can perform code motion \n", canPerformCodeMotionTimer.elapsed());
-  fprintf(timingFile, "Spent %2.3f seconds computing loop invariants  \n", computeLoopInvariantsTimer.elapsed()); 
+  fprintf(timingFile, "Spent %2.3f seconds building basic blocks      \n", buildBBTimer.elapsedSecs()); 
+  fprintf(timingFile, "Spent %2.3f seconds computing dominators       \n", computeDominatorTimer.elapsedSecs()); 
+  fprintf(timingFile, "Spent %2.3f seconds collecting natural loops   \n", collectNaturalLoopsTimer.elapsedSecs()); 
+  fprintf(timingFile, "Spent %2.3f seconds building local def maps    \n", buildLocalDefMapsTimer.elapsedSecs()); 
+  fprintf(timingFile, "Spent %2.3f seconds on can perform code motion \n", canPerformCodeMotionTimer.elapsedSecs());
+  fprintf(timingFile, "Spent %2.3f seconds computing loop invariants  \n", computeLoopInvariantsTimer.elapsedSecs()); 
    
-  double estimateOverall = buildBBTimer.elapsed() + computeDominatorTimer.elapsed() + collectNaturalLoopsTimer.elapsed() + \
-  buildLocalDefMapsTimer.elapsed() + canPerformCodeMotionTimer.elapsed() + computeLoopInvariantsTimer.elapsed();
+  double estimateOverall = buildBBTimer.elapsedSecs() + computeDominatorTimer.elapsedSecs() + collectNaturalLoopsTimer.elapsedSecs() + \
+  buildLocalDefMapsTimer.elapsedSecs() + canPerformCodeMotionTimer.elapsedSecs() + computeLoopInvariantsTimer.elapsedSecs();
   
   
-  fprintf(timingFile, "Spent %2.3f seconds loop hoisting on %ld loops \n", overallTimer.elapsed(), numLoops);
-  fprintf(timingFile, "      %2.3f seconds unaccounted for            \n", overallTimer.elapsed() - estimateOverall);
+  fprintf(timingFile, "Spent %2.3f seconds loop hoisting on %ld loops \n", overallTimer.elapsedSecs(), numLoops);
+  fprintf(timingFile, "      %2.3f seconds unaccounted for            \n", overallTimer.elapsedSecs() - estimateOverall);
   
-  double estimateInvariant = allOperandsAreLoopInvariantTimer.elapsed() + computeAliasTimer.elapsed() + \
-  collectSymExprAndDefTimer.elapsed() + calculateActualDefsTimer.elapsed();
+  double estimateInvariant = allOperandsAreLoopInvariantTimer.elapsedSecs() + computeAliasTimer.elapsedSecs() + \
+  collectSymExprAndDefTimer.elapsedSecs() + calculateActualDefsTimer.elapsedSecs();
   
   fprintf(timingFile, "\n");
   fprintf(timingFile, "In loop invariant computation:                 \n");
-  fprintf(timingFile, "Spent %2.3f seconds collecting sym/def Expr    \n", collectSymExprAndDefTimer.elapsed());
-  fprintf(timingFile, "Spent %2.3f seconds computing aliases          \n", computeAliasTimer.elapsed());
-  fprintf(timingFile, "Spent %2.3f seconds calculating actual defs    \n", calculateActualDefsTimer.elapsed());
-  fprintf(timingFile, "Spent %2.3f seconds in allOperandsAreInvariant \n", allOperandsAreLoopInvariantTimer.elapsed());
+  fprintf(timingFile, "Spent %2.3f seconds collecting sym/def Expr    \n", collectSymExprAndDefTimer.elapsedSecs());
+  fprintf(timingFile, "Spent %2.3f seconds computing aliases          \n", computeAliasTimer.elapsedSecs());
+  fprintf(timingFile, "Spent %2.3f seconds calculating actual defs    \n", calculateActualDefsTimer.elapsedSecs());
+  fprintf(timingFile, "Spent %2.3f seconds in allOperandsAreInvariant \n", allOperandsAreLoopInvariantTimer.elapsedSecs());
 
-  fprintf(timingFile, "      %2.3f seconds unaccounted for            \n\n", computeLoopInvariantsTimer.elapsed() - estimateInvariant);
+  fprintf(timingFile, "      %2.3f seconds unaccounted for            \n\n", computeLoopInvariantsTimer.elapsedSecs() - estimateInvariant);
   fprintf(timingFile, "_______________________________________________\n\n\n");
   
   
   fprintf(maxTimeFile, "For compilation of %s:                         \n", compileCommand );
-  fprintf(maxTimeFile, "Spent %2.3f seconds loop hoisting on %ld loops \n\n", overallTimer.elapsed(), numLoops);
+  fprintf(maxTimeFile, "Spent %2.3f seconds loop hoisting on %ld loops \n\n", overallTimer.elapsedSecs(), numLoops);
   fprintf(maxTimeFile, "_______________________________________________\n\n\n");
   
   fclose(timingFile);
