@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 //
 // Mark On statements/blocks as "fast" (no communication, small, etc.)
@@ -68,7 +82,6 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_GET_SVEC_MEMBER:
   case PRIM_GET_PRIV_CLASS:
   case PRIM_NEW_PRIV_CLASS:
-  case PRIM_NUM_PRIV_CLASSES:
 
   case PRIM_CHECK_NIL:
   case PRIM_GET_REAL:
@@ -88,11 +101,13 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_GET_SERIAL:
   case PRIM_SET_SERIAL:
 
+  case PRIM_START_RMEM_FENCE:
+  case PRIM_FINISH_RMEM_FENCE:
+
   case PRIM_STRING_COPY:
   case PRIM_CAST_TO_VOID_STAR:
   case PRIM_SIZEOF:
 
-  case PRIM_NEXT_UINT32:
   case PRIM_GET_USER_LINE:
   case PRIM_GET_USER_FILE:
     DEBUG_PRINTF(" *** OK (default): %s\n", call->primitive->name);
@@ -111,8 +126,8 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_OR_ASSIGN:
   case PRIM_XOR_ASSIGN:
     if (!isCallExpr(call->get(2))) { // callExprs checked in calling function
-      if (!call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE) &&
-          !call->get(2)->typeInfo()->symbol->hasFlag(FLAG_WIDE)) {
+      if (!call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE_REF) &&
+          !call->get(2)->typeInfo()->symbol->hasFlag(FLAG_WIDE_REF)) {
         DEBUG_PRINTF(" *** OK (PRIM_MOVE 1): %s\n", call->primitive->name);
         return true;
       }
@@ -130,7 +145,7 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_WIDE_GET_NODE:
   case PRIM_WIDE_GET_ADDR:
     // If this test is true, a remote get is required.
-    if (!(call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE) &&
+    if (!(call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE_REF) &&
           call->get(1)->getValType()->symbol->hasFlag(FLAG_WIDE_CLASS))) {
       DEBUG_PRINTF(" *** OK (PRIM_WIDE_GET_LOCALE, etc.): %s\n",
                    call->primitive->name);
@@ -142,7 +157,7 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_GET_UNION_ID:
   case PRIM_GET_MEMBER_VALUE:
   case PRIM_GET_SVEC_MEMBER_VALUE:
-    if (!call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE)) {
+    if (!call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE_REF)) {
       return true;
       DEBUG_PRINTF(" *** OK (PRIM_SET_UNION_ID, etc.): %s\n",
                    call->primitive->name);
@@ -167,7 +182,7 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_DEREF:
   case PRIM_SET_MEMBER:
   case PRIM_SET_SVEC_MEMBER:
-    if (!call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE) &&
+    if (!call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE_REF) &&
         !call->get(1)->typeInfo()->symbol->hasFlag(FLAG_WIDE_CLASS)) {
       DEBUG_PRINTF(" *** OK (PRIM_DEREF, etc.): %s\n", call->primitive->name);
       return true;
@@ -176,6 +191,9 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
 
   case PRIM_CHPL_COMM_GET:
   case PRIM_CHPL_COMM_PUT:
+  case PRIM_CHPL_COMM_ARRAY_GET:
+  case PRIM_CHPL_COMM_ARRAY_PUT:
+  case PRIM_CHPL_COMM_REMOTE_PREFETCH:
   case PRIM_CHPL_COMM_GET_STRD:
   case PRIM_CHPL_COMM_PUT_STRD:
     // These may involve communication, so are deemed slow.
@@ -210,17 +228,10 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
    // These may block, so are deemed slow.
    return false;
 
-  case PRIM_GC_CC_INIT:
-  case PRIM_GC_ADD_ROOT:
-  case PRIM_GC_ADD_NULL_ROOT:
-  case PRIM_GC_DELETE_ROOT:
-  case PRIM_GC_CLEANUP:
-    INT_FATAL("This primitive has not yet been implemented.");
-    break;
-
   case PRIM_NEW:
   case PRIM_INIT:
   case PRIM_NO_INIT:
+  case PRIM_TYPE_INIT:
   case PRIM_LOGICAL_FOLDER:
   case PRIM_TYPEOF:
   case PRIM_TYPE_TO_STRING:
@@ -233,9 +244,12 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_IS_TUPLE_TYPE:
   case PRIM_IS_STAR_TUPLE_TYPE:
   case PRIM_IS_SUBTYPE:
+  case PRIM_IS_WIDE_PTR:
   case PRIM_TUPLE_EXPAND:
   case PRIM_TUPLE_AND_EXPAND:
   case PRIM_QUERY:
+  case PRIM_QUERY_PARAM_FIELD:
+  case PRIM_QUERY_TYPE_FIELD:
   case PRIM_ERROR:
   case PRIM_WARNING:
 
@@ -252,7 +266,6 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_ACTUALS_LIST:
   case PRIM_YIELD:
 
-  case PRIM_USE:
   case PRIM_USED_MODULES_LIST:
 
   case PRIM_WHEN:
@@ -261,10 +274,10 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_CREATE_FN_TYPE:
 
   case PRIM_NUM_FIELDS:
+  case PRIM_IS_POD:
   case PRIM_FIELD_NUM_TO_NAME:
+  case PRIM_FIELD_NAME_TO_NUM:
   case PRIM_FIELD_VALUE_BY_NUM:
-  case PRIM_FIELD_ID_BY_NUM:
-  case PRIM_FIELD_VALUE_BY_NAME:
     INT_FATAL("This primitive should have been removed from the tree by now.");
     break;
 
@@ -272,12 +285,12 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_BLOCK_WHILEDO_LOOP:
   case PRIM_BLOCK_DOWHILE_LOOP:
   case PRIM_BLOCK_FOR_LOOP:
+  case PRIM_BLOCK_C_FOR_LOOP:
     return true;
  
    // These don't block in the Chapel sense, but they may require a system
     // call so we don't consider them eligible.
     //
-  case PRIM_FREE_TASK_LIST:
   case PRIM_ARRAY_ALLOC:
   case PRIM_ARRAY_FREE:
   case PRIM_ARRAY_FREE_ELTS:
@@ -288,8 +301,6 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
     // here until they are proven fast.
   case PRIM_GET_END_COUNT:
   case PRIM_SET_END_COUNT:
-  case PRIM_PROCESS_TASK_LIST:
-  case PRIM_EXECUTE_TASKS_IN_LIST:
   case PRIM_TO_LEADER:
   case PRIM_TO_FOLLOWER:
   case PRIM_DELETE:
@@ -300,7 +311,7 @@ isFastPrimitive(CallExpr *call, bool isLocal) {
   case PRIM_RT_ERROR:
   case PRIM_RT_WARNING:
   case PRIM_FTABLE_CALL:
-  case PRIM_VMT_CALL:
+  case PRIM_VIRTUAL_METHOD_CALL:
     return false;
 
   default:
@@ -315,7 +326,10 @@ static bool
 inLocalBlock(CallExpr *call) {
   for (Expr* parent = call->parentExpr; parent; parent = parent->parentExpr) {
     if (BlockStmt* blk = toBlockStmt(parent)) {
-      if (blk->blockInfo && blk->blockInfo->isPrimitive(PRIM_BLOCK_LOCAL))
+      // NOAKES 2014/11/25  Transitional. Do not trip over blockInfoGet for a Loop
+      if (blk->isLoopStmt() == true)
+        ;
+      else if (blk->blockInfoGet() && blk->blockInfoGet()->isPrimitive(PRIM_BLOCK_LOCAL))
         return true;
     }
   }
@@ -330,8 +344,7 @@ markFastSafeFn(FnSymbol *fn, int recurse, Vec<FnSymbol*> *visited) {
   if (fn->hasFlag(FLAG_EXPORT))
     return true;
 
-  if (fn->hasFlag(FLAG_EXTERN)) {
-    // consider a pragma to indicate that it would be "fast"
+  if (fn->hasFlag(FLAG_EXTERN) && !fn->hasFlag(FLAG_FAST_ON_SAFE_EXTERN)) {
     return false;
   }
 
@@ -342,7 +355,7 @@ markFastSafeFn(FnSymbol *fn, int recurse, Vec<FnSymbol*> *visited) {
 
   std::vector<CallExpr*> calls;
 
-  collectCallExprsSTL(fn, calls);
+  collectCallExprs(fn, calls);
 
   for_vector(CallExpr, call, calls) {
     DEBUG_PRINTF("\tcall %p (id=%d): ", call, call->id);
