@@ -1,13 +1,29 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
+#include "optimizations.h"
 
 #include "astutil.h"
 #include "bb.h"
 #include "expr.h"
-#include "optimizations.h"
 #include "passes.h"
+#include "stlUtil.h"
 #include "stmt.h"
 
 //
@@ -30,7 +46,7 @@ class AutoTrack {
  public:
   AutoTrack();
   ~AutoTrack();
- public: 
+ public:
   void move(CallExpr* call);        // Track a move primitive.
   void autoCopy(CallExpr* call);    // Track an autoCopy call.
   void autoDestroy(CallExpr* call); // Track an autoDestroy call.
@@ -178,13 +194,13 @@ void AutoTrack::insertLink(Link** head, CallExpr* call) {
   p->next = *head;
   *head = p;
 }
- 
+
 //
 // Remove autoCopy (and matching autoDestroy calls) that are
 // unnecessary within a function
 //
 static void removeUnnecessaryAutoCopyCalls(FnSymbol* fn) {
-  buildBasicBlocks(fn);
+  BasicBlock::buildBasicBlocks(fn);
 
   bool changed = false;
 
@@ -255,143 +271,18 @@ static void removeUnnecessaryAutoCopyCalls(FnSymbol* fn) {
 #endif
 } 
 
-// A Plain-Old Data type is one whose default-constructor, copy-constructor,
-// assignment operator and destructor are all trivial.  Trivial means that they
-// have no observable side-effects beyond the default behavior provided by
-// the compiler.
-// While it is possible for the user to write ones of these functions that
-// override the default and exhibit the same behavior as the compiler-supplied
-// ones and are thus behaviorally trivial, for practical purposes we choose to
-// treat as nontrivial any user-supplied versions of these functions.
-
-// This routine traverses all defined functions, and for those of interest
-// (being one of the four types above) determines if it is user-defined and
-// finds the class to which it belongs.  It then marks that class as being a
-// non-POD class (since the rules for it being a POD class have been violated).
-//
-// The assignment operator is not currently a member function, so we infer the
-// target type from the type LHS argument.
-// TODO: Add clauses for the copy-constructor and default-constructor.
-//
-// To yield a result that is strictly correct w.r.t. whether a class is POD or
-// non-POD, this routine should be run after resolution but before the
-// post-resolution pruning pass.  However, even though it is run after pruning,
-// the results are correct for our immediate use.  That is because if a function
-// is pruned, that means it is not called.  And if it is not called, whether it
-// is trivial or not is moot question.
-// TODO: Move this routine to a pass that precedes the post-resolution pruning pass.
-//
-static void
-markPODtypes()
-{
-  forv_Vec(FnSymbol, fn, gFnSymbols)
-  {
-    // Ignore invisible (compiler-defined) functions.
-    if (fn->hasFlag(FLAG_COMPILER_GENERATED))
-      continue;
-
-    // Decide whether to mark a class based on this function.
-    enum Flag flag = FLAG_UNKNOWN;
-    AggregateType* ct = NULL;
-
-    if (fn->hasFlag(FLAG_DESTRUCTOR))
-    {
-      // ~chpl_destroy(_mt, obj);
-      ArgSymbol* this_arg = fn->getFormal(1);
-      INT_ASSERT(!strcmp(this_arg->name, "this"));
-      ct = toAggregateType(this_arg->type);
-      if (ct)
-        flag = FLAG_HAS_USER_DESTRUCTOR;
-    }
-
-    if (fn->hasFlag(FLAG_INIT_COPY_FN))
-    {
-      // chpl__initCopy(obj);
-      if (fn->numFormals() > 0)
-      {
-        ArgSymbol* arg = fn->getFormal(1);
-        ct = toAggregateType(arg->type);
-        if (ct)
-          flag = FLAG_HAS_USER_INIT_COPY_FN;
-      }
-    }
-
-    // We should probably use a flag to distinguish assignment operators.  Then
-    // we can test easily that fn->hasFlag(FLAG_HAS_USER_DESTRUCTOR) and
-    // fn->hasFlag(FLAG_HAS_USER_ASSIGNMENT) are mutually exclusive.
-    if (!strcmp(fn->name,"="))
-    {
-      ArgSymbol* lhs = fn->getFormal(1);
-      ct = toAggregateType(lhs->type);
-      if (ct)
-        flag = FLAG_HAS_USER_ASSIGNMENT;
-    }
-
-    if (flag != FLAG_UNKNOWN)
-      ct->symbol->addFlag(flag);
-  }
-}
-
-
-static bool
-isPrimitiveCopy(Vec<Type*>& primitiveCopyTypeSet, Type* type) {
-  if (primitiveCopyTypeSet.set_in(type))
-    return true;
-  if (is_bool_type(type) || is_int_type(type) || is_uint_type(type) ||
-      is_real_type(type) || is_imag_type(type) || is_complex_type(type) ||
-      is_enum_type(type) ||
-      (isClass(type)))
-    return true;
-// These cases should be captured by the test for nontrivial assignment and/or
-//  destruction below.  Those cases still appearing in the conditional are
-//  implemented internally, rather than in module code, so until they are
-//  implemented in module code we have to carry along these special cases.
-//  if (isRecordWrappedType(type) ||
-//      type->symbol->hasFlag(FLAG_REF) ||
-//      type->symbol->hasFlag(FLAG_ITERATOR_RECORD))
-  if (isSyncType(type))
-    return false;
-  if (isRecord(type)) {
-    AggregateType* ct = toAggregateType(type);
-    INT_ASSERT(ct);
-
-    // If this class defines an assignment or initCopy or destructor, it is not
-    // POD.
-    if (ct->symbol->hasFlag(FLAG_HAS_USER_DESTRUCTOR) ||
-        ct->symbol->hasFlag(FLAG_HAS_USER_INIT_COPY_FN) ||
-        ct->symbol->hasFlag(FLAG_HAS_USER_ASSIGNMENT))
-      return false;
-
-    for_fields(field, ct) {
-      if (!isPrimitiveCopy(primitiveCopyTypeSet, field->type))
-        return false;
-    }
-    return true;
-  }
-  return false;
-}
-
 // We can remove the calls to chpl__initCopy (should actually be chpl__autoCopy)
 // and corresponding calls to chpl__autoDestroy for Plain-Old-Data (POD) types.
-// POD types are fundamental types, class types, and records that contain only
-// POD-typed fields (see isPODtype test above).
+// See isPOD() and propagateNotPOD().
 static void removePODinitDestroy()
 {
-  markPODtypes();
   compute_call_sites();
 
-  Vec<Type*> primitiveCopyTypeSet;
-  
   forv_Vec(FnSymbol, fn, gFnSymbols) {
     if (fn->hasFlag(FLAG_INIT_COPY_FN) || fn->hasFlag(FLAG_AUTO_COPY_FN))
     {
       // We expect both initCopy and autoCopy functions to have one argument
       // whose type is the same as the return type.
-      if (fn->numFormals() == 0 && fn->retType == dtNil)
-        // This is an oddity.  It should probably be pruned earlier, and this
-        // test turned into a per-pass verification or removed entirely.
-        continue; 
-
       INT_ASSERT(fn->numFormals() == 1);
 
       if (fn->getFormal(1)->type != fn->retType)
@@ -401,7 +292,7 @@ static void removePODinitDestroy()
         // type mismatch at the call sites, so we just punt.
         continue;
 
-      if (isPrimitiveCopy(primitiveCopyTypeSet, fn->retType)) {
+      if (isPOD(fn->retType)) {
         forv_Vec(CallExpr, call, *fn->calledBy) {
           Expr* actual = call->get(1);
           ArgSymbol* arg = actual_to_formal(actual);
