@@ -1,14 +1,28 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
+#include "countTokens.h"
 
 #include <cstring>
-#include "countTokens.h"
-#include "misc.h"
-#include "processTokens.h"
-#include "yy.h"
 
+#include "bison-chapel.h"
+#include "misc.h"
 
 bool countTokens = false;
 bool printTokens = false;
@@ -45,7 +59,7 @@ static void clearHist(int h[]) {
 }
 
 
-static void clearLine(void) {
+static void clearLine() {
   if (firstLineInFile) {
     firstLineInFile = 0;
   } else {
@@ -62,7 +76,7 @@ static void clearLine(void) {
   }
 }
 
-static void clearFile(void) {
+static void clearFile() {
   fileTokens = 0;
   blankLines = 0;
   commentLines = 0;
@@ -72,21 +86,21 @@ static void clearFile(void) {
   firstLineInFile = 1;
 }
 
-static void clearGlob(void) {
+static void clearGlob() {
   totTokens = 0;
   clearHist(totTokenHist);
   maxTokensPerLineTot = 0;
 }
 
 
-static void printSeparator(void) {
+static void printSeparator() {
   fflush(stdout);
-  fprintf(stderr, 
+  fprintf(stderr,
           "=============================================================\n");
 }
 
 
-static void printFileSummary(void) {
+static void printFileSummary() {
   if (printTokens) {
     int j;
 
@@ -95,7 +109,7 @@ static void printFileSummary(void) {
     fprintf(stderr, "LINES  = %d code, %d comment, %d blank\n", codeLines,
             commentLines, blankLines);
     fprintf(stderr, "MAX TOKENS/LINE = %d\n"
-            "AVG TOKENS/CODE LINE = %.2f\n", 
+            "AVG TOKENS/CODE LINE = %.2f\n",
             maxTokensPerLineInFile, (double)fileTokens/codeLines);
     fprintf(stderr, "HISTOGRAM:\n");
     for (j=0; j<=maxTokensPerLineInFile; j++) {
@@ -105,7 +119,7 @@ static void printFileSummary(void) {
 }
 
 
-static void initTokenCount(void) {
+static void initTokenCount() {
   if (printTokens && !countTokens) { // printTokens => countTokens
     countTokens = true;
   }
@@ -137,12 +151,12 @@ void startCountingFileTokens(const char* filename) {
 }
 
 
-void stopCountingFileTokens(void) {
+void stopCountingFileTokens(yyscan_t scanner) {
   tokenCountingOn = false;
-  
+
   if (printTokens) {
     if (strcmp(line, "") != 0) {
-      processNewline();
+      processNewline(scanner);
     }
 
     printSeparator();
@@ -153,7 +167,7 @@ void stopCountingFileTokens(void) {
 }
 
 
-void finishCountingTokens(void) {
+void finishCountingTokens() {
   if (countTokens) {
     if (printTokens) {
       fflush(stdout);
@@ -172,7 +186,7 @@ void countToken(const char* text) {
       int attempted = snprintf(&line[start], LINE_SIZE - start, " %s", text);
       if (attempted >= LINE_SIZE - start) {
         INT_FATAL("line length overflow");
-      } 
+      }
     }
     lineTokens++;
     fileTokens++;
@@ -183,7 +197,7 @@ void countToken(const char* text) {
 }
 
 
-void countNewline(void) {
+void countNewline() {
   if (tokenCountingOn && countTokens) {
     if (lineBlank) {
       blankLines++;
@@ -209,7 +223,7 @@ void countNewline(void) {
 }
 
 
-void countCommentLine(void) {
+void countCommentLine() {
   if (tokenCountingOn && countTokens) {
     lineBlank = false;
   }
@@ -231,7 +245,7 @@ void countSingleLineComment(const char* comment) {
 }
 
 
-void countMultiLineComment(char* comment) {
+void countMultiLineComment(const char* comment) {
   if (printTokens) {
     int start = strlen(line);
     int attempted =
