@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 // addInitCalls.c
 //////////////////////////////////////////////////////////////////////////////////
@@ -18,9 +32,11 @@
 //
 
 #include "passes.h"
-#include "stmt.h"
-#include "build.h"
+
 #include "astutil.h"
+#include "build.h"
+#include "stmt.h"
+#include "stringutil.h"
 
 static void addModuleInitBlocks();
 static void addInitGuards();
@@ -56,11 +72,11 @@ void addModuleInitBlocks() {
       if (parent != theProgram && parent != rootModule)
         initBlock->insertAtTail(new CallExpr(parent->initFn));
 
-    // Now, traverse my use statements, and call the initializer for each
-    // module I use.
+    // Call the initializer for each module I use.
     forv_Vec(ModuleSymbol, usedMod, mod->modUseList) {
-      if (usedMod == standardModule) continue;
-      initBlock->insertAtTail(new CallExpr(usedMod->initFn));
+      if (usedMod != standardModule) {
+        initBlock->insertAtTail(new CallExpr(usedMod->initFn));
+      }
     }
 
     if (initBlock->body.length > 0) fn->insertAtHead(initBlock);
@@ -189,15 +205,11 @@ static void addPrintModInitOrder(FnSymbol* fn)
   int myLen = strlen(s2);
   char lenStr[5];
   sprintf(lenStr, "%d", myLen);
-  Expr *es1 = buildStringLiteral(s1);
-  Expr *es2 = buildStringLiteral(s2);
+  Expr *es1 = buildCStringLiteral(s1);
+  Expr *es2 = buildCStringLiteral(s2);
   Expr *elen = buildIntLiteral(lenStr);
-  CallExpr* s1Init = new CallExpr(PRIM_MOVE, new SymExpr(s1tmp),
-                                  new CallExpr(PRIM_C_STRING_FROM_STRING,
-                                               es1));
-  CallExpr* s2Init = new CallExpr(PRIM_MOVE, new SymExpr(s2tmp),
-                                  new CallExpr(PRIM_C_STRING_FROM_STRING,
-                                               es2));
+  CallExpr* s1Init = new CallExpr(PRIM_MOVE, new SymExpr(s1tmp), es1);
+  CallExpr* s2Init = new CallExpr(PRIM_MOVE, new SymExpr(s2tmp), es2);
   CallExpr *printModInit = new CallExpr(gPrintModuleInitFn,
                                         new SymExpr(s1tmp),
                                         new SymExpr(s2tmp), elen);
