@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 //
 // cleanup
@@ -11,6 +25,7 @@
 //
 
 #include "astutil.h"
+#include "stlUtil.h"
 #include "build.h"
 #include "expr.h"
 #include "passes.h"
@@ -38,12 +53,7 @@ flatten_scopeless_block(BlockStmt* block) {
 // call expressions
 //
 static void normalize_nested_function_expressions(DefExpr* def) {
-  if ((!strncmp("_anon_record", def->sym->name, 12)) ||
-      (!strncmp("_parloopexpr", def->sym->name, 12)) ||
-      (!strncmp("_seqloopexpr", def->sym->name, 12)) ||
-      (!strncmp("_forallexpr", def->sym->name, 11)) ||
-      (!strncmp("_forallinit", def->sym->name, 11)) ||
-      def->sym->hasFlag(FLAG_COMPILER_NESTED_FUNCTION)) {
+  if (def->sym->hasFlag(FLAG_COMPILER_NESTED_FUNCTION)) {
     Expr* stmt = def->getStmtExpr();
     if (!stmt) {
       if (TypeSymbol* ts = toTypeSymbol(def->parentSymbol)) {
@@ -85,8 +95,8 @@ insertDestructureStatements(Expr* S1, Expr* S2, CallExpr* lhs, Expr* rhs) {
     buildIfStmt(new CallExpr("!=",
                              new SymExpr(new_IntSymbol(lhs->numActuals())),
                              new CallExpr(".", rhs->copy(),
-                                          new_StringSymbol("size"))),
-                new CallExpr("compilerError", new_StringSymbol("tuple size must match the number of grouped variables"), new_IntSymbol(0))));
+                                          new_CStringSymbol("size"))),
+                new CallExpr("compilerError", new_CStringSymbol("tuple size must match the number of grouped variables"), new_IntSymbol(0))));
 
   for_actuals(expr, lhs) {
     i++;
@@ -147,8 +157,16 @@ static void flatten_primary_methods(FnSymbol* fn) {
     DefExpr* def = fn->defPoint;
     def->remove();
     insertPoint->insertBefore(def);
-    if (fn->userString && fn->name != ts->name)
-      fn->userString = astr(ts->name, ".", fn->userString);
+    if (fn->userString && fn->name != ts->name) {
+      if (strncmp(fn->userString, "ref ", 4) == 0) {
+        // fn->userString of "ref foo()"
+        // Move "ref " before the type name so we end up with "ref Type.foo()"
+        // instead of "Type.ref foo()"
+        fn->userString = astr("ref ", ts->name, ".", fn->userString+4);
+      } else {
+        fn->userString = astr(ts->name, ".", fn->userString);
+      }
+    }
     if (ts->hasFlag(FLAG_SYNC))
       fn->addFlag(FLAG_SYNC);
     if (ts->hasFlag(FLAG_SINGLE))
@@ -161,9 +179,9 @@ static void flatten_primary_methods(FnSymbol* fn) {
 
 static void change_cast_in_where(FnSymbol* fn) {
   if (fn->where) {
-    Vec<BaseAST*> asts;
+    std::vector<BaseAST*> asts;
     collect_asts(fn->where, asts);
-    forv_Vec(BaseAST, ast, asts) {
+    for_vector(BaseAST, ast, asts) {
       if (CallExpr* call = toCallExpr(ast)) {
         if (call->isNamed("_cast")) {
           call->primitive = primitives[PRIM_IS_SUBTYPE];
@@ -176,25 +194,25 @@ static void change_cast_in_where(FnSymbol* fn) {
 
 
 void cleanup(void) {
-  Vec<BaseAST*> asts;
+  std::vector<BaseAST*> asts;
   collect_asts(rootModule, asts);
 
-  forv_Vec(BaseAST, ast, asts) {
+  for_vector(BaseAST, ast, asts) {
     SET_LINENO(ast);
     if (DefExpr* def = toDefExpr(ast)) {
       normalize_nested_function_expressions(def);
     }
   }
 
-  forv_Vec(BaseAST, ast, asts) {
-    SET_LINENO(ast);
-    if (BlockStmt* block = toBlockStmt(ast)) {
+  for_vector(BaseAST, ast1, asts) {
+    SET_LINENO(ast1);
+    if (BlockStmt* block = toBlockStmt(ast1)) {
       if (block->blockTag == BLOCK_SCOPELESS && block->list)
         flatten_scopeless_block(block);
-    } else if (CallExpr* call = toCallExpr(ast)) {
+    } else if (CallExpr* call = toCallExpr(ast1)) {
       if (call->isNamed("_build_tuple"))
         destructureTupleAssignment(call);
-    } else if (DefExpr* def = toDefExpr(ast)) {
+    } else if (DefExpr* def = toDefExpr(ast1)) {
       if (FnSymbol* fn = toFnSymbol(def->sym)) {
         flatten_primary_methods(fn);
         change_cast_in_where(fn);
