@@ -1,19 +1,31 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef __STDC_FORMAT_MACROS
 #define __STDC_FORMAT_MACROS
 #endif
-#include <inttypes.h>
-
-#include <cctype>
-#include <cstring>
-#include <cstdio>
 
+#include "codegen.h"
 
 #include "astutil.h"
+#include "stlUtil.h"
+#include "config.h"
 #include "driver.h"
 #include "expr.h"
 #include "files.h"
@@ -22,15 +34,21 @@
 #include "stmt.h"
 #include "stringutil.h"
 #include "symbol.h"
-#include "config.h"
-#include "codegen.h"
+#include "insertLineNumbers.h"
+
+#include <inttypes.h>
+
+#include <algorithm>
+#include <cctype>
+#include <cstring>
+#include <cstdio>
+#include <vector>
 
 // Global so that we don't have to pass around
 // to all of the codegen() routines
-GenInfo* gGenInfo;
-int gMaxVMT = -1;
-fileinfo gAllExternCode;
-int stmtCount = 0;
+GenInfo* gGenInfo   =  0;
+int      gMaxVMT    = -1;
+int      gStmtCount =  0;
 
 
 static const char*
@@ -44,7 +62,7 @@ subChar(Symbol* sym, const char* ch, const char* x) {
 }
 
 static void legalizeName(Symbol* sym) {
-  if (sym->hasFlag(FLAG_EXTERN))
+  if (!sym->isRenameable())
     return;
   for (const char* ch = sym->cname; *ch != '\0'; ch++) {
     switch (*ch) {
@@ -142,7 +160,7 @@ genGlobalString(const char* cname, const char* value) {
         info->module->getOrInsertGlobal(
           cname, llvm::IntegerType::getInt8PtrTy(info->module->getContext())));
     globalString->setInitializer(llvm::cast<llvm::GlobalVariable>(
-          new_StringSymbol(value)->codegen().val)->getInitializer());
+          new_CStringSymbol(value)->codegen().val)->getInitializer());
     globalString->setConstant(true);
     info->lvt->addGlobalValue(cname, globalString, GEN_PTR, true);
 #endif
@@ -164,6 +182,23 @@ genGlobalInt(const char* cname, int value) {
 #endif
   }
 }
+
+static void genGlobalInt32(const char *cname, int value) {
+  GenInfo *info = gGenInfo;
+  if (info->cfile) {
+    fprintf(info->cfile, "const int32_t %s = %d;\n", cname, value);
+  } else {
+#ifdef HAVE_LLVM
+    llvm::GlobalVariable *globalInt =
+        llvm::cast<llvm::GlobalVariable>(info->module->getOrInsertGlobal(
+            cname, llvm::IntegerType::getInt32Ty(info->module->getContext())));
+    globalInt->setInitializer(info->builder->getInt32(value));
+    globalInt->setConstant(true);
+    info->lvt->addGlobalValue(cname, globalInt, GEN_PTR, false);
+#endif
+  }
+}
+
 static void
 genClassIDs(Vec<TypeSymbol*> & typeSymbols) {
   genComment("Class Type Identification Numbers");
@@ -271,6 +306,8 @@ genVirtualMethodTable(Vec<TypeSymbol*>& types) {
         }
       }
     }
+    if (types.n == 0 || maxVMT == 0)
+      fprintf(hdrfile, "(chpl_fn_p)0");
     fprintf(hdrfile, "\n};\n");
   } else {
 #ifdef HAVE_LLVM
@@ -319,6 +356,58 @@ genVirtualMethodTable(Vec<TypeSymbol*>& types) {
   }
 }
 
+static void genFilenameTable() {
+  GenInfo *info = gGenInfo;
+  const char *tableName = "chpl_filenameTable";
+  const char *sizeName = "chpl_filenameTableSize";
+  if (info->cfile) {
+    FILE *hdrfile = info->cfile;
+
+    fprintf(hdrfile, "c_string %s[] = {\n", tableName);
+
+    bool first = true;
+    for (std::vector<std::string>::iterator it = gFilenameLookup.begin();
+         it != gFilenameLookup.end(); it++) {
+      if (!first)
+        fprintf(hdrfile, ",\n");
+      fprintf(hdrfile, "    \"%s\"", (*it).c_str());
+      first = false;
+    }
+
+    fprintf(hdrfile, "\n};\n");
+  } else {
+#ifdef HAVE_LLVM
+    std::vector<llvm::Constant *> table(gFilenameLookup.size());
+
+    llvm::Type *c_stringType =
+        llvm::IntegerType::getInt8PtrTy(info->module->getContext());
+
+    int idx = 0;
+    for (std::vector<std::string>::iterator it = gFilenameLookup.begin();
+         it != gFilenameLookup.end(); it++) {
+      table[idx++] = llvm::cast<llvm::GlobalVariable>(
+              new_CStringSymbol((*it).c_str())->codegen().val)->getInitializer();
+    }
+
+    llvm::ArrayType *filenameTableType =
+        llvm::ArrayType::get(c_stringType, table.size());
+
+    if (llvm::GlobalVariable *filenameTable =
+            info->module->getNamedGlobal(tableName)) {
+      filenameTable->eraseFromParent();
+    }
+
+    llvm::GlobalVariable *filenameTable = llvm::cast<llvm::GlobalVariable>(
+        info->module->getOrInsertGlobal(tableName, filenameTableType));
+    filenameTable->setInitializer(
+        llvm::ConstantArray::get(filenameTableType, table));
+    filenameTable->setConstant(true);
+    info->lvt->addGlobalValue(tableName, filenameTable, GEN_PTR, true);
+#endif
+  }
+  genGlobalInt32(sizeName, gFilenameLookup.size());
+}
+
 static int
 compareSymbol(const void* v1, const void* v2) {
   Symbol* s1 = *(Symbol* const *)v1;
@@ -342,32 +431,61 @@ compareSymbol(const void* v1, const void* v2) {
   return result;
 }
 
+static bool
+compareSymbol2(void* v1, void* v2) {
+  Symbol* s1 = (Symbol*)v1;
+  Symbol* s2 = (Symbol*)v2;
+  ModuleSymbol* m1 = s1->getModule();
+  ModuleSymbol* m2 = s2->getModule();
+  if (m1 != m2) {
+    if (m1->modTag < m2->modTag)
+      return 1;
+    if (m1->modTag > m2->modTag)
+      return 0;
+    return strcmp(m1->cname, m2->cname) < 0;
+  }
+
+  if (s1->linenum() != s2->linenum())
+    return s1->linenum() < s2->linenum();
+
+  int result = strcmp(s1->type->symbol->cname, s2->type->symbol->cname);
+  if (!result)
+    result = strcmp(s1->cname, s2->cname);
+
+  return result < 0;
+}
 
 //
 // given a name and up to two sets of names, return a name that is in
 // neither set and add the name to the first set; the second set may
 // be omitted; the returned name to be capped at fMaxCIdentLen if non-0
+// less how much can be added to it - maxCNameAddedChars
 //
 // the unique numbering is based on the map uniquifyNameCounts which
 // can be cleared to reset
 //
 int fMaxCIdentLen = 0;
 static const int maxUniquifyAddedChars = 25;
+// keep in sync with AggregateType::classStructName()
+static const int maxCNameAddedChars = 20;
 static char* longCNameReplacementBuffer = NULL;
 static Map<const char*, int> uniquifyNameCounts;
 static const char* uniquifyName(const char* name,
                                 Vec<const char*>* set1,
                                 Vec<const char*>* set2 = NULL) {
   const char* newName = name;
-  if (fMaxCIdentLen > 0 && (int)(strlen(newName)) > fMaxCIdentLen) {
+  if (fMaxCIdentLen > 0 &&
+      (int)(strlen(newName) + maxCNameAddedChars) > fMaxCIdentLen)
+  {
     // how much of the name to preserve
-    int prefixLen = fMaxCIdentLen - maxUniquifyAddedChars;
+    int prefixLen = fMaxCIdentLen - maxUniquifyAddedChars - maxCNameAddedChars;
     if (!longCNameReplacementBuffer) {
       longCNameReplacementBuffer = (char*)malloc(prefixLen+1);
       longCNameReplacementBuffer[prefixLen] = '\0';
     }
     strncpy(longCNameReplacementBuffer, newName, prefixLen);
     INT_ASSERT(longCNameReplacementBuffer[prefixLen] == '\0');
+    longCNameReplacementBuffer[prefixLen-1] = 'X'; //fyi truncation marker
     name = newName = astr(longCNameReplacementBuffer);
   }
   while (set1->set_in(newName) || (set2 && set2->set_in(newName))) {
@@ -386,11 +504,8 @@ static inline bool shouldCodegenAggregate(AggregateType* ct)
   // never codegen definitions of primitive or arithmetic types.
   if( toPrimitiveType(ct) ) return false;
 
-  // Needed special handling for complex types, since after complex2record
-  // they appear like normal records but we must not define them
-  // since they are defined in the runtime headers
-  // Added a flag, FLAG_NO_CODEGEN, to handle this case.
-  // This flag could be used for other similar cases if necessary.
+  // Don't codegen types with FLAG_NO_CODEGEN.  This is used for
+  // types that are defined in the runtime for example.
   if( ct->symbol->hasFlag(FLAG_NO_CODEGEN) ) return false;
 
   // Don't visit classes since they are prototyped individually all at once..
@@ -398,7 +513,7 @@ static inline bool shouldCodegenAggregate(AggregateType* ct)
   //   we do visit.
   if( isClass(ct) ) { // is it actually a class?
     if( ct->symbol->hasFlag(FLAG_REF) ||
-        ct->symbol->hasFlag(FLAG_WIDE) ||
+        ct->symbol->hasFlag(FLAG_WIDE_REF) ||
         ct->symbol->hasFlag(FLAG_DATA_CLASS)) return true;
     else return false;
   }
@@ -409,9 +524,10 @@ static inline bool shouldCodegenAggregate(AggregateType* ct)
 
 
 static void codegen_aggregate_def(AggregateType* ct) {
+  //DFS, check visited
   if (!shouldCodegenAggregate(ct)) return;
-  if (ct->symbol->codegenned) return;
-  ct->symbol->codegenned = true;
+  if (ct->symbol->hasFlag(FLAG_CODEGENNED)) return;
+  ct->symbol->addFlag(FLAG_CODEGENNED);
 
   // For reference or data class types, first generate
   // the referenced type
@@ -437,12 +553,171 @@ static void codegen_aggregate_def(AggregateType* ct) {
 }
 
 
+//
+// Produce compilation-time configuration info into a .c file and
+// #include that .c into the current codegen output file.
+//
+// Only put C data objects into this file, not Chapel ones, as it may
+// also be #include'd into a launcher, and those are C/C++ code.
+//
+// New generated variables should be added to runtime/include/chplcgfns.h
+//
+static const char* sCfgFname = "chpl_compilation_config";
+
+static void codegen_header_compilation_config() {
+  fileinfo cfgfile = { NULL, NULL, NULL };
+
+  openCFile(&cfgfile, sCfgFname, "c");
+#ifdef HAVE_LLVM
+  gChplCompilationConfig = cfgfile; // so LLVM backend can use it too.
+#endif
+
+  // follow convention of just not writing to the file if we can't open it
+  if (cfgfile.fptr != NULL) {
+    FILE* save_cfile = gGenInfo->cfile;
+
+    gGenInfo->cfile = cfgfile.fptr;
+
+    genComment("Compilation Info");
+
+    fprintf(cfgfile.fptr, "\n#include <stdio.h>\n");
+    fprintf(cfgfile.fptr, "\n#include \"chpltypes.h\"\n");
+
+    genGlobalString("chpl_compileCommand", compileCommand);
+    genGlobalString("chpl_compileVersion", compileVersion);
+    genGlobalString("CHPL_HOME",           CHPL_HOME);
+
+    genGlobalInt("CHPL_STACK_CHECKS", !fNoStackChecks);
+    genGlobalInt("CHPL_CACHE_REMOTE", fCacheRemote);
+
+    for (std::map<std::string, const char*>::iterator env=envMap.begin(); env!=envMap.end(); ++env) {
+      if (env->first != "CHPL_HOME") {
+        genGlobalString(env->first.c_str(), env->second);
+      }
+    }
+
+    // generate the "about" function
+    fprintf(cfgfile.fptr, "\nvoid chpl_program_about(void);\n");
+    fprintf(cfgfile.fptr, "\nvoid chpl_program_about() {\n");
+
+    fprintf(cfgfile.fptr,
+            "printf(\"%%s\", \"Compilation command: %s\\n\");\n",
+            compileCommand);
+    fprintf(cfgfile.fptr,
+            "printf(\"%%s\", \"Chapel compiler version: %s\\n\");\n",
+            compileVersion);
+    fprintf(cfgfile.fptr, "printf(\"Chapel environment:\\n\");\n");
+    fprintf(cfgfile.fptr,
+            "printf(\"%%s\", \"  CHPL_HOME: %s\\n\");\n",
+            CHPL_HOME);
+    for (std::map<std::string, const char*>::iterator env=envMap.begin(); env!=envMap.end(); ++env) {
+      if (env->first != "CHPL_HOME") {
+        fprintf(cfgfile.fptr,
+          "printf(\"%%s\", \"  %s: %s\\n\");\n",
+          env->first.c_str(),
+          env->second);
+      }
+    }
+
+    fprintf(cfgfile.fptr, "}\n");
+
+    genComment("Filename Lookup Table");
+    genFilenameTable();
+
+    closeCFile(&cfgfile);
+
+    gGenInfo->cfile = save_cfile;
+  }
+}
+
+
+static void protectNameFromC(Symbol* sym) {
+  //
+  // Symbols that start with 'chpl_' were presumably named by the
+  // implementation (compiler, internal modules, runtime) and
+  // sufficiently unique to not require further munging.
+  //
+  if (strncmp(sym->cname, "chpl_", 5) == 0) {
+    return;
+  }
+
+  //
+  // For now, we only rename our user and standard symbols.  Internal
+  // modules symbols should arguably similarly be protected, to ensure
+  // that we haven't inadvertantly used a name that some user library
+  // will; most file-level symbols should be protected by 'chpl_' or
+  // somesuch, but of course local symbols may not be, and can cause
+  // conflicts (at present, a local variable named 'socket' would).
+  // The challenges to handling MOD_INTERNAL symbols in the same way
+  // today is that things like chpl_string and uint64_t should not be
+  // renamed, and should arguably have FLAG_EXTERN on them; however,
+  // putting it on them causes it to bleed over onto type aliases in a
+  // way that breaks things and wasn't easy to fix.  So this remains
+  // a TODO (currently in Brad's court).
+  //
+  ModuleSymbol* symMod = sym->getModule();
+  if (symMod->modTag == MOD_INTERNAL) {
+    return;
+  }
+
+  //
+  // If this symbol is exported of an extern symbol then someone
+  // outside of Chapel is relying on it to have a certain name and we
+  // need to respect that.
+  //
+  if (!sym->isRenameable()) {
+    return;
+  }
+
+  //
+  // Walk from the symbol up to its enclosing module.  If the symbol
+  // is declared within an extern declaration, we should preserve its
+  // name for similar reasons.
+  //
+  if (sym != symMod) {
+    Symbol* parentSym = sym->defPoint->parentSymbol;
+    while (parentSym != symMod) {
+      if (parentSym->hasFlag(FLAG_EXTERN)) {
+        return;
+      }
+      parentSym = parentSym->defPoint->parentSymbol;
+    }
+  }
+
+  //
+  // For the sake of clarity, let's also avoid renaming arguments of
+  // exported functions.
+  //
+  if (toArgSymbol(sym)) {
+    Symbol* parentSym = sym->defPoint->parentSymbol;
+    if (parentSym->hasFlag(FLAG_EXPORT)) {
+      return;
+    }
+  }
+
+  //
+  // Rename the symbol
+  //
+  const char* oldName = sym->cname;
+  const char* newName = astr(oldName, "_chpl");
+  sym->cname = newName;
+  //
+  // Can we free this given how we create names?  free() doesn't like
+  // const char*, I don't want to just cast it away, and I'm not
+  // certain we can assume it isn't aliased to someting else, like
+  // sym->name...  In other cases, we seem to leak old names as
+  // well... :P
+  //
+  //  free(oldName);
+}
+
+
 // TODO: Split this into a number of smaller routines.<hilde>
 static void codegen_header() {
   GenInfo* info = gGenInfo;
   Vec<const char*> cnames;
   Vec<TypeSymbol*> types;
-  Vec<FnSymbol*> functions;
+  std::vector<FnSymbol*> functions;
   Vec<VarSymbol*> globals;
 
   // reserved symbol names that require renaming to compile
@@ -476,16 +751,46 @@ static void codegen_header() {
   //
   forv_Vec(FnSymbol, fn, gFnSymbols) {
     legalizeName(fn);
-    functions.add(fn);
+    functions.push_back(fn);
+  }
+  std::sort(functions.begin(), functions.end(), compareSymbol2);
+
+
+  //
+  // by default, mangle all Chapel symbols to avoid clashing with C
+  // identifiers.  This can be disabled via the --munge-user-idents
+  // flag.
+  //
+  if (fMungeUserIdents) {
+    forv_Vec(ModuleSymbol, sym, gModuleSymbols) {
+      protectNameFromC(sym);
+    }
+    forv_Vec(VarSymbol, sym, gVarSymbols) {
+      protectNameFromC(sym);
+    }
+    forv_Vec(ArgSymbol, sym, gArgSymbols) {
+      protectNameFromC(sym);
+    }
+    forv_Vec(TypeSymbol, sym, gTypeSymbols) {
+      protectNameFromC(sym);
+    }
+    forv_Vec(FnSymbol, sym, gFnSymbols) {
+      protectNameFromC(sym);
+    }
+    forv_Vec(EnumSymbol, sym, gEnumSymbols) {
+      protectNameFromC(sym);
+    }
+    forv_Vec(LabelSymbol, sym, gLabelSymbols) {
+      protectNameFromC(sym);
+    }
   }
-  qsort(functions.v, functions.n, sizeof(functions.v[0]), compareSymbol);
 
 
   //
   // mangle type names if they clash with other types
   //
   forv_Vec(TypeSymbol, ts, types) {
-    if (!ts->hasFlag(FLAG_EXTERN))
+    if (ts->isRenameable())
       ts->cname = uniquifyName(ts->cname, &cnames);
   }
   uniquifyNameCounts.clear();
@@ -528,7 +833,7 @@ static void codegen_header() {
   // constants, or other global variables
   //
   forv_Vec(VarSymbol, var, globals) {
-    if (!var->hasFlag(FLAG_EXTERN))
+    if (var->isRenameable())
       var->cname = uniquifyName(var->cname, &cnames);
   }
   uniquifyNameCounts.clear();
@@ -537,8 +842,8 @@ static void codegen_header() {
   // mangle function names if they clash with types, enum constants,
   // global variables, or other functions
   //
-  forv_Vec(FnSymbol, fn, functions) {
-    if (!fn->hasFlag(FLAG_USER_NAMED))
+  for_vector(FnSymbol, fn, functions) {
+    if (fn->isRenameable())
       fn->cname = uniquifyName(fn->cname, &cnames);
   }
   uniquifyNameCounts.clear();
@@ -569,9 +874,9 @@ static void codegen_header() {
       local.set_add(formal->cname);
     }
 
-    Vec<DefExpr*> defs;
+    std::vector<DefExpr*> defs;
     collectDefExprs(fn->body, defs);
-    forv_Vec(DefExpr, def, defs) {
+    for_vector(DefExpr, def, defs) {
       legalizeName(def->sym);
       // give temps cnames
       if (def->sym->hasFlag(FLAG_TEMP)) {
@@ -580,8 +885,8 @@ static void codegen_header() {
           if (!strncmp(def->sym->cname, "_t", 2))
             def->sym->cname = astr("T", def->sym->cname + 2);
         } else {
-          // temp name is _tmp
-          if (!strcmp(def->sym->cname, "_tmp"))
+          // temp name is tmp
+          if (!strcmp(def->sym->cname, "tmp"))
             def->sym->cname = astr("T");
         }
       }
@@ -590,34 +895,9 @@ static void codegen_header() {
     uniquifyNameCounts.clear();
   }
 
-  FILE* hdrfile = info->cfile;
-  genComment("Compilation Info");
+  codegen_header_compilation_config();
 
-  genGlobalString("chpl_compileCommand", compileCommand);
-  genGlobalString("chpl_compileVersion", compileVersion);
-  genGlobalString("CHPL_HOME", CHPL_HOME);
-  genGlobalString("CHPL_HOST_PLATFORM", CHPL_HOST_PLATFORM);
-  genGlobalString("CHPL_HOST_COMPILER", CHPL_HOST_COMPILER);
-  genGlobalString("CHPL_TARGET_PLATFORM", CHPL_TARGET_PLATFORM);
-  genGlobalString("CHPL_TARGET_COMPILER", CHPL_TARGET_COMPILER);
-  genGlobalString("CHPL_LOCALE_MODEL", CHPL_LOCALE_MODEL);
-  genGlobalString("CHPL_COMM", CHPL_COMM);
-  genGlobalString("CHPL_COMM_SUBSTRATE", CHPL_COMM_SUBSTRATE);
-  genGlobalString("CHPL_GASNET_SEGMENT", CHPL_GASNET_SEGMENT);
-  genGlobalString("CHPL_TASKS", CHPL_TASKS);
-  genGlobalString("CHPL_THREADS", CHPL_THREADS);
-  genGlobalString("CHPL_LAUNCHER", CHPL_LAUNCHER);
-  genGlobalString("CHPL_TIMERS", CHPL_TIMERS);
-  genGlobalString("CHPL_MEM", CHPL_MEM);
-  genGlobalString("CHPL_MAKE", CHPL_MAKE);
-  genGlobalString("CHPL_ATOMICS", CHPL_ATOMICS);
-  genGlobalString("CHPL_NETWORK_ATOMICS", CHPL_NETWORK_ATOMICS);
-  genGlobalString("CHPL_GMP", CHPL_GMP);
-  genGlobalString("CHPL_HWLOC", CHPL_HWLOC);
-  genGlobalString("CHPL_REGEXP", CHPL_REGEXP);
-  genGlobalString("CHPL_WIDE_POINTERS", CHPL_WIDE_POINTERS);
-  genGlobalString("CHPL_LLVM", CHPL_LLVM);
-  genGlobalString("CHPL_AUX_FILESYS", CHPL_AUX_FILESYS);
+  FILE* hdrfile = info->cfile;
 
   if( hdrfile ) {
     // This is done in runClang for LLVM version.
@@ -630,10 +910,17 @@ static void codegen_header() {
 
     fprintf(hdrfile, "#include \"stdchpl.h\"\n");
 
+    // Include the compilation config file
+    fprintf(hdrfile, "#include \"%s.c\"\n", sCfgFname);
+
+#ifdef HAVE_LLVM
     //include generated extern C header file
     if (externC && gAllExternCode.filename != NULL) {
       fprintf(hdrfile, "%s", astr("#include \"", gAllExternCode.filename, "\"\n"));
+      // If we wanted to, here is where we would re-enable
+      // the memory warning macros.
     }
+#endif
   }
 
   genClassIDs(types);
@@ -705,17 +992,17 @@ static void codegen_header() {
 
 
   genComment("Function Prototypes");
-  forv_Vec(FnSymbol, fnSymbol, functions) {
+  for_vector(FnSymbol, fnSymbol, functions) {
     fnSymbol->codegenPrototype();
   }
     
   genComment("Function Pointer Table");
-  forv_Vec(FnSymbol, fn, functions) {
-    if (fn->hasFlag(FLAG_BEGIN_BLOCK) ||
-        fn->hasFlag(FLAG_COBEGIN_OR_COFORALL_BLOCK) ||
-        fn->hasFlag(FLAG_ON_BLOCK)) {
-    ftableVec.add(fn);
-    ftableMap.put(fn, ftableVec.n-1);
+  for_vector(FnSymbol, fn2, functions) {
+    if (fn2->hasFlag(FLAG_BEGIN_BLOCK) ||
+        fn2->hasFlag(FLAG_COBEGIN_OR_COFORALL_BLOCK) ||
+        fn2->hasFlag(FLAG_ON_BLOCK)) {
+    ftableVec.add(fn2);
+    ftableMap[fn2] = ftableVec.n-1;
     }
   }
 
@@ -773,7 +1060,7 @@ static void codegen_header() {
     std::vector<llvm::Constant *> memDescTable;
     forv_Vec(const char*, memDesc, memDescsVec) {
       memDescTable.push_back(llvm::cast<llvm::GlobalVariable>(
-            new_StringSymbol(memDesc)->codegen().val)->getInitializer());
+            new_CStringSymbol(memDesc)->codegen().val)->getInitializer());
     }
     llvm::ArrayType *memDescTableType = llvm::ArrayType::get(
         llvm::IntegerType::getInt8PtrTy(info->module->getContext()),
@@ -795,8 +1082,6 @@ static void codegen_header() {
 
   genGlobalInt("chpl_mem_numDescs", memDescsVec.n);
 
-  genGlobalInt("chpl_no_stdmodules", fNoInternalModules);
-
   //
   // add table of private-broadcast constants
   //
@@ -915,7 +1200,7 @@ codegen_config() {
     fprintf(outfile, "initConfigVarTable();\n");
 
     forv_Vec(VarSymbol, var, gVarSymbols) {
-      if (var->hasFlag(FLAG_CONFIG) && !var->hasFlag(FLAG_TYPE_VARIABLE)) {
+      if (var->hasFlag(FLAG_CONFIG) && !var->isType()) {
         fprintf(outfile, "installConfigVar(\"%s\", \"", var->name);
         Type* type = var->type;
         if (type->symbol->hasFlag(FLAG_WIDE_CLASS))
@@ -963,15 +1248,19 @@ codegen_config() {
     info->builder->SetInsertPoint(createConfigBlock);
 
     llvm::Function *initConfigFunc = getFunctionLLVM("initConfigVarTable");
+#if HAVE_LLVM_VER >= 37
+    info->builder->CreateCall(initConfigFunc, {} );
+#else
     info->builder->CreateCall(initConfigFunc);
+#endif
 
     llvm::Function *installConfigFunc = getFunctionLLVM("installConfigVar");
 
     forv_Vec(VarSymbol, var, gVarSymbols) {
-      if (var->hasFlag(FLAG_CONFIG) && !var->hasFlag(FLAG_TYPE_VARIABLE)) {
+      if (var->hasFlag(FLAG_CONFIG) && !var->isType()) {
         std::vector<llvm::Value *> args (3);
         args[0] = info->builder->CreateLoad(
-            new_StringSymbol(var->name)->codegen().val);
+            new_CStringSymbol(var->name)->codegen().val);
 
         Type* type = var->type;
         if (type->symbol->hasFlag(FLAG_WIDE_CLASS)) {
@@ -984,22 +1273,22 @@ codegen_config() {
           type = type->getField("addr")->type;
         }
         args[1] = info->builder->CreateLoad(
-            new_StringSymbol(type->symbol->name)->codegen().val);
+            new_CStringSymbol(type->symbol->name)->codegen().val);
 
         if (var->getModule()->modTag == MOD_INTERNAL) {
           args[2] = info->builder->CreateLoad(
-              new_StringSymbol("Built-in")->codegen().val);
+              new_CStringSymbol("Built-in")->codegen().val);
         }
         else {
           args[2] =info->builder->CreateLoad(
-              new_StringSymbol(var->getModule()->name)->codegen().val);
+              new_CStringSymbol(var->getModule()->name)->codegen().val);
         }
 
         info->builder->CreateCall(installConfigFunc, args);
       }
     }
     info->builder->CreateRetVoid();
-    llvm::verifyFunction(*createConfigFunc);
+    //llvm::verifyFunction(*createConfigFunc);
 #endif
   }
 }
@@ -1053,20 +1342,24 @@ void codegen(void) {
 
   SET_LINENO(rootModule);
 
-  fileinfo hdrfile, mainfile;
-  GenInfo *info = gGenInfo;
+  fileinfo hdrfile  = { NULL, NULL, NULL };
+  fileinfo mainfile = { NULL, NULL, NULL };
+
+  GenInfo* info     = gGenInfo;
+
   INT_ASSERT(info);
+
   if( llvmCodegen ) {
 #ifdef HAVE_LLVM
-    fileinfo nullfile = {NULL, NULL, NULL};
-    hdrfile = mainfile = nullfile; 
     if( fHeterogeneous )
       INT_FATAL("fHeretogeneous not yet supported with LLVM");
+
     prepareCodegenLLVM();
 #endif
   } else {
-    openCFile(&hdrfile, "chpl__header", "h");
-    openCFile(&mainfile, "_main", "c");
+    openCFile(&hdrfile,  "chpl__header", "h");
+    openCFile(&mainfile, "_main",        "c");
+
     fprintf(mainfile.fptr, "#include \"chpl__header.h\"\n");
 
     codegen_makefile(&mainfile);
@@ -1091,77 +1384,76 @@ void codegen(void) {
 
     finishCodegenLLVM();
 #endif 
-    return;
-  }
-
-  if (fHeterogeneous) {
-    codegenTypeStructureInclude(mainfile.fptr);
-    forv_Vec(TypeSymbol, ts, gTypeSymbols) {
-      if ((ts->type != dtOpaque) &&
-          (!toPrimitiveType(ts->type) ||
-           !toPrimitiveType(ts->type)->isInternalType)) {
-        registerTypeToStructurallyCodegen(ts);
+  } else {
+    if (fHeterogeneous) {
+      codegenTypeStructureInclude(mainfile.fptr);
+      forv_Vec(TypeSymbol, ts, gTypeSymbols) {
+        if ((ts->type != dtOpaque) &&
+            (!toPrimitiveType(ts->type) ||
+             !toPrimitiveType(ts->type)->isInternalType)) {
+          registerTypeToStructurallyCodegen(ts);
+        }
       }
     }
+
+    ChainHashMap<char*, StringHashFns, int> filenames;
+    forv_Vec(ModuleSymbol, currentModule, allModules) {
+      mysystem(astr("# codegen-ing module", currentModule->name),
+               "generating comment for --print-commands option");
+
+      // Macs are case-insensitive when it comes to files, so
+      // the following bit of code creates a unique filename
+      // with case-insensitivity taken into account
+
+      // create the lowercase filename
+      char lowerFilename[FILENAME_MAX];
+      sprintf(lowerFilename, "%s", currentModule->name);
+      for (unsigned int i=0; i<strlen(lowerFilename); i++) {
+        lowerFilename[i] = tolower(lowerFilename[i]);
+      }
+
+      // create a filename by bumping a version number until we get a
+      // filename we haven't seen before
+      char filename[FILENAME_MAX];
+      sprintf(filename, "%s", lowerFilename);
+      int version = 1;
+      while (filenames.get(filename)) {
+        version++;
+        sprintf(filename, "%s%d", lowerFilename, version);
+      }
+      filenames.put(filename, 1);
+
+      // build the real filename using that version number -- preserves
+      // case by default by going back to currentModule->name rather
+      // than using the lowercase filename
+      if (version == 1) {
+        sprintf(filename, "%s", currentModule->name);
+      } else {
+        sprintf(filename, "%s%d", currentModule->name, version);
+      }
+
+      fileinfo modulefile;
+      openCFile(&modulefile, filename, "c");
+      info->cfile = modulefile.fptr;
+
+      currentModule->codegenDef();
+      closeCFile(&modulefile);
+      fprintf(mainfile.fptr, "#include \"%s%s\"\n", filename, ".c");
+    }
+
+    if (fHeterogeneous)
+      codegenTypeStructures(hdrfile.fptr);
+
+    info->cfile = hdrfile.fptr;
+    codegen_header_addons();
+
+    closeCFile(&hdrfile);
+    closeCFile(&mainfile);
   }
 
-  ChainHashMap<char*, StringHashFns, int> filenames;
-  forv_Vec(ModuleSymbol, currentModule, allModules) {
-    mysystem(astr("# codegen-ing module", currentModule->name),
-             "generating comment for --print-commands option");
-
-    // Macs are case-insensitive when it comes to files, so
-    // the following bit of code creates a unique filename
-    // with case-insensitivity taken into account
-
-    // create the lowercase filename
-    char lowerFilename[FILENAME_MAX];
-    sprintf(lowerFilename, "%s", currentModule->name);
-    for (unsigned int i=0; i<strlen(lowerFilename); i++) {
-      lowerFilename[i] = tolower(lowerFilename[i]);
-    }
-
-    // create a filename by bumping a version number until we get a
-    // filename we haven't seen before
-    char filename[FILENAME_MAX];
-    sprintf(filename, "%s", lowerFilename);
-    int version = 1;
-    while (filenames.get(filename)) {
-      version++;
-      sprintf(filename, "%s%d", lowerFilename, version);
-    }
-    filenames.put(filename, 1);
-
-    // build the real filename using that version number -- preserves
-    // case by default by going back to currentModule->name rather
-    // than using the lowercase filename
-    if (version == 1) {
-      sprintf(filename, "%s", currentModule->name);
-    } else {
-      sprintf(filename, "%s%d", currentModule->name, version);
-    }
-    
-    fileinfo modulefile;
-    openCFile(&modulefile, filename, "c");
-    info->cfile = modulefile.fptr;
-    
-    currentModule->codegenDef();
-    closeCFile(&modulefile);
-    fprintf(mainfile.fptr, "#include \"%s%s\"\n", filename, ".c");
-  }
-
-  if (fHeterogeneous) 
-    codegenTypeStructures(hdrfile.fptr);
-
-  info->cfile = hdrfile.fptr;
-  codegen_header_addons();
-
-  closeCFile(&hdrfile);
-  closeCFile(&mainfile);
-
   if (fPrintEmittedCodeSize)
   {
-    fprintf(stderr, "Statements emitted: %d\n", stmtCount);
+    fprintf(stderr, "Statements emitted: %d\n", gStmtCount);
   }
 }
 
@@ -1252,14 +1544,14 @@ std::string numToString(int64_t num)
 std::string int64_to_string(int64_t i)
 {
   char buf[32];
-  sprintf(buf, "%"PRId64, i);
+  sprintf(buf, "%" PRId64, i);
   std::string ret(buf);
   return ret;
 }
 std::string uint64_to_string(uint64_t i)
 {
   char buf[32];
-  sprintf(buf, "%"PRIu64, i);
+  sprintf(buf, "%" PRIu64, i);
   std::string ret(buf);
   return ret;
 }
