@@ -1,12 +1,32 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
+#include "insertLineNumbers.h"
+
+#include <algorithm>
+
+#include "passes.h"
 
 #include "astutil.h"
 #include "expr.h"
-#include "passes.h"
 #include "stmt.h"
+#include "stringutil.h"
 #include "symbol.h"
 
 //
@@ -19,6 +39,37 @@
 // number and a filename.
 //
 
+
+std::vector<std::string> gFilenameLookup;
+// Caches lookups into our filename vector
+std::map<std::string, int> gFilenameLookupCache;
+
+static int getFilenameLookupPosition(std::string name) {
+  std::map<std::string, int>::iterator it = gFilenameLookupCache.find(name);
+  int idx;
+  if (it != gFilenameLookupCache.end()) {
+    // Use cached position in the table
+    idx = it->second;
+  } else {
+    // not found - new add new filename to the cache
+    std::vector<std::string>::iterator vIt =
+        std::find(gFilenameLookup.begin(), gFilenameLookup.end(), name);
+    if (vIt == gFilenameLookup.end()) {
+      // Not in the lookup table either, add it (This case should only be for
+      // command line casts, doesn't really matter if something else sneaks in
+      // though)
+      gFilenameLookup.push_back(name);
+      idx = gFilenameLookup.size() - 1;
+    } else {
+      idx = vIt - gFilenameLookup.begin();
+    }
+    gFilenameLookupCache.insert(std::pair<std::string, int>(name, idx));
+  }
+  INT_ASSERT(idx >= 0);
+  INT_ASSERT((size_t)idx < gFilenameLookup.size());
+  return idx;
+}
+
 static void moveLinenoInsideArgBundle();
 
 //
@@ -48,7 +99,7 @@ static ArgSymbol* newLine(FnSymbol* fn) {
 }
 
 static ArgSymbol* newFile(FnSymbol* fn) {
-  ArgSymbol* file = new ArgSymbol(INTENT_CONST_REF, "_fn", dtString);
+  ArgSymbol* file = new ArgSymbol(INTENT_CONST_REF, "_fn", dtInt[INT_SIZE_32]);
   fn->insertFormalAtTail(file);
   filenameMap.put(fn, file);
   queue.add(fn);
@@ -92,16 +143,21 @@ insertLineNumber(CallExpr* call) {
     } else if (call->isPrimitive(PRIM_GET_USER_LINE)) {
       call->replace(new SymExpr(line));
     }
-  } else if (!strcmp(fn->name, "chpl__heapAllocateGlobals") ||
+  } else if (fn->hasFlag(FLAG_EXTERN) ||
+             (fn->hasFlag(FLAG_EXPORT) && !fn->hasFlag(FLAG_INSERT_LINE_FILE_INFO)) ||
+             !strcmp(fn->name, "chpl__heapAllocateGlobals") ||
+             !strcmp(fn->name, "chpl__initStringLiterals") ||
              !strcmp(fn->name, "chpl__initModuleGuards") ||
              !strcmp(fn->name, "chpl_gen_main") ||
-             ((mod->modTag == MOD_USER || mod->modTag == MOD_MAIN) && 
+             ftableMap.count(fn) ||
+             (mod->modTag == MOD_USER && 
               !fn->hasFlag(FLAG_COMPILER_GENERATED) && !fn->hasFlag(FLAG_INLINE)) ||
              (developer && strcmp(fn->name, "halt"))) {
     // hilde sez: This special casing is suspect.  Can't we key off a flag?
 
     // call is in user code; insert AST line number and filename
     // or developer flag is on and the call is not the halt() call
+    // or the call is via the ftable
     if (call->isResolved() &&
         call->isResolved()->hasFlag(FLAG_COMMAND_LINE_SETTING)) {
       call->insertAtTail(new_IntSymbol(0));
@@ -112,23 +168,15 @@ insertLineNumber(CallExpr* call) {
       INT_ASSERT(var);
       INT_ASSERT(var->immediate);
       INT_ASSERT(var->immediate->const_kind == CONST_KIND_STRING);
-      call->insertAtTail(new_StringSymbol(astr("<command line setting of '",
-                                               var->immediate->v_string,
-                                               "'>")));
+      const char *cmdLineSetting =
+          astr("<command line setting of '", var->immediate->v_string, "'>");
+      int filenameIdx = getFilenameLookupPosition(cmdLineSetting);
+      call->insertAtTail(new_IntSymbol(filenameIdx, INT_SIZE_32));
     } else {
-      if (fCLineNumbers) {
-        if (!gCLine) {
-          gCLine = new VarSymbol("__LINE__", dtInt[INT_SIZE_DEFAULT]);
-          rootModule->block->insertAtTail(new DefExpr(gCLine));
-          gCFile = new VarSymbol("__FILE__", dtString);
-          rootModule->block->insertAtTail(new DefExpr(gCFile));
-        }
-        call->insertAtTail(gCLine);
-        call->insertAtTail(gCFile);
-      } else {
-        call->insertAtTail(new_IntSymbol(call->linenum()));
-        call->insertAtTail(new_StringSymbol(call->fname()));
-      }
+      call->insertAtTail(new_IntSymbol(call->linenum()));
+
+      int filenameIdx = getFilenameLookupPosition(call->fname());
+      call->insertAtTail(new_IntSymbol(filenameIdx, INT_SIZE_32));
     }
   } else if (file) {
     // call is in non-user code, but the function already has line
@@ -149,12 +197,18 @@ insertLineNumber(CallExpr* call) {
 
 static bool isClassMethodCall(CallExpr* call) {
   FnSymbol* fn = call->isResolved();
-  if (fn && fn->hasFlag(FLAG_METHOD) && fn->_this) {
+  if (fn && fn->isMethod() && fn->_this) {
     if (AggregateType* ct = toAggregateType(fn->_this->typeInfo())) {
       if (fn->numFormals() > 0 &&
           fn->getFormal(1)->typeInfo() == fn->_this->typeInfo()) {
         if (isClass(ct) || ct->symbol->hasFlag(FLAG_WIDE_CLASS)) {
-          return true;
+          // TODO: When strings are records, remove this protective if.
+          // isClassMethodCall is only used in insertNilChecks() and widened
+          // strings gain FLAG_WIDE_CLASS, but it doesn't make sense to check
+          // strings against nil.
+          if (strcmp(ct->symbol->name, "__wide_chpl_string")) {
+            return true;
+          }
         }
       }
     }
@@ -163,24 +217,37 @@ static bool isClassMethodCall(CallExpr* call) {
 }
 
 
-// insert nil checks primitives in front of all member accesses
-static void insertNilChecks()
-{
+// insert nil checks primitives in front of most member accesses
+//
+// Exceptions:
+//   1) The Chapel specification indicates that it is acceptable to
+//      invoke the destructor on NIL, so we avoid doing so when
+//      handling a call to a destructor.
+//
+static void insertNilChecks() {
   forv_Vec(CallExpr, call, gCallExprs) {
     // A member access is one of these primitives or a method call.
     // A method call is expected to access its "this" argument.
-    if (call->isPrimitive(PRIM_GET_MEMBER) ||
+    if (call->isPrimitive(PRIM_GET_MEMBER)       ||
         call->isPrimitive(PRIM_GET_MEMBER_VALUE) ||
-        call->isPrimitive(PRIM_SET_MEMBER) ||
-        call->isPrimitive(PRIM_GETCID) ||
-        call->isPrimitive(PRIM_TESTCID) ||
+        call->isPrimitive(PRIM_SET_MEMBER)       ||
+        call->isPrimitive(PRIM_GETCID)           ||
+        call->isPrimitive(PRIM_TESTCID)          ||
         isClassMethodCall(call)) {
-      Expr* stmt = call->getStmtExpr();
-      SET_LINENO(stmt);
-      AggregateType* ct = toAggregateType(call->get(1)->typeInfo());
+      Expr*          arg0 = call->get(1);
+      AggregateType* ct   = toAggregateType(arg0->typeInfo());
+
       if (ct && (isClass(ct) || ct->symbol->hasFlag(FLAG_WIDE_CLASS))) {
-        stmt->insertBefore(new CallExpr(PRIM_CHECK_NIL, 
-                                        call->get(1)->copy()));
+        FnSymbol* fn = call->isResolved();
+
+        // Avoid inserting a nil-check if this is a call to a destructor
+        if (fn == NULL || fn->hasFlag(FLAG_DESTRUCTOR) == false) {
+          Expr* stmt = call->getStmtExpr();
+
+          SET_LINENO(stmt);
+
+          stmt->insertBefore(new CallExpr(PRIM_CHECK_NIL, arg0->copy()));
+        }
       }
     }
   }
@@ -189,6 +256,22 @@ static void insertNilChecks()
 void insertLineNumbers() {
   compute_call_sites();
 
+  // Temporary vector that stores some constant filenames that are used
+  // directly by the runtime. The index for these matter, and are provided to
+  // the runtime as defines in chpl-linefile-support.h
+  std::vector<std::string> constantFilenames;
+  // Put a null string into the iterator at the first position, some runtime
+  // calls will pass in NULL for their filename, we can then use this null
+  // string to deal with that case.
+  constantFilenames.push_back("");
+  // Add "<internal>" to the filename table if it didn't make it in there, some
+  // runtime functions use this name directly, and it doesn't always end up in
+  // the table otherwise
+  constantFilenames.push_back("<internal>");
+
+  gFilenameLookup.insert(gFilenameLookup.begin(), constantFilenames.begin(),
+                         constantFilenames.end());
+
   if (!fNoNilChecks) {
     insertNilChecks();
   }
@@ -235,8 +318,8 @@ static void moveLinenoInsideArgBundle()
     //  than the expected number.  Both block types below expect an
     //  argument bundle, and the on-block expects an additional argument
     //  that is the locale on which it should be executed.
-    if ((fn->numFormals() > 2 && fn->hasFlag(FLAG_ON_BLOCK)) ||
-        (fn->numFormals() > 1 && !fn->hasFlag(FLAG_ON_BLOCK) &&
+    if ((fn->numFormals() > 4 && fn->hasFlag(FLAG_ON_BLOCK)) ||
+        (fn->numFormals() > 5 && !fn->hasFlag(FLAG_ON_BLOCK) &&
          (fn->hasFlag(FLAG_BEGIN_BLOCK) ||
           fn->hasFlag(FLAG_COBEGIN_OR_COFORALL_BLOCK)))) {
 
@@ -248,21 +331,29 @@ static void moveLinenoInsideArgBundle()
       DefExpr* lineArg = toDefExpr(fn->formals.tail);
       lineArg->remove();
 
-      // In the body of the wrapper, create local lineno and fname variables 
-      // initialized from the corresponding fields in the argument bundle.
+      // In the body of the wrapper, create a local lineno variable initialized
+      // from the new corresponding field in the argument bundle.
       DefExpr* bundleArg = toDefExpr(fn->formals.tail);
       AggregateType* bundleType = toAggregateType(bundleArg->sym->typeInfo());
+
       VarSymbol* lineField = newTemp("_ln", lineArg->sym->typeInfo());
       bundleType->fields.insertAtTail(new DefExpr(lineField));
-      VarSymbol* fileField = newTemp("_fn", fileArg->sym->typeInfo());
-      bundleType->fields.insertAtTail(new DefExpr(fileField));
-      VarSymbol* fileLocal = newTemp("_fn", fileArg->sym->typeInfo());
+
       VarSymbol* lineLocal = newTemp("_ln", lineArg->sym->typeInfo());
-      fn->insertAtHead("'move'(%S, '.v'(%S, %S))", fileLocal, bundleArg->sym, fileField);
-      fn->insertAtHead(new DefExpr(fileLocal));
+
       fn->insertAtHead("'move'(%S, '.v'(%S, %S))", lineLocal, bundleArg->sym, lineField);
       fn->insertAtHead(new DefExpr(lineLocal));
 
+      // Same thing, just for the filename index now.
+
+      VarSymbol* fileField = newTemp("_fn", fileArg->sym->typeInfo());
+      bundleType->fields.insertAtTail(new DefExpr(fileField));
+
+      VarSymbol* fileLocal = newTemp("_fn", fileArg->sym->typeInfo());
+
+      fn->insertAtHead("'move'(%S, '.v'(%S, %S))", fileLocal, bundleArg->sym, fileField);
+      fn->insertAtHead(new DefExpr(fileLocal));
+
       // Replace references to the (removed) arguments with
       // references to these local variables.
       SymbolMap update;
@@ -278,8 +369,10 @@ static void moveLinenoInsideArgBundle()
         Expr* fileActual = call->argList.tail->remove();
         Expr* lineActual = call->argList.tail->remove();
         Expr* bundleActual = call->argList.tail;
-        call->insertBefore(new CallExpr(PRIM_SET_MEMBER, bundleActual->copy(), lineField, lineActual));
-        call->insertBefore(new CallExpr(PRIM_SET_MEMBER, bundleActual->copy(), fileField, fileActual));
+        call->insertBefore(new CallExpr(PRIM_SET_MEMBER, bundleActual->copy(),
+                                        lineField, lineActual));
+        call->insertBefore(new CallExpr(PRIM_SET_MEMBER, bundleActual->copy(),
+                                        fileField, fileActual));
       }
     }
   }
