@@ -1,19 +1,31 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "passes.h"
 #include "resolveIntents.h"
-#include "view.h"
 
 bool intentsResolved = false;
 
 static IntentTag constIntentForType(Type* t) {
   if (isSyncType(t) ||
       isRecordWrappedType(t) ||  // domain, array, or distribution
-      isRecord(t) ||  // may eventually want to decide based on size
-      is_string_type(t)) {  
+      isRecord(t)) { // may eventually want to decide based on size
     return INTENT_CONST_REF;
   } else if (is_bool_type(t) ||
              is_int_type(t) ||
@@ -28,9 +40,10 @@ static IntentTag constIntentForType(Type* t) {
              t == dtOpaque ||
              t == dtTaskID ||
              t == dtFile ||
-             t == dtTaskList ||
              t == dtNil ||
              t == dtStringC ||
+             t == dtStringCopy ||
+             t == dtCVoidPtr ||
              t->symbol->hasFlag(FLAG_EXTERN)) {
     return INTENT_CONST_IN;
   }
@@ -50,14 +63,14 @@ IntentTag blankIntentForType(Type* t) {
              is_imag_type(t) ||
              is_complex_type(t) ||
              is_enum_type(t) ||
-             is_string_type(t) ||
              t == dtStringC ||
+             t == dtStringCopy ||
+             t == dtCVoidPtr ||
              isClass(t) ||
              isRecord(t) ||
              isUnion(t) ||
              t == dtTaskID ||
              t == dtFile ||
-             t == dtTaskList ||
              t == dtNil ||
              t == dtOpaque ||
              t->symbol->hasFlag(FLAG_DOMAIN) ||
@@ -69,23 +82,26 @@ IntentTag blankIntentForType(Type* t) {
   return INTENT_BLANK;
 }
 
+IntentTag concreteIntent(IntentTag existingIntent, Type* t) {
+  if (existingIntent == INTENT_BLANK) {
+    return blankIntentForType(t);
+  } else if (existingIntent == INTENT_CONST) {
+    return constIntentForType(t);
+  } else {
+    return existingIntent;
+  }
+}
+
+static IntentTag blankIntentForThisArg(Type* t) {
+  // todo: be honest when 't' is an array or domain
+  return INTENT_CONST_IN;
+}
+
 void resolveArgIntent(ArgSymbol* arg) {
-  if (arg->intent == INTENT_BLANK) {
-    if (arg->hasFlag(FLAG_ARG_THIS)) {
-      //
-      // BLC: If we eventually support 'var' intents for 'this' as
-      // part of the opt_this_intent_tag rule in the parser, we would
-      // need to cue off of that information in setting the intent for
-      // "this" here.
-      //
-      arg->intent = INTENT_CONST_IN;
-    } else {
-      arg->intent = blankIntentForType(arg->type);
-    }
-  }
-  if (arg->intent == INTENT_CONST) {
-    arg->intent = constIntentForType(arg->type);
-  }
+  arg->intent = 
+    arg->hasFlag(FLAG_ARG_THIS) && arg->intent == INTENT_BLANK ?
+    blankIntentForThisArg(arg->type) :
+    concreteIntent(arg->intent, arg->type);
 }
 
 void resolveIntents() {
