@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "astutil.h"
 #include "expr.h"
@@ -18,7 +32,7 @@ void returnStarTuplesByRefArgs() {
   compute_call_sites();
 
   forv_Vec(FnSymbol, fn, gFnSymbols) {
-    if ((fn->retType->symbol->hasFlag(FLAG_STAR_TUPLE)) || (fn->retType->symbol->hasFlag(FLAG_FIXED_STRING))) {
+    if ((fn->retType->symbol->hasFlag(FLAG_STAR_TUPLE))) {
       SET_LINENO(fn);
 
       //
@@ -31,7 +45,7 @@ void returnStarTuplesByRefArgs() {
       ArgSymbol* arg = new ArgSymbol(INTENT_REF, "_ret", ret->type->refType);
       fn->insertFormalAtTail(arg);
       fn->retType = dtVoid;
-      fn->insertBeforeReturn(new CallExpr(PRIM_MOVE, arg, ret));
+      fn->insertBeforeReturnAfterLabel(new CallExpr(PRIM_MOVE, arg, ret));
       CallExpr* call = toCallExpr(fn->body->body.tail);
       INT_ASSERT(call && call->isPrimitive(PRIM_RETURN));
       call->get(1)->replace(new SymExpr(gVoid));
@@ -71,6 +85,9 @@ void returnStarTuplesByRefArgs() {
   // SET/GET_SVEC_MEMBER primitives
   //
   forv_Vec(CallExpr, call, gCallExprs) {
+    if (! call->inTree())
+      continue;
+
     if (call->isPrimitive(PRIM_SET_MEMBER) ||
         call->isPrimitive(PRIM_GET_MEMBER) ||
         call->isPrimitive(PRIM_GET_MEMBER_VALUE)) {
