@@ -1,21 +1,38 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef __STDC_FORMAT_MACROS
 #define __STDC_FORMAT_MACROS
 #endif
-#include <cstdlib>
-#include <inttypes.h>
+
+#include "resolution.h"
+
 #include "astutil.h"
 #include "caches.h"
 #include "chpl.h"
 #include "expr.h"
-#include "resolution.h"
 #include "stmt.h"
+#include "stringutil.h"
 #include "symbol.h"
 
+#include <cstdlib>
+#include <inttypes.h>
 
 static int explainInstantiationLine = -2;
 static ModuleSymbol* explainInstantiationModule = NULL;
@@ -56,7 +73,7 @@ explainInstantiation(FnSymbol* fn) {
           len += sprintf(msg+len, "%s = ", arg->name);
         if (VarSymbol* vs = toVarSymbol(e->value)) {
           if (vs->immediate && vs->immediate->const_kind == NUM_KIND_INT)
-            len += sprintf(msg+len, "%"PRId64, vs->immediate->int_value());
+            len += sprintf(msg+len, "%" PRId64, vs->immediate->int_value());
           else if (vs->immediate && vs->immediate->const_kind == CONST_KIND_STRING)
             len += sprintf(msg+len, "\"%s\"", vs->immediate->v_string);
           else
@@ -71,7 +88,7 @@ explainInstantiation(FnSymbol* fn) {
       }
     }
   }
-  len += sprintf(msg+len, ")");
+  sprintf(msg+len, ")");
   if (callStack.n) {
     USR_PRINT(callStack.v[callStack.n-1], msg);
   } else {
@@ -99,37 +116,65 @@ copyGenericSub(SymbolMap& subs, FnSymbol* root, FnSymbol* fn, Symbol* key, Symbo
 
 
 static void
-instantiate_tuple(FnSymbol* fn) {
+instantiate_tuple_signature(FnSymbol* fn) {
   AggregateType* tuple = toAggregateType(fn->retType);
   //
   // tuple is the return type for the type constructor
   // tuple is NULL for the default constructor
   //
 
+  fn->numPreTupleFormals = fn->formals.length;
+  
   int64_t size = toVarSymbol(fn->substitutions.v[0].value)->immediate->int_value();
-  Expr* last = fn->body->body.last();
-  for (int i = 1; i <= size; i++) {
+  
+  for (int i = 1; i <= size; ++i) {
     const char* name = astr("x", istr(i));
-    ArgSymbol* arg = new ArgSymbol(INTENT_BLANK, name, dtAny, NULL, new SymExpr(gTypeDefaultToken));
-    if (tuple)
-      arg->addFlag(FLAG_TYPE_VARIABLE);
-    fn->insertFormalAtTail(arg);
-    last->insertBefore(new CallExpr(PRIM_SET_MEMBER, fn->_this,
-                                    new_IntSymbol(i), arg));
-    if (tuple)
+    ArgSymbol* formal = new ArgSymbol(INTENT_BLANK, name, dtAny, NULL, new SymExpr(gTypeDefaultToken));
+    
+    if (tuple) {
+      formal->addFlag(FLAG_TYPE_VARIABLE);
       tuple->fields.insertAtTail(new DefExpr(new VarSymbol(name)));
+    }
+    
+    fn->insertFormalAtTail(formal);
   }
+  
   fn->removeFlag(FLAG_TUPLE);
+  
+  fn->addFlag(FLAG_PARTIAL_TUPLE);
   fn->addFlag(FLAG_ALLOW_REF);
 }
 
 
+static void
+instantiate_tuple_body(FnSymbol* fn) {
+  Expr* last = fn->body->body.last();
+  int numPreTupleFormals = fn->numPreTupleFormals;
+  
+  for (int i = numPreTupleFormals + 1; i <= fn->formals.length; ++i) {
+    ArgSymbol* formal = fn->getFormal(i);
+    
+    last->insertBefore(new CallExpr(PRIM_SET_MEMBER, fn->_this, new_IntSymbol(i - numPreTupleFormals), formal));
+  }
+  
+  fn->removeFlag(FLAG_PARTIAL_TUPLE);
+}
+
+
+static void
+getTupleArgAndType(FnSymbol* fn, ArgSymbol*& arg, AggregateType*& ct) {
+  INT_ASSERT(fn->numFormals() == 1); // expected of the original function
+  arg = fn->getFormal(1);
+  ct = toAggregateType(arg->type);
+  INT_ASSERT(!isReferenceType(ct));
+}
+
 static void
 instantiate_tuple_hash( FnSymbol* fn) {
-  if (fn->numFormals() != 1)
-    INT_FATAL(fn, "tuple hash function has more than one argument");
-  ArgSymbol* arg = fn->getFormal(1);
-  AggregateType* ct = toAggregateType(arg->type);
+  ArgSymbol* arg;
+  AggregateType* ct;
+  getTupleArgAndType(fn, arg, ct);
+
   CallExpr* call = NULL;
   bool first = true;
   for (int i=1; i<ct->fields.length; i++) {
@@ -146,86 +191,97 @@ instantiate_tuple_hash( FnSymbol* fn) {
                                          new_IntSymbol(17)));
     }
   }
+  
   // YAH, make sure that we do not return a negative hash value for now
   call = new CallExpr( "&", new_IntSymbol( 0x7fffffffffffffffLL, INT_SIZE_64), call);
   CallExpr* ret = new CallExpr(PRIM_RETURN, new CallExpr("_cast", dtInt[INT_SIZE_64]->symbol, call));
+  
   fn->body->replace( new BlockStmt( ret));
   normalize(fn);
 }
 
-
 static void
-instantiate_tuple_initCopy(FnSymbol* fn) {
-  if (fn->numFormals() != 1)
-    INT_FATAL(fn, "tuple initCopy function has more than one argument");
-  ArgSymbol  *arg = fn->getFormal(1);
-  AggregateType  *ct = toAggregateType(arg->type);
-  CallExpr *call = new CallExpr("_build_tuple_always");
+instantiate_tuple_init(FnSymbol* fn) {
+  ArgSymbol* arg;
+  AggregateType* ct;
+  getTupleArgAndType(fn, arg, ct);
+  if (!arg->hasFlag(FLAG_TYPE_VARIABLE))
+    INT_FATAL(fn, "_defaultOf function not provided a type argument");
+
+  // Similar to build_record_init_function in buildDefaultFunctions, we need
+  // to call the type specified default initializer
+  CallExpr* call = new CallExpr(ct->defaultInitializer);
   BlockStmt* block = new BlockStmt();
-  for (int i=1; i<ct->fields.length; i++) {
-    call->insertAtTail(new CallExpr("chpl__initCopy", new CallExpr(arg, new_IntSymbol(i))));
+  
+  // Need to insert all required arguments into this call
+  for_formals(formal, ct->defaultInitializer) {
+    VarSymbol* tmp = newTemp(formal->name);
+    if (!strcmp(formal->name, "size"))
+      block->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_QUERY_PARAM_FIELD, arg, new_CStringSymbol(formal->name))));
+    else if (!formal->hasFlag(FLAG_IS_MEME)) {
+      if (formal->isParameter()) {
+        tmp->addFlag(FLAG_PARAM);
+      }
+      block->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_INIT, formal->type->symbol)));
+    }
+    block->insertAtHead(new DefExpr(tmp));
+    call->insertAtTail(new NamedExpr(formal->name, new SymExpr(tmp)));
   }
   block->insertAtTail(new CallExpr(PRIM_RETURN, call));
   fn->body->replace(block);
   normalize(fn);
 }
 
-
 static void
-instantiate_tuple_autoCopy(FnSymbol* fn) {
-  if (fn->numFormals() != 1)
-    INT_FATAL(fn, "tuple autoCopy function has more than one argument");
-  ArgSymbol  *arg = fn->getFormal(1);
-  AggregateType  *ct = toAggregateType(arg->type);
-  CallExpr *call = new CallExpr("_build_tuple_always_allow_ref");
+instantiate_tuple_initCopy_or_autoCopy(FnSymbol* fn,
+                                       const char* build_tuple_fun,
+                                       const char* copy_fun)
+{
+  ArgSymbol* arg;
+  AggregateType* ct;
+  getTupleArgAndType(fn, arg, ct);
+
+  CallExpr *call = new CallExpr(build_tuple_fun);
   BlockStmt* block = new BlockStmt();
+  
   for (int i=1; i<ct->fields.length; i++) {
-    Symbol* tmp = newTemp();
-    block->insertAtTail(new DefExpr(tmp));
-//     if (ct->symbol->hasFlag(FLAG_STAR_TUPLE)) {
-//       block->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_SVEC_MEMBER_VALUE, arg, new_IntSymbol(i))));
-//     } else {
-    block->insertAtTail(new CallExpr(PRIM_MOVE, tmp, new CallExpr(PRIM_GET_MEMBER_VALUE, arg, new_StringSymbol(astr("x", istr(i))))));
-      //    }
-    call->insertAtTail(new CallExpr("chpl__autoCopy", tmp));
+    CallExpr* member = new CallExpr(arg, new_IntSymbol(i));
+    DefExpr* def = toDefExpr(ct->fields.get(i+1));
+    INT_ASSERT(def);
+    if (isReferenceType(def->sym->type))
+      // If it is a reference, pass it through.
+      call->insertAtTail(member);
+    else
+      // Otherwise, construct it.
+      call->insertAtTail(new CallExpr(copy_fun, member));
   }
+  
   block->insertAtTail(new CallExpr(PRIM_RETURN, call));
   fn->body->replace(block);
   normalize(fn);
 }
 
+static void
+instantiate_tuple_initCopy(FnSymbol* fn) {
+  instantiate_tuple_initCopy_or_autoCopy(fn,
+                                         "_build_tuple",
+                                         "chpl__initCopy");
+}
+
+static void
+instantiate_tuple_autoCopy(FnSymbol* fn) {
+  instantiate_tuple_initCopy_or_autoCopy(fn,
+                                         "_build_tuple_always_allow_ref",
+                                         "chpl__autoCopy");
+}
+
 
 static TypeSymbol*
 getNewSubType(FnSymbol* fn, Symbol* key, TypeSymbol* value) {
-  if (getSyncFlags(value).any() &&
-      strcmp(fn->name, "_construct__tuple") &&
-      !fn->hasFlag(FLAG_REF)) {
-    if (!getSyncFlags(fn).any() ||
-        (fn->hasFlag(FLAG_METHOD) &&
-         (value->type->instantiatedFrom != fn->_this->type))) {
-      // allow types to be instantiated to sync types
-      if (!key->hasFlag(FLAG_TYPE_VARIABLE)) {
-        // instantiation of a non-type formal of sync type loses sync
-
-        // unless sync is explicitly specified as the generic
-        if (isSyncType(key->type))
-          return value;
-        if (fn->hasFlag(FLAG_NO_SYNC_DEMOTION))
-          // Special case for the chpl_here_alloc and free functions, wherein want to 
-          // allocate and free the _sync object and not the variable it wraps.
-          // Sync variables are handled specially, so they normally appear to have the
-          // type of the value they wrap, and moves involving them cause read?? and
-          // write?? calls to be inserted as needed.
-          return value;
-
-        TypeSymbol* nt = toTypeSymbol(value->type->substitutions.v[0].value);
-        return getNewSubType(fn, key, nt);
-      }
-    }
-  } else if (value->hasFlag(FLAG_REF) &&
-             !fn->hasFlag(FLAG_REF) &&
-             !fn->hasFlag(FLAG_ALLOW_REF) &&
-             !fn->hasFlag(FLAG_TUPLE)) {
+  if (value->hasFlag(FLAG_REF) &&
+      !fn->hasFlag(FLAG_REF) &&
+      !fn->hasFlag(FLAG_ALLOW_REF) &&
+      !fn->hasFlag(FLAG_TUPLE)) {
     // instantiation of a formal of ref type loses ref
     return getNewSubType(fn, key, value->getValType()->symbol);
   }
@@ -238,7 +294,7 @@ evaluateWhereClause(FnSymbol* fn) {
   if (fn->where) {
     whereStack.add(fn);
     resolveFormals(fn);
-    resolveBlock(fn->where);
+    resolveBlockStmt(fn->where);
     whereStack.pop();
     SymExpr* se = toSymExpr(fn->where->body.last());
     if (!se)
@@ -367,11 +423,11 @@ static void renameInstantiatedTypeString(TypeSymbol* sym, VarSymbol* var)
 }
 
 static void
-renameInstantiatedType(TypeSymbol* sym, SymbolMap* subs, FnSymbol* fn) {
+renameInstantiatedType(TypeSymbol* sym, SymbolMap& subs, FnSymbol* fn) {
   if (sym->name[strlen(sym->name)-1] == ')') {
     // avoid "strange" instantiated type names based on partial instantiation
     //  instead of C(int,real)(imag) this results in C(int,real,imag)
-    char* buf = (char*)malloc(strlen(sym->name));
+    char* buf = (char*)malloc(strlen(sym->name) + 1);
     memcpy(buf, sym->name, strlen(sym->name));
     buf[strlen(sym->name)-1] = '\0';
     sym->name = astr(buf, ",");
@@ -382,7 +438,7 @@ renameInstantiatedType(TypeSymbol* sym, SymbolMap* subs, FnSymbol* fn) {
   sym->cname = astr(sym->cname, "_");
   bool first = false;
   for_formals(formal, fn) {
-    if (Symbol* value = subs->get(formal)) {
+    if (Symbol* value = subs.get(formal)) {
       if (TypeSymbol* ts = toTypeSymbol(value)) {
         if (!first && sym->hasFlag(FLAG_TUPLE)) {
           if (sym->hasFlag(FLAG_STAR_TUPLE)) {
@@ -410,13 +466,12 @@ renameInstantiatedType(TypeSymbol* sym, SymbolMap* subs, FnSymbol* fn) {
         VarSymbol* var = toVarSymbol(value);
         if (var && var->immediate) {
           Immediate* immediate = var->immediate;
-          if (var->type == dtString)
+          if (var->type == dtString || var->type == dtStringC)
             renameInstantiatedTypeString(sym, var);
-          else if (immediate->const_kind == NUM_KIND_UINT &&
-                   immediate->num_index == INT_SIZE_1) {
+          else if (immediate->const_kind == NUM_KIND_BOOL) {
             // Handle boolean types specially.
-            const char* name4bool = immediate->uint_value() ? "true" : "false";
-            const char* cname4bool = immediate->uint_value() ? "T" : "F";
+            const char* name4bool = immediate->bool_value() ? "true" : "false";
+            const char* cname4bool = immediate->bool_value() ? "T" : "F";
             sym->name = astr(sym->name, name4bool);
             sym->cname = astr(sym->cname, cname4bool);
           } else {
@@ -437,10 +492,135 @@ renameInstantiatedType(TypeSymbol* sym, SymbolMap* subs, FnSymbol* fn) {
   sym->name = astr(sym->name, ")");
 }
 
+/** Instantiate a type
+ *
+ * \param fn   Type constructor we are working on
+ * \param subs Type substitutions to be made during instantiation
+ * \param call The call that is being resolved (used for scope)
+ * \param type The generic type we wish to instantiate
+ */
+static Type*
+instantiateTypeForTypeConstructor(FnSymbol* fn, SymbolMap& subs, CallExpr* call, Type* type) {
+  INT_ASSERT(isAggregateType(type));
+  AggregateType* ct = toAggregateType(type);
 
+  Type* newType = NULL;
+  newType = ct->symbol->copy()->type;
+
+  Type *oldParentTy = NULL;
+  Type* newParentTy = NULL;
+  AggregateType* newCt = toAggregateType(newType);
+
+  // Get the right super type if we are using a super constructor.
+  // This only matters for generic parent types.
+  if (ct->dispatchParents.n > 0) {
+    if(AggregateType *parentTy = toAggregateType(ct->dispatchParents.v[0])){
+      if (parentTy->symbol->hasFlag(FLAG_GENERIC)) {
+        // Set the type of super to be the instantiated
+        // parent with substitutions.
+
+        CallExpr* parentTyCall = new CallExpr(astr("_type_construct_", parentTy->symbol->name));
+        // Pass the special formals to the superclass type constructor.
+        for_formals(arg, fn) {
+          if (arg->hasFlag(FLAG_PARENT_FIELD)) {
+            Symbol* value = subs.get(arg);
+            if (!value) {
+              value = arg;
+              // Or error?
+            }
+            parentTyCall->insertAtTail(value);
+          }
+        }
+        call->insertBefore(parentTyCall);
+        resolveCallAndCallee(parentTyCall);
+
+        oldParentTy = parentTy;
+        newParentTy = parentTyCall->isResolved()->retType;
+        parentTyCall->remove();
+
+        // Now adjust the super field's type.
+
+        DefExpr* superDef = NULL;
+
+        // Find the super field
+        for_alist(tmp, newCt->fields) {
+          DefExpr* def = toDefExpr(tmp);
+          INT_ASSERT(def);
+          if (VarSymbol* field = toVarSymbol(def->sym)) {
+            if (field->hasFlag(FLAG_SUPER_CLASS)) {
+              superDef = def;
+            }
+          }
+        }
+
+        if (superDef) {
+          superDef->sym->type = newParentTy;
+          INT_ASSERT(newCt->getField("super")->typeInfo() == newParentTy);
+        }
+
+      }
+    }
+  }
+
+  //
+  // mark star tuples, add star flag
+  //
+  if (!fn->hasFlag(FLAG_TUPLE) && newType->symbol->hasFlag(FLAG_TUPLE)) {
+    bool markStar = true;
+    Type* starType = NULL;
+    form_Map(SymbolMapElem, e, subs) {
+      TypeSymbol* ts = toTypeSymbol(e->value);
+      INT_ASSERT(ts && ts->type);
+      if (starType == NULL) {
+        starType = ts->type;
+      } else if (starType != ts->type) {
+        markStar = false;
+        break;
+      }
+    }
+    if (markStar)
+      newType->symbol->addFlag(FLAG_STAR_TUPLE);
+  }
+
+  renameInstantiatedType(newType->symbol, subs, fn);
+  fn->retType->symbol->defPoint->insertBefore(new DefExpr(newType->symbol));
+  newType->symbol->copyFlags(fn);
+  if (isSyncType(newType))
+    newType->defaultValue = NULL;
+  newType->substitutions.copy(fn->retType->substitutions);
+  // Add dispatch parents, but replace parent type with
+  // instantiated parent type.
+  forv_Vec(Type, t, fn->retType->dispatchParents) {
+    Type *useT = t;
+    if (t == oldParentTy) useT = newParentTy;
+    newType->dispatchParents.add(useT);
+  }
+
+  forv_Vec(Type, t, fn->retType->dispatchParents) {
+    Type *useT = t;
+    if (t == oldParentTy) useT = newParentTy;
+    bool inserted = useT->dispatchChildren.add_exclusive(newType);
+    INT_ASSERT(inserted);
+  }
+  if (newType->dispatchChildren.n)
+    INT_FATAL(fn, "generic type has subtypes");
+  newType->instantiatedFrom = fn->retType;
+  newType->substitutions.map_union(subs);
+  newType->symbol->removeFlag(FLAG_GENERIC);
+
+  return newType;
+}
+
+/** Instantiate enough of the function for it to make it through the candidate
+ *  filtering and disambiguation process.
+ *
+ * \param fn   Generic function to instantiate
+ * \param subs Type substitutions to be made during instantiation
+ * \param call Call that is being resolved
+ */
 FnSymbol*
-instantiate(FnSymbol* fn, SymbolMap* subs, CallExpr* call) {
-  form_Map(SymbolMapElem, e, *subs) {
+instantiateSignature(FnSymbol* fn, SymbolMap& subs, CallExpr* call) {
+  form_Map(SymbolMapElem, e, subs) {
     if (TypeSymbol* ts = toTypeSymbol(e->value)) {
       if (ts->type->symbol->hasFlag(FLAG_GENERIC))
         INT_FATAL(fn, "illegal instantiation with a generic type");
@@ -465,11 +645,15 @@ instantiate(FnSymbol* fn, SymbolMap* subs, CallExpr* call) {
   // substitutions to refer to the root function's formal arguments
   //
   SymbolMap all_subs;
-  if (fn->instantiatedFrom)
-    form_Map(SymbolMapElem, e, fn->substitutions)
+  if (fn->instantiatedFrom) {
+    form_Map(SymbolMapElem, e, fn->substitutions) {
       all_subs.put(e->key, e->value);
-  form_Map(SymbolMapElem, e, *subs)
+    }
+  }
+  
+  form_Map(SymbolMapElem, e, subs) {
     copyGenericSub(all_subs, root, fn, e->key, e->value);
+  }
 
   //
   // use cached instantiation if possible
@@ -489,83 +673,47 @@ instantiate(FnSymbol* fn, SymbolMap* subs, CallExpr* call) {
   //
   Type* newType = NULL;
   if (fn->hasFlag(FLAG_TYPE_CONSTRUCTOR)) {
-    INT_ASSERT(isAggregateType(fn->retType));
-    newType = fn->retType->symbol->copy()->type;
-
-    //
-    // mark star tuples, add star flag
-    //
-    if (!fn->hasFlag(FLAG_TUPLE) && newType->symbol->hasFlag(FLAG_TUPLE)) {
-      bool markStar = true;
-      Type* starType = NULL;
-      form_Map(SymbolMapElem, e, *subs) {
-        TypeSymbol* ts = toTypeSymbol(e->value);
-        INT_ASSERT(ts && ts->type);
-        if (starType == NULL) {
-          starType = ts->type;
-        } else if (starType != ts->type) {
-          markStar = false;
-          break;
-        }
-      }
-      if (markStar)
-        newType->symbol->addFlag(FLAG_STAR_TUPLE);
-    }
-
-    renameInstantiatedType(newType->symbol, subs, fn);
-    fn->retType->symbol->defPoint->insertBefore(new DefExpr(newType->symbol));
-    newType->symbol->copyFlags(fn);
-    if (isSyncType(newType))
-      newType->defaultValue = NULL;
-    newType->substitutions.copy(fn->retType->substitutions);
-    newType->dispatchParents.copy(fn->retType->dispatchParents);
-    forv_Vec(Type, t, fn->retType->dispatchParents) {
-      bool inserted = t->dispatchChildren.add_exclusive(newType);
-      INT_ASSERT(inserted);
-    }
-    if (newType->dispatchChildren.n)
-      INT_FATAL(fn, "generic type has subtypes");
-    newType->instantiatedFrom = fn->retType;
-    newType->substitutions.map_union(*subs);
-    newType->symbol->removeFlag(FLAG_GENERIC);
+    newType = instantiateTypeForTypeConstructor(fn, subs, call, fn->retType);
   }
 
   //
   // instantiate function
   //
+  
   SymbolMap map;
-
-  if (newType)
+  
+  if (newType) {
     map.put(fn->retType->symbol, newType->symbol);
-
-  FnSymbol* newFn = fn->copy(&map);
-
+  }
+  
+  FnSymbol* newFn = fn->partialCopy(&map);
+  
   addCache(genericsCache, root, newFn, &all_subs);
 
-  //printf("newFn: %d %s\n", newFn->id, newFn->cname);
-
   newFn->removeFlag(FLAG_GENERIC);
   newFn->addFlag(FLAG_INVISIBLE_FN);
   newFn->instantiatedFrom = fn;
+  newFn->substitutions.map_union(all_subs);
 
-  if (call)
+  if (call) {
     newFn->instantiationPoint = getVisibilityBlock(call);
-
+  }
 
   Expr* putBefore = fn->defPoint;
   if( !putBefore->list ) {
     putBefore = call->parentSymbol->defPoint;
   }
+  
   putBefore->insertBefore(new DefExpr(newFn));
 
   //
   // add parameter instantiations to parameter map
   //
-  for (int i = 0; i < subs->n; i++) {
-    if (ArgSymbol* arg = toArgSymbol(subs->v[i].key)) {
+  for (int i = 0; i < subs.n; i++) {
+    if (ArgSymbol* arg = toArgSymbol(subs.v[i].key)) {
       if (arg->intent == INTENT_PARAM) {
         Symbol* key = map.get(arg);
-        Symbol* val = subs->v[i].value;
+        Symbol* val = subs.v[i].value;
         if (!key || !val || isTypeSymbol(val))
           INT_FATAL("error building parameter map in instantiation");
         paramMap.put(key, val);
@@ -594,11 +742,11 @@ instantiate(FnSymbol* fn, SymbolMap* subs, CallExpr* call) {
   //
   for_formals(formal, fn) {
     ArgSymbol* newFormal = toArgSymbol(map.get(formal));
-    if (Symbol* value = subs->get(formal)) {
+    if (Symbol* value = subs.get(formal)) {
       INT_ASSERT(formal->intent == INTENT_PARAM || isTypeSymbol(value));
       if (formal->intent == INTENT_PARAM) {
         newFormal->intent = INTENT_BLANK;
-        newFormal->instantiatedParam = true;
+        newFormal->addFlag(FLAG_INSTANTIATED_PARAM);
         if (newFormal->type->symbol->hasFlag(FLAG_GENERIC))
           newFormal->type = paramMap.get(newFormal)->type;
       } else {
@@ -621,26 +769,30 @@ instantiate(FnSymbol* fn, SymbolMap* subs, CallExpr* call) {
     newType->defaultTypeConstructor = newFn;
     newFn->retType = newType;
   }
+  
+  if (fn->hasFlag(FLAG_TUPLE)) {
+    instantiate_tuple_signature(newFn);
+  }
 
-  if (!strcmp(fn->name, "chpl__defaultHash") &&
+  if (!strcmp(fn->name, "_defaultOf") &&
       fn->getFormal(1)->type->symbol->hasFlag(FLAG_TUPLE))
+    instantiate_tuple_init(newFn);
+
+  if (!strcmp(fn->name, "chpl__defaultHash") && fn->getFormal(1)->type->symbol->hasFlag(FLAG_TUPLE)) {
     instantiate_tuple_hash(newFn);
+  }
 
-  if (fn->hasFlag(FLAG_INIT_COPY_FN) &&
-      fn->getFormal(1)->type->symbol->hasFlag(FLAG_TUPLE))
+  if (fn->hasFlag(FLAG_INIT_COPY_FN) && fn->getFormal(1)->type->symbol->hasFlag(FLAG_TUPLE)) {
     instantiate_tuple_initCopy(newFn);
+  }
 
-  if (fn->hasFlag(FLAG_AUTO_COPY_FN) &&
-      fn->getFormal(1)->type->symbol->hasFlag(FLAG_TUPLE))
+  if (fn->hasFlag(FLAG_AUTO_COPY_FN) && fn->getFormal(1)->type->symbol->hasFlag(FLAG_TUPLE)) {
     instantiate_tuple_autoCopy(newFn);
-
-  newFn->substitutions.append(all_subs);
-
-  if (fn->hasFlag(FLAG_TUPLE))
-    instantiate_tuple(newFn);
-
-  if (newFn->numFormals() > 1 && newFn->getFormal(1)->type == dtMethodToken)
+  }
+  
+  if (newFn->numFormals() > 1 && newFn->getFormal(1)->type == dtMethodToken) {
     newFn->getFormal(2)->type->methods.add(newFn);
+  }
 
   newFn->tag_generic();
 
@@ -658,8 +810,46 @@ instantiate(FnSymbol* fn, SymbolMap* subs, CallExpr* call) {
   if (!newFn->hasFlag(FLAG_GENERIC) && explainInstantiationLine) {
     explainInstantiation(newFn);
   }
-
+  
   checkInstantiationLimit(fn);
+  
+  return newFn;
+}
 
+/** Finish copying and instantiating the partially instantiated function.
+ * 
+ * TODO: See if more code from instantiateSignature can be moved into this
+ *       function.
+ * 
+ * \param fn   Generic function to finish instantiating
+ */
+void
+instantiateBody(FnSymbol* fn) {
+  if (fn->hasFlag(FLAG_PARTIAL_COPY)) {
+    fn->finalizeCopy();
+
+    if (fn->hasFlag(FLAG_PARTIAL_TUPLE)) {
+      instantiate_tuple_body(fn);
+    }
+  }
+}
+
+/** Fully instantiate a generic function given a map of substitutions and a
+ *  call site.
+ * 
+ * \param fn   Generic function to instantiate
+ * \param subs Type substitutions to be made during instantiation
+ * \param call Call that is being resolved
+ */
+FnSymbol*
+instantiate(FnSymbol* fn, SymbolMap& subs, CallExpr* call) {
+  FnSymbol* newFn;
+  
+  newFn = instantiateSignature(fn, subs, call);
+  
+  if (newFn != NULL) {
+    instantiateBody(newFn);
+  }
+  
   return newFn;
 }
