@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 // Get realpath on linux
 #ifndef _XOPEN_SOURCE
@@ -11,49 +25,53 @@
 #define _XOPEN_SOURCE_EXTENDED 1
 #endif
 
-#include "beautify.h"
 #include "files.h"
+
+#include "beautify.h"
+#include "driver.h"
 #include "misc.h"
 #include "mysystem.h"
 #include "stringutil.h"
 #include "tmpdirname.h"
-#include <cstring>
-#include <cstdlib>
+
 #include <pwd.h>
-#include <cerrno>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
 
+#include <cstring>
+#include <cstdlib>
+#include <cerrno>
 #include <string>
+#include <map>
 
-char executableFilename[FILENAME_MAX+1] = "a.out";
-char saveCDir[FILENAME_MAX+1] = "";
-char ccflags[256] = "";
-char ldflags[256] = "";
-bool ccwarnings = false;
+#include <sys/types.h>
+#include <sys/stat.h>
 
-extern bool fFastFlag;
+char               executableFilename[FILENAME_MAX + 1] = "a.out";
+char               saveCDir[FILENAME_MAX + 1]           = "";
+
+std::string ccflags;
+std::string ldflags;
+
+int                numLibFlags                          = 0;
+const char**       libFlag                              = NULL;
+
+Vec<const char*>   incDirs;
 
 // directory for intermediates; tmpdir or saveCDir
-static const char* intDirName = NULL;
-
-static const int MAX_CHARS_PER_PID = 32;
-
-int numLibFlags = 0;
-const char** libFlag = NULL;
-
-Vec<const char*> incDirs;
+static const char* intDirName        = NULL;
 
+static const int   MAX_CHARS_PER_PID = 32;
 
 void addLibInfo(const char* libName) {
   static int libSpace = 0;
 
   numLibFlags++;
+
   if (numLibFlags > libSpace) {
     libSpace = 2*numLibFlags;
     libFlag = (const char**)realloc(libFlag, libSpace*sizeof(char*));
   }
+
   libFlag[numLibFlags-1] = astr(libName);
 }
 
@@ -61,7 +79,6 @@ void addIncInfo(const char* incDir) {
   incDirs.add(astr(incDir));
 }
 
-
 void ensureDirExists(const char* dirname, const char* explanation) {
   const char* mkdircommand = "mkdir -p ";
   const char* command = astr(mkdircommand, dirname);
@@ -70,50 +87,75 @@ void ensureDirExists(const char* dirname, const char* explanation) {
 }
 
 
-static const char* removeSpacesFromString(char* str) {
-  if (strchr(str, ' ')) {
-    char* str2 = (char*)malloc(strlen(str)+1);
-    char* dst = str2;
-    char* src = str;
-    while (*src != '\0') {
-      *dst = *src;
-      src++;
-      if (*dst != ' ')
+static void removeSpacesBackslashesFromString(char* str)
+{
+  char* src = str;
+  char* dst = str;
+  while (*src != '\0')
+  {
+    *dst = *src++;
+    if (*dst != ' ' && *dst != '\\')
         dst++;
-    }
-    *dst = '\0';
-    return str2;
-  } else {
-    return str;
   }
+  *dst = '\0';
 }
 
 
-static void ensureTmpDirExists(void) {
+/*
+ * Find the default tmp directory. Try getting the tmp dir from the ISO/IEC
+ * 9945 env var options first, then P_tmpdir, then "/tmp".
+ */
+static const char* getTempDir() {
+  const char* possibleDirsInEnv[] = {"TMPDIR", "TMP", "TEMP", "TEMPDIR"};
+  for (unsigned int i = 0; i < (sizeof(possibleDirsInEnv) / sizeof(char*)); i++) {
+    const char* curDir = getenv(possibleDirsInEnv[i]);
+    if (curDir != NULL) {
+      return curDir;
+    }
+  }
+#ifdef P_tmpdir
+  return P_tmpdir;
+#else
+  return "/tmp";
+#endif
+}
+
+
+const char* makeTempDir(const char* dirPrefix) {
+  const char* tmpdirprefix = astr(getTempDir(), "/", dirPrefix);
+  const char* tmpdirsuffix = ".deleteme";
+
+  pid_t mypid = getpid();
+#ifdef DEBUGTMPDIR
+  mypid = 0;
+#endif
+
+  char mypidstr[MAX_CHARS_PER_PID];
+  snprintf(mypidstr, MAX_CHARS_PER_PID, "-%d", (int)mypid);
+
+  struct passwd* passwdinfo = getpwuid(geteuid());
+  const char* userid;
+  if (passwdinfo == NULL) {
+    userid = "anon";
+  } else {
+    userid = passwdinfo->pw_name;
+  }
+  char* myuserid = strdup(userid);
+  removeSpacesBackslashesFromString(myuserid);
+
+  const char* tmpDir = astr(tmpdirprefix, myuserid, mypidstr, tmpdirsuffix);
+  ensureDirExists(tmpDir, "making temporary directory");
+
+  free(myuserid); myuserid = NULL;
+
+  return tmpDir;
+}
+
+static void ensureTmpDirExists() {
   if (saveCDir[0] == '\0') {
     if (tmpdirname == NULL) {
-      const char* tmpdirprefix = "/tmp/chpl-";
-      const char* tmpdirsuffix = ".deleteme";
-      
-      pid_t mypid = getpid();
-#ifdef DEBUGTMPDIR
-      mypid = 0;
-#endif
-      
-      char mypidstr[MAX_CHARS_PER_PID];
-      snprintf(mypidstr, MAX_CHARS_PER_PID, "-%d", (int)mypid);
-      
-      struct passwd* passwdinfo = getpwuid(geteuid());
-      const char* userid;
-      if (passwdinfo == NULL) {
-        userid = "anon";
-      } else {
-        userid = removeSpacesFromString(passwdinfo->pw_name);
-      }
-      
-      tmpdirname = astr(tmpdirprefix, userid, mypidstr, tmpdirsuffix);
+      tmpdirname = makeTempDir("chpl-");
       intDirName = tmpdirname;
-      ensureDirExists(intDirName, "making temporary directory");
     }
   } else {
     if (intDirName != saveCDir) {
@@ -124,7 +166,13 @@ static void ensureTmpDirExists(void) {
 }
 
 
-void deleteTmpDir(void) {
+void deleteDir(const char* dirname) {
+  const char* cmd = astr("rm -rf ", dirname);
+  mysystem(cmd, astr("removing directory: ", dirname));
+}
+
+
+void deleteTmpDir() {
   static int inDeleteTmpDir = 0; // break infinite recursion
 
   if (inDeleteTmpDir) {
@@ -139,10 +187,7 @@ void deleteTmpDir(void) {
         strcmp(tmpdirname, "//") == 0) {
       INT_FATAL("tmp directory name looks fishy");
     }
-    const char* rmdircommand = "rm -r ";
-    const char* command = astr(rmdircommand, tmpdirname);
-
-    mysystem(command, "removing temporary directory");
+    deleteDir(tmpdirname);
     tmpdirname = NULL;
   }
 #endif
@@ -154,20 +199,9 @@ void deleteTmpDir(void) {
 const char* genIntermediateFilename(const char* filename) {
   const char* slash = "/";
 
-  ensureTmpDirExists();    
+  ensureTmpDirExists();
 
-  const char* newfilename = astr(intDirName, slash, filename);
-
-  return newfilename;
-}
-
-// MPF - genIntermediateFilename is a better name, declared in files.h,
-// but didn't want to modify all the code here yet so we have
-// this 2nd name for the same routine. 
-static
-const char* genIntFilename(const char* filename)
-{
-  return genIntermediateFilename(filename);
+  return astr(intDirName, slash, filename);
 }
 
 static const char* stripdirectories(const char* filename) {
@@ -185,19 +219,22 @@ static const char* stripdirectories(const char* filename) {
 
 const char* objectFileForCFile(const char* inputFilename) {
   const char* pathlessFilename = stripdirectories(inputFilename);
-  const char* objFilename = genIntFilename(astr(pathlessFilename, ".o"));
+  const char* objFilename = genIntermediateFilename(astr(pathlessFilename, ".o"));
   return objFilename;
 }
 
-static FILE* openfile(const char* filename, const char* mode = "w", 
-                      bool fatal = true) {
+static FILE* openfile(const char* filename,
+                      const char* mode  = "w",
+                      bool        fatal = true) {
   FILE* newfile;
 
   newfile = fopen(filename, mode);
+
   if (newfile == NULL) {
     const char* errorstr = "opening ";
-    const char* errormsg = astr(errorstr, filename, ": ", 
-                                     strerror(errno));
+    const char* errormsg = astr(errorstr,
+                                filename, ": ",
+                                strerror(errno));
 
     if (fatal) {
       USR_FATAL(errormsg);
@@ -234,31 +271,30 @@ void openCFile(fileinfo* fi, const char* name, const char* ext) {
   else
     fi->filename = astr(name);
 
-  fi->pathname = genIntFilename(fi->filename);
-  fi->fptr = fopen(fi->pathname, "w");
+  fi->pathname = genIntermediateFilename(fi->filename);
+  openfile(fi, "w");
 }
 
-void appendCFile(fileinfo* fi, const char* name, const char* ext) {
-  if (ext)
-    fi->filename = astr(name, ".", ext);
-  else
-    fi->filename = astr(name);
-  
-  fi->pathname = genIntFilename(fi->filename);
-  fi->fptr = fopen(fi->pathname, "a+");
-}
 void closeCFile(fileinfo* fi, bool beautifyIt) {
   fclose(fi->fptr);
-  if (beautifyIt && saveCDir[0])
+  //
+  // We should beautify if (1) we were asked to and (2) either (a) we
+  // were asked to save the C code or (b) we're debugging and were
+  // asked to codegen cpp #line information.
+  //
+  // TODO: With some refactoring, we could simply do the #line part of
+  // beautify without also improving indentation and such which could
+  // save some time.
+  //
+  if (beautifyIt && (saveCDir[0] || (debugCCode && printCppLineno)))
     beautify(fi);
 }
 
-
 fileinfo* openTmpFile(const char* tmpfilename, const char* mode) {
   fileinfo* newfile = (fileinfo*)malloc(sizeof(fileinfo));
 
   newfile->filename = astr(tmpfilename);
-  newfile->pathname = genIntFilename(tmpfilename);
+  newfile->pathname = genIntermediateFilename(tmpfilename);
   openfile(newfile, mode);
 
   return newfile;
@@ -275,11 +311,12 @@ void closeInputFile(FILE* infile) {
 }
 
 
-static const char** inputFilenames = {NULL};
+static const char** inputFilenames = NULL;
 
 
 static bool checkSuffix(const char* filename, const char* suffix) {
   const char* dot = strrchr(filename, '.');
+
   return (dot && strcmp(dot+1, suffix) == 0);
 }
 
@@ -304,7 +341,7 @@ bool isChplSource(const char* filename) {
   return retval;
 }
 
-static bool isRecognizedSource(char* filename) {
+static bool isRecognizedSource(const char* filename) {
   return (isCSource(filename) ||
           isCHeader(filename) ||
           isObjFile(filename) ||
@@ -312,83 +349,128 @@ static bool isRecognizedSource(char* filename) {
 }
 
 
-void testInputFiles(int numFilenames, char* filename[]) {
-  inputFilenames = (const char**)malloc((numFilenames+1)*sizeof(char*));
-  int i;
+void addSourceFiles(int numNewFilenames, const char* filename[]) {
+  static int numInputFiles = 0;
+  int cursor = numInputFiles;
   char achar;
-  for (i=0; i<numFilenames; i++) {
+  numInputFiles += numNewFilenames;
+  inputFilenames = (const char**)realloc(inputFilenames, 
+                                         (numInputFiles+1)*sizeof(char*));
+
+  for (int i = 0; i < numNewFilenames; i++, cursor++) {
     if (!isRecognizedSource(filename[i])) {
-      USR_FATAL(astr("file '", filename[i], 
-                          "' does not have a recognized suffix"));
+      USR_FATAL(astr("file '",
+                     filename[i],
+                     "' does not have a recognized suffix"));
     }
     // WE SHOULDN"T TRY TO OPEN .h files, just .c and .chpl and .o
     if (!isCHeader(filename[i])) {
       FILE* testfile = openInputFile(filename[i]);
       if (fscanf(testfile, "%c", &achar) != 1) {
-        USR_FATAL(astr("source file '", filename[i], 
+        USR_FATAL(astr("source file '",
+                       filename[i],
                        "' is either empty or a directory"));
       }
-      
+
       closeInputFile(testfile);
     }
-    inputFilenames[i] = astr(filename[i]);
-  }
-  inputFilenames[i] = NULL;
 
-  if (!foundChplSource)
+    inputFilenames[cursor] = astr(filename[i]);
+  }
+
+  inputFilenames[cursor] = NULL;
+
+  if (!foundChplSource && fUseIPE == false)
     USR_FATAL("Command line contains no .chpl source files");
 }
 
+void addSourceFile(const char* filename) {
+  const char* filenamearr[1] = {filename};
+  addSourceFiles(1, filenamearr);
+}
+
 
 const char* nthFilename(int i) {
   return inputFilenames[i];
 }
 
 
-const char* createGDBFile(int argc, char* argv[]) {
-  const char* gdbfilename = genIntFilename("gdb.commands");
-  FILE* gdbfile = openfile(gdbfilename);
+const char* createDebuggerFile(const char* debugger, int argc, char* argv[]) {
+  const char* dbgfilename = genIntermediateFilename(astr(debugger, ".commands"));
+  FILE* dbgfile = openfile(dbgfilename);
   int i;
 
-  fprintf(gdbfile, "set args");
+  if (strcmp(debugger, "gdb") == 0) {
+    fprintf(dbgfile, "set args");
+  } else if (strcmp(debugger, "lldb") == 0) {
+    fprintf(dbgfile, "settings set -- target.run-args");
+  } else {
+      INT_FATAL(astr("createDebuggerFile doesn't know how to handle the given "
+                     "debugger: '", debugger, "'"));
+  }
   for (i=1; i<argc; i++) {
-    if (strcmp(argv[i], "--gdb") != 0) {
-      fprintf(gdbfile, " %s", argv[i]);
+    if (strcmp(argv[i], astr("--", debugger)) != 0) {
+      fprintf(dbgfile, " %s", argv[i]);
     }
   }
-  fprintf(gdbfile, "\n");
-  closefile(gdbfile);
-  mysystem(astr("cat ", CHPL_HOME, "/compiler/etc/gdb.commands >> ", 
-                gdbfilename), 
-           "appending gdb commands", 0);
 
+  fprintf(dbgfile, "\n");
+  closefile(dbgfile);
+  mysystem(astr("cat ", CHPL_HOME, "/compiler/etc/", debugger, ".commands >> ",
+                dbgfilename),
+           astr("appending ", debugger, " commands"),
+           false);
 
-  return gdbfilename;
+  return dbgfilename;
 }
 
+std::string runPrintChplEnv(std::map<std::string, const char*> varMap) {
+  // Run printchplenv script, passing currently known CHPL_vars as well
+  std::string command = "";
 
-static const char* mysystem_getresult(const char* command, 
-                                      const char* description,
-                                      int ignorestatus) {
-  const char* systemFilename = "system.out.tmp";
-  const char* fullSystemFilename = genIntFilename(systemFilename);
-  char* result = (char*)malloc(256*sizeof(char));
-  mysystem(astr(command, " > ", fullSystemFilename), description, ignorestatus);
-  fileinfo* systemFile = openTmpFile(systemFilename, "r");
-  if (fscanf(systemFile->fptr, "%s", result) != 1){
-    strcpy(result,"");
+  // Pass known variables in varMap into printchplenv by appending to command
+  for (std::map<std::string, const char*>::iterator ii=varMap.begin(); ii!=varMap.end(); ++ii)
+  {
+    command += ii->first + "=" + std::string(ii->second) + " ";
   }
-  closefile(systemFile);
-  return astr(result);  // canonicalize
+
+  // Toss stderr away until printchplenv supports a '--suppresswarnings' flag
+  command += std::string(CHPL_HOME) + "/util/printchplenv --simple 2> /dev/null";
+
+  return runCommand(command);
 }
 
+std::string runCommand(std::string& command) {
+  // Run arbitrary command and return result
+  char buffer[256];
+  std::string result = "";
+  std::string error = "";
 
-const char* runUtilScript(const char* script) {
-  return mysystem_getresult(astr(CHPL_HOME, "/util/", script), 
-                            astr("running $CHPL_HOME/util/", script), 0);
+  // Call command
+  FILE* pipe = popen(command.c_str(), "r");
+  if (!pipe) {
+    error = "running " + command;
+    USR_FATAL(error.c_str());
+  }
+
+  // Read output of command into result via buffer
+  while (!feof(pipe)) {
+    if (fgets(buffer, 256, pipe) != NULL) {
+      result += buffer;
+    }
+  }
+
+  if (pclose(pipe)) {
+    error = command + " did not run successfully";
+    USR_FATAL(error.c_str());
+  }
+
+  return result;
 }
 
 const char* getIntermediateDirName() {
+  ensureTmpDirExists();
+
   return intDirName;
 }
 
@@ -438,7 +520,7 @@ static void genObjFiles(FILE* makefile) {
         fprintf(makefile, "\t%s \\\n", inputFilename);
       } else {
         const char* pathlessFilename = stripdirectories(inputFilename);
-        const char* objFilename = genIntFilename(astr(pathlessFilename, ".o"));
+        const char* objFilename = genIntermediateFilename(astr(pathlessFilename, ".o"));
         fprintf(makefile, "\t%s \\\n", objFilename);
       }
     }
@@ -465,9 +547,22 @@ void codegen_makefile(fileinfo* mainfile, const char** tmpbinname, bool skip_com
   const char* strippedExeFilename = stripdirectories(executableFilename);
   const char* exeExt = "";
   const char* tmpbin = "";
+  std::string chplmakeallvars = "\0";
+
 
   fprintf(makefile.fptr, "CHPL_MAKE_HOME = %s\n\n", CHPL_HOME);
-  fprintf(makefile.fptr, "TMPDIRNAME = %s\n", tmpDirName);
+  fprintf(makefile.fptr, "TMPDIRNAME = %s\n\n", tmpDirName);
+
+  // Generate one variable containing all envMap information to pass to printchplenv
+  for (std::map<std::string, const char*>::iterator env=envMap.begin(); env!=envMap.end(); ++env)
+  {
+    if(!useDefaultEnv(env->first)) {
+      chplmakeallvars += env->first + "=" + std::string(env->second) + " ";
+    }
+  }
+
+  fprintf(makefile.fptr, "\nCHPL_MAKE_ALL_VARS = %s\n", chplmakeallvars.c_str());
+
 
   // LLVM builds just use the makefile for the launcher and
   // so want to skip the actual program generation.
@@ -493,27 +588,20 @@ void codegen_makefile(fileinfo* mainfile, const char** tmpbinname, bool skip_com
   // factor of 5 or so in time in running the test system, as opposed
   // to specifying BINNAME on the C compiler command line.
 
-  fprintf(makefile.fptr, "COMP_GEN_CFLAGS =");
-  if (ccwarnings) {
-    fprintf(makefile.fptr, " $(WARN_GEN_CFLAGS)");
-  }
-  if (debugCCode) {
-    fprintf(makefile.fptr, " $(DEBUG_CFLAGS)");
-  }
-  if (optimizeCCode) {
-    fprintf(makefile.fptr, " $(OPT_CFLAGS)");
-  }
-  if (fieeefloat) {
-    fprintf(makefile.fptr, " $(IEEE_FLOAT_GEN_CFLAGS)");
-  } else {
-    fprintf(makefile.fptr, " $(NO_IEEE_FLOAT_GEN_CFLAGS)");
-  }
+  fprintf(makefile.fptr, "COMP_GEN_WARN = %i\n", ccwarnings);
+  fprintf(makefile.fptr, "COMP_GEN_DEBUG = %i\n", debugCCode);
+  fprintf(makefile.fptr, "COMP_GEN_OPT = %i\n", optimizeCCode);
+  fprintf(makefile.fptr, "COMP_GEN_SPECIALIZE = %i\n", specializeCCode);
+  fprintf(makefile.fptr, "COMP_GEN_FLOAT_OPT = %i\n", ffloatOpt);
+  
+  fprintf(makefile.fptr, "COMP_GEN_USER_CFLAGS =");
+
   if (fLibraryCompile && (fLinkStyle==LS_DYNAMIC))
     fprintf(makefile.fptr, " $(SHARED_LIB_CFLAGS)");
   forv_Vec(const char*, dirName, incDirs) {
     fprintf(makefile.fptr, " -I%s", dirName);
   }
-  fprintf(makefile.fptr, " %s\n", ccflags);
+  fprintf(makefile.fptr, " %s\n", ccflags.c_str());
 
   fprintf(makefile.fptr, "COMP_GEN_LFLAGS =");
   if (!fLibraryCompile) {
@@ -527,7 +615,7 @@ void codegen_makefile(fileinfo* mainfile, const char** tmpbinname, bool skip_com
     else
       fprintf(makefile.fptr, " $(LIB_STATIC_FLAG)" );
   }
-  fprintf(makefile.fptr, " %s\n", ldflags);
+  fprintf(makefile.fptr, " %s\n", ldflags.c_str());
 
   fprintf(makefile.fptr, "TAGS_COMMAND = ");
   if (developer && saveCDir[0] && !printCppLineno) {
@@ -551,14 +639,6 @@ void codegen_makefile(fileinfo* mainfile, const char** tmpbinname, bool skip_com
     fprintf(makefile.fptr, " %s", libFlag[i]);
   fprintf(makefile.fptr, "\n");
 
-  // MPF - we want to allow the runtime to make use of debug/optimize
-  // information
-  if (debugCCode) {
-    fprintf(makefile.fptr, "DEBUG = 1\n");
-  }
-  if (optimizeCCode) {
-    fprintf(makefile.fptr, "OPTIMIZE = 1\n");
-  }
   fprintf(makefile.fptr, "\n");
   fprintf(makefile.fptr, "\n");
 
@@ -591,8 +671,9 @@ static const char* searchPath(Vec<const char*> path, const char* filename,
       if (foundfile == NULL) {
         foundfile = fullfilename;
       } else if (!noWarn) {
-        USR_WARN("Ambiguous module source file -- using %s over %s", 
-                 cleanFilename(foundfile), cleanFilename(fullfilename));
+        USR_WARN("Ambiguous module source file -- using %s over %s",
+                 cleanFilename(foundfile),
+                 cleanFilename(fullfilename));
       }
     }
   }
@@ -641,37 +722,52 @@ void addFlagModulePath(const char* newpath) {
 // search path, we'll add all unique directories specified via -M and
 // CHPL_MODULE_PATH.
 //
-void addDashMsToUserPath(void) {
+void addDashMsToUserPath() {
   forv_Vec(const char*, dirname, flagModPath) {
     addUsrDirToModulePath(dirname);
   }
 }
 
 
-void setupModulePaths(void) {
-  const char* modulesRoot = (fMinimalModules ? "modules-minimal" : "modules");
+void setupModulePaths() {
+  const char* modulesRoot = 0;
+
+  if (fMinimalModules == true)
+    modulesRoot = "modules/minimal";
+
+  else if (fUseIPE == true)
+    modulesRoot = "modules/ipe";
+
+  else
+    modulesRoot = "modules";
 
   intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/localeModels/",
                       CHPL_LOCALE_MODEL));
-  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/threads/", 
-                      CHPL_THREADS));
-  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/tasks/", 
+  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/tasktable/",
+                      fEnableTaskTracking ? "on" : "off"));
+  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/tasks/",
                       CHPL_TASKS));
-  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/comm/", 
+  intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal/comm/",
                       CHPL_COMM));
   intModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/internal"));
-  stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/standard/gen/", 
+
+  stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/standard/gen/",
                       CHPL_TARGET_PLATFORM,
                       "-", CHPL_TARGET_COMPILER));
+
   stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/standard"));
+  stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/packages"));
   stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/layouts"));
   stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/dists"));
   stdModPath.add(astr(CHPL_HOME, "/", modulesRoot, "/dists/dims"));
+
   const char* envvarpath = getenv("CHPL_MODULE_PATH");
+
   if (envvarpath) {
     char path[FILENAME_MAX+1];
     strncpy(path, envvarpath, FILENAME_MAX);
     char* colon = NULL;
+
     do {
       char* start = colon ? colon+1 : path;
       colon = strchr(start, ':');
@@ -698,8 +794,9 @@ void addModulePathFromFilename(const char* origfilename) {
 }
 
 
-const char* modNameToFilename(const char* modName, bool isInternal, 
-                              bool* isStandard) {
+const char* modNameToFilename(const char* modName,
+                              bool        isInternal,
+                              bool*       isStandard) {
   const char* filename = astr(modName, ".chpl");
   const char* fullfilename;
   if (isInternal) {
@@ -713,14 +810,49 @@ const char* modNameToFilename(const char* modName, bool isInternal,
 }
 
 const char* stdModNameToFilename(const char* modName) {
-  const char* fullfilename = searchPath(stdModPath, astr(modName, ".chpl"), 
+  const char* fullfilename = searchPath(stdModPath,
+                                        astr(modName, ".chpl"),
                                         NULL);
+
   if (fullfilename == NULL) {
     USR_FATAL("Can't find standard module '%s'\n", modName);
   }
+
   return fullfilename;
 }
 
+const char* filenameToModulename(const char* filename) {
+  const char* moduleName = astr(filename);
+  const char* firstSlash = strrchr(moduleName, '/');
+
+  if (firstSlash) {
+    moduleName = firstSlash + 1;
+  }
+
+  return asubstr(moduleName, strrchr(moduleName, '.'));
+}
+
+//
+// Return a fully qualified path name for the internal file with the specified baseName
+//
+
+const char* pathNameForInternalFile(const char* baseName) {
+  const char* fileName = astr(baseName, ".chpl");
+
+  return searchPath(intModPath, fileName, NULL, true);
+}
+
+//
+// Return a fully qualified path name for the standard file with the specified baseName
+// Generate a warning if there is a user file that might define the same module
+//
+
+const char* pathNameForStandardFile(const char* baseName) {
+  const char* fileName     = astr(baseName, ".chpl");
+  const char* userFileName = searchPath(usrModPath, fileName, NULL, false);
+
+  return searchPath(stdModPath, fileName, userFileName, false);
+}
 
 static void helpPrintPath(Vec<const char*> path) {
   forv_Vec(const char*, dirname, path) {
@@ -728,8 +860,11 @@ static void helpPrintPath(Vec<const char*> path) {
   }
 }
 
-void printModuleSearchPath(void) {
+void printModuleSearchPath() {
   fprintf(stderr, "module search dirs:\n");
+  if (developer) {
+    helpPrintPath(intModPath);
+  }
   helpPrintPath(usrModPath);
   helpPrintPath(stdModPath);
   fprintf(stderr, "end of module search dirs\n");
@@ -812,32 +947,60 @@ static int sys_getcwd(char** path_out)
 {
   int sz = 128;
   char* buf;
-  char* got;
-  int err = 0;
 
   buf = (char*) malloc(sz);
   if( !buf ) return ENOMEM;
+
   while( 1 ) {
-    got = getcwd(buf, sz);
-    if( got != NULL ) break;
-    else if( errno == ERANGE ) {
+    if ( getcwd(buf, sz) != NULL ) {
+      break;
+
+    } else if ( errno == ERANGE ) {
       // keep looping but with bigger buffer.
-      sz = 2*sz;
-      got = (char*) realloc(buf, sz);
-      if( ! got ) {
+      sz *= 2;
+
+      /*
+       * Realloc may return NULL, in which case we will need to free the memory
+       * initially pointed to by buf.  This is why we store the result of the
+       * call in newP instead of directly into buf.  If a non-NULL value is
+       * returned we update the buf pointer.
+       */
+      void* newP = realloc(buf, sz);
+
+      if (newP != NULL) {
+        buf = static_cast<char*>(newP);
+
+      } else {
         free(buf);
         return ENOMEM;
       }
+
     } else {
       // Other error, stop.
-      err = errno;
+      free(buf);
+      return errno;
     }
   }
 
   *path_out = buf;
-  return err;
+  return 0;
 }
 
+
+/*
+ * Returns the current working directory. Does not report failures. Use
+ * sys_getcwd() if you need error reports.
+ */
+const char* getCwd() {
+  const char* result = getcwd(NULL, PATH_MAX);
+  if (result) {
+    return result;
+  } else {
+    return "";
+  }
+}
+
+
 // Find the path to the running program
 // (or return NULL if we couldn't figure it out).
 // The return value must be freed by the caller.
@@ -902,7 +1065,7 @@ char* findProgramPath(const char *argv0)
     end = strchr(start, ':');
     if( end == NULL ) end = path_end;
     else end[0] = '\0'; // replace ':' with '\0'
-  
+
     real = dirHasFile(start, argv0);
     if( real ) break;
 
@@ -930,7 +1093,7 @@ bool isSameFile(const char* pathA, const char* pathB)
   rc = stat(pathB, &statsB);
   if( rc != 0 ) return false;
 
-  // is the inode the same? 
+  // is the inode the same?
   if( statsA.st_dev == statsB.st_dev &&
       statsA.st_ino == statsB.st_ino ) {
     return true;
@@ -938,4 +1101,3 @@ bool isSameFile(const char* pathA, const char* pathB)
 
   return false;
 }
-
