@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 
 // Merges sequences of loads or sequences of stores
@@ -37,14 +51,32 @@
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/ADT/SmallSet.h"
 #include "llvm/ADT/Statistic.h"
+
+#if HAVE_LLVM_VER >= 35
+#include "llvm/IR/InstIterator.h"
+#include "llvm/IR/CallSite.h"
+#else
 #include "llvm/Support/InstIterator.h"
 #include "llvm/Support/CallSite.h"
+#endif
+
+
+#if HAVE_LLVM_VER >= 35
+#include "llvm/IR/Verifier.h"
+#else
 #include "llvm/Analysis/Verifier.h"
+#endif
+
 #include "llvm/Analysis/AliasAnalysis.h"
 #include "llvm/Analysis/MemoryDependenceAnalysis.h"
 #include "llvm/Transforms/Utils/ValueMapper.h"
 
+#if HAVE_LLVM_VER >= 37
+#include "llvm/IR/GetElementPtrTypeIterator.h"
+#endif
+
 #include <cstdio>
+#include <list>
 
 using namespace llvm;
 
@@ -725,7 +757,13 @@ bool AggregateGlobalOpsOpt::runOnFunction(Function &F) {
   }
 
   //MD = &getAnalysis<MemoryDependenceAnalysis>();
+#if HAVE_LLVM_VER >= 37
+  TD = & F.getParent()->getDataLayout();
+#elif HAVE_LLVM_VER >= 35
+  TD = & getAnalysisIfAvailable<DataLayoutPass>()->getDataLayout();
+#else
   TD = getAnalysisIfAvailable<DataLayout>();
+#endif
   //TLI = &getAnalysis<TargetLibraryInfo>();
 
   // Walk all instruction in the function.
@@ -756,7 +794,11 @@ bool AggregateGlobalOpsOpt::runOnFunction(Function &F) {
   }
 
   if( extraChecks ) {
+#if HAVE_LLVM_VER >= 35
+    assert(!verifyFunction(F, &errs()));
+#else
     verifyFunction(F);
+#endif
   }
 
   //MD = 0;
