@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 //===----------------------------------------------------------------------===//
 // Chapel LLVM Wide Opt 
@@ -31,9 +45,18 @@
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/ADT/SmallSet.h"
 #include "llvm/ADT/Statistic.h"
+
+#if HAVE_LLVM_VER >= 35
+#include "llvm/IR/InstIterator.h"
+#include "llvm/IR/CallSite.h"
+#include "llvm/IR/Verifier.h"
+#else
 #include "llvm/Support/InstIterator.h"
 #include "llvm/Support/CallSite.h"
 #include "llvm/Analysis/Verifier.h"
+#endif
+
+
 #include "llvm/Transforms/Utils/ValueMapper.h"
 
 #include <cstdio>
@@ -79,6 +102,7 @@ namespace {
     return tempVar;
   }
 
+#if 0
   void typeChangeReplaceInstWithInst(Instruction* Old, Instruction* New)
   {
     assert(Old->getParent());
@@ -98,6 +122,7 @@ namespace {
  
     Old->eraseFromParent();
   }
+#endif
 
   // Like the version in BasicBlockUtils but assumes New is already
   // in the block.
@@ -1218,7 +1243,12 @@ namespace {
 
         for(Function::use_iterator UI = F->use_begin(), UE = F->use_end();
                 UI!=UE; ) {
+#if HAVE_LLVM_VER >= 35
+          Use &U = *UI;
+          User *Old = U.getUser();
+#else
           User *Old = *UI;
+#endif
           ++UI;
           CallSite CS(Old);
           if (CS.getInstruction()) {
@@ -1350,7 +1380,11 @@ namespace {
             NF->dump();
           }
           if( extraChecks ) {
+#if HAVE_LLVM_VER >= 35
+            assert(!verifyFunction(*NF, &errs()));
+#else
             verifyFunction(*NF);
+#endif
           }
         }
 
@@ -1399,7 +1433,11 @@ namespace {
                 AI != AE; ++AI) {
           GlobalAlias *ga = &*AI;
 
+#if HAVE_LLVM_VER >= 35
+          GlobalValue *gv = dyn_cast<GlobalValue>(ga->getAliasee());
+#else
           GlobalValue *gv = const_cast<GlobalValue*>(ga->getAliasedGlobal());
+#endif
           Type *old_type = ga->getType();
           Type *new_type = convertTypeGlobalToWide(&M, info, ga->getType());
           if (new_type == old_type) {
@@ -1407,8 +1445,21 @@ namespace {
           }
 
           Constant *init = ConstantExpr::getPointerCast(gv, new_type);
-          GlobalAlias *new_alias = new GlobalAlias(new_type, ga->getLinkage(),
+#if HAVE_LLVM_VER >= 37
+          GlobalAlias *new_alias = GlobalAlias::create(
+              llvm::PointerType::get(new_type, 0 /*addr space*/ ),
+              ga->getLinkage(),
               "", init, &M);
+#elif HAVE_LLVM_VER >= 35
+          GlobalAlias *new_alias = GlobalAlias::create(new_type,
+              0, // address space
+              ga->getLinkage(),
+              "", init, &M);
+#else
+          GlobalAlias *new_alias = new GlobalAlias(new_type,
+              ga->getLinkage(),
+              "", init, &M);
+#endif
 
           Constant *cast_ptr = ConstantExpr::getPointerCast(new_alias, ga->getType());
 
@@ -1499,7 +1550,11 @@ namespace {
         }
 
         if( extraChecks ) {
-          verifyFunction(*F);
+#if HAVE_LLVM_VER >= 35
+            assert(!verifyFunction(*F, &errs()));
+#else
+            verifyFunction(*F);
+#endif
         }
 
         if( debugPassTwo ) {
@@ -1589,7 +1644,11 @@ namespace {
         }
 
         if( extraChecks ) {
+#if HAVE_LLVM_VER >= 35
+          assert(!verifyFunction(*F, &errs()));
+#else
           verifyFunction(*F);
+#endif
         }
       }
 
@@ -1647,8 +1706,8 @@ static
 bool containsGlobalPointers(unsigned gSpace, SmallSet<Type*, 10> & set, Type* t)
 {
   // All primitive types do not need to change.
-  if(t->isPrimitiveType() || t->isIntegerTy()) return false;
-  assert(!t->isVoidTy());
+  if(t->isFloatingPointTy() || t->isX86_MMXTy() || t->isLabelTy() ||
+     t->isMetadataTy() || t->isVoidTy() || t->isIntegerTy()) return false;
 
   // Pointer types return true if they are in our address space.
   if(t->isPointerTy()){
@@ -1657,7 +1716,7 @@ bool containsGlobalPointers(unsigned gSpace, SmallSet<Type*, 10> & set, Type* t)
 
   // If it's something we've already enumerated,
   // it doesn't get to say yes.
-  if( set.insert(t) ) {
+  if( llvm_small_set_insert(set, t) ) {
     // We added it to the set, we can continue.
   } else {
     // It was already in the set, don't bother
