@@ -1,23 +1,42 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ *
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ *
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
+#include "misc.h"
+
+#include "baseAST.h"
+#include "chpl.h"
+#include "driver.h"
+#include "expr.h"
+#include "files.h"
+#include "parser.h"
+#include "resolution.h"
+#include "stmt.h"
+#include "stringutil.h"
+#include "symbol.h"
+#include "version.h"
 
 #include <csignal>
 #include <fcntl.h>
 #include <sys/stat.h>
-#include "driver.h"
-#include "../main/version.h"
-#include "files.h"
-#include "chpl.h"
-#include "baseAST.h"
-#include "symbol.h"
-#include "expr.h"
-#include "stmt.h"
-#include "stringutil.h"
-#include "misc.h"
-#include "yy.h"
-#include "../resolution/resolution.h"
+
+
+static const char* help_url = "http://chapel.cray.com/bugs.html";
 
 static void cleanup_for_exit(void) {
   deleteTmpDir();
@@ -42,8 +61,16 @@ static int err_print;
 static int err_ignore;
 static FnSymbol* err_fn = NULL;
 
-bool forceWidePtrs() {
-  return !strcmp(CHPL_LOCALE_MODEL, "numa");
+//
+// Chances are that all non-flat locale models will require wide
+// pointers.  Ultimately, we'd like to have such decisions be made by
+// param fields/methods within the locale models themselves, but that
+// would require a fairly large refactoring, so for now, we
+// special-case 'flat' with the expectation that most other locale
+// models will not be flat.
+//
+static bool forceWidePtrs() {
+  return (strcmp(CHPL_LOCALE_MODEL, "flat") != 0);
 }
 
 bool forceWidePtrsForLocal() {
@@ -54,8 +81,14 @@ bool requireWideReferences() {
   return !fLocal || forceWidePtrs();
 }
 
+//
+// If the --no-local flag is used, or the locale model is not 'flat'
+// (i.e., has sub-locales that an on-clause might target), we should
+// require on-clauses to be "outlined" (i.e., we should not assume the
+// on-clause is a no-op and execute the associated statement locally.
+//
 bool requireOutlinedOn() {
-  return !fLocal || forceWidePtrs();
+  return !fLocal || strcmp(CHPL_LOCALE_MODEL, "flat") != 0;
 }
 
 const char* cleanFilename(const char* name) {
@@ -69,7 +102,7 @@ const char* cleanFilename(const char* name) {
 }
 
 
-static const char* cleanFilename(BaseAST* ast) {
+const char* cleanFilename(BaseAST* ast) {
   const char* astFname = ast->fname();
   if (astFname)
     return cleanFilename(astFname);
@@ -100,9 +133,7 @@ print_user_internal_error() {
   fprintf(stderr, "%s ", error);
   char version[128];
   get_version(version);
-  fprintf(stderr, "chpl Version %s\n", version);
-  if (err_fatal)
-    clean_exit(1);
+  fprintf(stderr, "chpl Version %s", version);
 }
 
 
@@ -144,8 +175,8 @@ setupError(const char *filename, int lineno, int tag) {
 }
 
 
-static void
-printDevelErrorHeader(BaseAST* ast) {
+static bool
+printErrorHeader(BaseAST* ast) {
   if (!err_print) {
     if (Expr* expr = toExpr(ast)) {
       Symbol* parent = expr->parentSymbol;
@@ -172,7 +203,7 @@ printDevelErrorHeader(BaseAST* ast) {
             fprintf(stderr, "constructor '%s':\n", err_fn->name+11);
           } else {
             fprintf(stderr, "%s '%s':\n",
-                    (err_fn->hasFlag(FLAG_ITERATOR_FN) ? "iterator" : "function"),
+                    (err_fn->isIterator() ? "iterator" : "function"),
                     err_fn->name);
           }
         }
@@ -180,9 +211,33 @@ printDevelErrorHeader(BaseAST* ast) {
     }
   }
 
+  bool have_ast_line = false;
+  const char* filename;
+  int linenum;
 
-  if (ast && ast->linenum())
-    fprintf(stderr, "%s:%d: ", cleanFilename(ast), ast->linenum());
+  if ( ast && ast->linenum() ) {
+    have_ast_line = true;
+    filename = cleanFilename(ast);
+    linenum = ast->linenum();
+  } else {
+    have_ast_line = false;
+    if ( !err_print && currentAstLoc.filename && currentAstLoc.lineno > 0 ) {
+      // Use our best guess for the line number for user errors,
+      // but don't do that for err_print (USR_PRINT) notes that don't
+      // come with line numbers.
+      filename = cleanFilename(currentAstLoc.filename);
+      linenum = currentAstLoc.lineno;
+    } else {
+      filename = NULL;
+      linenum = -1;
+    }
+  }
+
+  bool guess = filename && !have_ast_line;
+
+  if (filename) {
+    fprintf(stderr, "%s:%d: ", filename, linenum);
+  }
 
   if (err_print) {
     fprintf(stderr, "note: ");
@@ -196,15 +251,50 @@ printDevelErrorHeader(BaseAST* ast) {
     fprintf(stderr, "warning: ");
   }
 
-  if (!err_user && !developer) {
-    print_user_internal_error();
+  if (!err_user) {
+    if (!developer) {
+      print_user_internal_error();
+    }
   }
-
+  return guess;
 }
 
-static void printDevelErrorFooter(void) {
-  if (developer)
+
+static void printErrorFooter(bool guess) {
+  //
+  // For developers, indicate the compiler source location where an
+  // internal error was generated.
+  //
+  if (developer && !err_user)
     fprintf(stderr, " [%s:%d]", err_filename, err_lineno);
+
+  //
+  // For users and developers, if the source line was a guess (i.e., an
+  // AST was not passed to the INT_FATAL() macro and we relied on the
+  // global SET_LINENO() information instead), indicate that.
+  //
+  if (guess) {
+    fprintf(stderr, "\nNote: This source location is a guess.");
+  }
+
+  //
+  // Apologize for our internal errors to the end-user
+  //
+  if (!developer && !err_user) {
+    fprintf(stderr, "\n\n"
+            "Internal errors indicate a bug in the Chapel compiler (\"It's us, not you\"),\n"
+            "and we're sorry for the hassle.  We would appreciate your reporting this bug -- \n"
+            "please see %s for instructions.  In the meantime,\n"
+            "the filename + line number above may be useful in working around the issue.\n\n", 
+            help_url);
+
+    //
+    // and exit if it's fatal (isn't it always?)
+    //
+    if (err_fatal) {
+      clean_exit(1);
+    }
+  }
 }
 
 
@@ -265,23 +355,28 @@ void handleError(const char *fmt, ...) {
   if (err_ignore)
     return;
 
-  printDevelErrorHeader(NULL);
+  bool guess = printErrorHeader(NULL);
 
-  if (!err_user && !developer)
-    return;
+  //
+  // Only print out the arguments if this is a user error or we're
+  // in developer mode.
+  //
+  if (err_user || developer) {
+    va_list args;
 
-  va_list args;
-
-  va_start(args, fmt);
-  vfprintf(stderr, fmt, args);
-  va_end(args);
-
-  printDevelErrorFooter();
+    va_start(args, fmt);
+    vfprintf(stderr, fmt, args);
+    va_end(args);
+  }
 
+  printErrorFooter(guess);
   fprintf(stderr, "\n");
 
   printCallStackOnError();
 
+  if (!err_user && !developer)
+    return;
+
   if (exit_immediately) {
     if (ignore_errors_for_pass) {
       exit_end_of_pass = true;
@@ -312,25 +407,27 @@ static void vhandleError(FILE* file, BaseAST* ast, const char *fmt, va_list args
   if (err_ignore)
     return;
 
+  bool guess = false;
   if (file == stderr)
-    printDevelErrorHeader(ast);
+    guess = printErrorHeader(ast);
 
-  if (!err_user && !developer)
-    return;
-
-  vfprintf(file, fmt, args);
+  if (err_user || developer) {
+    vfprintf(file, fmt, args);
+  }
 
   if (fPrintIDonError && ast)
     fprintf(file, " [%d]", ast->id);
 
   if (file == stderr)
-    printDevelErrorFooter();
-
+    printErrorFooter(guess);
   fprintf(file, "\n");
 
   if (file == stderr)
     printCallStackOnError();
 
+  if (!err_user && !developer)
+    return;
+
   if (exit_immediately) {
     if (ignore_errors_for_pass) {
       exit_end_of_pass = true;
