@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _chpl_comm_compiler_llvm_support_h_
 #define _chpl_comm_compiler_llvm_support_h_
@@ -21,27 +35,27 @@
 // where sync_scope = loadOrStoreInst->getSynchScope()
 //       atomic_ordering = loadOrStoreInst->getOrdering()
 
-static ___always_inline
+static inline
 void chpl_gen_comm_get_ctl(void *dst_addr, wide_ptr_t src, int64_t n, int64_t ctl)
 {
   c_nodeid_t src_node = chpl_wide_ptr_get_node(src);
   void* src_addr = chpl_wide_ptr_get_address(src);
-  chpl_gen_comm_get(dst_addr, src_node, src_addr, sizeof(uint8_t), CHPL_TYPE_uint8_t, n, -1, "");
+  chpl_gen_comm_get(dst_addr, src_node, src_addr, sizeof(uint8_t)*n, CHPL_TYPE_uint8_t, -1, 0);
 }
 
-static ___always_inline
+static inline
 void chpl_gen_comm_put_ctl(wide_ptr_t dst, void *src_addr, int64_t n, int64_t ctl)
 {
   c_nodeid_t dst_node = chpl_wide_ptr_get_node(dst);
   void* dst_addr = chpl_wide_ptr_get_address(dst);
-  chpl_gen_comm_put(src_addr, dst_node, dst_addr, sizeof(uint8_t), CHPL_TYPE_uint8_t, n, -1, "");
+  chpl_gen_comm_put(src_addr, dst_node, dst_addr, sizeof(uint8_t)*n, CHPL_TYPE_uint8_t, -1, 0);
 }
 
 // This function implements memcpy/memmove when both the source and destination
 // are possibly remote. The Chapel compiler does not normally generate
 // such code, but it could appear during optimization. Note that the
 // dst and src regions could overlap.
-static ___always_inline
+static inline
 void chpl_gen_comm_getput(wide_ptr_t dst, wide_ptr_t src, int64_t n)
 {
   c_nodeid_t src_node = chpl_wide_ptr_get_node(src);
@@ -70,7 +84,7 @@ void chpl_gen_comm_getput(wide_ptr_t dst, wide_ptr_t src, int64_t n)
   }
 }
 
-static ___always_inline
+static inline
 void chpl_gen_comm_memset(wide_ptr_t dst, int8_t src, int64_t n)
 {
   c_nodeid_t dst_node = chpl_wide_ptr_get_node(dst);
