@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _chpl_comm_h_
 #define _chpl_comm_h_
@@ -27,18 +41,12 @@ extern c_nodeid_t chpl_nodeID; // unique ID for each node: 0, 1, 2, ...
 // (hopefully) unique to the running image, and never changed again.
 extern int32_t chpl_numNodes; // number of nodes
 
-extern int32_t chpl_numPrivateObjects;
-extern void** chpl_privateObjects; // privatized array and domain objects
-
 extern int chpl_verbose_comm;     // set via startVerboseComm
 extern int chpl_comm_diagnostics; // set via startCommDiagnostics
 extern int chpl_verbose_mem;      // set via startVerboseMem
 
-static ___always_inline
-int32_t chpl_numPrivatizedClasses(void) { return chpl_numPrivateObjects; }
+size_t chpl_comm_getenvMaxHeapSize(void);
 
-extern void chpl_newPrivatizedClass(void*);
-extern void* chpl_getPrivatizedClass(int32_t);
 
 //
 // Shared interface (implemented in the compiler generated code)
@@ -66,6 +74,53 @@ extern const int chpl_heterogeneous;
 // Comm layer-specific interface
 //
 
+// chpl_comm_nb_handle_t must be defined in the comm layer header
+// chpl-comm-task-decls.h
+
+// Do a GET in a nonblocking fashion, returning a handle which can be used to
+// wait for the GET to complete. The destination buffer must not be modified
+// before the request completes (after waiting on the returned handle)
+chpl_comm_nb_handle_t chpl_comm_get_nb(void* addr, c_nodeid_t node, void* raddr,
+                                       size_t size, int32_t typeIndex,
+                                       int ln, int32_t fn);
+
+// Do a PUT in a nonblocking fashion, returning a handle which can be used to
+// wait for the PUT to complete. The source buffer must not be modified before
+// the request completes (after waiting on the returned handle)
+chpl_comm_nb_handle_t chpl_comm_put_nb(void *addr, c_nodeid_t node, void* raddr,
+                                       size_t size, int32_t typeIndex,
+                                       int ln, int32_t fn);
+
+// Returns nonzero iff the handle has already been waited for and has
+// been cleared out in a call to chpl_comm_{wait,try}_some.
+int chpl_comm_test_nb_complete(chpl_comm_nb_handle_t h);
+
+// Wait on handles created by chpl_comm_start_....  ignores completed handles.
+// Clears out completed handles so that calling chpl_comm_nb_handle_is_complete
+// on them returns nonzero.
+void chpl_comm_wait_nb_some(chpl_comm_nb_handle_t* h, size_t nhandles);
+
+// Try handles created by chpl_comm_start_....  ignores completed handles.
+// Clears out completed handles so that calling chpl_comm_nb_handle_is_complete
+// on them returns nonzero.  Returns nonzero iff at least one completion is
+// detected.
+int chpl_comm_try_nb_some(chpl_comm_nb_handle_t* h, size_t nhandles);
+
+// Returns whether or not the passed wide address is known to be in
+// a communicable memory region and known to be readable. That is,
+// GET to that address should succeed without an access violation
+// or other memory protection error.
+// Returns 1 if the entire passed region can be read with a GET.
+// Returns 0 if it might not be safely readable.
+//
+// This function returns 1 if the requested region is known to be in a
+// pre-mapped registered memory region that can be read with GET without
+// any segmentation errors. For GASNET_SEGMENT=everything, for example,
+// this function always returns 0 since although any address is in the
+// segment, not all addresses are readable without causing a segmentation
+// violation on the remote locale.
+int chpl_comm_addr_gettable(c_nodeid_t node, void* start, size_t len);
+
 //
 // returns the maximum number of threads that can be handled
 // by this communication layer (used to ensure numThreadsPerLocale is
@@ -156,7 +211,7 @@ void chpl_comm_broadcast_global_vars(int numGlobals);
 //
 // Note that this routine is currently used only during program
 // initialization, so it is arguably not as performance critical as
-// other more core communication routines (like puts, gets, forks).
+// other more core communication routines (like puts, gets, executeOns).
 //
 // The third argument, 'tid' (type ID) is intended for use when
 // targeting heterogeneous architectures where byte swapping may be
@@ -164,7 +219,7 @@ void chpl_comm_broadcast_global_vars(int numGlobals);
 // currently in use on any platforms, but is being retained in the
 // event that we wish to re-enable this capability in the future.
 // 
-void chpl_comm_broadcast_private(int id, int32_t size, int32_t tid);
+void chpl_comm_broadcast_private(int id, size_t size, int32_t tid);
 
 //
 // Barrier for synchronization between all top-level locales; currently
@@ -210,8 +265,8 @@ void chpl_comm_exit(int all, int status);
 //   size and locale are part of p
 //
 void  chpl_comm_put(void* addr, c_nodeid_t node, void* raddr,
-                    int32_t elemSize, int32_t typeIndex, int32_t len,
-                    int ln, chpl_string fn);
+                    size_t size, int32_t typeIndex,
+                    int ln, int32_t fn);
 
 //
 // get 'size' bytes of remote data at 'raddr' on locale 'locale' to
@@ -221,8 +276,8 @@ void  chpl_comm_put(void* addr, c_nodeid_t node, void* raddr,
 //   size and locale are part of p
 //
 void  chpl_comm_get(void *addr, c_nodeid_t node, void* raddr,
-                    int32_t elemSize, int32_t typeIndex, int32_t len,
-                    int ln, chpl_string fn);
+                    size_t size, int32_t typeIndex,
+                    int ln, int32_t fn);
 
 //
 // put the number of elements pointed out by count array, with strides pointed
@@ -236,18 +291,18 @@ void  chpl_comm_get(void *addr, c_nodeid_t node, void* raddr,
 //   Proposal for Extending the UPC Memory Copy Library Functions and Supporting 
 //   Extensions to GASNet, Version 2.0. Author: Dan Bonachea 
 //
-void  chpl_comm_put_strd(void* dstaddr, void* dststrides, int32_t dstlocale, 
-                     void* srcaddr, void* srcstrides, void* count,
-                     int32_t stridelevels, int32_t elemSize, int32_t typeIndex, 
-                     int ln, chpl_string fn);
+void  chpl_comm_put_strd(void* dstaddr, size_t* dststrides, int32_t dstlocale, 
+                     void* srcaddr, size_t* srcstrides, size_t* count,
+                     int32_t stridelevels, size_t elemSize, int32_t typeIndex, 
+                     int ln, int32_t fn);
 
 //
 // same as chpl_comm_puts(), but do get instead
 //
-void  chpl_comm_get_strd(void* dstaddr, void* dststrides, int32_t srclocale, 
-                     void* srcaddr, void* srcstrides, void* count,
-                     int32_t stridelevels, int32_t elemSize, int32_t typeIndex, 
-                     int ln, chpl_string fn);
+void  chpl_comm_get_strd(void* dstaddr, size_t* dststrides, int32_t srclocale, 
+                     void* srcaddr, size_t* srcstrides, size_t* count,
+                     int32_t stridelevels, size_t elemSize, int32_t typeIndex, 
+                     int ln, int32_t fn);
 
 //
 // Get a local copy of a wide string.
@@ -256,30 +311,33 @@ void  chpl_comm_get_strd(void* dstaddr, void* dststrides, int32_t srclocale,
 // a locally-allocated char[] and the locale field is set to "here".
 // The local char[] buffer is leaked. :(
 //
-void chpl_gen_comm_wide_string_get(void* addr,
-  c_nodeid_t node, void* raddr, int32_t elemSize, int32_t typeIndex, int32_t len,
-                                   int ln, chpl_string fn);
+void chpl_gen_comm_wide_string_get(void *addr, c_nodeid_t node, void *raddr,
+                                   size_t size, int32_t typeIndex,
+                                   int ln, int32_t fn);
 
 //
-// remote fork should launch a thread on locale that runs function f
-// passing it arg where the size of arg is stored in arg_size
+// Runs a function f on a remote locale, passing it
+// arg where size of arg is stored in arg_size.
+//
+// This call will block the current task until the remote function has
+// completed. Use chpl_comm_execute_on_nb if you do not want to wait.
 // notes:
-//   multiple forks to the same locale should be handled concurrently
+//   multiple executeOns to the same locale should be handled concurrently
 //
-void chpl_comm_fork(c_nodeid_t node, c_sublocid_t subloc,
-                    chpl_fn_int_t fid, void *arg, int32_t arg_size);
+void chpl_comm_execute_on(c_nodeid_t node, c_sublocid_t subloc,
+                          chpl_fn_int_t fid, void *arg, size_t arg_size);
 
 //
-// non-blocking fork
+// non-blocking execute_on
 //
-void chpl_comm_fork_nb(c_nodeid_t node, c_sublocid_t subloc,
-                       chpl_fn_int_t fid, void *arg, int32_t arg_size);
+void chpl_comm_execute_on_nb(c_nodeid_t node, c_sublocid_t subloc,
+                             chpl_fn_int_t fid, void *arg, size_t arg_size);
 
 //
-// fast (non-forking) fork (i.e., run in handler)
+// fast execute_on (i.e., run in handler)
 //
-void chpl_comm_fork_fast(c_nodeid_t node, c_sublocid_t subloc,
-                         chpl_fn_int_t fid, void *arg, int32_t arg_size);
+void chpl_comm_execute_on_fast(c_nodeid_t node, c_sublocid_t subloc,
+                         chpl_fn_int_t fid, void *arg, size_t arg_size);
 
 
 //
@@ -292,7 +350,12 @@ void chpl_comm_fork_fast(c_nodeid_t node, c_sublocid_t subloc,
 //
 int chpl_comm_numPollingTasks(void);
 
-
+// Some communication layers need to be periodically invoked
+// in order to make progress. This call gives the comm layer
+// an opportunity to move puts,gets, etc along while the
+// current thread is idle (e.g. when we are waiting on
+// an atomic variable for other tasks to finish).
+void chpl_comm_make_progress(void);
 
 //
 // Comm diagnostics stuff
@@ -300,12 +363,14 @@ int chpl_comm_numPollingTasks(void);
 typedef struct _chpl_commDiagnostics {
   uint64_t get;
   uint64_t get_nb;
-  uint64_t get_nb_test;
-  uint64_t get_nb_wait;
   uint64_t put;
-  uint64_t fork;
-  uint64_t fork_fast;
-  uint64_t fork_nb;
+  uint64_t put_nb;
+  uint64_t test_nb;
+  uint64_t wait_nb;
+  uint64_t try_nb;
+  uint64_t execute_on;
+  uint64_t execute_on_fast;
+  uint64_t execute_on_nb;
 } chpl_commDiagnostics;
 
 void chpl_startVerboseComm(void);
@@ -313,26 +378,17 @@ void chpl_stopVerboseComm(void);
 void chpl_startVerboseCommHere(void);
 void chpl_stopVerboseCommHere(void);
 
-void chpl_startCommDiagnostics(void);
+void chpl_startCommDiagnostics(void); // this one implemented by comm layers
+void chpl_gen_startCommDiagnostics(void); // this one implemented in chpl-comm.c
 void chpl_stopCommDiagnostics(void);
+void chpl_gen_stopCommDiagnostics(void);
 void chpl_startCommDiagnosticsHere(void);
+void chpl_gen_startCommDiagnosticsHere(void);
 void chpl_stopCommDiagnosticsHere(void);
+void chpl_gen_stopCommDiagnosticsHere(void);
 void chpl_resetCommDiagnosticsHere(void);
 void chpl_getCommDiagnosticsHere(chpl_commDiagnostics *cd);
 
-//
-// These are still supported because our extern record support is
-//  still a bit lacking.
-//
-uint64_t chpl_numCommGets(void);
-uint64_t chpl_numCommNBGets(void);
-uint64_t chpl_numCommTestNBGets(void);
-uint64_t chpl_numCommWaitNBGets(void);
-uint64_t chpl_numCommPuts(void);
-uint64_t chpl_numCommForks(void);
-uint64_t chpl_numCommFastForks(void);
-uint64_t chpl_numCommNBForks(void);
-
 #else // LAUNCHER
 
 #define chpl_comm_barrier(x)
@@ -341,4 +397,9 @@ uint64_t chpl_numCommNBForks(void);
 
 #endif // LAUNCHER
 
+// Warn if runtime uses e.g. chpl_comm_get
+// (it should use chpl_gen_comm_get)
+#include "chpl-comm-warning-macros.h"
+
 #endif
+
