@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _chpl_tasks_h_
 #define _chpl_tasks_h_
@@ -10,6 +24,7 @@
 
 #include <stdint.h>
 #include "chpltypes.h"
+#include "chpl-tasks-prvdata.h"
 
 #ifdef CHPL_TASKS_MODEL_H
 #include CHPL_TASKS_MODEL_H
@@ -35,9 +50,9 @@
 void      chpl_sync_lock(chpl_sync_aux_t *);
 void      chpl_sync_unlock(chpl_sync_aux_t *);
 void      chpl_sync_waitFullAndLock(chpl_sync_aux_t *,
-                                       int32_t, chpl_string);
+                                       int32_t, int32_t);
 void      chpl_sync_waitEmptyAndLock(chpl_sync_aux_t *,
-                                        int32_t, chpl_string);
+                                        int32_t, int32_t);
 void      chpl_sync_markAndSignalFull(chpl_sync_aux_t *);     // and unlock
 void      chpl_sync_markAndSignalEmpty(chpl_sync_aux_t *);    // and unlock
 chpl_bool chpl_sync_isFull(void *, chpl_sync_aux_t *);
@@ -49,26 +64,26 @@ void      chpl_sync_destroyAux(chpl_sync_aux_t *);
 
 typedef chpl_sync_aux_t chpl_single_aux_t;
 
-static ___always_inline
+static inline
 void chpl_single_lock(chpl_sync_aux_t * s) { chpl_sync_lock(s); }
-static ___always_inline
+static inline
 void chpl_single_unlock(chpl_sync_aux_t * s) { chpl_sync_unlock(s); }
-static ___always_inline
+static inline
 void chpl_single_waitFullAndLock(chpl_sync_aux_t * s,
-                                 int32_t lineno, chpl_string filename) {
+                                 int32_t lineno, int32_t filename) {
   chpl_sync_waitFullAndLock(s,lineno,filename);
 }
-static ___always_inline
+static inline
 void chpl_single_markAndSignalFull(chpl_sync_aux_t * s) {
   chpl_sync_markAndSignalFull(s);
 }
-static ___always_inline
+static inline
 chpl_bool chpl_single_isFull(void *val_ptr, chpl_sync_aux_t *s) {
   return chpl_sync_isFull(val_ptr, s);
 }
-static ___always_inline
+static inline
 void chpl_single_initAux(chpl_sync_aux_t * s) { chpl_sync_initAux(s); }
-static ___always_inline
+static inline
 void chpl_single_destroyAux(chpl_sync_aux_t * s) { chpl_sync_destroyAux(s); }
 
 
@@ -101,10 +116,10 @@ void chpl_task_callMain(void (*chpl_main)(void));
 //
 // The following is an optional callback into the tasking layer from
 // the main task indicating that the standard internal modules have
-// been initialized.  It gives the tasking layer the ability to make
-// use of functionality in the internal modules (like the task
-// tracking table) which are not yet available in
-// chpl_task_callMain().
+// been initialized.  It gives the tasking layer the ability to wait
+// to make use of functionality in the internal modules (like the task
+// tracking table) which are not yet available at the time of the call
+// to chpl_task_callMain().
 //
 #ifndef CHPL_TASK_STD_MODULES_INITIALIZED
 #define CHPL_TASK_STD_MODULES_INITIALIZED()
@@ -125,14 +140,12 @@ void chpl_task_addToTaskList(
          chpl_fn_int_t,      // function to call for task
          void*,              // argument to the function
          c_sublocid_t,       // desired sublocale
-         chpl_task_list_p*,  // task list
+         void**,             // task list
          c_nodeid_t,         // locale (node) where task list resides
          chpl_bool,          // is begin{} stmt?  (vs. cobegin or coforall)
          int,                // line at which function begins
-         chpl_string);       // name of file containing functions
-void chpl_task_processTaskList(chpl_task_list_p);
-void chpl_task_executeTasksInList(chpl_task_list_p);
-void chpl_task_freeTaskList(chpl_task_list_p);
+         int32_t);          // name of file containing functions
+void chpl_task_executeTasksInList(void**);
 
 //
 // Launch a task that is the logical continuation of some other task,
@@ -178,7 +191,7 @@ void chpl_task_yield(void);
 //
 // Suspend.
 //
-void chpl_task_sleep(int);
+void chpl_task_sleep(double);
 
 //
 // Get and set dynamic serial state.
@@ -186,8 +199,41 @@ void chpl_task_sleep(int);
 chpl_bool chpl_task_getSerial(void);
 void      chpl_task_setSerial(chpl_bool);
 
+// The type for task private data, chpl_task_prvData_t,
+// is defined in chpl-tasks-prvdata.h in order to support
+// proper initialization order with a task model .h
+
+// Get pointer to task private data.
+#ifndef CHPL_TASK_GET_PRVDATA_IMPL_DECL
+chpl_task_prvData_t* chpl_task_getPrvData(void);
+#endif
+
 //
-// Returns the the number of sublocales the tasking layer knows about,
+// Can this tasking layer support remote caching?
+//
+// (In practice this answers: "Are tasks bound to specific pthreads
+// or, if not, does the tasking layer make memory consistency calls
+// whenever it might move a task from one pthread to another?"  Remote
+// caching uses pthread-specific data (TLS) extensively, so it turns
+// itself off when it's used with a tasking layer that can't support
+// that.)
+//
+#ifndef CHPL_TASK_SUPPORTS_REMOTE_CACHE_IMPL_DECL
+int chpl_task_supportsRemoteCache(void);
+#endif
+
+//
+// Returns the maximum width of parallelism the tasking layer expects
+// to be able to provide on the calling (sub)locale.  With some
+// exceptions for uncommon cases, this is the greatest parallel
+// speedup a program should expect to achieve.  Running more active
+// tasks than this will typically add overhead but not concurrency,
+// and thus increase execution wall time rather than decrease it.
+//
+uint32_t chpl_task_getMaxPar(void);
+
+//
+// Returns the number of sublocales the tasking layer knows about,
 // within the span of hardware it is managing tasks on.
 //
 c_sublocid_t chpl_task_getNumSublocales(void);
@@ -248,18 +294,28 @@ uint32_t chpl_task_getNumIdleThreads(void);
 int32_t chpl_task_getenvNumThreadsPerLocale(void);
 
 //
-// This returns the minimum task call stack size, whether specified in
-// the environment or the default.  It is common to all of the tasking
-// implementations and so is implemented in runtime/src/chpl-tasks.c.
+// This returns any task call stack size specified in the environment.
+// If the environment doesn't specify a call stack size, it returns 0.
+// It is common to all tasking implementations and so is implemented
+// in runtime/src/chpl-tasks.c.
 //
-size_t chpl_task_getMinCallStackSize(void);
+size_t chpl_task_getEnvCallStackSize(void);
+
+//
+// This returns the default task call stack size.
+// It is common to all tasking implementations and so is implemented
+// in runtime/src/chpl-tasks.c.
+//
+size_t chpl_task_getDefaultCallStackSize(void);
 
 //
 // These are service functions provided to the runtime by the module
 // code.
 //
-extern void chpl_taskRunningCntInc(int64_t _ln, chpl_string _fn);
-extern void chpl_taskRunningCntDec(int64_t _ln, chpl_string _fn);
+extern void chpl_taskRunningCntInc(int64_t _ln, int32_t _fn);
+extern void chpl_taskRunningCntDec(int64_t _ln, int32_t _fn);
+
+#include "chpl-tasks-callbacks.h"
 
 #else // LAUNCHER
 
