@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _chpl_threads_h_
 #define _chpl_threads_h_
@@ -107,5 +121,6 @@ void chpl_thread_mutexInit(chpl_thread_mutex_p);
 chpl_thread_mutex_p chpl_thread_mutexNew(void);
 void chpl_thread_mutexLock(chpl_thread_mutex_p);
 void chpl_thread_mutexUnlock(chpl_thread_mutex_p);
+void chpl_thread_mutexDestroy(chpl_thread_mutex_p);
 
 #endif
