@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _chpl_wide_ptr_impl_h_
 #define _chpl_wide_ptr_impl_h_
@@ -23,8 +37,6 @@
 // chpl_error needed for some of the packed wide pointer error cases.
 #include "error.h"
 
-#define WIDE_PTR_INLINE ___always_inline
-
 // Methods for working with wide pointers (packed or not) and localeID_t.
 // There are several variants of many of these in order to simplify
 // different code generation tasks. The versions that take in a pointer
@@ -42,7 +54,7 @@
   #define CHPL_PTR_MASK ( (1ULL << CHPL_PTR_BITS) - 1ULL)
 #endif
 
-static WIDE_PTR_INLINE
+static inline
 wide_ptr_t chpl_return_wide_ptr_node(c_nodeid_t node, void* addr)
 {
 #ifndef CHPL_WIDE_POINTER_PACKED
@@ -68,7 +80,7 @@ wide_ptr_t chpl_return_wide_ptr_node(c_nodeid_t node, void* addr)
 #endif
 }
 
-static WIDE_PTR_INLINE
+static inline
 void chpl_check_wide_ptr(wide_ptr_t ptr)
 {
 #ifndef CHPL_WIDE_POINTER_PACKED
@@ -83,7 +95,7 @@ void chpl_check_wide_ptr(wide_ptr_t ptr)
 #endif
 }
 
-static WIDE_PTR_INLINE
+static inline
 wide_ptr_t chpl_return_wide_ptr_loc(chpl_localeID_t loc, void * addr)
 {
 #ifndef CHPL_WIDE_POINTER_PACKED
@@ -99,14 +111,14 @@ wide_ptr_t chpl_return_wide_ptr_loc(chpl_localeID_t loc, void * addr)
 #endif
 }
 
-static WIDE_PTR_INLINE
+static inline
 wide_ptr_t chpl_return_wide_ptr_loc_ptr(const chpl_localeID_t* loc, void * addr)
 {
   return chpl_return_wide_ptr_loc(*loc, addr);
 }
 
 
-static WIDE_PTR_INLINE
+static inline
 c_nodeid_t chpl_wide_ptr_get_node(wide_ptr_t ptr)
 {
 #ifndef CHPL_WIDE_POINTER_PACKED
@@ -119,7 +131,7 @@ c_nodeid_t chpl_wide_ptr_get_node(wide_ptr_t ptr)
 #endif
 }
 
-static WIDE_PTR_INLINE
+static inline
 void* chpl_wide_ptr_get_address(wide_ptr_t ptr)
 {
 #ifndef CHPL_WIDE_POINTER_PACKED
@@ -133,7 +145,7 @@ void* chpl_wide_ptr_get_address(wide_ptr_t ptr)
 #endif
 }
 
-static WIDE_PTR_INLINE
+static inline
 chpl_localeID_t chpl_wide_ptr_get_localeID(wide_ptr_t ptr)
 {
   chpl_localeID_t loc;
@@ -147,7 +159,7 @@ chpl_localeID_t chpl_wide_ptr_get_localeID(wide_ptr_t ptr)
   return loc;
 }
 
-static WIDE_PTR_INLINE
+static inline
 void chpl_wide_ptr_read_localeID(wide_ptr_t ptr,
                                  chpl_localeID_t* loc)
 {
@@ -155,7 +167,7 @@ void chpl_wide_ptr_read_localeID(wide_ptr_t ptr,
 }
 
 
-static WIDE_PTR_INLINE
+static inline
 wide_ptr_t chpl_return_wide_ptr_add(wide_ptr_t ptr, size_t amt)
 {
 #ifndef CHPL_WIDE_POINTER_PACKED
