@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _CHPL_LAUNCH_H_
 #define _CHPL_LAUNCH_H_
@@ -24,6 +38,7 @@ int chpl_launch_using_system(char* command, char* argv0);
 char* chpl_get_enviro_keys(char sep);
 
 void chpl_compute_real_binary_name(const char* argv0);
+const char* chpl_get_real_binary_wrapper(void);
 const char* chpl_get_real_binary_name(void);
 
 //
@@ -31,7 +46,7 @@ const char* chpl_get_real_binary_name(void);
 //
 int chpl_launch(int argc, char* argv[], int32_t numLocales);
 int chpl_launch_handle_arg(int argc, char* argv[], int argNum, 
-                           int32_t lineno, chpl_string filename);
+                           int32_t lineno, int32_t filename);
 void chpl_launch_print_help(void);
 
 #endif
