@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _chpltypes_H_
 #define _chpltypes_H_
@@ -11,7 +25,11 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h> // for ptrdiff_t
 #include <sys/time.h> // for struct timeval
+#ifndef __cplusplus
+#include <complex.h>
+#endif
 
 // C types usable from Chapel.
 typedef char c_char;
@@ -28,10 +46,20 @@ typedef unsigned long long c_ulonglong;
 typedef float c_float;
 typedef double c_double;
 typedef void* c_void_ptr;
+typedef uintptr_t c_uintptr;
+typedef intptr_t c_intptr;
+typedef ptrdiff_t c_ptrdiff;
+
+// C++ does not support c99 bools
+#ifndef __cplusplus
+typedef _Bool chpl_bool;
+#else
+typedef bool chpl_bool;
+#endif
+
 #define c_nil NULL
-static inline c_int is_c_nil(void* x) { return x==NULL; }
+static inline chpl_bool is_c_nil(void* x) { return (chpl_bool)(x==c_nil); }
 static inline void* c_pointer_return(void* x) { return x; }
-typedef const char* c_string;
 
 typedef enum {
   CHPL_TYPE_chpl_bool,
@@ -142,7 +170,6 @@ typedef void* _chpl_value;
 typedef void* chpl_opaque;
 
 #define nilRef 0
-#define chpl_emptyCstring ""
 
 // macros for specifying the correct C literal type
 #define INT8( i)   ((int8_t)(INT8_C(i)))
@@ -155,13 +182,6 @@ typedef void* chpl_opaque;
 #define UINT64( i) ((uint64_t)(UINT64_C(i)))
 
 
-// C++ does not support c99 bools
-#ifndef __cplusplus
-typedef _Bool chpl_bool;
-#else
-typedef bool chpl_bool;
-#endif
-
 typedef int8_t chpl_bool8;
 typedef int16_t chpl_bool16;
 typedef int32_t chpl_bool32;
@@ -183,10 +203,11 @@ typedef float               _real32;
 typedef double              _real64;
 typedef float               _imag32;
 typedef double              _imag64;
-typedef struct __complex64 { _real32 re; _real32 im; } _complex64;
-typedef struct __complex128 { _real64 re; _real64 im; } _complex128;
-typedef const char*         chpl_string;
-typedef int64_t              _symbol;
+#ifndef __cplusplus
+typedef float complex       _complex64;
+typedef double complex      _complex128;
+#endif
+typedef int64_t             _symbol;
 
 // macros for Chapel min/max -> C stdint.h or values.h min/max
 #define MIN_INT8            INT8_MIN
@@ -212,40 +233,6 @@ typedef int64_t              _symbol;
 #define MAX_FLOAT32         FLT_MAX
 #define MAX_FLOAT64         DBL_MAX
 
-static ___always_inline
-int8_t ascii(c_string s) {
-  return (int8_t) *s;
-}
-
-struct chpl_chpl____wide_chpl_string_s;
-
-//
-// stopgap formatting
-//
-chpl_string chpl_format(chpl_string format, ...)
-  __attribute__((format(printf, 1, 2)));
-
-// Uses the system allocator.
-char* chpl_glom_strings(int numstrings, ...);
-
-chpl_bool string_contains(c_string x, c_string y);
-chpl_string string_copy(c_string x, int32_t lineno, c_string filename);
-chpl_string chpl_wide_string_copy(struct chpl_chpl____wide_chpl_string_s* x, int32_t lineno, chpl_string filename);
-void chpl_string_widen(struct chpl_chpl____wide_chpl_string_s* x, chpl_string from, int32_t lineno, chpl_string filename);
-void chpl_comm_wide_get_string(chpl_string* local, struct chpl_chpl____wide_chpl_string_s* x, int32_t tid, int32_t lineno, chpl_string filename);
-chpl_string string_concat(c_string x, c_string y, int32_t lineno, c_string filename);
-int string_index_of(c_string x, c_string y);
-chpl_string string_index(c_string x, int i, int32_t lineno, c_string filename);
-chpl_string string_select(c_string x, int low, int high, int stride, int32_t lineno, c_string filename);
-int32_t chpl_string_compare(c_string x, c_string y);
-int64_t string_length(c_string x);
-
-// implement string_from_c_string and c_string_from_string primitives.
-void string_from_c_string(chpl_string *ret, c_string str, int haslen, int64_t len, int32_t lineno, chpl_string filename);
-void wide_string_from_c_string(struct chpl_chpl____wide_chpl_string_s *ret, c_string str, int haslen, int64_t len, int32_t lineno, chpl_string filename);
-void c_string_from_string(c_string* ret, chpl_string* str, int32_t lineno, chpl_string filename);
-void c_string_from_wide_string(c_string* ret, struct chpl_chpl____wide_chpl_string_s* str, int32_t lineno, chpl_string filename);
-
 int64_t real2int( _real64 f);       // return the raw bytes of the float
 int64_t object2int( _chpl_object o);  // return the ptr
 
@@ -257,6 +244,53 @@ typedef struct chpl_main_argument_s {
   int32_t return_value;
 } chpl_main_argument;
 
+#ifndef __cplusplus
+_complex128 _chpl_complex128(_real64 re, _real64 im);
+_complex64 _chpl_complex64(_real32 re, _real32 im);
+
+_real64* complex128GetRealRef(_complex128* cplx);
+_real64* complex128GetImagRef(_complex128* cplx);
+_real32* complex64GetRealRef(_complex64* cplx);
+_real32* complex64GetImagRef(_complex64* cplx);
+
+/* 128 bit complex operators for LLVM use */
+static inline _complex128 complexMultiply128(_complex128 c1, _complex128 c2) {
+  return c1*c2;
+}
+static inline _complex128 complexDivide128(_complex128 c1, _complex128 c2) {
+  return c1/c2;
+}
+static inline _complex128 complexAdd128(_complex128 c1, _complex128 c2) {
+  return c1+c2;
+}
+static inline _complex128 complexSubtract128(_complex128 c1, _complex128 c2) {
+  return c1-c2;
+}
+static inline _complex128 complexUnaryMinus128(_complex128 c1) {
+  return -c1;
+}
+
+/* 64 bit complex operators for LLVM use */
+static inline _complex64 complexMultiply64(_complex64 c1, _complex64 c2) {
+  return c1*c2;
+}
+static inline _complex64 complexDivide64(_complex64 c1, _complex64 c2) {
+  return c1/c2;
+}
+static inline _complex64 complexAdd64(_complex64 c1, _complex64 c2) {
+  return c1+c2;
+}
+static inline _complex64 complexSubtract64(_complex64 c1, _complex64 c2) {
+  return c1-c2;
+}
+static inline _complex64 complexUnaryMinus64(_complex64 c1) {
+  return -c1;
+}
+#endif
+
+/* This should be moved somewhere else, but where is the question */
 const char* chpl_get_argument_i(chpl_main_argument* args, int32_t i);
 
+#include "chpl-string-support.h"
+
 #endif
