@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _chpl_locale_model_h_
 #define _chpl_locale_model_h_
@@ -27,7 +41,7 @@ typedef struct {
 // This is the external copy constructor for a chpl_localeID_t, specified
 // by the module code for a numa locale model.
 //
-static ___always_inline
+static inline
 chpl_localeID_t chpl__initCopy_chpl_rt_localeID_t(chpl_localeID_t initial) {
   return initial;
 }
@@ -36,18 +50,18 @@ chpl_localeID_t chpl__initCopy_chpl_rt_localeID_t(chpl_localeID_t initial) {
 // These functions are used by the module code to assemble and
 // disassemble global locale IDs.
 //
-static ___always_inline
+static inline
 chpl_localeID_t chpl_rt_buildLocaleID(c_nodeid_t node, c_sublocid_t subloc) {
   chpl_localeID_t loc = { node, subloc };
   return loc;
 }
 
-static ___always_inline
+static inline
 c_nodeid_t chpl_rt_nodeFromLocaleID(chpl_localeID_t loc) {
   return loc.node;
 }
 
-static ___always_inline
+static inline
 c_sublocid_t chpl_rt_sublocFromLocaleID(chpl_localeID_t loc) {
   return loc.subloc;
 }
