@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 /* A C implementation based on the C++ deque in
  * stl_deque from gnu gcc C++ STL implementation.
@@ -29,13 +43,15 @@ extern "C" {
 #include <string.h>
 #include <errno.h>
 
-#ifndef SIMPLE_TEST
+#ifndef CHPL_RT_UNIT_TEST
 #include "chpl-mem.h"
-#define deque_calloc(nmemb, size) chpl_mem_allocManyZero(nmemb, size, CHPL_RT_MD_IO_BUFFER, __LINE__, __FILE__)
-#define deque_free(ptr) chpl_mem_free(ptr, __LINE__, __FILE__)
+#define deque_calloc(nmemb, size) chpl_mem_allocManyZero(nmemb, size, CHPL_RT_MD_IO_BUFFER, 0, 0)
+#define deque_free(ptr) chpl_mem_free(ptr, 0, 0)
+#define deque_memcpy(dest, src, num) chpl_memcpy(dest, src, num)
 #else
 #define deque_calloc(nmemb, size) calloc(nmemb,size)
 #define deque_free(ptr) free(ptr)
+#define deque_memcpy(dest, src, num) memcpy(dest, src, num)
 #endif
 
 typedef struct deque_node_s {
@@ -129,7 +145,7 @@ void deque_it_forward_n(const ssize_t item_size, deque_iterator_t* it, ssize_t n
 static inline
 void deque_it_get_cur(const ssize_t item_size, const deque_iterator_t it, void* out)
 {
-  memcpy(out, it.cur, item_size);
+  deque_memcpy(out, it.cur, item_size);
 }
 
 static inline
@@ -141,7 +157,7 @@ void* deque_it_get_cur_ptr(const ssize_t item_size, const deque_iterator_t it)
 static inline
 void deque_it_set_cur(const ssize_t item_size, const deque_iterator_t it, void* in)
 {
-  memcpy(it.cur, in, item_size);
+  deque_memcpy(it.cur, in, item_size);
 }
 
 static inline
@@ -172,14 +188,14 @@ void _deque_destroy_nodes(deque_node_t* start, deque_node_t* finish)
 }
 
 static inline
-err_t _deque_create_nodes(const ssize_t item_size, deque_node_t* start, deque_node_t* finish)
+qioerr _deque_create_nodes(const ssize_t item_size, deque_node_t* start, deque_node_t* finish)
 {
   deque_node_t* cur;
   for( cur = start; cur < finish; ++cur ) {
     cur->data = deque_calloc( __deque_buf_size(item_size), item_size );
     if( ! cur->data ) {
       _deque_destroy_nodes(start, cur);
-      return ENOMEM;
+      return QIO_ENOMEM;
     }
   }
 
@@ -187,20 +203,20 @@ err_t _deque_create_nodes(const ssize_t item_size, deque_node_t* start, deque_no
 }
 
 static inline
-err_t _deque_initialize_map(const ssize_t item_size, deque_t* d, ssize_t num_elements)
+qioerr _deque_initialize_map(const ssize_t item_size, deque_t* d, ssize_t num_elements)
 {
   const ssize_t buf_size = __deque_buf_size(item_size);
   const ssize_t num_nodes = (num_elements / buf_size) + 1;
   deque_node_t* nstart;
   deque_node_t* nfinish;
-  err_t err;
+  qioerr err;
 
-  if( num_elements < 0 ) return EINVAL;
+  if( num_elements < 0 ) QIO_RETURN_CONSTANT_ERROR(EINVAL, "negative num_elements");
 
   d->map_size = _DEQUE_MAX(_DEQUE_INITIAL_MAP_SIZE, num_nodes + 2);
   d->map = (deque_node_t*) deque_calloc(d->map_size, sizeof(deque_node_t));
   if( ! d->map ) {
-    return ENOMEM;
+    return QIO_ENOMEM;
   }
 
   // For "small" maps (needing less than _M_map_size nodes), allocation
@@ -228,7 +244,7 @@ err_t _deque_initialize_map(const ssize_t item_size, deque_t* d, ssize_t num_ele
 }
 
 static inline
-err_t deque_init(const ssize_t item_size, deque_t* d, ssize_t num_elements)
+qioerr deque_init(const ssize_t item_size, deque_t* d, ssize_t num_elements)
 {
   return _deque_initialize_map(item_size, d, num_elements);
 }
@@ -285,10 +301,10 @@ ssize_t deque_size(const ssize_t item_size, deque_t* d)
 // These functions are in the C file.
 void _deque_map_copy_forward(deque_node_t* start, deque_node_t* end, deque_node_t* dst);
 void _deque_map_copy_backward(deque_node_t* start, deque_node_t* end, deque_node_t* dst_end);
-err_t _deque_reallocate_map(const ssize_t item_size, const ssize_t buf_size, deque_t* d, ssize_t nodes_to_add, char add_at_front);
+qioerr _deque_reallocate_map(const ssize_t item_size, const ssize_t buf_size, deque_t* d, ssize_t nodes_to_add, char add_at_front);
 
 static inline
-err_t _deque_reserve_map_at_back(const ssize_t item_size, deque_t* d, ssize_t nodes_to_add)
+qioerr _deque_reserve_map_at_back(const ssize_t item_size, deque_t* d, ssize_t nodes_to_add)
 {
   if( nodes_to_add + 1 > d->map_size - (d->finish.node - d->map) ) {
     return _deque_reallocate_map(item_size, __deque_buf_size(item_size), d, nodes_to_add, 0);
@@ -298,7 +314,7 @@ err_t _deque_reserve_map_at_back(const ssize_t item_size, deque_t* d, ssize_t no
 }
 
 static inline
-err_t _deque_reserve_map_at_front(const ssize_t item_size, deque_t* d, ssize_t nodes_to_add)
+qioerr _deque_reserve_map_at_front(const ssize_t item_size, deque_t* d, ssize_t nodes_to_add)
 {
   if( nodes_to_add > (d->start.node - d->map) ) {
     return _deque_reallocate_map(item_size, __deque_buf_size(item_size), d, nodes_to_add, 1);
@@ -309,23 +325,23 @@ err_t _deque_reserve_map_at_front(const ssize_t item_size, deque_t* d, ssize_t n
 
 // Called only if _M_impl._M_finish._M_cur == _M_impl._M_finish._M_last - 1.
 static inline
-err_t _deque_push_back_aux(const ssize_t item_size, deque_t* d, void* value)
+qioerr _deque_push_back_aux(const ssize_t item_size, deque_t* d, void* value)
 {
   void *newdata;
-  err_t err;
+  qioerr err;
 
   err = _deque_reserve_map_at_back(item_size, d, 1);
   if( err ) return err;
 
   newdata = deque_calloc( __deque_buf_size(item_size), item_size );
   if( !newdata ) {
-    return ENOMEM;
+    return QIO_ENOMEM;
   }
 
   (d->finish.node + 1)->data = newdata;
 
   //construct(d->finish.cur, v);
-  memcpy(d->finish.cur, value, item_size);
+  deque_memcpy(d->finish.cur, value, item_size);
 
   _deque_set_node(item_size, __deque_buf_size(item_size), & d->finish, d->finish.node + 1);
   d->finish.cur = d->finish.first;
@@ -335,17 +351,17 @@ err_t _deque_push_back_aux(const ssize_t item_size, deque_t* d, void* value)
 
 // Called only if _M_impl._M_start._M_cur == _M_impl._M_start._M_first.
 static inline
-err_t _deque_push_front_aux(const ssize_t item_size, deque_t* d, void* value)
+qioerr _deque_push_front_aux(const ssize_t item_size, deque_t* d, void* value)
 {
   void *newdata;
-  err_t err;
+  qioerr err;
 
   err = _deque_reserve_map_at_front(item_size, d, 1);
   if( err ) return err;
 
   newdata = deque_calloc( __deque_buf_size(item_size), item_size );
   if( !newdata ) {
-    return ENOMEM;
+    return QIO_ENOMEM;
   }
 
   (d->start.node - 1)->data = newdata;
@@ -355,7 +371,7 @@ err_t _deque_push_front_aux(const ssize_t item_size, deque_t* d, void* value)
   d->start.cur = PTR_ADDBYTES(d->start.last, -item_size);
 
   //construct(d->start.cur, v); // calls the constructor
-  memcpy(d->start.cur, value, item_size);
+  deque_memcpy(d->start.cur, value, item_size);
 
   return 0;
 }
@@ -392,11 +408,11 @@ void _deque_pop_front_aux(const ssize_t item_size, deque_t* d)
 }
 
 static inline
-err_t deque_push_front(const ssize_t item_size, deque_t* d, void* value)
+qioerr deque_push_front(const ssize_t item_size, deque_t* d, void* value)
 {
   if( d->start.cur != d->start.first ) {
     //construct(d->start.cur - item_size, value);
-    memcpy(PTR_ADDBYTES(d->start.cur, -item_size), value, item_size);
+    deque_memcpy(PTR_ADDBYTES(d->start.cur, -item_size), value, item_size);
 
     d->start.cur = PTR_ADDBYTES(d->start.cur, -item_size);
 
@@ -407,11 +423,11 @@ err_t deque_push_front(const ssize_t item_size, deque_t* d, void* value)
 }
 
 static inline
-err_t deque_push_back(const ssize_t item_size, deque_t* d, void* value)
+qioerr deque_push_back(const ssize_t item_size, deque_t* d, void* value)
 {
   if( d->finish.cur != PTR_ADDBYTES(d->finish.last, -item_size) ) {
     //construct(d->finish.cur, value);
-    memcpy(d->finish.cur, value, item_size);
+    deque_memcpy(d->finish.cur, value, item_size);
 
     d->finish.cur = PTR_ADDBYTES(d->finish.cur, item_size);
 
