@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _QBUFFER_H_
 #define _QBUFFER_H_
@@ -10,15 +24,29 @@
 extern "C" {
 #endif
 
+// This macro set to obtain the portable format macro PRIu64 for debug output.
+#define __STDC_FORMAT_MACROS 1
+// This macro set to obtain SIZE_MAX
+#define __STDC_LIMIT_MACROS 1
+
 #include "sys_basic.h"
+#include "qio_error.h"
 #ifndef __cplusplus
 #include <stdbool.h>
 #endif
 #include "chpl-atomics.h"
 
-// This macro set to obtain the portable format macro PRIu64 for debug output.
-#define __STDC_FORMAT_MACROS 1
 #include <inttypes.h>
+#include <stdint.h>
+
+// Last resort way to get SIZE_MAX. This should be correct,
+// but we'd rather use the system's definition... which should
+// theoretically be provided by the above (__STDC_LIMIT_MACROS+stdint.h)
+// but that isn't happening for me on GCC 4.7.2 when this file is included
+// by a C++ program.
+#ifndef SIZE_MAX
+#define SIZE_MAX (~((size_t)0))
+#endif
 
 #include <sys/uio.h>
 #include "deque.h"
@@ -124,17 +152,16 @@ void qbytes_free_null(qbytes_t* b);
 // unmap the data
 void qbytes_free_munmap(qbytes_t* b);
 // free the data
-void qbytes_free_sys_free(qbytes_t* b);
 void qbytes_free_qio_free(qbytes_t* b);
 
 void _qbytes_init_generic(qbytes_t* ret, void* give_data, int64_t len, qbytes_free_t free_function);
-err_t qbytes_create_generic(qbytes_t** out, void* give_data, int64_t len, qbytes_free_t free_function);
-err_t _qbytes_init_iobuf(qbytes_t* ret);
-err_t qbytes_create_iobuf(qbytes_t** out);
-err_t _qbytes_init_calloc(qbytes_t* ret, int64_t len);
+qioerr qbytes_create_generic(qbytes_t** out, void* give_data, int64_t len, qbytes_free_t free_function);
+qioerr _qbytes_init_iobuf(qbytes_t* ret);
+qioerr qbytes_create_iobuf(qbytes_t** out);
+qioerr _qbytes_init_calloc(qbytes_t* ret, int64_t len);
 
 // The caller is responsible for calling qbytes_release on the return value.
-err_t qbytes_create_calloc(qbytes_t** out, int64_t len);
+qioerr qbytes_create_calloc(qbytes_t** out, int64_t len);
 
 static inline
 int64_t qbytes_len(qbytes_t* b)
@@ -201,6 +228,7 @@ qbuffer_iter_t qbuffer_iter_null(void) {
 
 void debug_print_qbuffer_iter(qbuffer_iter_t* iter);
 void debug_print_qbuffer(qbuffer_t* buf);
+void debug_print_iovec(const struct iovec* iov, int iovcnt, size_t maxbytes);
 
 
 static inline
@@ -213,16 +241,16 @@ int qbuffer_is_initialized(qbuffer_t* buf) {
 }
 
 /* Initialize a buffer */
-err_t qbuffer_init(qbuffer_t* buf);
+qioerr qbuffer_init(qbuffer_t* buf);
 
 /* Destroy a buffer inited with qbuffer_init */
-err_t qbuffer_destroy(qbuffer_t* buf);
+qioerr qbuffer_destroy(qbuffer_t* buf);
 
 /* Destroy a buffer and free() the pointer */
-err_t qbuffer_destroy_free(qbuffer_t* buf);
+qioerr qbuffer_destroy_free(qbuffer_t* buf);
 
 /* Create a reference-counted buffer ptr */
-err_t qbuffer_create(qbuffer_ptr_t* out);
+qioerr qbuffer_create(qbuffer_ptr_t* out);
 
 /* Increment a reference count
  */
@@ -244,14 +272,14 @@ void qbuffer_extend_front(qbuffer_t* buf);
 
 /* Append a bytes_t to a buffer group.
  */
-err_t qbuffer_append(qbuffer_t* buf, qbytes_t* bytes, int64_t skip_bytes, int64_t len_bytes);
+qioerr qbuffer_append(qbuffer_t* buf, qbytes_t* bytes, int64_t skip_bytes, int64_t len_bytes);
 
 /* Append a buffer to another buffer; the bytes will be shared (reference counts increased) */
-err_t qbuffer_append_buffer(qbuffer_t* buf, qbuffer_t* src, qbuffer_iter_t src_start, qbuffer_iter_t src_end);
+qioerr qbuffer_append_buffer(qbuffer_t* buf, qbuffer_t* src, qbuffer_iter_t src_start, qbuffer_iter_t src_end);
 
 /* Prepend a bytes_t to a buffer group.
  */
-err_t qbuffer_prepend(qbuffer_t* buf, qbytes_t* bytes, int64_t skip_bytes, int64_t len_bytes);
+qioerr qbuffer_prepend(qbuffer_t* buf, qbytes_t* bytes, int64_t skip_bytes, int64_t len_bytes);
 
 /* trim functions remove parts that are completely in the area
  * to be removed. */
@@ -259,8 +287,8 @@ void qbuffer_trim_front(qbuffer_t* buf, int64_t remove_bytes);
 void qbuffer_trim_back(qbuffer_t* buf, int64_t remove_bytes);
 
 /* Remove a part from the front or back. */
-err_t qbuffer_pop_front(qbuffer_t* buf);
-err_t qbuffer_pop_back(qbuffer_t* buf);
+qioerr qbuffer_pop_front(qbuffer_t* buf);
+qioerr qbuffer_pop_back(qbuffer_t* buf);
 
 /* Without changing any memory, changes the offsets
  * used in the buffer (offset_start, offset_end, and offsets in the parts).
@@ -389,7 +417,7 @@ int64_t qbuffer_len(qbuffer_t* buf)
 /* Turn a range from a qbuffer into an io-vector. Note that this contains
  * pointers into the qbuffer and is only valid until the qbuffer is changed...
  */
-err_t qbuffer_to_iov(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, 
+qioerr qbuffer_to_iov(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, 
                    size_t max_iov, struct iovec *iov_out, 
                    qbytes_t** bytes_out /* can be NULL */,
                    size_t *iovcnt_out);
@@ -400,29 +428,29 @@ err_t qbuffer_to_iov(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end,
  * It is the responsibility of the caller to call
  * qbytes_release on the result of this function.
  */
-err_t qbuffer_flatten(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, qbytes_t** bytes_out);
+qioerr qbuffer_flatten(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, qbytes_t** bytes_out);
 
 /* Create a reference-sharing version of the buffer,
  * starting from current iterator position.
  */
-//err_t qbuffer_clone(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, qbuffer_ptr_t* buf_out);
+//qioerr qbuffer_clone(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, qbuffer_ptr_t* buf_out);
 
 /* Copies bytes from start to end in buffer to ptr.
  * Returns an error if we would exceed ret_len
  * */
-err_t qbuffer_copyout(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, void* ptr, size_t ret_len);
+qioerr qbuffer_copyout(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, void* ptr, size_t ret_len);
 
 /* Copies bytes from ptr to start to end in buffer.
  * Returns an error if we would exceed ret_len
  * */
-err_t qbuffer_copyin(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, const void* ptr, size_t ret_len);
+qioerr qbuffer_copyin(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, const void* ptr, size_t ret_len);
 
-err_t qbuffer_copyin_buffer(qbuffer_t* dst, qbuffer_iter_t dst_start, qbuffer_iter_t dst_end,
+qioerr qbuffer_copyin_buffer(qbuffer_t* dst, qbuffer_iter_t dst_start, qbuffer_iter_t dst_end,
                             qbuffer_t* src, qbuffer_iter_t src_start, qbuffer_iter_t src_end);
 
 /* Overwrites the qbuffer buffers with a fixed byte.
  * */
-err_t qbuffer_memset(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, unsigned char byte);
+qioerr qbuffer_memset(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, unsigned char byte);
 
 // How many bytes to try to store on stack in some functions that don't
 // really want to call malloc
@@ -431,10 +459,12 @@ err_t qbuffer_memset(qbuffer_t* buf, qbuffer_iter_t start, qbuffer_iter_t end, u
 #ifdef _chplrt_H_
 
 #include "chpl-mem.h"
-#define qio_malloc(size) chpl_mem_alloc(size, CHPL_RT_MD_IO_BUFFER, __LINE__, __FILE__)
-#define qio_calloc(nmemb, size) chpl_mem_allocManyZero(nmemb, size, CHPL_RT_MD_IO_BUFFER, __LINE__, __FILE__)
-#define qio_realloc(ptr, size) chpl_mem_realloc(ptr, size, CHPL_RT_MD_IO_BUFFER, __LINE__, __FILE__)
-#define qio_free(ptr) chpl_mem_free(ptr, __LINE__, __FILE__)
+#define qio_malloc(size) chpl_mem_alloc(size, CHPL_RT_MD_IO_BUFFER, 0, 0)
+#define qio_calloc(nmemb, size) chpl_mem_allocManyZero(nmemb, size, CHPL_RT_MD_IO_BUFFER, 0, 0)
+#define qio_realloc(ptr, size) chpl_mem_realloc(ptr, size, CHPL_RT_MD_IO_BUFFER, 0, 0)
+#define qio_valloc(size) chpl_valloc(size)
+#define qio_free(ptr) chpl_mem_free(ptr, 0, 0)
+#define qio_memcpy(dest, src, num) chpl_memcpy(dest, src, num)
 
 static inline char* qio_strdup(const char* ptr)
 {
@@ -450,26 +480,35 @@ typedef chpl_bool qio_bool;
 #define qio_malloc(size) malloc(size)
 #define qio_calloc(nmemb, size) calloc(nmemb,size)
 #define qio_realloc(ptr, size) realloc(ptr, size)
+#define qio_valloc(size) valloc(size)
 #define qio_free(ptr) free(ptr)
-#define sys_free(ptr) free(ptr)
 #define qio_strdup(ptr) strdup(ptr)
+#define qio_memcpy(dest, src, num) memcpy(dest, src, num)
 
 typedef bool qio_bool;
 
 #endif
 
-// Declare MAX_ON_STACK bytes. We declare it as uint64_t to
-// make sure it's aligned as well as malloc would be.
+// Declare MAX_ON_STACK bytes. We declare it with the original
+// type to make sure it's aligned as well as malloc would be.
 #define MAYBE_STACK_SPACE(type,onstack) \
   type onstack[MAX_ON_STACK/sizeof(type)]
 
 #define MAYBE_STACK_ALLOC(type, count, ptr, onstack) \
 { \
-  size_t size = count * sizeof(type); \
-  if( size <= sizeof(onstack) ) { \
-    ptr = onstack; \
+  /* check for integer overflow or negative count */ \
+  if( count >= 0 && \
+      (uint64_t) count <= (SIZE_MAX / sizeof(type)) ) { \
+    /* check that count is positive and small enough to go on the stack */ \
+    if( (size_t) count <= (sizeof(onstack)/sizeof(type)) ) { \
+      ptr = onstack; \
+    } else { \
+      ptr = (type*) qio_malloc(count*sizeof(type)); \
+    } \
   } else { \
-    ptr = (type*) qio_malloc(size); \
+    /* handle integer overflow */ \
+    ptr = NULL; \
+    assert(0 && "size overflow in MAYBE_STACK_ALLOC"); \
   } \
 }
 
@@ -481,9 +520,38 @@ typedef bool qio_bool;
   } \
 }
 
-#define VOID_PTR_DIFF(a,b) (((intptr_t) (a)) - ((intptr_t) (b)))
-#define VOID_PTR_ADD(ptr,amt) ((void*)(((char*) (ptr)) + (amt)))
-#define VOID_PTR_ALIGN(ptr,align) (((uintptr_t)ptr) & (align - 1))
+/* Returns the difference between two pointers,
+   but returns 0 if either pointer is NULL.
+ */
+static inline intptr_t qio_ptr_diff(void* a, void* b)
+{
+  if( a == NULL || b == NULL ) return 0;
+  return ((intptr_t)a) - ((intptr_t)b);
+}
+
+/* Returns 1 if there is space for nbytes between cur and end
+   (cur might represent a current buffer position and end might
+    represent the boundary.)
+   Takes into account the possibility that cur or end might be NULL;
+   returns 0 if either is NULL and nbytes > 0.
+    */
+static inline int qio_space_in_ptr_diff(intptr_t nbytes, void* end, void* cur)
+{
+  return nbytes <= qio_ptr_diff(end,cur);
+}
+
+static inline void* qio_ptr_add(void* ptr, intptr_t amt)
+{
+  return ((void*)(((char*) (ptr)) + (amt)));
+}
+
+/* Returns the number of bytes between ptr and its alignment.
+   align must be a power of 2.
+ */
+static inline uintptr_t qio_ptr_align(void* ptr, uintptr_t align)
+{
+ return (((uintptr_t)ptr) & (align - 1));
+}
 
 #ifdef __cplusplus
 } // end extern "C"
