@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _QIO_FORMATTED_H_
 #define _QIO_FORMATTED_H_
@@ -10,6 +24,7 @@
 #include "qio.h"
 
 #include "bswap.h"
+#include "error.h"
 
 // true 1 false 0   __bool_true_false_are_defined
 #include <stdbool.h>
@@ -34,31 +49,31 @@ void qio_set_glocale(void);
 
 // Read/Write methods for Binary I/O
 
-static ___always_inline
-err_t qio_channel_read_int8(const int threadsafe, qio_channel_t* restrict ch, int8_t* restrict ptr) {
+static inline
+qioerr qio_channel_read_int8(const int threadsafe, qio_channel_t* restrict ch, int8_t* restrict ptr) {
   return qio_channel_read_amt(threadsafe, ch, ptr, 1);
 }
 
-static ___always_inline
-err_t qio_channel_write_int8(const int threadsafe, qio_channel_t* restrict ch, int8_t x) {
+static inline
+qioerr qio_channel_write_int8(const int threadsafe, qio_channel_t* restrict ch, int8_t x) {
   return qio_channel_write_amt(threadsafe, ch, &x, 1);
 }
 
-static ___always_inline
-err_t qio_channel_read_uint8(const int threadsafe, qio_channel_t* restrict ch, uint8_t* restrict ptr) {
+static inline
+qioerr qio_channel_read_uint8(const int threadsafe, qio_channel_t* restrict ch, uint8_t* restrict ptr) {
   return qio_channel_read_amt(threadsafe, ch, ptr, 1);
 }
 
-static ___always_inline
-err_t qio_channel_write_uint8(const int threadsafe, qio_channel_t* restrict ch, uint8_t x) {
+static inline
+qioerr qio_channel_write_uint8(const int threadsafe, qio_channel_t* restrict ch, uint8_t x) {
   return qio_channel_write_amt(threadsafe, ch, &x, 1);
 }
 
 
 
-static ___always_inline
-err_t qio_channel_read_int16(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int16_t* restrict ptr) {
-  err_t err;
+static inline
+qioerr qio_channel_read_int16(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int16_t* restrict ptr) {
+  qioerr err;
   int16_t x;
   err = qio_channel_read_amt(threadsafe, ch, &x, 2);
   if( err ) return err;
@@ -68,16 +83,16 @@ err_t qio_channel_read_int16(const int threadsafe, const int byteorder, qio_chan
   return 0;
 }
 
-static ___always_inline
-err_t qio_channel_write_int16(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int16_t x) {
+static inline
+qioerr qio_channel_write_int16(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int16_t x) {
   if( byteorder == QIO_BIG ) x = htobe16(x);
   if( byteorder == QIO_LITTLE ) x = htole16(x);
   return qio_channel_write_amt(threadsafe, ch, &x, 2);
 }
 
-static ___always_inline
-err_t qio_channel_read_uint16(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint16_t* restrict ptr) {
-  err_t err;
+static inline
+qioerr qio_channel_read_uint16(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint16_t* restrict ptr) {
+  qioerr err;
   uint16_t x;
   err = qio_channel_read_amt(threadsafe, ch, &x, 2);
   if( err ) return err;
@@ -87,17 +102,17 @@ err_t qio_channel_read_uint16(const int threadsafe, const int byteorder, qio_cha
   return 0;
 }
 
-static ___always_inline
-err_t qio_channel_write_uint16(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint16_t x) {
+static inline
+qioerr qio_channel_write_uint16(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint16_t x) {
   if( byteorder == QIO_BIG ) x = htobe16(x);
   if( byteorder == QIO_LITTLE ) x = htole16(x);
   return qio_channel_write_amt(threadsafe, ch, &x, 2);
 }
 
 
-static ___always_inline
-err_t qio_channel_read_int32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int32_t* restrict ptr) {
-  err_t err;
+static inline
+qioerr qio_channel_read_int32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int32_t* restrict ptr) {
+  qioerr err;
   int32_t x;
   err = qio_channel_read_amt(threadsafe, ch, &x, 4);
   if( err ) return err;
@@ -107,16 +122,16 @@ err_t qio_channel_read_int32(const int threadsafe, const int byteorder, qio_chan
   return 0;
 }
 
-static ___always_inline
-err_t qio_channel_write_int32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int32_t x) {
+static inline
+qioerr qio_channel_write_int32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int32_t x) {
   if( byteorder == QIO_BIG ) x = htobe32(x);
   if( byteorder == QIO_LITTLE ) x = htole32(x);
   return qio_channel_write_amt(threadsafe, ch, &x, 4);
 }
 
-static ___always_inline
-err_t qio_channel_read_uint32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint32_t* restrict ptr) {
-  err_t err;
+static inline
+qioerr qio_channel_read_uint32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint32_t* restrict ptr) {
+  qioerr err;
   uint32_t x;
   err = qio_channel_read_amt(threadsafe, ch, &x, 4);
   if( err ) return err;
@@ -126,17 +141,17 @@ err_t qio_channel_read_uint32(const int threadsafe, const int byteorder, qio_cha
   return 0;
 }
 
-static ___always_inline
-err_t qio_channel_write_uint32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint32_t x) {
+static inline
+qioerr qio_channel_write_uint32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint32_t x) {
   if( byteorder == QIO_BIG ) x = htobe32(x);
   if( byteorder == QIO_LITTLE ) x = htole32(x);
   return qio_channel_write_amt(threadsafe, ch, &x, 4);
 }
 
-static ___always_inline
-err_t qio_channel_read_int64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int64_t* restrict ptr) {
+static inline
+qioerr qio_channel_read_int64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int64_t* restrict ptr) {
   int64_t x;
-  err_t err;
+  qioerr err;
   err = qio_channel_read_amt(threadsafe, ch, &x, 8);
   if( err ) return err;
   if( byteorder == QIO_BIG ) x = be64toh(x);
@@ -145,17 +160,17 @@ err_t qio_channel_read_int64(const int threadsafe, const int byteorder, qio_chan
   return 0;
 }
 
-static ___always_inline
-err_t qio_channel_write_int64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int64_t x) {
+static inline
+qioerr qio_channel_write_int64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, int64_t x) {
   if( byteorder == QIO_BIG ) x = htobe64(x);
   if( byteorder == QIO_LITTLE ) x = htole64(x);
   return qio_channel_write_amt(threadsafe, ch, &x, 8);
 }
 
-static ___always_inline
-err_t qio_channel_read_uint64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint64_t* restrict ptr) {
+static inline
+qioerr qio_channel_read_uint64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint64_t* restrict ptr) {
   uint64_t x;
-  err_t err;
+  qioerr err;
   err = qio_channel_read_amt(threadsafe, ch, &x, 8);
   if( err ) return err;
   if( byteorder == QIO_BIG ) x = be64toh(x);
@@ -164,22 +179,22 @@ err_t qio_channel_read_uint64(const int threadsafe, const int byteorder, qio_cha
   return 0;
 }
 
-static ___always_inline
-err_t qio_channel_write_uint64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint64_t x) {
+static inline
+qioerr qio_channel_write_uint64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, uint64_t x) {
   if( byteorder == QIO_BIG ) x = htobe64(x);
   if( byteorder == QIO_LITTLE ) x = htole64(x);
   return qio_channel_write_amt(threadsafe, ch, &x, 8);
 }
 
 // Reading/writing varints in the same format as Google Protocol Buffers.
-err_t qio_channel_read_uvarint(const int threadsafe, qio_channel_t* restrict ch, uint64_t* restrict ptr);
-err_t qio_channel_read_svarint(const int threadsafe, qio_channel_t* restrict ch, int64_t* restrict ptr);
-err_t qio_channel_write_uvarint(const int threadsafe, qio_channel_t* restrict ch, uint64_t num);
-err_t qio_channel_write_svarint(const int threadsafe, qio_channel_t* restrict ch, int64_t num);
+qioerr qio_channel_read_uvarint(const int threadsafe, qio_channel_t* restrict ch, uint64_t* restrict ptr);
+qioerr qio_channel_read_svarint(const int threadsafe, qio_channel_t* restrict ch, int64_t* restrict ptr);
+qioerr qio_channel_write_uvarint(const int threadsafe, qio_channel_t* restrict ch, uint64_t num);
+qioerr qio_channel_write_svarint(const int threadsafe, qio_channel_t* restrict ch, int64_t num);
 
 
-static ___always_inline
-err_t qio_channel_read_int(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, void* restrict ptr, size_t len, int issigned) {
+static inline
+qioerr qio_channel_read_int(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, void* restrict ptr, size_t len, int issigned) {
 
   ssize_t signed_len = len;
   if( issigned ) signed_len = - signed_len;
@@ -202,12 +217,12 @@ err_t qio_channel_read_int(const int threadsafe, const int byteorder, qio_channe
     case -8:
       return qio_channel_read_uint64(threadsafe, byteorder, ch,(uint64_t*)ptr);
     default:
-      return EINVAL;
+      QIO_RETURN_CONSTANT_ERROR(EINVAL, "bad integer size");
   }
 }
 
-static ___always_inline
-err_t qio_channel_write_int(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, const void* restrict ptr, size_t len, int issigned) {
+static inline
+qioerr qio_channel_write_int(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, const void* restrict ptr, size_t len, int issigned) {
 
   ssize_t signed_len = len;
   if( issigned ) signed_len = - signed_len;
@@ -230,17 +245,17 @@ err_t qio_channel_write_int(const int threadsafe, const int byteorder, qio_chann
     case -8:
       return qio_channel_write_int64(threadsafe, byteorder, ch, *(const int64_t*) ptr);
     default:
-      return EINVAL;
+      QIO_RETURN_CONSTANT_ERROR(EINVAL, "bad integer size");
   }
 }
 
-static ___always_inline
-err_t qio_channel_read_float32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, float* restrict ptr) {
+static inline
+qioerr qio_channel_read_float32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, float* restrict ptr) {
   union {
    uint32_t i;
    float f;
   } x;
-  err_t err;
+  qioerr err;
 
   err = qio_channel_read_amt(threadsafe, ch, &x, 4);
   if( err ) return err;
@@ -250,8 +265,8 @@ err_t qio_channel_read_float32(const int threadsafe, const int byteorder, qio_ch
   return 0;
 }
 
-static ___always_inline
-err_t qio_channel_write_float32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, float x) {
+static inline
+qioerr qio_channel_write_float32(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, float x) {
   union {
    uint32_t i;
    float f;
@@ -265,13 +280,13 @@ err_t qio_channel_write_float32(const int threadsafe, const int byteorder, qio_c
 }
 
 
-static ___always_inline
-err_t qio_channel_read_float64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, double* restrict ptr) {
+static inline
+qioerr qio_channel_read_float64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, double* restrict ptr) {
   union {
    uint64_t i;
    double f;
   } x;
-  err_t err;
+  qioerr err;
 
   err = qio_channel_read_amt(threadsafe, ch, &x, 8);
   if( err ) return err;
@@ -282,8 +297,8 @@ err_t qio_channel_read_float64(const int threadsafe, const int byteorder, qio_ch
 }
 
 
-static ___always_inline
-err_t qio_channel_write_float64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, double x) {
+static inline
+qioerr qio_channel_write_float64(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, double x) {
   union {
    uint64_t i;
    double f;
@@ -296,33 +311,33 @@ err_t qio_channel_write_float64(const int threadsafe, const int byteorder, qio_c
   return qio_channel_write_amt(threadsafe, ch, &u, 8);
 }
 
-static ___always_inline
-err_t qio_channel_read_float(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, void* restrict ptr, size_t len) {
+static inline
+qioerr qio_channel_read_float(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, void* restrict ptr, size_t len) {
   switch ( len ) {
     case 4:
       return qio_channel_read_float32(threadsafe, byteorder, ch, (float*) ptr);
     case 8:
       return qio_channel_read_float64(threadsafe, byteorder, ch, (double*) ptr);
     default:
-      return EINVAL;
+      QIO_RETURN_CONSTANT_ERROR(EINVAL, "bad float size");
   }
 }
 
-static ___always_inline
-err_t qio_channel_write_float(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, const void* restrict ptr, size_t len) {
+static inline
+qioerr qio_channel_write_float(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, const void* restrict ptr, size_t len) {
   switch ( len ) {
     case 4:
       return qio_channel_write_float32(threadsafe, byteorder, ch, *(const float*) ptr);
     case 8:
       return qio_channel_write_float64(threadsafe, byteorder, ch, *(const double*) ptr);
     default:
-      return EINVAL;
+      QIO_RETURN_CONSTANT_ERROR(EINVAL, "bad float size");
   }
 }
 
-static ___always_inline
-err_t qio_channel_read_complex(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, void* restrict re_ptr, void* restrict im_ptr, size_t len) {
-  err_t err;
+static inline
+qioerr qio_channel_read_complex(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, void* restrict re_ptr, void* restrict im_ptr, size_t len) {
+  qioerr err;
   switch ( len ) {
     case 4:
       err = qio_channel_read_float32(threadsafe, byteorder, ch, (float*) re_ptr);
@@ -337,14 +352,14 @@ err_t qio_channel_read_complex(const int threadsafe, const int byteorder, qio_ch
       }
       break;
     default:
-      err = EINVAL;
+      QIO_GET_CONSTANT_ERROR(err, EINVAL, "bad complex size");
   }
   return err;
 }
 
-static ___always_inline
-err_t qio_channel_write_complex(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, const void* restrict re_ptr, const void* restrict im_ptr, size_t len) {
-  err_t err;
+static inline
+qioerr qio_channel_write_complex(const int threadsafe, const int byteorder, qio_channel_t* restrict ch, const void* restrict re_ptr, const void* restrict im_ptr, size_t len) {
+  qioerr err;
   switch ( len ) {
     case 4:
       err = qio_channel_write_float32(threadsafe, byteorder, ch, *(const float*) re_ptr);
@@ -359,54 +374,66 @@ err_t qio_channel_write_complex(const int threadsafe, const int byteorder, qio_c
       }
       break;
     default:
-      err = EINVAL;
+      QIO_GET_CONSTANT_ERROR(err, EINVAL, "bad complex size");
   }
   return err;
 }
 
-// string binary style:
-// -1 -- 1 byte of length before
-// -2 -- 2 bytes of length before
-// -4 -- 4 bytes of length before
-// -8 -- 8 bytes of length before
-// -10 -- variable byte length before (hi-bit 1 means more, little endian)
-// -0x01XX -- read until terminator XX is read
-//  + -- nonzero positive -- read exactly this length.
-err_t qio_channel_read_string(const int threadsafe, const int byteorder, const int64_t str_style, qio_channel_t* restrict ch, const char* restrict * restrict out, int64_t* restrict len_out, ssize_t maxlen);
+#define QIO_BINARY_STRING_STYLE_LEN1B_DATA -1
+#define QIO_BINARY_STRING_STYLE_LEN2B_DATA -2
+#define QIO_BINARY_STRING_STYLE_LEN4B_DATA -4
+#define QIO_BINARY_STRING_STYLE_LEN8B_DATA -8
+#define QIO_BINARY_STRING_STYLE_LENvB_DATA -10
+#define QIO_BINARY_STRING_STYLE_DATA_NULL -0x0100
+#define QIO_BINARY_STRING_STYLE_TOEOF -0xff00
 
 // string binary style:
-// -1 -- 1 byte of length before
-// -2 -- 2 bytes of length before
-// -4 -- 4 bytes of length before
-// -8 -- 8 bytes of length before
-// -10 -- variable byte length before (hi-bit 1 means more, little endian)
-// -0x01XX -- read until terminator XX is read
+// QIO_BINARY_STRING_STYLE_LEN1B_DATA -1 -- 1 byte of length before
+// QIO_BINARY_STRING_STYLE_LEN2B_DATA -2 -- 2 bytes of length before
+// QIO_BINARY_STRING_STYLE_LEN4B_DATA -4 -- 4 bytes of length before
+// QIO_BINARY_STRING_STYLE_LEN8B_DATA -8 -- 8 bytes of length before
+// QIO_BINARY_STRING_STYLE_LENvB_DATA -10 -- variable byte length before
+//                                        (hi-bit 1 means more, little endian)
+// QIO_BINARY_STRING_STYLE_TOEOF -0xff00 -- read until end or up to maxlen
+// BINARY_STRING_STYLE_DATA_NULL|0xXX -0x01XX -- read until terminator XX
 //  + -- nonzero positive -- read exactly this length.
-err_t qio_channel_write_string(const int threadsafe, const int byteorder, const int64_t str_style, qio_channel_t* restrict ch, const char* restrict ptr, ssize_t len);
+qioerr qio_channel_read_string(const int threadsafe, const int byteorder, const int64_t str_style, qio_channel_t* restrict ch, const char* restrict * restrict out, int64_t* restrict len_out, ssize_t maxlen_bytes);
+
+// string binary style:
+// QIO_BINARY_STRING_STYLE_LEN1B_DATA -1 -- 1 byte of length before
+// QIO_BINARY_STRING_STYLE_LEN2B_DATA -2 -- 2 bytes of length before
+// QIO_BINARY_STRING_STYLE_LEN4B_DATA -4 -- 4 bytes of length before
+// QIO_BINARY_STRING_STYLE_LEN8B_DATA -8 -- 8 bytes of length before
+// QIO_BINARY_STRING_STYLE_LENvB_DATA -10 -- variable byte length before
+//                                        (hi-bit 1 means more, little endian)
+// QIO_BINARY_STRING_STYLE_TOEOF -0xff00 -- read until end or up to maxlen
+// BINARY_STRING_STYLE_DATA_NULL|0xXX -0x01XX -- read until terminator XX
+//  + -- nonzero positive -- read exactly this length.
+qioerr qio_channel_write_string(const int threadsafe, const int byteorder, const int64_t str_style, qio_channel_t* restrict ch, const char* restrict ptr, ssize_t len);
 
 
-err_t qio_channel_scan_bool(const int threadsafe, qio_channel_t* restrict ch, uint8_t* restrict out);
-err_t qio_channel_print_bool(const int threadsafe, qio_channel_t* restrict ch, uint8_t num);
+qioerr qio_channel_scan_bool(const int threadsafe, qio_channel_t* restrict ch, uint8_t* restrict out);
+qioerr qio_channel_print_bool(const int threadsafe, qio_channel_t* restrict ch, uint8_t num);
 
-err_t qio_channel_scan_int(const int threadsafe, qio_channel_t* restrict ch, void* restrict out, size_t len, int issigned);
-err_t qio_channel_scan_float(const int threadsafe, qio_channel_t* restrict ch, void* restrict out, size_t len);
-err_t qio_channel_scan_imag(const int threadsafe, qio_channel_t* restrict ch, void* restrict out, size_t len);
-err_t qio_channel_print_int(const int threadsafe, qio_channel_t* restrict ch, const void* restrict ptr, size_t len, int issigned);
-err_t qio_channel_print_float(const int threadsafe, qio_channel_t* restrict ch, const void* restrict ptr, size_t len);
-err_t qio_channel_print_imag(const int threadsafe, qio_channel_t* restrict ch, const void* restrict ptr, size_t len);
+qioerr qio_channel_scan_int(const int threadsafe, qio_channel_t* restrict ch, void* restrict out, size_t len, int issigned);
+qioerr qio_channel_scan_float(const int threadsafe, qio_channel_t* restrict ch, void* restrict out, size_t len);
+qioerr qio_channel_scan_imag(const int threadsafe, qio_channel_t* restrict ch, void* restrict out, size_t len);
+qioerr qio_channel_print_int(const int threadsafe, qio_channel_t* restrict ch, const void* restrict ptr, size_t len, int issigned);
+qioerr qio_channel_print_float(const int threadsafe, qio_channel_t* restrict ch, const void* restrict ptr, size_t len);
+qioerr qio_channel_print_imag(const int threadsafe, qio_channel_t* restrict ch, const void* restrict ptr, size_t len);
 
-err_t qio_channel_scan_complex(const int threadsafe, qio_channel_t* restrict ch, void* restrict re_out, void* restrict im_out, size_t len);
-err_t qio_channel_print_complex(const int threadsafe, qio_channel_t* restrict ch, const void* restrict re_ptr, const void* im_ptr, size_t len);
+qioerr qio_channel_scan_complex(const int threadsafe, qio_channel_t* restrict ch, void* restrict re_out, void* restrict im_out, size_t len);
+qioerr qio_channel_print_complex(const int threadsafe, qio_channel_t* restrict ch, const void* restrict re_ptr, const void* im_ptr, size_t len);
 
 // These methods read or write UTF-8 characters (code points).
 
 #include "utf8-decoder.h"
 
-err_t _qio_channel_read_char_slow_unlocked(qio_channel_t* restrict ch, int32_t* restrict chr);
+qioerr _qio_channel_read_char_slow_unlocked(qio_channel_t* restrict ch, int32_t* restrict chr);
 
 static inline
-err_t qio_channel_read_char(const int threadsafe, qio_channel_t* restrict ch, int32_t* restrict chr) {
-  err_t err;
+qioerr qio_channel_read_char(const int threadsafe, qio_channel_t* restrict ch, int32_t* restrict chr) {
+  qioerr err;
   uint32_t codepoint=0, state;
   
   if( qio_glocale_utf8 == 0 ) {
@@ -423,14 +450,14 @@ err_t qio_channel_read_char(const int threadsafe, qio_channel_t* restrict ch, in
   // Fast path: an entire multi-byte sequence
   // is stored in the buffers.
   if( qio_glocale_utf8 > 0 &&
-      4 <= VOID_PTR_DIFF(ch->cached_end, ch->cached_cur) ) {
+      qio_space_in_ptr_diff(4, ch->cached_end, ch->cached_cur) ) {
     if( qio_glocale_utf8 == QIO_GLOCALE_UTF8 ) {
       state = 0;
       while( 1 ) {
         qio_utf8_decode(&state,
                         &codepoint,
                         *(unsigned char*)ch->cached_cur);
-        ch->cached_cur = VOID_PTR_ADD(ch->cached_cur,1);
+        ch->cached_cur = qio_ptr_add(ch->cached_cur,1);
         if (state <= 1) {
           break;
         }
@@ -439,12 +466,12 @@ err_t qio_channel_read_char(const int threadsafe, qio_channel_t* restrict ch, in
         *chr = codepoint;
       } else {
         *chr = 0xfffd; // replacement character
-        err = EILSEQ;
+        QIO_GET_CONSTANT_ERROR(err, EILSEQ, "");
       }
     } else if( qio_glocale_utf8 == QIO_GLOCALE_ASCII ) {
       // character == byte.
       *chr = *(unsigned char*)ch->cached_cur;
-      ch->cached_cur = VOID_PTR_ADD(ch->cached_cur,1);
+      ch->cached_cur = qio_ptr_add(ch->cached_cur,1);
       err = 0;
     }
   } else {
@@ -505,13 +532,13 @@ int qio_nbytes_char(int32_t chr)
 // dst is a pointer to a buffer containing room for the encoded characters
 // (use qio_nbytes_char to find out the size).
 static inline
-err_t qio_encode_char_buf(char* dst, int32_t chr)
+qioerr qio_encode_char_buf(char* dst, int32_t chr)
 {
-  err_t err = 0;
+  qioerr err = 0;
   if( qio_glocale_utf8 > 0 ) {
     if( qio_glocale_utf8 == QIO_GLOCALE_UTF8 ) {
       if( chr < 0 ) {
-        err = EILSEQ;
+        QIO_GET_CONSTANT_ERROR(err, EILSEQ, "");
       } else if( chr < 0x80 ) {
         // OK, we got a 1-byte character; case #1
         *(unsigned char*)dst = (unsigned char) chr;
@@ -541,20 +568,21 @@ err_t qio_encode_char_buf(char* dst, int32_t chr)
     memset(&ps, 0, sizeof(mbstate_t));
     got = wcrtomb(dst, chr, &ps);
     if( got == (size_t) -1 ) {
-      err = EILSEQ;
+      QIO_GET_CONSTANT_ERROR(err, EILSEQ, "");
     }
 #else
-    err = ENOSYS;
+    QIO_GET_CONSTANT_ERROR(err, ENOSYS, "missing wctype.h");
 #endif
   }
   return err;
 }
 
-// Returns NULL if it's an illegal character OR we're out of memory.
-const char* qio_encode_to_string(int32_t chr);
+// Returns NULL if it's an illegal character OR we're out of memory; otherwise,
+// a newly-allocated ASCIIZ string containing the result.
+c_string_copy qio_encode_to_string(int32_t chr);
 
 static inline
-err_t qio_decode_char_buf(int32_t* restrict chr, int* restrict nbytes, const char* buf, ssize_t buflen)
+qioerr qio_decode_char_buf(int32_t* restrict chr, int* restrict nbytes, const char* buf, ssize_t buflen)
 {
   const char* start = buf;
   const char* end = start + buflen;
@@ -576,12 +604,12 @@ err_t qio_decode_char_buf(int32_t* restrict chr, int* restrict nbytes, const cha
       }
       if( state == UTF8_ACCEPT ) {
         *chr = codepoint;
-        *nbytes = VOID_PTR_DIFF(buf, start);
+        *nbytes = qio_ptr_diff((void*) buf, (void*) start);
         return 0;
       } else {
         *chr = 0xfffd; // replacement character
-        *nbytes = VOID_PTR_DIFF(buf, start);
-        return EILSEQ;
+        *nbytes = qio_ptr_diff((void*) buf, (void*) start);
+        QIO_RETURN_CONSTANT_ERROR(EILSEQ, "");
       }
     } else if( qio_glocale_utf8 == QIO_GLOCALE_ASCII ) {
       // character == byte.
@@ -592,7 +620,7 @@ err_t qio_decode_char_buf(int32_t* restrict chr, int* restrict nbytes, const cha
       } else {
         *chr = -1;
         *nbytes = 0;
-        return EILSEQ;
+        QIO_RETURN_CONSTANT_ERROR(EILSEQ, "");
       }
     }
   } else {
@@ -612,12 +640,12 @@ err_t qio_decode_char_buf(int32_t* restrict chr, int* restrict nbytes, const cha
       // errno should be EILSEQ.
       *chr = -3; // invalid character... think 0xfffd for unicode
       *nbytes = 1;
-      return EILSEQ;
+      QIO_RETURN_CONSTANT_ERROR(EILSEQ, "");
     } else if( got == (size_t) -2 ) {
       // continue as long as we have an incomplete char.
       *chr = -3; // invalid character... think 0xfffd for unicode
       *nbytes = 1;
-      return EILSEQ;
+      QIO_RETURN_CONSTANT_ERROR(EILSEQ, "");
     } else {
       // OK!
       // mbrtowc already set the character.
@@ -626,20 +654,21 @@ err_t qio_decode_char_buf(int32_t* restrict chr, int* restrict nbytes, const cha
       return 0;
     }
 #else
-    return ENOSYS;
+    QIO_GET_CONSTANT_ERROR(err, ENOSYS, "missing wctype.h");
 #endif
   }
-  assert(0);
-  return EILSEQ; // this should never be reached.
+  *chr = 0; // Makes GCC stop complaining about *chr not being set in other locations
+  chpl_internal_error("qio_decode_char_buf");
+  QIO_RETURN_CONSTANT_ERROR(EILSEQ, ""); // this should never be reached.
 }
 
 
-err_t _qio_channel_write_char_slow_unlocked(qio_channel_t* restrict ch, int32_t chr);
+qioerr _qio_channel_write_char_slow_unlocked(qio_channel_t* restrict ch, int32_t chr);
 
 static inline
-err_t qio_channel_write_char(const int threadsafe, qio_channel_t* restrict ch, int32_t chr)
+qioerr qio_channel_write_char(const int threadsafe, qio_channel_t* restrict ch, int32_t chr)
 {
-  err_t err;
+  qioerr err;
 
   if( qio_glocale_utf8 == 0 ) {
     qio_set_glocale();
@@ -653,36 +682,36 @@ err_t qio_channel_write_char(const int threadsafe, qio_channel_t* restrict ch, i
   err = 0;
 
   if( qio_glocale_utf8 > 0 &&
-      4 <= VOID_PTR_DIFF(ch->cached_end, ch->cached_cur) ) {
+      qio_space_in_ptr_diff(4, ch->cached_end, ch->cached_cur) ) {
     if( qio_glocale_utf8 == QIO_GLOCALE_UTF8 ) {
       if( chr < 0 ) {
-        err = EILSEQ;
+        QIO_GET_CONSTANT_ERROR(err, EILSEQ, "");
       } else if( chr < 0x80 ) {
         // OK, we got a 1-byte character; case #1
         *(unsigned char*)ch->cached_cur = (unsigned char) chr;
-        ch->cached_cur = VOID_PTR_ADD(ch->cached_cur,1);
+        ch->cached_cur = qio_ptr_add(ch->cached_cur,1);
       } else if( chr < 0x800 ) {
         // OK, we got a fits-in-2-bytes character; case #2
         *(unsigned char*)ch->cached_cur = (0xc0 | (chr >> 6));
         *(((unsigned char*)ch->cached_cur)+1) = (0x80 | (chr & 0x3f));
-        ch->cached_cur = VOID_PTR_ADD(ch->cached_cur,2);
+        ch->cached_cur = qio_ptr_add(ch->cached_cur,2);
       } else if( chr < 0x10000 ) {
         // OK, we got a fits-in-3-bytes character; case #3
         *(unsigned char*)ch->cached_cur = (0xe0 | (chr >> 12));
         *(((unsigned char*)ch->cached_cur)+1) = (0x80 | ((chr >> 6) & 0x3f));
         *(((unsigned char*)ch->cached_cur)+2) = (0x80 | (chr & 0x3f));
-        ch->cached_cur = VOID_PTR_ADD(ch->cached_cur,3);
+        ch->cached_cur = qio_ptr_add(ch->cached_cur,3);
       } else {
         // OK, we got a fits-in-4-bytes character; case #4
         *(unsigned char*)ch->cached_cur = (0xf0 | (chr >> 18));
         *(((unsigned char*)ch->cached_cur)+1) = (0x80 | ((chr >> 12) & 0x3f));
         *(((unsigned char*)ch->cached_cur)+2) = (0x80 | ((chr >> 6) & 0x3f));
         *(((unsigned char*)ch->cached_cur)+3) = (0x80 | (chr & 0x3f));
-        ch->cached_cur = VOID_PTR_ADD(ch->cached_cur,4);
+        ch->cached_cur = qio_ptr_add(ch->cached_cur,4);
       }
     } else if( qio_glocale_utf8 == QIO_GLOCALE_ASCII ) {
       *(unsigned char*)ch->cached_cur = (unsigned char) chr;
-      ch->cached_cur = VOID_PTR_ADD(ch->cached_cur,1);
+      ch->cached_cur = qio_ptr_add(ch->cached_cur,1);
     }
   } else {
     err = _qio_channel_write_char_slow_unlocked(ch, chr);
@@ -706,16 +735,17 @@ int qio_unicode_supported(void) {
   return qio_glocale_utf8 == QIO_GLOCALE_UTF8;
 }
 
-err_t qio_channel_skip_past_newline(const int threadsafe, qio_channel_t* restrict ch, int skipOnlyWs);
+qioerr qio_channel_skip_past_newline(const int threadsafe, qio_channel_t* restrict ch, int skipOnlyWs);
 
-err_t qio_channel_write_newline(const int threadsafe, qio_channel_t* restrict ch);
+qioerr qio_channel_write_newline(const int threadsafe, qio_channel_t* restrict ch);
 
-err_t qio_channel_scan_string(const int threadsafe, qio_channel_t* restrict ch, const char* restrict * restrict out, int64_t* restrict len_out, ssize_t maxlen);
+qioerr qio_channel_scan_string(const int threadsafe, qio_channel_t* restrict ch, const char* restrict * restrict out, int64_t* restrict len_out, ssize_t maxlen_bytes);
 
+// reads match exactly - skipping whitespace before it if skipwsbefore is set.
 // returns 0 if it matched, or EFORMAT if it did not.
-err_t qio_channel_scan_literal(const int threadsafe, qio_channel_t* restrict ch, const char* restrict match, ssize_t len, int skipws);
+qioerr qio_channel_scan_literal(const int threadsafe, qio_channel_t* restrict ch, const char* restrict match, ssize_t len, int skipwsbefore);
 // Chapel needs another name for the same routine.
-err_t qio_channel_scan_literal_2(const int threadsafe, qio_channel_t* ch, /* const char* */ void* match, ssize_t len, int skipws);
+qioerr qio_channel_scan_literal_2(const int threadsafe, qio_channel_t* ch, /* const char* */ void* match, ssize_t len, int skipwsbefore);
 
 typedef struct qio_truncate_info_ {
   ssize_t max_columns;
@@ -731,22 +761,25 @@ typedef struct qio_truncate_info_ {
 // Quote a string according to a style (we have this one for some error
 // situations in which it's undesireable to use the stdout channel
 // because of e.g. Chapel module initialization order)
-err_t qio_quote_string(uint8_t string_start, uint8_t string_end, uint8_t string_format, const char* restrict ptr, ssize_t len, const char** out, qio_truncate_info_t* ti);
+qioerr qio_quote_string(uint8_t string_start, uint8_t string_end, uint8_t string_format, const char* restrict ptr, ssize_t len, const char** out, qio_truncate_info_t* ti);
 // like qio_quote_string, but only get length information.
-err_t qio_quote_string_length(uint8_t string_start, uint8_t string_end, uint8_t string_format, const char* restrict ptr, ssize_t len, qio_truncate_info_t* ti);
-
-const char* qio_quote_string_chpl(const char* ptr, ssize_t len);
+qioerr qio_quote_string_length(uint8_t string_start, uint8_t string_end, uint8_t string_format, const char* restrict ptr, ssize_t len, qio_truncate_info_t* ti);
 
 // Prints a string according to the style.
-err_t qio_channel_print_string(const int threadsafe, qio_channel_t* restrict ch, const char* restrict ptr, ssize_t len);
+qioerr qio_channel_print_string(const int threadsafe, qio_channel_t* restrict ch, const char* restrict ptr, ssize_t len);
 
 // Prints a string as-is (this really just calls qio_channel_write_amt).
-err_t qio_channel_print_literal(const int threadsafe, qio_channel_t* restrict ch, const char* restrict ptr, ssize_t len);
-err_t qio_channel_print_literal_2(const int threadsafe, qio_channel_t* ch, /*const char* */ void* ptr, ssize_t len);
-
+qioerr qio_channel_print_literal(const int threadsafe, qio_channel_t* restrict ch, const char* restrict ptr, ssize_t len);
+qioerr qio_channel_print_literal_2(const int threadsafe, qio_channel_t* ch, /*const char* */ void* ptr, ssize_t len);
 
 
+int32_t qio_skip_json_object_unlocked(qio_channel_t* restrict ch);
+int32_t qio_skip_json_array_unlocked(qio_channel_t* restrict ch);
+int32_t qio_skip_json_value_unlocked(qio_channel_t* restrict ch);
+int32_t qio_skip_json_string_unlocked(qio_channel_t* restrict ch);
+int32_t qio_skip_json_field_unlocked(qio_channel_t* restrict ch);
 
+qioerr qio_channel_skip_json_field(const int threadsafe, qio_channel_t* ch);
 
 enum {
   QIO_CONV_UNK = 0,
@@ -801,7 +834,7 @@ typedef struct qio_conv_s {
 
 void qio_conv_destroy(qio_conv_t* spec);
 void qio_conv_init(qio_conv_t* spec_out);
-qioerr qio_conv_parse(const char* fmt, size_t start, uint64_t* end_out, int scanning, qio_conv_t* spec_out, qio_style_t* style_out);
+qioerr qio_conv_parse(c_string fmt, size_t start, uint64_t* end_out, int scanning, qio_conv_t* spec_out, qio_style_t* style_out);
 
 // These error codes can be used by callers to qio_conv_parse
 qioerr qio_format_error_too_many_args(void);
