@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _CHPL_REGEXP_H_
 #define _CHPL_REGEXP_H_
@@ -48,16 +62,16 @@ void qio_regexp_create_compile_flags_2(void* str, int64_t str_len, void* flags,
   qio_regexp_create_compile_flags((const char*) str, str_len, (const char*) flags, flags_len, isUtf8, compiled);
 }
 
-void qio_regexp_retain(qio_regexp_t* regexp);
+void qio_regexp_retain(const qio_regexp_t* regexp);
 void qio_regexp_release(qio_regexp_t* regexp);
 
-void qio_regexp_get_options(qio_regexp_t* regexp, qio_regexp_options_t* options);
+void qio_regexp_get_options(const qio_regexp_t* regexp, qio_regexp_options_t* options);
 
-void qio_regexp_get_pattern(qio_regexp_t* regexp, const char** pattern);
+void qio_regexp_get_pattern(const qio_regexp_t* regexp, const char** pattern);
 
-int64_t qio_regexp_get_ncaptures(qio_regexp_t* regexp);
-qio_bool qio_regexp_ok(qio_regexp_t* regexp);
-const char* qio_regexp_error(qio_regexp_t* regexp);
+int64_t qio_regexp_get_ncaptures(const qio_regexp_t* regexp);
+qio_bool qio_regexp_ok(const qio_regexp_t* regexp);
+const char* qio_regexp_error(const qio_regexp_t* regexp);
 
 enum {
   QIO_REGEXP_ANCHOR_UNANCHORED = 0,
@@ -121,6 +135,6 @@ int64_t qio_regexp_replace(qio_regexp_t* regexp, const char* repl, int64_t repl_
 //  - if there was a match, put it after the match
 //  - if there was an error, we do not adjust the channel position afterwards
 //
-err_t qio_regexp_channel_match(qio_regexp_t* regexp, const int threadsafe, struct qio_channel_s* ch, int64_t maxlen, int anchor, qio_bool can_discard, qio_bool keep_unmatched, qio_bool keep_whole_pattern, qio_regexp_string_piece_t* submatch, int64_t nsubmatch);
+qioerr qio_regexp_channel_match(const qio_regexp_t* regexp, const int threadsafe, struct qio_channel_s* ch, int64_t maxlen, int anchor, qio_bool can_discard, qio_bool keep_unmatched, qio_bool keep_whole_pattern, qio_regexp_string_piece_t* submatch, int64_t nsubmatch);
 
 #endif
