@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef _QIO_STYLE_H_
 #define _QIO_STYLE_H_
@@ -26,6 +40,7 @@ typedef uint8_t style_char_t;
 #define QIO_STYLE_ELEMENT_TUPLE 5
 #define QIO_STYLE_ELEMENT_BYTE_ORDER 6
 #define QIO_STYLE_ELEMENT_IS_NATIVE_BYTE_ORDER 7
+#define QIO_STYLE_ELEMENT_SKIP_UNKNOWN_FIELDS 8
 
 
 #define QIO_STRING_FORMAT_WORD 0
@@ -33,6 +48,7 @@ typedef uint8_t style_char_t;
 #define QIO_STRING_FORMAT_CHPL 2
 #define QIO_STRING_FORMAT_JSON 3
 #define QIO_STRING_FORMAT_TOEND 4
+#define QIO_STRING_FORMAT_TOEOF 5
 
 #define QIO_COMPLEX_FORMAT_READ_ANY 0
 #define QIO_COMPLEX_FORMAT_READ_STRICT 0x10
@@ -101,6 +117,8 @@ typedef struct qio_style_s {
                                and nonprinting characters c = \uABCD
        QIO_STRING_FORMAT_TOEND string is as-is; reading reads until string_end;
                                returned string includes string_end.
+       QIO_STRING_FORMAT_TOEOF string is as-is; reading reads until EOF;
+                               returned string includes string_end.
      */
   uint8_t string_format;
 
@@ -129,14 +147,36 @@ typedef struct qio_style_s {
   uint8_t uppercase; // numeric stuff is uppercase
   uint8_t leftjustify; // 1 == left, 0 == right
 
-  // floating point options
-  uint8_t showpoint; // integer floating point values include a decimal point
-                     // with some level of precision (maybe just . or maybe .00000 for %g)
-  uint8_t showpointzero;  // integer floating point values get a .0
-                          // if they would otherwise have be printed without a .0
+  // more numeric options that make the most sense for floating point but
+  // also apply to integers. Thes only apply to printing (not reading).
+  uint8_t showpoint; // floating point values with no fractional portion
+                     // will get a decimal point
+                     // with some level of precision (maybe just . or maybe
+                     // .00000 for %g). This applies also to integers.
+                     // when reading, this number has no impact on floating
+                     // point values, but for integers it causes any \.0*
+                     // to be consumed after the number.
 
+  uint8_t showpointzero;  // floating point values with no fractional portion
+                          // get a .0 if they would otherwise have be printed
+                          // without a .0
+                          // Since this setting exists to distinguish
+                          // printed floating point values from printed
+                          // integers, it does not apply to integers. You could
+                          // use precision = 1 for that.
+                          // Also, it has no impact on numbers printed with
+                          // an exponent for the same reason.
+
+  // numeric printing and scanning choice
   int32_t precision; // for floating point, number after decimal point.
                      // or number of significant digits in realfmt 2.
+                     // for integers, this is always the number
+                     // of .000 zeros to print
+                     // when reading, this number has no impact on floating
+                     // point values, but for integers it causes any \.0*
+                     // to be consumed after the number when precision > 0.
+
+  // realfmt does not apply to integers.
   uint8_t realfmt; //0 -> print with %g; 1 -> print with %f; 2 -> print with %e
 
   // Other data type choices
@@ -166,6 +206,12 @@ typedef struct qio_style_s {
   // QIO_TUPLE_FORMAT_CHPL make it look like (a,b,c)
   // QIO_TUPLE_FORMAT_JSON make it look like a JSON array [1,2]
   uint8_t tuple_style;
+
+  // If this is set, skip any unknown record/class fields
+  // when reading (ie the data to read might have more fields
+  // than a record, and the additional fields will be ignored).
+  uint8_t skip_unknown_fields;
+
 } qio_style_t;
 
 typedef qio_style_t _qio_style_ptr_t;
@@ -236,7 +282,7 @@ qio_style_t qio_style_default(void)
 static inline
 void qio_style_copy(qio_style_t* dst, const qio_style_t* src)
 {
-  memcpy(dst, src, sizeof(qio_style_t));
+  qio_memcpy(dst, src, sizeof(qio_style_t));
 }
 
 static inline
