@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "chplrt.h"
 
@@ -16,6 +30,7 @@
 #include <string.h>
 #include <inttypes.h>
 #include <math.h>
+#include <complex.h>
 
 static int scanningNCounts(void) {
   static int answer = -1;
@@ -45,9 +60,9 @@ static int illegalFirstUnsChar(char c) {
 #define _type(base, width) base##width##_t
 
 #define _define_string_to_int_precise(base, width, uns)                 \
-  _type(base,width) chpl_string_to_##base##width##_t_precise(const char* str, \
-                                                             int* invalid,    \
-                                                             char* invalidCh) { \
+  _type(base,width) c_string_to_##base##width##_t_precise(c_string str, \
+                                                          int* invalid,    \
+                                                          char* invalidCh) { \
     char* endPtr;                                                       \
     _type(base, width) val = (_type(base, width))strtol(str, &endPtr, 10);  \
     *invalid = (*str == '\0' || *endPtr != '\0');                       \
@@ -61,9 +76,9 @@ static int illegalFirstUnsChar(char c) {
   }
 
 #define _define_string_to_bigint_precise(base, width, uns, format)      \
-  _type(base, width)  chpl_string_to_##base##width##_t_precise(const char* str, \
-                                                               int* invalid,  \
-                                                               char* invalidCh) { \
+  _type(base, width)  c_string_to_##base##width##_t_precise(c_string str, \
+                                                            int* invalid,  \
+                                                            char* invalidCh) { \
     _type(base, width)  val;                                            \
     int numbytes;                                                       \
     int numitems = sscanf(str, format"%n", &val, &numbytes);            \
@@ -102,10 +117,10 @@ _define_string_to_int_precise(uint, 32, 1)
 _define_string_to_bigint_precise(uint, 64, 1, "%" SCNu64)
 
 
-chpl_bool chpl_string_to_chpl_bool(const char* str, int lineno, const char* filename) {
-  if (chpl_string_compare(str, "true") == 0) {
+chpl_bool c_string_to_chpl_bool(c_string str, int lineno, int32_t filename) {
+  if (string_compare(str, "true") == 0) {
     return true;
-  } else if (chpl_string_compare(str, "false") == 0) {
+  } else if (string_compare(str, "false") == 0) {
     return false;
   } else {
     const char* message = 
@@ -120,9 +135,9 @@ chpl_bool chpl_string_to_chpl_bool(const char* str, int lineno, const char* file
 #define _real_type(base, width) _##base##width
 
 #define _define_string_to_float_precise(base, width, format)            \
-  _real_type(base, width) chpl_string_to_##base##width##_precise(const char* str, \
-                                                                 int* invalid,   \
-                                                                 char* invalidCh) { \
+  _real_type(base, width) c_string_to_##base##width##_precise(c_string str, \
+                                                              int* invalid,   \
+                                                              char* invalidCh) { \
     _real_type(base, width) val;                                        \
     int numbytes;                                                       \
     int numitems = sscanf(str, format"%n", &val, &numbytes);            \
@@ -149,9 +164,9 @@ _define_string_to_float_precise(real, 32, "%f")
 _define_string_to_float_precise(real, 64, "%lf")
 
 #define _define_string_to_imag_precise(base, width, format)             \
-  _real_type(base, width) chpl_string_to_##base##width##_precise(const char* str, \
-                                                                 int* invalid,   \
-                                                                 char* invalidCh) { \
+  _real_type(base, width) c_string_to_##base##width##_precise(c_string str, \
+                                                              int* invalid,   \
+                                                              char* invalidCh) { \
     _real_type(base, width) val;                                        \
     int numbytes;                                                       \
     char i = '\0';                                                      \
@@ -188,22 +203,24 @@ _define_string_to_imag_precise(imag, 64, "%lf")
 
 
 #define _define_string_to_complex_precise(base, width, format, halfwidth) \
-  _real_type(base, width) chpl_string_to_##base##width##_precise(const char* str, \
-                                                                 int* invalid,   \
-                                                                 char* invalidCh) { \
-    _real_type(base, width) val = {0.0, 0.0};                           \
+  _##base##width c_string_to_##base##width##_precise(c_string str,      \
+                                                     int* invalid,      \
+                                                     char* invalidCh) { \
+    _##base##width val = 0.0;                                           \
+    _real_type(real, halfwidth) val_re = 0.0;                           \
+    _real_type(real, halfwidth) val_im = 0.0;                           \
     /* check for pure imaginary case first */                           \
-    val.im = chpl_string_to_imag##halfwidth##_precise(str, invalid, invalidCh); \
+    val_im = c_string_to_imag##halfwidth##_precise(str, invalid, invalidCh); \
     if (*invalid) {                                                     \
       int numbytes = -1;                                                \
       char sign = '\0';                                                 \
       char i = '\0';                                                    \
       int numitems;                                                     \
       int posAfterReal, posBeforeSign, posBeforeImag;                   \
-      val.im = 0.0; /* reset */                                         \
+      val_im = 0.0; /* reset */                                         \
       numitems = sscanf(str, format"%n %n%c %n"format"%c%n",            \
-                        &(val.re), &posAfterReal, &posBeforeSign,       \
-                        &sign, &posBeforeImag, &(val.im), &i,           \
+                        &(val_re), &posAfterReal, &posBeforeSign,       \
+                        &sign, &posBeforeImag, &(val_im), &i,           \
                         &numbytes);                                     \
       if (scanningNCounts()) {                                          \
         if (numitems == 3) {                                            \
@@ -240,7 +257,7 @@ _define_string_to_imag_precise(imag, 64, "%lf")
           *invalidCh = i;                                               \
         } else if (numbytes == strlen(str)) {                           \
           if (sign == '-') {                                            \
-            val.im = -val.im;                                           \
+            val_im = -val_im;                                           \
           }                                                             \
           *invalid = 0;                                                 \
           *invalidCh = '\0';                                            \
@@ -253,6 +270,7 @@ _define_string_to_imag_precise(imag, 64, "%lf")
         *invalidCh = *str;                                              \
       }                                                                 \
     }                                                                   \
+    val = val_re + val_im*_Complex_I;                                   \
     return val;                                                         \
   }
 
@@ -263,13 +281,19 @@ _define_string_to_complex_precise(complex, 128, "%lf", 64)
 
 
 #define _define_string_to_type(base, width)                             \
-  _type(base, width) chpl_string_to_##base##width##_t(const char* str, int lineno, \
-                                                      const char* filename) {   \
+  _type(base, width) c_string_to_##base##width##_t(c_string str, int lineno, \
+                                                   int32_t filename) {   \
     int invalid;                                                        \
     char invalidStr[2] = "\0\0";                                        \
-    _type(base, width) val = chpl_string_to_##base##width##_t_precise(str,    \
-                                                                      &invalid, \
-                                                                      invalidStr); \
+    _type(base, width) val = 0;                                         \
+    if (!str) {                                                         \
+      invalid = 1;                                                      \
+    } else {                                                            \
+      val = c_string_to_##base##width##_t_precise(str,                  \
+                                                  &invalid,             \
+                                                  invalidStr);          \
+    }                                                                   \
+                                                                        \
     if (invalid) {                                                      \
       const char* message;                                              \
       if (invalid == 2) {                                               \
@@ -298,13 +322,13 @@ _define_string_to_type(uint, 32)
 _define_string_to_type(uint, 64)
 
 #define _define_string_to_real_type(base, width)                        \
-  _real_type(base, width) chpl_string_to_##base##width(const char* str, int lineno, \
-                                                       const char* filename) {   \
+  _##base##width c_string_to_##base##width(c_string str, int lineno,    \
+                                           int32_t filename) {         \
     int invalid;                                                        \
     char invalidStr[2] = "\0\0";                                        \
-    _real_type(base, width) val = chpl_string_to_##base##width##_precise(str,    \
-                                                                         &invalid, \
-                                                                         invalidStr); \
+    _##base##width val = c_string_to_##base##width##_precise(str,       \
+                                                             &invalid, \
+                                                             invalidStr); \
     if (invalid) {                                                      \
       const char* message;                                              \
       if (invalid == 2) {                                               \
@@ -331,65 +355,68 @@ _define_string_to_real_type(complex, 64)
 _define_string_to_real_type(complex, 128)
 
 
-/*
- *  int and uint to string
- */
-#define integral_to_string(type, format)        \
-  chpl_string type##_to_chpl_string(type x) {   \
-    char buffer[256];                           \
-    sprintf(buffer, format, x);                 \
-    return string_copy(buffer, 0, NULL);        \
-  }
+c_string_copy
+integral_to_c_string_copy(int64_t x, uint32_t size, chpl_bool isSigned)
+{
+  char buffer[256];
+  const char* format = "";
+  enum {UNSIGNED = 0<<16, SIGNED = 1<<16 };
+  switch (SIGNED * isSigned + size)
+  {
+   default:
+    chpl_error("Unexpected case in integral_to_c_string_copy", -1, 0);
+    break;
 
-integral_to_string(int8_t, "%" PRId8)
-integral_to_string(int16_t, "%" PRId16)
-integral_to_string(int32_t, "%" PRId32)
-integral_to_string(int64_t, "%" PRId64)
-integral_to_string(uint8_t, "%" PRIu8)
-integral_to_string(uint16_t, "%" PRIu16)
-integral_to_string(uint32_t, "%" PRIu32)
-integral_to_string(uint64_t, "%" PRIu64)
+   case UNSIGNED + 1: format = "%" PRIu8;  break;
+   case UNSIGNED + 2: format = "%" PRIu16; break;
+   case UNSIGNED + 4: format = "%" PRIu32; break;
+   case UNSIGNED + 8: format = "%" PRIu64; break;
+   case   SIGNED + 1: format = "%" PRId8;  break;
+   case   SIGNED + 2: format = "%" PRId16; break;
+   case   SIGNED + 4: format = "%" PRId32; break;
+   case   SIGNED + 8: format = "%" PRId64; break;
+  }
+  sprintf(buffer, format, x);
+  return string_copy(buffer, 0, 0);
+}
 
 /*
  *  real and imag to string
  */
-static void ensureDecimal(char* buffer) {
-  /* add decimal if one does not exist */
+static char* ensureDecimal(char* buffer) {
+  char* last = buffer + strlen(buffer);
   if (!strchr(buffer, '.') && !strchr(buffer, 'e')) {
-    if (strchr(buffer, 'i')) {
-      buffer[strlen(buffer)-1] = '\0';
-      strcat(buffer, ".0i");
-    } else {
-      strcat(buffer, ".0");
-    }
+    strcat(buffer, ".0");
+    last += 2;
   }
+  return last;
 }
 
 #define NANSTRING "nan"
 #define NEGINFSTRING "-inf"
 #define POSINFSTRING "inf"
 
-// the above strings are copied below so the return value of real_to_string
-// can be freed unconditionally
-#define real_to_string(type, format)           \
-  chpl_string type##_to_chpl_string(type x) {  \
-    if (isnan(x)) {                            \
-      return string_copy(NANSTRING, 0, NULL);  \
-    } else if (isinf(x)) {                     \
-      if (x < 0) {                             \
-        return string_copy(NEGINFSTRING, 0, NULL);  \
-      } else {                                 \
-        return string_copy(POSINFSTRING, 0, NULL);  \
-      }                                        \
-    } else {                                   \
-      char buffer[256];                        \
-      sprintf(buffer, format, x);              \
-      ensureDecimal(buffer);                   \
-      return string_copy(buffer, 0, NULL);     \
-    }                                          \
-  }
+// Note: This function is thread-safe, since the stack-allocated buffer is
+// private to the thread, and each returned c_string_copy is a unique object.
+c_string_copy
+real_to_c_string_copy(_real64 x, chpl_bool isImag)
+{
+  if (isnan(x)) {
+    return string_copy(NANSTRING, 0, 0);
+  } else if (isinf(x)) {
+    if (x < 0) {
+      return string_copy(NEGINFSTRING, 0, 0);
+    } else {
+      return string_copy(POSINFSTRING, 0, 0);
+    }
+  } else {
+    char buffer[256];
+    char* last;
 
-real_to_string(_real32, "%lg")
-real_to_string(_real64, "%lg")
-real_to_string(_imag32, "%lgi")
-real_to_string(_imag64, "%lgi")
+    sprintf(buffer, "%lg", x);
+    last = ensureDecimal(buffer);
+    if (isImag)
+      strcat(last, "i");
+    return string_copy(buffer, 0, 0);
+  }
+}
