@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef CHPL_HAS_GMP
 
@@ -20,13 +34,13 @@
 #include "chpl-comm.h"
 
 static void* chpl_gmp_alloc(size_t sz) {
-  return chpl_mem_alloc(sz, CHPL_RT_MD_GMP, __LINE__, __FILE__);
+  return chpl_mem_alloc(sz, CHPL_RT_MD_GMP, 0, 0);
 }
 static void* chpl_gmp_realloc(void* ptr, size_t old_size, size_t new_size) {
-  return chpl_mem_realloc( ptr, new_size, CHPL_RT_MD_GMP, __LINE__, __FILE__);
+  return chpl_mem_realloc( ptr, new_size, CHPL_RT_MD_GMP, 0, 0);
 }
 static void chpl_gmp_free(void* ptr, size_t old_size) {
-  chpl_mem_free( ptr, __LINE__, __FILE__);
+  chpl_mem_free( ptr, 0, 0);
 }
 
 void chpl_gmp_init(void) {
@@ -43,7 +57,9 @@ void chpl_gmp_get_mpz(mpz_t ret, int64_t src_locale, __mpz_struct from)
   ret[0]._mp_size = from._mp_size;
 
   // Next, use GASNET to move the pointer data.
-  chpl_comm_get( ret[0]._mp_d, src_locale, from._mp_d, sizeof(mp_limb_t), CHPL_TYPE_uint64_t, ret[0]._mp_alloc, __LINE__, __FILE__);
+  chpl_gen_comm_get(ret[0]._mp_d, src_locale, from._mp_d,
+                    sizeof(mp_limb_t) * ret[0]._mp_alloc, CHPL_TYPE_uint64_t,
+                    0, 0);
 
   //gmp_printf("got %Zd\n", ret);
 }
@@ -72,11 +88,11 @@ void chpl_gmp_mpz_print(mpz_t x)
 }
 
 
-chpl_string chpl_gmp_mpz_get_str(int base, mpz_t x)
+c_string_copy chpl_gmp_mpz_get_str(int base, mpz_t x)
 {
   size_t len = mpz_sizeinbase(x, base);
 
-  char* str = (char*)chpl_mem_calloc(len+1, CHPL_RT_MD_GLOM_STRINGS_DATA, 0, 0);
+  char* str = (char*)chpl_mem_calloc(1, len+1, CHPL_RT_MD_GLOM_STRINGS_DATA, 0, 0);
 
   mpz_get_str(str, base, x);
 
