@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "chplrt.h"
 
@@ -12,9 +26,12 @@
 #include "chpltypes.h"
 #include "chpl-comm.h"
 #include "chplcgfns.h"
+#include "chpl-linefile-support.h"
 #include "config.h"
 #include "error.h"
 
+#include "chpl-comm-compiler-macros.h"
+
 #include <assert.h>
 #include <string.h>
 #include <stdint.h>
@@ -23,23 +40,28 @@
 #include <inttypes.h>
 
 
+static void
+printMemAllocs(chpl_mem_descInt_t description, int64_t threshold,
+               int32_t lineno, int32_t filename);
+
+
 //
 // This is in the modules, in MemTracking.chpl.
 //
-extern void chpl_memTracking_returnConfigVals(chpl_bool*,
-                                              chpl_bool*,
-                                              chpl_bool*,
-                                              chpl_bool*,
-                                              size_t*,
-                                              size_t*,
-                                              c_string*,
-                                              c_string*);
+extern void chpl_memTracking_returnConfigVals(chpl_bool* memTrack,
+                                              chpl_bool* memStats,
+                                              chpl_bool* memLeaksByType,
+                                              c_string* memLeaksByDesc,
+                                              chpl_bool* memLeaks,
+                                              size_t* memMax,
+                                              size_t* memThreshold,
+                                              c_string* memLog,
+                                              c_string* memLeaksLog);
 
 chpl_bool chpl_memTrack = false;
 
-#undef malloc
-#undef calloc
-#undef free
+// memory layer hasn't been initialized, need to use the system allocator
+#include "chpl-mem-no-warning-macros.h"
 
 typedef struct memTableEntry_struct { /* table entry */
   size_t number;
@@ -47,7 +69,7 @@ typedef struct memTableEntry_struct { /* table entry */
   chpl_mem_descInt_t description;
   void* memAlloc;
   int32_t lineno;
-  c_string filename;
+  int32_t filename;
   struct memTableEntry_struct* nextInBucket;
 } memTableEntry;
 
@@ -62,14 +84,15 @@ static int hashSize = 0;
 
 static memTableEntry** memTable = NULL;
 
-static _Bool memLeaks = false;
-static _Bool memLeaksTable = false;
 static _Bool memStats = false;
+static _Bool memLeaksByType = false;
+static c_string memLeaksByDesc = "";
+static _Bool memLeaks = false;
 static size_t memMax = 0;
 static size_t memThreshold = 0;
-static c_string memLog = "";
+static c_string memLog = NULL;
 static FILE* memLogFile = NULL;
-static c_string memLeaksLog = "";
+static c_string memLeaksLog = NULL;
 
 static size_t totalMem = 0;       /* total memory currently allocated */
 static size_t maxMem = 0;         /* maximum total memory during run  */
@@ -100,8 +123,9 @@ void chpl_setMemFlags(void) {
   //
   chpl_memTracking_returnConfigVals(&local_memTrack,
                                     &memStats,
+                                    &memLeaksByType,
+                                    &memLeaksByDesc,
                                     &memLeaks,
-                                    &memLeaksTable,
                                     &memMax,
                                     &memThreshold,
                                     &memLog,
@@ -109,14 +133,15 @@ void chpl_setMemFlags(void) {
 
   if (local_memTrack
       || memStats
+      || memLeaksByType
+      || (memLeaksByDesc && strcmp(memLeaksByDesc, ""))
       || memLeaks
-      || memLeaksTable
       || memMax > 0
-      || strcmp(memLeaksLog, "") != 0) {
+      || memLeaksLog != NULL) {
     chpl_memTrack = true;
   }
 
-  if (strcmp(memLog, "") == 0) {
+  if (!memLog) {
     memLogFile = stdout;
   } else {
     if (chpl_numNodes == 1) {
@@ -150,7 +175,7 @@ static unsigned hash(void* memAlloc, int hashSize) {
 }
 
 
-static void increaseMemStat(size_t chunk, int32_t lineno, chpl_string filename) {
+static void increaseMemStat(size_t chunk, int32_t lineno, int32_t filename) {
   totalMem += chunk;
   totalAllocated += chunk;
   if (memMax && (totalMem > memMax)) {
@@ -194,8 +219,9 @@ resizeTable(int direction) {
   hashSizeIndex = newHashSizeIndex;
 }
 
-
-static void addMemTableEntry(void* memAlloc, size_t number, size_t size, chpl_mem_descInt_t description, int32_t lineno, chpl_string filename) {
+static void addMemTableEntry(void *memAlloc, size_t number, size_t size,
+                             chpl_mem_descInt_t description, int32_t lineno,
+                             int32_t filename) {
   unsigned hashValue;
   memTableEntry* memEntry;
 
@@ -214,7 +240,7 @@ static void addMemTableEntry(void* memAlloc, size_t number, size_t size, chpl_me
   memEntry->description = description;
   memEntry->memAlloc = memAlloc;
   memEntry->lineno = lineno;
-  c_string_from_string(&memEntry->filename, &filename, 0, NULL);
+  memEntry->filename = filename;
   memEntry->number = number;
   memEntry->size = size;
   increaseMemStat(number*size, lineno, filename);
@@ -256,18 +282,24 @@ static memTableEntry* removeMemTableEntry(void* address) {
 }
 
 
-uint64_t chpl_memoryUsed(int32_t lineno, chpl_string filename) {
-  if (!chpl_memTrack)
-    chpl_error("invalid call to memoryUsed(); rerun with --memTrack",
-               lineno, filename);
+uint64_t chpl_memoryUsed(int32_t lineno, int32_t filename) {
+  if (!chpl_memTrack) {
+    chpl_warning("invalid call to memoryUsed(); rerun with --memTrack",
+                 lineno, filename);
+    return 0;
+  }
+
   return (uint64_t)totalMem;
 }
 
 
-void chpl_printMemStat(int32_t lineno, chpl_string filename) {
-  if (!chpl_memTrack)
-    chpl_error("invalid call to printMemStat(); rerun with --memTrack",
-               lineno, filename);
+void chpl_printMemAllocStats(int32_t lineno, int32_t filename) {
+  if (!chpl_memTrack) {
+    chpl_warning("invalid call to printMemAllocStats(); rerun with --memTrack",
+                 lineno, filename);
+    return;
+  }
+
   chpl_sync_lock(&memTrack_sync);
   fprintf(memLogFile, "=================\n");
   fprintf(memLogFile, "Memory Statistics\n");
@@ -289,10 +321,10 @@ void chpl_printMemStat(int32_t lineno, chpl_string filename) {
     fprintf(memLogFile, "==============================================================\n");
     for (i = 0; i < chpl_numNodes; i++) {
       static size_t m1, m2, m3, m4;
-      chpl_comm_get(&m1, i, &totalMem, sizeof(size_t), -1 /* broke for hetero */, 1, lineno, filename);
-      chpl_comm_get(&m2, i, &maxMem, sizeof(size_t), -1 /* broke for hetero */, 1, lineno, filename);
-      chpl_comm_get(&m3, i, &totalAllocated,  sizeof(size_t), -1 /* broke for hetero */, 1, lineno, filename);
-      chpl_comm_get(&m4, i, &totalFreed, sizeof(size_t), -1 /*broke for hetero */, 1, lineno, filename);
+      chpl_gen_comm_get(&m1, i, &totalMem, sizeof(size_t), -1 /* broke for hetero */, lineno, filename);
+      chpl_gen_comm_get(&m2, i, &maxMem, sizeof(size_t), -1 /* broke for hetero */, lineno, filename);
+      chpl_gen_comm_get(&m3, i, &totalAllocated,  sizeof(size_t), -1 /* broke for hetero */, lineno, filename);
+      chpl_gen_comm_get(&m4, i, &totalFreed, sizeof(size_t), -1 /*broke for hetero */, lineno, filename);
       fprintf(memLogFile, "%-9d  %-9zu  %-9zu  %-9zu  %-9zu\n", i, m1, m2, m3, m4);
     }
     fprintf(memLogFile, "==============================================================\n");
@@ -301,17 +333,27 @@ void chpl_printMemStat(int32_t lineno, chpl_string filename) {
 }
 
 
-static int leakedMemTableEntryCmp(const void* p1, const void* p2) {
+
+static int memTableEntryCmp(const void* p1, const void* p2) {
   return *(size_t*)p2 - *(size_t*)p1;
 }
 
-static void chpl_printLeakedMemTable(void) {
+
+static void printMemAllocsByType(_Bool forLeaks,
+                                 int32_t lineno, int32_t filename) {
   size_t* table;
   memTableEntry* me;
   int i;
   const int numberWidth   = 9;
   const int numEntries = CHPL_RT_MD_NUM+chpl_mem_numDescs;
 
+  if (!chpl_memTrack) {
+    chpl_warning("invalid call to printMemAllocsByType(); rerun with "
+                 "--memTrack",
+                 lineno, filename);
+    return;
+  }
+
   table = (size_t*)calloc(numEntries, 3*sizeof(size_t));
 
   for (i = 0; i < hashSize; i++) {
@@ -322,13 +364,23 @@ static void chpl_printLeakedMemTable(void) {
     }
   }
 
-  qsort(table, numEntries, 3*sizeof(size_t), leakedMemTableEntryCmp);
+  qsort(table, numEntries, 3*sizeof(size_t), memTableEntryCmp);
+
+  if (forLeaks) {
+    fprintf(memLogFile, "====================\n");
+    fprintf(memLogFile, "Leaked Memory Report\n");
+    fprintf(memLogFile, "==============================================================\n");
+    fprintf(memLogFile, "Number of leaked allocations\n");
+    fprintf(memLogFile, "           Total leaked memory (bytes)\n");
+  }
+  else {
+    fprintf(memLogFile, "================================\n");
+    fprintf(memLogFile, "Memory Allocation Report by Type\n");
+    fprintf(memLogFile, "==============================================================\n");
+    fprintf(memLogFile, "Number of allocations\n");
+    fprintf(memLogFile, "           Total allocated bytes\n");
+  }
 
-  fprintf(memLogFile, "====================\n");
-  fprintf(memLogFile, "Leaked Memory Report\n");
-  fprintf(memLogFile, "==============================================================\n");
-  fprintf(memLogFile, "Number of leaked allocations\n");
-  fprintf(memLogFile, "           Total leaked memory (bytes)\n");
   fprintf(memLogFile, "                      Description of allocation\n");
   fprintf(memLogFile, "==============================================================\n");
   for (i = 0; i < 3*(CHPL_RT_MD_NUM+chpl_mem_numDescs); i += 3) {
@@ -345,74 +397,96 @@ static void chpl_printLeakedMemTable(void) {
 }
 
 
-void chpl_reportMemInfo() {
-  if (memStats) {
-    fprintf(memLogFile, "\n");
-    chpl_printMemStat(0, 0);
-  }
-  if (memLeaks) {
-    fprintf(memLogFile, "\n");
-    chpl_printLeakedMemTable();
-  }
-  if (memLeaksTable) {
-    fprintf(memLogFile, "\n");
-    chpl_printMemTable(0, 0, 0);
-  }
-  if (memLogFile && memLogFile != stdout)
-    fclose(memLogFile);
-  if (memLeaksLog && strcmp(memLeaksLog, "")) {
-    memLogFile = fopen(memLeaksLog, "a");
-    fprintf(memLogFile, "\nCompiler Command : %s\n", chpl_compileCommand);
-    fprintf(memLogFile, "Execution Command: %s\n\n", chpl_executionCommand);
-    chpl_printMemStat(0, 0);
-    fprintf(memLogFile, "\n");
-    chpl_printLeakedMemTable();
-    fclose(memLogFile);
-  }
+void chpl_printMemAllocsByType(int32_t lineno, int32_t filename) {
+  printMemAllocsByType(false /* forLeaks */, lineno, filename);
+}
+
+
+static chpl_mem_descInt_t
+find_desc(const char* descString)
+{
+  const int numEntries = CHPL_RT_MD_NUM+chpl_mem_numDescs;
+  for (chpl_mem_descInt_t i = 0; i < numEntries; ++i)
+    if (! strcmp(descString, chpl_mem_descString(i)))
+      return i;
+  return -1;
+}
+
+
+// Print only those memTable entries that match the given descriptor string and
+// exceed the given threshold.
+// If no match is found, all entries (exceeding the given threshold) are printed.
+void chpl_printMemAllocsByDesc(const char* descString, int64_t threshold,
+                               int32_t lineno, int32_t filename)
+{
+  chpl_mem_descInt_t description = find_desc(descString);
+  printMemAllocs(description, threshold, lineno, filename);
 }
 
 
 static int descCmp(const void* p1, const void* p2) {
   memTableEntry* m1 = *(memTableEntry**)p1;
   memTableEntry* m2 = *(memTableEntry**)p2;
+  c_string m1Filename;
+  c_string m2Filename;
 
   int val = strcmp(chpl_mem_descString(m1->description), chpl_mem_descString(m2->description));
-  if (val == 0 && m1->filename && m2->filename)
-    val = strcmp(m1->filename, m2->filename);
+  if (val == 0 && m1->filename && m2->filename) {
+    m1Filename = chpl_lookupFilename(m1->filename);
+    m2Filename = chpl_lookupFilename(m2->filename);
+    val = strcmp(m1Filename, m2Filename);
+  }
   if (val == 0)
     val = (m1->lineno < m2->lineno) ? -1 : ((m1->lineno > m2->lineno) ? 1 : 0);
   return val;
 }
 
 
-void chpl_printMemTable(int64_t threshold, int32_t lineno, chpl_string filename) {
+// If description is -1, print all entries; otherwise print only those with the
+// matching CHPL_RT_MD_ descriptor.
+// Print only those entries exceeding threshold.
+void chpl_printMemAllocs(int64_t threshold, int32_t lineno, int32_t filename) {
+  printMemAllocs(-1, threshold, lineno, filename);
+}
+
+static void
+printMemAllocs(chpl_mem_descInt_t description, int64_t threshold,
+               int32_t lineno, int32_t filename) {
   const int numberWidth   = 9;
   const int precision     = sizeof(uintptr_t) * 2;
   const int addressWidth  = precision+4;
   const int descWidth     = 33;
   int filenameWidth       = strlen("Allocated Memory (Bytes)");
   int totalWidth;
+  int filenameLength;
 
   memTableEntry* memEntry;
+  c_string memEntryFilename;
   int n, i;
   char* loc;
   memTableEntry** table;
 
-  if (!chpl_memTrack)
-    chpl_error("The printMemTable function only works with the --memTrack flag", lineno, filename);
+  if (!chpl_memTrack) {
+    chpl_warning("invalid call to printMemAllocs(); rerun with --memTrack",
+                 lineno, filename);
+    return;
+  }
 
   n = 0;
   filenameWidth = strlen("Allocated Memory (Bytes)");
   for (i = 0; i < hashSize; i++) {
     for (memEntry = memTable[i]; memEntry != NULL; memEntry = memEntry->nextInBucket) {
       size_t chunk = memEntry->number * memEntry->size;
-      if (chunk >= threshold) {
-        n += 1;
-        if (memEntry->filename) {
-          int filenameLength = strlen(memEntry->filename);
-          if (filenameLength > filenameWidth)
-            filenameWidth = filenameLength;
-        }
+      if (chunk < threshold)
+        continue;
+      if (description != -1 && memEntry->description != description)
+        continue;
+      n += 1;
+      if (memEntry->filename) {
+        memEntryFilename = chpl_lookupFilename(memEntry->filename);
+        filenameLength = strlen(memEntryFilename);
+        if (filenameLength > filenameWidth)
+          filenameWidth = filenameLength;
       }
     }
   }
@@ -440,9 +514,11 @@ void chpl_printMemTable(int64_t threshold, int32_t lineno, chpl_string filename)
   for (i = 0; i < hashSize; i++) {
     for (memEntry = memTable[i]; memEntry != NULL; memEntry = memEntry->nextInBucket) {
       size_t chunk = memEntry->number * memEntry->size;
-      if (chunk >= threshold) {
-        table[n++] = memEntry;
-      }
+      if (chunk < threshold)
+        continue;
+      if (description != -1 && memEntry->description != description)
+        continue;
+      table[n++] = memEntry;
     }
   }
   qsort(table, n, sizeof(memTableEntry*), descCmp);
@@ -451,10 +527,12 @@ void chpl_printMemTable(int64_t threshold, int32_t lineno, chpl_string filename)
 
   for (i = 0; i < n; i++) {
     memEntry = table[i];
-    if (memEntry->filename)
-      sprintf(loc, "%s:%"PRId32, memEntry->filename, memEntry->lineno);
-    else
+    if (memEntry->filename) {
+      memEntryFilename = chpl_lookupFilename(memEntry->filename);
+      sprintf(loc, "%s:%" PRId32, memEntryFilename, memEntry->lineno);
+    } else {
       sprintf(loc, "--");
+    }
     fprintf(memLogFile, "%-*s%-*zu%-*zu%-*zu%-*s%#-*.*" PRIxPTR "\n",
            filenameWidth+numberWidth, loc,
            numberWidth, memEntry->number,
@@ -473,61 +551,84 @@ void chpl_printMemTable(int64_t threshold, int32_t lineno, chpl_string filename)
 }
 
 
+void chpl_reportMemInfo() {
+  if (memStats) {
+    fprintf(memLogFile, "\n");
+    chpl_printMemAllocStats(0, 0);
+  }
+  if (memLeaksByType) {
+    fprintf(memLogFile, "\n");
+    printMemAllocsByType(true /* forLeaks */, 0, 0);
+  }
+  if (memLeaksByDesc && strcmp(memLeaksByDesc, "")) {
+    fprintf(memLogFile, "\n");
+    chpl_printMemAllocsByDesc(memLeaksByDesc, memThreshold, 0, 0);
+  }
+  if (memLeaks) {
+    fprintf(memLogFile, "\n");
+    printMemAllocs(-1, memThreshold, 0, 0);
+  }
+  if (memLogFile && memLogFile != stdout)
+    fclose(memLogFile);
+  if (memLeaksLog && strcmp(memLeaksLog, "")) {
+    memLogFile = fopen(memLeaksLog, "a");
+    fprintf(memLogFile, "\nCompiler Command : %s\n", chpl_compileCommand);
+    fprintf(memLogFile, "Execution Command: %s\n\n", chpl_executionCommand);
+    chpl_printMemAllocStats(0, 0);
+    fprintf(memLogFile, "\n");
+    printMemAllocsByType(true /* forLeaks */, 0, 0);
+    fclose(memLogFile);
+  }
+}
+
+
 void chpl_track_malloc(void* memAlloc, size_t number, size_t size,
                        chpl_mem_descInt_t description,
-                       int32_t lineno, chpl_string filename) {
-  c_string c_filename;
-
+                       int32_t lineno, int32_t filename) {
   if (number * size > memThreshold) {
-    if (chpl_memTrack) {
+    if (chpl_memTrack && chpl_mem_descTrack(description)) {
       chpl_sync_lock(&memTrack_sync);
       addMemTableEntry(memAlloc, number, size, description, lineno, filename);
       chpl_sync_unlock(&memTrack_sync);
     }
     if (chpl_verbose_mem) {
-      c_string_from_string(&c_filename, &filename, 0, NULL);
-      fprintf(memLogFile,
-              "%" FORMAT_c_nodeid_t ": %s:%" PRId32
-              ": allocate %zuB of %s at %p\n",
-              chpl_nodeID, (c_filename ? c_filename : "--"), lineno,
-              number*size, chpl_mem_descString(description), memAlloc);
+      fprintf(memLogFile, "%" FORMAT_c_nodeid_t ": %s:%" PRId32
+                          ": allocate %zuB of %s at %p\n",
+              chpl_nodeID, (filename ? chpl_lookupFilename(filename) : "--"),
+              lineno, number * size, chpl_mem_descString(description),
+              memAlloc);
     }
   }
 }
 
 
-void chpl_track_free(void* memAlloc, int32_t lineno, chpl_string filename) {
+void chpl_track_free(void* memAlloc, int32_t lineno, int32_t filename) {
   memTableEntry* memEntry = NULL;
-  c_string c_filename;
-
   if (chpl_memTrack) {
     chpl_sync_lock(&memTrack_sync);
     memEntry = removeMemTableEntry(memAlloc);
     if (memEntry) {
       if (chpl_verbose_mem) {
-        c_string_from_string(&c_filename, &filename, 0, NULL);
-        fprintf(memLogFile,
-                "%" FORMAT_c_nodeid_t ": %s:%" PRId32
-                ": free %zuB of %s at %p\n",
-                chpl_nodeID, (c_filename ? c_filename : "--"), lineno,
-                memEntry->number*memEntry->size,
+        fprintf(memLogFile, "%" FORMAT_c_nodeid_t ": %s:%" PRId32
+                            ": free %zuB of %s at %p\n",
+                chpl_nodeID, (filename ? chpl_lookupFilename(filename) : "--"),
+                lineno, memEntry->number * memEntry->size,
                 chpl_mem_descString(memEntry->description), memAlloc);
       }
       free(memEntry);
     }
     chpl_sync_unlock(&memTrack_sync);
   } else if (chpl_verbose_mem && !memEntry) {
-    c_string_from_string(&c_filename, &filename, 0, NULL);
-    fprintf(memLogFile,
-            "%" FORMAT_c_nodeid_t ": %s:%" PRId32 ": free at %p\n",
-            chpl_nodeID, (c_filename ? c_filename : "--"), lineno, memAlloc);
+    fprintf(memLogFile, "%" FORMAT_c_nodeid_t ": %s:%" PRId32 ": free at %p\n",
+            chpl_nodeID, (filename ? chpl_lookupFilename(filename) : "--"),
+            lineno, memAlloc);
   }
 }
 
 
 void chpl_track_realloc_pre(void* memAlloc, size_t size,
                          chpl_mem_descInt_t description,
-                         int32_t lineno, chpl_string filename) {
+                         int32_t lineno, int32_t filename) {
   memTableEntry* memEntry = NULL;
 
   if (chpl_memTrack && size > memThreshold) {
@@ -545,22 +646,19 @@ void chpl_track_realloc_pre(void* memAlloc, size_t size,
 void chpl_track_realloc_post(void* moreMemAlloc,
                          void* memAlloc, size_t size,
                          chpl_mem_descInt_t description,
-                         int32_t lineno, chpl_string filename) {
-  c_string c_filename;
-
+                         int32_t lineno, int32_t filename) {
   if (size > memThreshold) {
-    if (chpl_memTrack) {
+    if (chpl_memTrack && chpl_mem_descTrack(description)) {
       chpl_sync_lock(&memTrack_sync);
       addMemTableEntry(moreMemAlloc, 1, size, description, lineno, filename);
       chpl_sync_unlock(&memTrack_sync);
     }
     if (chpl_verbose_mem) {
-      c_string_from_string(&c_filename, &filename, 0, NULL);
-      fprintf(memLogFile,
-              "%" FORMAT_c_nodeid_t ": %s:%" PRId32
-              ": reallocate %zuB of %s at %p -> %p\n",
-              chpl_nodeID, (c_filename ? c_filename : "--"), lineno,
-              size, chpl_mem_descString(description), memAlloc, moreMemAlloc);
+      fprintf(memLogFile, "%" FORMAT_c_nodeid_t ": %s:%" PRId32
+                          ": reallocate %zuB of %s at %p -> %p\n",
+              chpl_nodeID, (filename ? chpl_lookupFilename(filename) : "--"),
+              lineno, size, chpl_mem_descString(description), memAlloc,
+              moreMemAlloc);
     }
   }
 }
