@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "chplrt.h"
 
@@ -13,6 +27,10 @@
 
 #include "chplcgfns.h"
 #include "chpl-gen-includes.h"
+#include "chpl-linefile-support.h"
+
+// Don't get warning macros for chpl_comm_get etc
+#include "chpl-comm-no-warning-macros.h"
 
 #include <errno.h>
 #include <stdio.h>
@@ -28,15 +46,59 @@ static int mysystem(const char* command, const char* description,
   int status = system(command);
 
   if (status == -1) {
-    chpl_error("system() fork failed", 0, "(command-line)");
+    chpl_error("system() fork failed", 0, CHPL_FILE_IDX_COMMAND_LINE);
   } else if (status != 0 && !ignorestatus) {
-    chpl_error(description, 0, "(command-line)");
+    chpl_error(description, 0, CHPL_FILE_IDX_COMMAND_LINE);
   }
 
   return status;
 }
 
 // Chapel interface
+chpl_comm_nb_handle_t chpl_comm_put_nb(void *addr, c_nodeid_t node, void* raddr,
+                                       size_t size, int32_t typeIndex,
+                                       int ln, int32_t fn)
+{
+  assert(node == 0);
+  chpl_memcpy(raddr, addr, size);
+  return NULL;
+}
+
+chpl_comm_nb_handle_t chpl_comm_get_nb(void* addr, c_nodeid_t node, void* raddr,
+                                       size_t size, int32_t typeIndex,
+                                       int ln, int32_t fn)
+{
+  assert(node == 0);
+  chpl_memcpy(addr, raddr, size);
+  return NULL;
+}
+
+int chpl_comm_test_nb_complete(chpl_comm_nb_handle_t h)
+{
+  return ((void*) h) == NULL;
+}
+
+void chpl_comm_wait_nb_some(chpl_comm_nb_handle_t* h, size_t nhandles)
+{
+  size_t i;
+  for( i = 0; i < nhandles; i++ ) {
+    assert(h[i] == NULL);
+  }
+}
+
+int chpl_comm_try_nb_some(chpl_comm_nb_handle_t* h, size_t nhandles)
+{
+  size_t i;
+  for( i = 0; i < nhandles; i++ ) {
+    assert(h[i] == NULL);
+  }
+  return 0;
+}
+
+int chpl_comm_addr_gettable(c_nodeid_t node, void* start, size_t len)
+{
+  return 0;
+}
 
 int32_t chpl_comm_getMaxThreads(void) {
   return 0;
@@ -76,7 +138,7 @@ void chpl_comm_desired_shared_heap(void** start_p, size_t* size_p) {
 
 void chpl_comm_broadcast_global_vars(int numGlobals) { }
 
-void chpl_comm_broadcast_private(int id, int32_t sizee, int32_t tid) { }
+void chpl_comm_broadcast_private(int id, size_t sizee, int32_t tid) { }
 
 void chpl_comm_barrier(const char *msg) { }
 
@@ -85,28 +147,28 @@ void chpl_comm_pre_task_exit(int all) { }
 void chpl_comm_exit(int all, int status) { }
 
 void  chpl_comm_put(void* addr, int32_t locale, void* raddr,
-                    int32_t size, int32_t typeIndex, int32_t len,
-                    int ln, chpl_string fn) {
+                    size_t size, int32_t typeIndex,
+                    int ln, int32_t fn) {
   assert(locale==0);
 
-  memmove(raddr, addr, size*len);
+  memmove(raddr, addr, size);
 }
 
 void  chpl_comm_get(void* addr, int32_t locale, void* raddr,
-                    int32_t size, int32_t typeIndex, int32_t len,
-                    int ln, chpl_string fn) {
+                    size_t size, int32_t typeIndex,
+                    int ln, int32_t fn) {
   assert(locale==0);
 
-  memmove(addr, raddr, size*len);
+  memmove(addr, raddr, size);
 }
 
-void  chpl_comm_put_strd(void* dstaddr_arg, void* dststrides, int32_t dstlocale,
-                         void* srcaddr_arg, void* srcstrides, void* count,
-                         int32_t stridelevels, int32_t elemSize, int32_t typeIndex,
-                         int ln, chpl_string fn)
+void  chpl_comm_put_strd(void* dstaddr_arg, size_t* dststrides, int32_t dstlocale,
+                         void* srcaddr_arg, size_t* srcstrides, size_t* count,
+                         int32_t stridelevels, size_t elemSize, int32_t typeIndex,
+                         int ln, int32_t fn)
 {
   const size_t strlvls = (size_t)stridelevels;
-  int i,j,k,l,m,t,total,off,x,carry;
+  size_t i,j,k,l,m,t,total,off,x,carry;
 
   int8_t* dstaddr,*dstaddr1,*dstaddr2,*dstaddr3;
   int8_t* srcaddr,*srcaddr1,*srcaddr2,*srcaddr3;
@@ -120,16 +182,16 @@ void  chpl_comm_put_strd(void* dstaddr_arg, void* dststrides, int32_t dstlocale,
   assert(dstlocale==0);
 
   //Only count[0] and strides are meassured in number of bytes.
-  cnt[0] = ((int32_t*)count)[0] * elemSize;
+  cnt[0] = count[0] * elemSize;
   if (strlvls>0) {
-    srcstr[0] = ((int32_t*)srcstrides)[0] * elemSize;
-    dststr[0] = ((int32_t*)dststrides)[0] * elemSize;
+    srcstr[0] = srcstrides[0] * elemSize;
+    dststr[0] = dststrides[0] * elemSize;
     for (i=1;i<strlvls;i++) {
-      srcstr[i] = ((int32_t*)srcstrides)[i] * elemSize;
-      dststr[i] = ((int32_t*)dststrides)[i] * elemSize;
-      cnt[i] = ((int32_t*)count)[i];
+      srcstr[i] = srcstrides[i] * elemSize;
+      dststr[i] = dststrides[i] * elemSize;
+      cnt[i] = count[i];
     }
-    cnt[strlvls] = ((int32_t*)count)[strlvls];
+    cnt[strlvls] = count[strlvls];
   }
 
   switch(strlvls) {
@@ -254,13 +316,13 @@ void  chpl_comm_put_strd(void* dstaddr_arg, void* dststrides, int32_t dstlocale,
   }
 }
 
-void  chpl_comm_get_strd(void* dstaddr_arg, void* dststrides, int32_t srclocale,
-                         void* srcaddr_arg, void* srcstrides, void* count,
-                         int32_t stridelevels, int32_t elemSize, int32_t typeIndex,
-                         int ln, chpl_string fn)
+void  chpl_comm_get_strd(void* dstaddr_arg, size_t* dststrides, int32_t srclocale,
+                         void* srcaddr_arg, size_t* srcstrides, size_t* count,
+                         int32_t stridelevels, size_t elemSize, int32_t typeIndex,
+                         int ln, int32_t fn)
 {
   const size_t strlvls = (size_t)stridelevels;
-  int i,j,k,l,m,t,total,off,x,carry;
+  size_t i,j,k,l,m,t,total,off,x,carry;
 
   int8_t* dstaddr,*dstaddr1,*dstaddr2,*dstaddr3;
   int8_t* srcaddr,*srcaddr1,*srcaddr2,*srcaddr3;
@@ -273,16 +335,16 @@ void  chpl_comm_get_strd(void* dstaddr_arg, void* dststrides, int32_t srclocale,
   assert(srclocale==0);
 
   //Only count[0] and strides are meassured in number of bytes.
-  cnt[0] = ((int32_t*)count)[0] * elemSize;
+  cnt[0] = count[0] * elemSize;
   if (strlvls>0) {
-    srcstr[0] = ((int32_t*)srcstrides)[0] * elemSize;
-    dststr[0] = ((int32_t*)dststrides)[0] * elemSize;
+    srcstr[0] = srcstrides[0] * elemSize;
+    dststr[0] = dststrides[0] * elemSize;
     for (i=1;i<strlvls;i++) {
-      srcstr[i] = ((int32_t*)srcstrides)[i] * elemSize;
-      dststr[i] = ((int32_t*)dststrides)[i] * elemSize;
-      cnt[i] = ((int32_t*)count)[i];
+      srcstr[i] = srcstrides[i] * elemSize;
+      dststr[i] = dststrides[i] * elemSize;
+      cnt[i] = count[i];
       }
-    cnt[strlvls] = ((int32_t*)count)[strlvls];
+    cnt[strlvls] = count[strlvls];
   }
 
   switch(strlvls) {
@@ -409,12 +471,12 @@ void  chpl_comm_get_strd(void* dstaddr_arg, void* dststrides, int32_t srclocale,
 
 typedef struct {
   chpl_fn_int_t fid;
-  int           arg_size;
+  size_t        arg_size;
   char          arg[0];       // variable-sized data here
 } fork_t;
 
-void chpl_comm_fork(c_nodeid_t node, c_sublocid_t subloc,
-                    chpl_fn_int_t fid, void *arg, int32_t arg_size) {
+void chpl_comm_execute_on(c_nodeid_t node, c_sublocid_t subloc,
+                    chpl_fn_int_t fid, void *arg, size_t arg_size) {
   assert(node==0);
 
   chpl_ftable_call(fid, arg);
@@ -428,27 +490,27 @@ static void fork_nb_wrapper(fork_t* f) {
   chpl_mem_free(f, 0, 0);
 }
 
-void chpl_comm_fork_nb(c_nodeid_t node, c_sublocid_t subloc,
-                       chpl_fn_int_t fid, void *arg, int32_t arg_size) {
+void chpl_comm_execute_on_nb(c_nodeid_t node, c_sublocid_t subloc,
+                       chpl_fn_int_t fid, void *arg, size_t arg_size) {
   fork_t *info;
-  int     info_size;
+  size_t  info_size;
 
   assert(node==0);
 
   info_size = sizeof(fork_t) + arg_size;
   info = (fork_t*)chpl_mem_allocMany(info_size, sizeof(char),
-                                     CHPL_RT_MD_COMM_FORK_SEND_NB_INFO, 0, 0);
+                                     CHPL_RT_MD_COMM_FRK_SND_INFO, 0, 0);
   info->fid = fid;
   info->arg_size = arg_size;
   if (arg_size)
-    memcpy(&(info->arg), arg, arg_size);
+    chpl_memcpy(&(info->arg), arg, arg_size);
   chpl_task_startMovedTask((chpl_fn_p)fork_nb_wrapper, (void*)info,
                            subloc, chpl_nullTaskID, false);
 }
 
-// Same as chpl_comm_fork()
-void chpl_comm_fork_fast(c_nodeid_t node, c_sublocid_t subloc,
-                         chpl_fn_int_t fid, void *arg, int32_t arg_size) {
+// Same as chpl_comm_execute_on()
+void chpl_comm_execute_on_fast(c_nodeid_t node, c_sublocid_t subloc,
+                         chpl_fn_int_t fid, void *arg, size_t arg_size) {
   assert(node==0);
 
   chpl_ftable_call(fid, arg);
@@ -456,6 +518,10 @@ void chpl_comm_fork_fast(c_nodeid_t node, c_sublocid_t subloc,
 
 int chpl_comm_numPollingTasks(void) { return 0; }
 
+void chpl_comm_make_progress(void)
+{
+}
+
 void chpl_startVerboseComm() { }
 void chpl_stopVerboseComm() { }
 void chpl_startVerboseCommHere() { }
@@ -467,13 +533,7 @@ void chpl_startCommDiagnosticsHere() { }
 void chpl_stopCommDiagnosticsHere() { }
 
 void chpl_resetCommDiagnosticsHere() { }
-void chpl_getCommDiagnosticsHere(chpl_commDiagnostics *cd) { }
+void chpl_getCommDiagnosticsHere(chpl_commDiagnostics *cd) {
+  memset(cd, 0, sizeof(chpl_commDiagnostics));
+}
 
-uint64_t chpl_numCommGets(void) { return 0; }
-uint64_t chpl_numCommNBGets(void) { return 0; }
-uint64_t chpl_numCommTestNBGets(void) { return 0; }
-uint64_t chpl_numCommWaitNBGets(void) { return 0; }
-uint64_t chpl_numCommPuts(void) { return 0; }
-uint64_t chpl_numCommForks(void) { return 0; }
-uint64_t chpl_numCommFastForks(void) { return 0; }
-uint64_t chpl_numCommNBForks(void) { return 0; }
