@@ -1,9 +1,24 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "chplrt.h"
+#include "chpl-linefile-support.h"
 
 #include "error.h"
 #include "chplexit.h"
@@ -20,7 +35,19 @@
 
 int verbosity = 1;
 
-void chpl_warning(const char* message, int32_t lineno, chpl_string filename) {
+void chpl_warning(const char* message, int32_t lineno, int32_t filenameIdx) {
+  const char* filename = NULL;
+  // squash warnings if --quiet flag is used
+  if (verbosity == 0) {
+    return;
+  }
+  if (filenameIdx != 0)
+    filename = chpl_lookupFilename(filenameIdx);
+  chpl_warning_explicit(message, lineno, filename);
+}
+
+void chpl_warning_explicit(const char *message, int32_t lineno,
+                           const char *filename) {
   // squash warnings if --quiet flag is used
   if (verbosity == 0) {
     return;
@@ -34,7 +61,6 @@ void chpl_warning(const char* message, int32_t lineno, chpl_string filename) {
     fprintf(stderr, "warning: %s\n", message);
 }
 
-
 #ifndef LAUNCHER
 static atomic_flag thisLocaleAlreadyExiting;
 void chpl_error_init(void) {
@@ -53,7 +79,8 @@ static void spinhaltIfAlreadyExiting(void) {
 #endif
 }
 
-static void chpl_error_common(const char* message, int32_t lineno, chpl_string filename) {
+void chpl_error_explicit(const char *message, int32_t lineno,
+                         const char *filename) {
   spinhaltIfAlreadyExiting();
   fflush(stdout);
   if (lineno > 0)
@@ -62,15 +89,17 @@ static void chpl_error_common(const char* message, int32_t lineno, chpl_string f
     fprintf(stderr, "%s: error: %s", filename, message);
   else
     fprintf(stderr, "error: %s", message);
-}
-
-
-void chpl_error(const char* message, int32_t lineno, chpl_string filename) {
-  chpl_error_common(message, lineno, filename);
   fprintf(stderr, "\n");
   chpl_exit_any(1);
 }
 
+void chpl_error(const char *message, int32_t lineno, int32_t filenameIdx) {
+  const char *filename = NULL;
+  if (filenameIdx != 0)
+    filename= chpl_lookupFilename(filenameIdx);
+  chpl_error_explicit(message, lineno, filename);
+}
+
 
 void chpl_internal_error(const char* message) {
   spinhaltIfAlreadyExiting();
