@@ -1,8 +1,26 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
+// get popen/pclose
+#ifndef _XOPEN_SOURCE
+#define _XOPEN_SOURCE 600
+#endif
 #include <libgen.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -15,31 +33,7 @@
 #include "chpl-mem.h"
 #include "chpltypes.h"
 #include "error.h"
-
-// FIX ME WHEN WE SORT OUT THIS BATCH SCHEDULER + LAUNCHER STUFF
-// THIS SHOULD GO INTO SOME HEADER FILE
-typedef enum {
-  aprun_cc,   // binding policy
-  aprun_n,    // cores per locale
-  aprun_d,    // num locales
-  aprun_N,    // locales per node
-  aprun_j,    // cpus per node (newer versions of aprun)
-  aprun_none
-} aprun_arg_t;
-
-void initAprunAttributes(void);
-const char* getCoresPerLocaleStr(void);
-int getCoresPerLocale(void);
-const char* getLocalesPerNodeStr(void);
-int getLocalesPerNode(void);
-const char* getCPUsPerCUStr(void);
-int getCPUsPerCU(void);
-const char* getNumLocalesStr(void);
-const char* getAprunArgStr(aprun_arg_t arg); // possibly inline
-int getAprunArg(aprun_arg_t argt);           // possibly inline
-//
-// FIX ME ABOVE
-//
+#include "../aprun/aprun-utils.h"
 
 #define baseExpectFilename ".chpl-expect-"
 #define EXPECT "expect"
@@ -137,7 +131,7 @@ static char* genQsubOptions(char* genFilename, char* projectString, qsubVersion
     fprintf(qsubScript, "#PBS -N Chpl-%.10s\n", genFilename);
   } else {
     optionString = chpl_mem_allocMany(maxOptLength, sizeof(char),
-                                      CHPL_RT_MD_COMMAND_BUFFER, -1, "");
+                                      CHPL_RT_MD_COMMAND_BUFFER, -1, 0);
     length += snprintf(optionString + length, maxOptLength - length,
                        "-z -V -I -N Chpl-%.10s", genFilename);
   }
@@ -167,7 +161,6 @@ static char* genQsubOptions(char* genFilename, char* projectString, qsubVersion
     }
   }
   switch (qsub) {
-  case pbspro:
   case unknown:
     if (generate_qsub_script) {
       fprintf(qsubScript, "#PBS -l mppwidth=%d\n", numLocales);
@@ -179,6 +172,19 @@ static char* genQsubOptions(char* genFilename, char* projectString, qsubVersion
                          numLocales, procsPerNode, numCoresPerLocale);
     }
     break;
+  case pbspro:
+    if (generate_qsub_script) {
+      // We always want to use scatter since we use one PE per node
+      fprintf(qsubScript, "#PBS -l place=scatter\n");
+      fprintf(qsubScript, "#PBS -l select=%d:ncpus=%d\n", numLocales, numCoresPerLocale);
+    } else {
+      // We always want to use scatter since we use one PE per node
+      length += snprintf(optionString + length, maxOptLength - length,
+                         " -l place=scatter");
+      length += snprintf(optionString + length, maxOptLength - length,
+                         " -l select=%d:ncpus=%d", numLocales, numCoresPerLocale);
+    }
+    break;
   case moab:
     if (generate_qsub_script) {
       fprintf(qsubScript, "#PBS -l nodes=%d\n", numLocales);
@@ -222,9 +228,7 @@ static char** chpl_launch_create_argv(int argc, char* argv[],
   FILE *outfile;
   pid_t mypid;
   int numCoresPerLocale;
-  int CPUsPerCU;
-  int LocalesPerNode;
-  const char *ccArg = _ccArg ? _ccArg : "none";
+  char **aprun_cmd;
 
   if (basenamePtr == NULL) {
       basenamePtr = argv[0];
@@ -243,8 +247,6 @@ static char** chpl_launch_create_argv(int argc, char* argv[],
 
   initAprunAttributes();
   numCoresPerLocale = getCoresPerLocale();
-  CPUsPerCU = getCPUsPerCU();
-  LocalesPerNode = getLocalesPerNode();
 
   qsubOptions = genQsubOptions(basenamePtr, projectString,
                                determineQsubVersion(),
@@ -311,14 +313,10 @@ static char** chpl_launch_create_argv(int argc, char* argv[],
     fprintf(expectFile, "send \"cd \\$PBS_O_WORKDIR\\n\"\n");
     fprintf(expectFile, "expect -re \"\\n$chpl_prompt\" {}\n");
     if (verbosity > 2) {
-      fprintf(expectFile, "send \"aprun -cc %s -q -b %s%d %s%d %s%d ", ccArg,
-              getCoresPerLocaleStr(), numCoresPerLocale,
-              getNumLocalesStr(), 1 /* only run on one locale */,
-              getLocalesPerNodeStr(), LocalesPerNode);
-      if (CPUsPerCU >= 0) {
-        fprintf(expectFile, "%s%d ", getCPUsPerCUStr(), CPUsPerCU);
-      }
-      fprintf(expectFile, "ls %s\\n\"\n", chpl_get_real_binary_name());
+      fprintf(expectFile, "send \"aprun -q %s%d ",
+              getNumLocalesStr(), 1 /* only run on one locale */);
+      fprintf(expectFile, "ls %s %s\\n\"\n",
+          chpl_get_real_binary_wrapper(), chpl_get_real_binary_name());
       fprintf(expectFile, "expect {\n");
       fprintf(expectFile, "  \"failed: chdir\" {send_user "
               "\"error: %s must be launched from and/or stored on a "
@@ -337,21 +335,17 @@ static char** chpl_launch_create_argv(int argc, char* argv[],
     outfile = expectFile;
   }
 
-  if (verbosity < 2) {
-    fprintf(outfile, "-q ");
-  }
-  fprintf(outfile, "-cc %s %s%d %s%d %s%d ", ccArg,
-          getCoresPerLocaleStr(), numCoresPerLocale,
-          getNumLocalesStr(), numLocales,
-          getLocalesPerNodeStr(), LocalesPerNode);
-  if (CPUsPerCU >= 0) {
-    fprintf(outfile, "%s%d ", getCPUsPerCUStr(), CPUsPerCU);
-  }
-  fprintf(outfile, "%s", chpl_get_real_binary_name());
-  for (i=1; i<argc; i++) {
-    fprintf(outfile, " '%s'", argv[i]);
+  aprun_cmd = chpl_create_aprun_cmd(argc, argv, numLocales, _ccArg);
+
+  for (i=1; ; i++) {
+    if (aprun_cmd[i] != NULL)
+      fprintf(outfile, "%s ", aprun_cmd[i]);
+    else
+      break;
   }
 
+  free(aprun_cmd);
+
   if (generate_qsub_script) {
     fprintf(qsubScript, "\n\n");
     fclose(qsubScript);
@@ -420,7 +414,7 @@ int chpl_launch(int argc, char* argv[], int32_t numLocales) {
 
 
 int chpl_launch_handle_arg(int argc, char* argv[], int argNum,
-                           int32_t lineno, chpl_string filename) {
+                           int32_t lineno, int32_t filename) {
   int numArgs = 0;
   if (!strcmp(argv[argNum], CHPL_WALLTIME_FLAG)) {
     walltime = argv[argNum+1];
