@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include <libgen.h>
 #include <stdio.h>
@@ -94,7 +108,7 @@ static int getNumCoresPerLocale(void) {
 static void genNumLocalesOptions(FILE* slurmFile, sbatchVersion sbatch, 
                                  int32_t numLocales,
                                  int32_t numCoresPerLocale) {
-  char* queue = getenv("CHPL_LAUNCHER_QUEUE");
+  //char* queue = getenv("CHPL_LAUNCHER_QUEUE");
   char* walltime = getenv("CHPL_LAUNCHER_WALLTIME");
   char* constraint = getenv("CHPL_LAUNCHER_CONSTRAINT");
 
@@ -128,6 +142,8 @@ static void genNumLocalesOptions(FILE* slurmFile, sbatchVersion sbatch,
   case torque:
     fprintf(slurmFile, "#SBATCH --nodes=%d\n", numLocales);
     break;
+  default:
+    break;
   }
 }
 
@@ -171,7 +187,7 @@ static char* chpl_launch_create_command(int argc, char* argv[],
     if (getenv("CHPL_LAUNCHER_USE_SBATCH") != NULL) {
 //    fprintf(slurmFile, "#SBATCH -joe\n");  
     if (outputfn!=NULL) 
-      fprintf(slurmFile, "#SBATCH -o %s.%%j.out\n", outputfn);
+      fprintf(slurmFile, "#SBATCH -o %s\n", outputfn);
     else
       fprintf(slurmFile, "#SBATCH -o %s.%%j.out\n", argv[0]);
 //    fprintf(slurmFile, "cd $SBATCH_O_WORKDIR\n");
@@ -198,7 +214,7 @@ static char* chpl_launch_create_command(int argc, char* argv[],
   fprintf(expectFile, "spawn -noecho salloc ");
   fprintf(expectFile, "-J %.10s ",basenamePtr); // pass 
   fprintf(expectFile, "-N %d ",numLocales); 
-  fprintf(expectFile, "--ntasks-per-node=1 ",numLocales); 
+  fprintf(expectFile, "--ntasks-per-node=1 ");
   fprintf(expectFile, "--exclusive "); //  give exclusive access to the nodes
   fprintf(expectFile, "--time=%s ",walltime); 
   if (constraint) {
@@ -229,7 +245,7 @@ static char* chpl_launch_create_command(int argc, char* argv[],
 
   size = strlen(baseCommand) + 1;
 
-  command = chpl_mem_allocMany(size, sizeof(char), CHPL_RT_MD_COMMAND_BUFFER, -1, "");
+  command = chpl_mem_allocMany(size, sizeof(char), CHPL_RT_MD_COMMAND_BUFFER, -1, 0);
   
   sprintf(command, "%s", baseCommand);
 
@@ -268,7 +284,7 @@ int chpl_launch(int argc, char* argv[], int32_t numLocales) {
 
 
 int chpl_launch_handle_arg(int argc, char* argv[], int argNum,
-                           int32_t lineno, chpl_string filename) {
+                           int32_t lineno, int32_t filename) {
   return 0;
 }
 
