@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include <stdio.h>
 #include <sys/stat.h>
@@ -180,26 +194,34 @@ chpl_run_utility1K(const char *command, char *const argv[], char *outbuf, int ou
 //
 char** chpl_bundle_exec_args(int argc, char *const argv[],
                               int largc, char *const largv[]) {
-  int len = argc+largc+1;
+  int len = argc+largc+2;
+  int newargc = 0;
   char **newargv = chpl_mem_allocMany(len, sizeof(char*),
-                                      CHPL_RT_MD_COMMAND_BUFFER, -1, "");
+                                      CHPL_RT_MD_COMMAND_BUFFER, -1, 0);
   if (!newargv) {
     chpl_internal_error("Could not allocate memory");
   }
 
   newargv[len-1] = NULL;
+  newargv[len-2] = NULL;
 
+  // add any launcher args
   if (largc > 0) {
-    // launcher args
     memcpy(newargv, largv, largc*sizeof(char *));
+    newargc = largc;
   }
   if (argc > 0) {
-    // binary
+    // if there is a wrapper, add it after the launcher args
+    if (strcmp(chpl_get_real_binary_wrapper(), "") != 0) {
+      newargv[newargc++] = (char *) chpl_get_real_binary_wrapper();
+    }
+
+    // add the _real binary (after launchers args or wrapper)
     chpl_compute_real_binary_name(argv[0]);
-    newargv[largc] = (char *) chpl_get_real_binary_name();
+    newargv[newargc++] = (char *) chpl_get_real_binary_name();
     if (argc > 1) {
-      // other args (skip binary name)
-      memcpy(newargv+largc+1, argv+1, (argc-1)*sizeof(char *));
+      // add args passed to main (skip original binary) after _real binary
+      memcpy(newargv+newargc, argv+1, (argc-1)*sizeof(char *));
     }
   }
 
@@ -302,14 +324,14 @@ char* chpl_get_enviro_keys(char sep)
       }
     }
     if( pass == 0 ) ret = chpl_mem_allocMany(k+1, sizeof(char),
-                                             CHPL_RT_MD_COMMAND_BUFFER,-1,"");
+                                             CHPL_RT_MD_COMMAND_BUFFER,-1,0);
   }
 
   return ret;
 }
 
 int handleNonstandardArg(int* argc, char* argv[], int argNum, 
-                         int32_t lineno, chpl_string filename) {
+                         int32_t lineno, int32_t filename) {
   int numHandled = chpl_launch_handle_arg(*argc, argv, argNum, 
                                           lineno, filename);
   if (numHandled == 0) {
@@ -351,7 +373,7 @@ void chpl_compute_real_binary_name(const char* argv0) {
   int exe_length = strlen(launcher_exe_suffix);
   int length;
   const char* real_suffix = getenv("CHPL_LAUNCHER_SUFFIX");
-  
+
   if (NULL == real_suffix) {
     real_suffix = launcher_real_suffix;
   }
@@ -372,6 +394,21 @@ void chpl_compute_real_binary_name(const char* argv0) {
   strcpy(cursor, launcher_real_suffix);
 }
 
+// Undocumented means to wrap the _real binary. Useful for things like timing
+// the _real executable or running it through some other program. Note that
+// this will not work with launchers that add arguments after the first
+// positional argument expecting it to be the binary. e.g. amudprun adds
+// __AMUDP_SLAVE_PROCESS__* and the hostname:port. Returns the value of
+// CHPL_LAUNCHER_REAL_WRAPPER if set, "" otherwise
+const char* chpl_get_real_binary_wrapper(void) {
+  const char* real_wrapper = getenv("CHPL_LAUNCHER_REAL_WRAPPER");
+  if (real_wrapper != NULL) {
+    return real_wrapper;
+  } else {
+    return "";
+  }
+}
+
 const char* chpl_get_real_binary_name(void) {
   return &chpl_real_binary_name[0];
 }
@@ -385,13 +422,13 @@ int main(int argc, char* argv[]) {
 
   // Set up main argument parsing.
   chpl_gen_main_arg.argv = chpl_mem_allocMany(argc, sizeof(char*),
-                                      CHPL_RT_MD_COMMAND_BUFFER, -1, "");
+                                      CHPL_RT_MD_COMMAND_BUFFER, -1, 0);
   chpl_gen_main_arg.argv[0] = argv[0];
   chpl_gen_main_arg.argc = 1;
   chpl_gen_main_arg.return_value = 0;
 
   CreateConfigVarTable();
-  parseArgs(&argc, argv);
+  parseArgs(true, parse_normally, &argc, argv);
 
   execNumLocales = getArgNumLocales();
 
