@@ -1,7 +1,21 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 // squelch warning on Mac OS X
 #ifdef _POSIX_C_SOURCE
@@ -9,7 +23,7 @@
 #endif
 #define _POSIX_C_SOURCE 20112L
 
-#ifndef SIMPLE_TEST
+#ifndef CHPL_RT_UNIT_TEST
 #include "chplrt.h"
 #endif
 
@@ -29,6 +43,22 @@
 #include <string.h>
 #include <limits.h>
 
+#if defined(__APPLE__)
+#include <sys/param.h>
+#include <sys/mount.h>
+#define SYS_HAS_STATFS 1
+#elif defined(__linux__) || defined(__CYGWIN__)
+#include <sys/vfs.h>
+#define SYS_HAS_STATFS 1
+#else
+#define SYS_HAS_STATFS 0
+#endif
+
+#if defined(SYS_HAS_LLAPI)
+#include <lustre/lustreapi.h>
+#include <sys/ioctl.h>
+#endif
+
 // preadv/pwritev are available
 // only on linux/glibc 2.10 or later
 #ifdef __GLIBC__
@@ -41,6 +71,28 @@
 #endif
 #endif
 
+#ifdef __CYGWIN__
+#undef HAS_PREADV
+#undef HAS_PWRITEV
+
+// PAGE_SIZE is declared in both Cygwin headers and Windows ones.
+// We don't really care who wins but want to get rid of the warning.
+#undef PAGE_SIZE
+
+
+#include <io.h> // _get_osfhandle
+//#include <ntddk.h>
+//#include <winternl.h>
+//#include <ntifs.h>
+#include <windows.h>
+#include <sys/cygwin.h> // for cygwin_internal
+
+#define REPLACE_CYGWIN_PREADWRITE 1
+
+#endif
+
+
+
 // Should be available in sys_xsi_strerror_r.c
 extern int sys_xsi_strerror_r(int errnum, char* buf, size_t buflen);
 
@@ -267,6 +319,12 @@ err_t sys_posix_madvise(void* addr, size_t len, int advice)
 
 
 
+// Some systems use "No error" and some use "Success"
+// if you perror(0); others print an error. So
+// using "No error" is consistent with some systems
+// and makes the most sense to us.
+static const char* error_string_no_error = "No error";
+
 static
 const char* extended_errors[] = {
   "end of file",
@@ -294,9 +352,11 @@ err_t sys_strerror_internal(err_t error, char** string_out, size_t extra_space)
 
   err_out = 0;
 
-  if( EXTEND_ERROR_OFFSET <= error
-                          && error < EXTEND_ERROR_OFFSET+EXTEND_ERROR_NUM) {
-    errmsg = extended_errors[error - EXTEND_ERROR_OFFSET];
+  if( error == 0 ||
+      (EXTEND_ERROR_OFFSET <= error
+                           && error < EXTEND_ERROR_OFFSET+EXTEND_ERROR_NUM) ) {
+    if( error == 0 ) errmsg = error_string_no_error;
+    else errmsg = extended_errors[error - EXTEND_ERROR_OFFSET];
     buf_sz = strlen(errmsg) + 1;
     buf = (char*) qio_malloc(buf_sz + extra_space);
     if( ! buf ) return ENOMEM;
@@ -373,7 +433,7 @@ const char* sys_strerror_syserr_str(qioerr error, err_t* err_in_strerror)
     start = strlen(ret);
     ret[start] = ':';
     ret[start+1] = ' ';
-    memcpy(&ret[start+2], msg, msg_len);
+    qio_memcpy(&ret[start+2], msg, msg_len);
     ret[start+2+msg_len] = '\0';
   }
   return ret;
@@ -529,6 +589,99 @@ err_t sys_lstat(const char* path, struct stat* buf)
   return err_out;
 }
 
+#ifdef SYS_HAS_LLAPI
+err_t sys_lustre_get_stripe_size(fd_t fd, int64_t* size_out) 
+{
+  struct lov_user_md_v1 *lum;
+  size_t lum_size = sizeof(*lum) + LOV_MAX_STRIPE_COUNT * sizeof(struct lov_user_ost_data_v1);
+  int rc = 0;
+  err_t err = 0;
+
+  lum = qio_calloc(lum_size, 1);
+
+  lum->lmm_magic = LOV_USER_MAGIC_V1;
+  lum->lmm_stripe_count = LOV_MAX_STRIPE_COUNT;
+
+  STARTING_SLOW_SYSCALL;
+  rc = ioctl(fd, LL_IOC_LOV_GETSTRIPE, lum);
+  *size_out = lum->lmm_stripe_size;
+
+  if (rc < 0)  {
+    *size_out = 0;
+    err = errno;
+  }
+  DONE_SLOW_SYSCALL;
+
+  qio_free(lum);
+
+  return err;
+}
+#endif
+
+// TAKZ - on Mac, the types in the statfs structure become signed or unsigned
+// based upon whether or not they have 64 bit inodes. This leads to some rather
+// messy error handling and checking, since we want to avoid overflow in the
+// case that we have either signed types (in the case of 32 bit inodes) or
+// large (64 bit) unsigned types (in the case of 64 bit inodes). We therefore
+// use the linux convention for statfs that fields that are undefined for a
+// given file system are set to 0 (and we handle this in safe_inode_cast).
+#define safe_inode_cast(t) (t < 0 ? 0 : (uint64_t)t)
+
+err_t sys_fstatfs(fd_t fd, sys_statfs_t* buf)
+{
+    err_t err_out;
+    int got;
+
+#if SYS_HAS_STATFS
+    struct statfs tmp;
+    got = fstatfs(fd, &tmp);
+
+#if defined(__APPLE__)
+    buf->f_bsize   = (int64_t)tmp.f_iosize;
+    buf->f_type    = safe_inode_cast(tmp.f_type);
+    buf->f_blocks  = safe_inode_cast(tmp.f_blocks);
+    buf->f_bfree   = safe_inode_cast(tmp.f_bfree);
+    buf->f_bavail  = safe_inode_cast(tmp.f_bavail);
+    buf->f_files   = safe_inode_cast(tmp.f_files);
+    buf->f_ffree   = safe_inode_cast(tmp.f_ffree);
+    buf->f_namelen = safe_inode_cast(MNAMELEN);
+#else // linux or cygwin
+    // We don't have to deal with possible conversion from signed to unsiged
+    // numbers here, since in linux the field will be set to 0 if it is
+    // undefined for the FS. Since we know the field is >= 0 we can get rid of
+    // all the branching logic that we had for apple
+    buf->f_bsize   = (int64_t)tmp.f_bsize;
+    buf->f_type    = (uint64_t)tmp.f_type;
+    buf->f_blocks  = (uint64_t)tmp.f_blocks;
+    buf->f_bfree   = (uint64_t)tmp.f_bfree;
+    buf->f_bavail  = (uint64_t)tmp.f_bavail;
+    buf->f_files   = (uint64_t)tmp.f_files;
+    buf->f_ffree   = (uint64_t)tmp.f_ffree;
+    buf->f_namelen = (uint64_t)tmp.f_namelen;
+#endif
+
+#else
+    // unable to get fstatfs, so set all the fields of the struct to be 0 to say
+    // that we were unable to get any information
+    buf->f_bsize   = 0;
+    buf->f_type    = 0;
+    buf->f_blocks  = 0;
+    buf->f_bfree   = 0;
+    buf->f_bavail  = 0;
+    buf->f_files   = 0;
+    buf->f_ffree   = 0;
+    buf->f_namelen = 0;
+    got = ENOSYS;
+#endif
+    if (got != -1) {
+        err_out = 0;
+    } else{
+        err_out = errno;
+    }
+
+    return err_out;
+}
+
 err_t sys_mkstemp(char* template_, fd_t* fd_out)
 {
   int got;
@@ -629,7 +782,7 @@ err_t sys_read(int fd, void* buf, size_t count, ssize_t* num_read_out)
 
   STARTING_SLOW_SYSCALL;
   got = read(fd, buf, count);
-  if( got != -1 ) {
+  if( got >= 0 ) {
     *num_read_out = got;
     if( got == 0 && count != 0 ) err_out = EEOF;
     else err_out = 0;
@@ -649,7 +802,7 @@ err_t sys_write(int fd, const void* buf, size_t count, ssize_t* num_written_out)
 
   STARTING_SLOW_SYSCALL;
   got = write(fd, buf, count);
-  if( got != -1 ) {
+  if( got >= 0 ) {
     *num_written_out = got;
     err_out = 0;
   } else {
@@ -661,23 +814,130 @@ err_t sys_write(int fd, const void* buf, size_t count, ssize_t* num_written_out)
   return err_out;
 }
 
+
+#ifdef __CYGWIN__
+static
+err_t get_errcode_from_winerr(DWORD win_error)
+{
+  uintptr_t res = cygwin_internal(CW_GET_ERRNO_FROM_WINERROR,
+                                  win_error,
+                                  EACCES);
+  return res;
+}
+#endif
+
+static inline
+err_t do_pread(int fd, void* buf, size_t count, off_t offset, ssize_t *num_read)
+{
+#ifdef REPLACE_CYGWIN_PREADWRITE
+  ssize_t got;
+  err_t error;
+  HANDLE handle = (HANDLE) _get_osfhandle(fd);
+  DWORD win_to_read;
+  DWORD win_num_read;
+  OVERLAPPED overlapped;
+  BOOL win_did_read;
+  DWORD win_error;
+
+  win_to_read = count;
+  win_num_read = 0;
+  memset(&overlapped, 0, sizeof(OVERLAPPED));
+  overlapped.Offset = offset;
+  overlapped.OffsetHigh = offset >> (8*sizeof(DWORD));
+
+  win_did_read = ReadFile(handle, buf, win_to_read, &win_num_read, &overlapped);
+  if( win_did_read ) {
+    got = win_num_read;
+    error = 0;
+  } else {
+    got = -1;
+    win_error = GetLastError();
+    // Cygwin turns ERROR_HANDLE_EOF into ENODATA
+    // but if it stopped doing that we could check for it here:
+    //if( win_error == ERROR_HANDLE_EOF ) error = ENODATA; else
+    error = get_errcode_from_winerr(win_error);
+  }
+
+  if( got == -1 && error == ENODATA ) {
+    // this is how cygwin reports EOF
+    got = 0;
+    error = 0;
+  }
+  *num_read = got;
+  assert(got < 0 || (size_t) got <= count); // can't read more than requested!
+  return error;
+#else
+  ssize_t got;
+  got = pread(fd, buf, count, offset);
+  if( got < 0 ) {
+    *num_read = 0;
+    return errno;
+  }
+  assert(got <= count); // can't read more than requested!
+  *num_read = got;
+  return 0;
+#endif
+}
+
+static inline
+err_t do_pwrite(int fd, const void* buf, size_t count, off_t offset, ssize_t *num_written)
+{
+#ifdef REPLACE_CYGWIN_PREADWRITE
+  ssize_t got;
+  err_t error;
+  HANDLE handle = (HANDLE) _get_osfhandle(fd);
+  DWORD win_to_write;
+  DWORD win_num_wrote;
+  OVERLAPPED overlapped;
+  BOOL win_did_write;
+  DWORD win_error;
+
+  win_to_write = count;
+  win_num_wrote = 0;
+  memset(&overlapped, 0, sizeof(OVERLAPPED));
+  overlapped.Offset = offset;
+  overlapped.OffsetHigh = offset >> (8*sizeof(DWORD));
+
+  win_did_write = WriteFile(handle, buf, win_to_write, &win_num_wrote, &overlapped);
+  if( win_did_write ) {
+    got = win_num_wrote;
+    error = 0;
+  } else {
+    got = -1;
+    win_error = GetLastError();
+    error = get_errcode_from_winerr(win_error);
+  }
+
+  assert(got < 0 || (size_t) got <= count); // can't read more than requested!
+  *num_written = got;
+  return error;
+#else
+  ssize_t got;
+  got = pwrite(fd, buf, count, offset);
+  if( got < 0 ) {
+    *num_written = 0;
+    return errno;
+  }
+  assert(got <= count); // can't write more than requested!
+  *num_written = got;
+  return 0;
+#endif
+}
+
 err_t sys_pread(int fd, void* buf, size_t count, off_t offset, ssize_t* num_read_out)
 {
   ssize_t got;
   err_t err_out;
 
   STARTING_SLOW_SYSCALL;
-  got = pread(fd, buf, count, offset);
-  #ifdef __CYGWIN__
-  if( got == -1 && errno == ENODATA ) got = 0;
-  #endif
+  got = 0;
+  err_out = do_pread(fd, buf, count, offset, &got);
   if( got != -1 ) {
     *num_read_out = got;
     if( got == 0 && count != 0 ) err_out = EEOF;
     else err_out = 0;
   } else {
     *num_read_out = 0;
-    err_out = errno;
   }
   DONE_SLOW_SYSCALL;
 
@@ -690,13 +950,13 @@ err_t sys_pwrite(int fd, const void* buf, size_t count, off_t offset, ssize_t* n
   err_t err_out;
 
   STARTING_SLOW_SYSCALL;
-  got = pwrite(fd, buf, count, offset);
+  got = 0;
+  err_out = do_pwrite(fd, buf, count, offset, &got);
   if( got != -1 ) {
     *num_written_out = got;
     err_out = 0;
   } else {
     *num_written_out = 0;
-    err_out = errno;
   }
   DONE_SLOW_SYSCALL;
 
@@ -842,17 +1102,14 @@ err_t sys_preadv(fd_t fd, const struct iovec* iov, int iovcnt, off_t seek_to_off
   err_out = 0;
   got_total = 0;
   for( i = 0; i < iovcnt; i++ ) {
-    got = pread(fd, iov[i].iov_base, iov[i].iov_len, seek_to_offset + got_total);
-    #ifdef __CYGWIN__
-    if( got == -1 && errno == ENODATA ) got = 0;
-    #endif
-    if( got != -1 ) {
+    got = 0;
+    err_out = do_pread(fd, iov[i].iov_base, iov[i].iov_len, seek_to_offset + got_total, &got);
+    if( got >= 0 ) {
       got_total += got;
     } else {
-      err_out = errno;
       break;
     }
-    if( got != iov[i].iov_len ) {
+    if( (size_t) got != iov[i].iov_len ) {
       break;
     }
   }
@@ -918,14 +1175,14 @@ err_t sys_pwritev(fd_t fd, const struct iovec* iov, int iovcnt, off_t seek_to_of
   err_out = 0;
   got_total = 0;
   for( i = 0; i < iovcnt; i++ ) {
-    got = pwrite(fd, iov[i].iov_base, iov[i].iov_len, seek_to_offset + got_total);
-    if( got != -1 ) {
+    got = 0;
+    err_out = do_pwrite(fd, iov[i].iov_base, iov[i].iov_len, seek_to_offset + got_total, &got);
+    if( got >= 0 ) {
       got_total += got;
     } else {
-      err_out = errno;
       break;
     }
-    if( got != iov[i].iov_len ) {
+    if( (size_t) got != iov[i].iov_len ) {
       break;
     }
   }
@@ -1079,7 +1336,7 @@ err_t sys_accept(fd_t sockfd, sys_sockaddr_t* addr_out, fd_t* fd_out)
 
   got = accept(sockfd, (struct sockaddr*) & addr_out->addr, &addr_len);
   if( got != -1 ) {
-    if( addr_len > sizeof(sys_sockaddr_storage_t) ) {
+    if( addr_len > (socklen_t) sizeof(sys_sockaddr_storage_t) ) {
       fprintf(stderr, "Warning: address truncated in sys_accept\n");
     }
     addr_out->len = addr_len;
@@ -1173,7 +1430,7 @@ int sys_getaddrinfo_socktype(sys_addrinfo_ptr_t a) {return a->ai_socktype;}
 int sys_getaddrinfo_protocol(sys_addrinfo_ptr_t a) {return a->ai_protocol;}
 sys_sockaddr_t sys_getaddrinfo_addr(sys_addrinfo_ptr_t a) {
   sys_sockaddr_t ret;
-  memcpy(&ret.addr, a->ai_addr, a->ai_addrlen);
+  qio_memcpy(&ret.addr, a->ai_addr, a->ai_addrlen);
   ret.len = a->ai_addrlen;
   return ret;
 }
@@ -1510,32 +1767,41 @@ err_t sys_unlink(const char* path)
   return err_out;
 }
 
+// This routine returns a malloc'd string through its path_out pointer.
+// The caller is responsible for freeing that memory.
 err_t sys_getcwd(const char** path_out)
 {
-  int sz = 128;
-  char* buf;
-  char* got;
-  err_t err = 0;
+  int   sz  = 128;
+  char* buf = (char*) qio_malloc(sz);
+  err_t err = (buf == 0) ? ENOMEM : 0;
 
-  buf = (char*) qio_malloc(sz);
-  if( !buf ) return ENOMEM;
-  while( 1 ) {
-    got = getcwd(buf, sz);
-    if( got != NULL ) break;
-    else if( errno == ERANGE ) {
-      // keep looping but with bigger buffer.
-      sz = 2*sz;
-      got = (char*) qio_realloc(buf, sz);
-      if( ! got ) {
+  // getcwd() returns 0 if the provided buffer is too small
+  // If this happens, grow the buffer and try again
+  while (err == 0 && getcwd(buf, sz) == 0) {
+    if (errno == ERANGE) {
+      int   newSz  = 2 * sz;
+      char* newBuf = (char*) qio_realloc(buf, newSz);
+
+      if (newBuf == 0) {
         qio_free(buf);
-        return ENOMEM;
+        err = ENOMEM;
+      } else {
+        sz  = newSz;
+        buf = newBuf;
       }
+
     } else {
-      // Other error, stop.
       err = errno;
     }
   }
 
+  if (err != 0) {
+    qio_free(buf);
+    sz  = 0;
+    buf = 0;
+  }
+
   *path_out = buf;
+
   return err;
 }
