@@ -1,21 +1,31 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #define _GNU_SOURCE
 
-#ifdef __OPTIMIZE__
-// Turn assert() into a no op if the C compiler defines the macro above.
-#define NDEBUG
-#endif
-
 #include "chplrt.h"
 #include "chplcast.h"
 #include "chplcgfns.h" // for chpl_ftable
 #include "chpl-comm.h"
 #include "chpl-locale-model.h"
 #include "chpl-mem.h"
+#include "chplsys.h"
 #include "chpl-tasks.h"
 #include "error.h"
 #include <assert.h>
@@ -30,6 +40,7 @@
 #include <sys/syscall.h>
 #include <sys/types.h>
 #include <stdio.h>
+#include <math.h>
 
 #include "myth.h"
 
@@ -42,8 +53,8 @@
 
 typedef struct {
   c_sublocid_t requestedSubloc;  // requested sublocal for task
-  chpl_bool    serial_state;     // true: serialize execution
-}task_private_data_t;
+  chpl_task_prvData_t prvdata;
+} chpl_task_prvDataImpl_t;
 
 typedef struct {
   // A task which acquires pthread_mutex_lock must not migrate to
@@ -55,17 +66,17 @@ static int tasking_layer_active = 0;
 static int worker_in_cs_beforeinit = 0;
 static thread_local_data* s_tld;
 
-static const task_private_data_t s_def_chpl_data=
-             { c_sublocid_any_val, false };
+static const chpl_task_prvDataImpl_t s_def_chpl_data=
+             { .requestedSubloc = c_sublocid_any_val };
 
 // task-private data of main task
-static task_private_data_t s_main_chpl_data =
-             { c_sublocid_any_val, false };
+static chpl_task_prvDataImpl_t s_main_chpl_data =
+             { .requestedSubloc = c_sublocid_any_val };
 
-static inline task_private_data_t* getTaskPrivateData(void) {
+static inline chpl_task_prvDataImpl_t* getTaskPrivateData(void) {
   if (tasking_layer_active){
-    if (myth_wsapi_get_hint_size(myth_self())==sizeof(task_private_data_t)){
-      return (task_private_data_t*)myth_wsapi_get_hint_ptr(myth_self());
+    if (myth_wsapi_get_hint_size(myth_self())==sizeof(chpl_task_prvDataImpl_t)){
+      return (chpl_task_prvDataImpl_t*)myth_wsapi_get_hint_ptr(myth_self());
     }
   }
   return &s_main_chpl_data;
@@ -193,7 +204,7 @@ void chpl_sync_unlock(chpl_sync_aux_t *s) {
 }
 
 void chpl_sync_waitFullAndLock(chpl_sync_aux_t *s, int32_t lineno,
-    chpl_string filename) {
+    c_string filename) {
   assert(!is_worker_in_cs());
   {
     //wait until F/E bit is empty, and acquire lock
@@ -202,7 +213,7 @@ void chpl_sync_waitFullAndLock(chpl_sync_aux_t *s, int32_t lineno,
 }
 
 void chpl_sync_waitEmptyAndLock(chpl_sync_aux_t *s, int32_t lineno,
-    chpl_string filename) {
+    c_string filename) {
   assert(!is_worker_in_cs());
   {
     myth_felock_wait_lock(s->lock, 0);
@@ -269,7 +280,7 @@ typedef struct {
   chpl_fn_p fp;
   void* arg;
   chpl_bool count_running;
-  task_private_data_t chpl_data;
+  chpl_task_prvDataImpl_t chpl_data;
 } moved_task_wrapper_desc_t;
 
 static void* moved_task_wrapper(void* a) {
@@ -292,7 +303,7 @@ void chpl_task_init(void) {
   char *env;
   int i;
   void *chpl_data_ptr=&s_main_chpl_data;
-  size_t chpl_data_size=sizeof(task_private_data_t);
+  size_t chpl_data_size=sizeof(chpl_task_prvDataImpl_t);
 
   //
   // This threading layer does not have any inherent limit on the number
@@ -312,16 +323,17 @@ void chpl_task_init(void) {
     if ((lim = atoi(env)) > 0)
       numThreadsPerLocale = lim;
   }
-  // override by --numThreadsPerLocale
-  if ((lim=chpl_task_getenvNumThreadsPerLocale()) > 0)
-    numThreadsPerLocale = lim;
+  // override by CHPL_RT_NUM_THREADS_PER_LOCALE
+  if ((lim=chpl_task_getenvNumThreadsPerLocale())>0)
+    numThreadsPerLocale=lim;
   // limit by comm layer limit
-  if ((lim = chpl_comm_getMaxThreads()) > 0){
-    numThreadsPerLocale = (numThreadsPerLocale > lim)?lim:numThreadsPerLocale;
+  if ((lim=chpl_comm_getMaxThreads())>0){
+    numThreadsPerLocale=(numThreadsPerLocale > lim)?lim:numThreadsPerLocale;
   }
 
   s_num_workers = numThreadsPerLocale;
-  s_stack_size = chpl_task_getMinCallStackSize();
+  if ((s_stack_size=chpl_task_getEnvCallStackSize())==0)
+    s_stack_size=chpl_task_getDefaultCallStackSize();
   assert(s_stack_size > 0);
   assert(!is_worker_in_cs());
   s_tld = chpl_mem_allocMany(numThreadsPerLocale + numCommTasks,
@@ -343,7 +355,7 @@ int chpl_task_createCommTask(chpl_fn_p fn, void* arg) {
   //Since return value is always ignored, this cast is legal unless the definition is changed.
   opt.stack_size = 0;
   opt.switch_immediately = 0;
-  opt.custom_data_size = sizeof(task_private_data_t);
+  opt.custom_data_size = sizeof(chpl_task_prvDataImpl_t);
   opt.custom_data = (void*)&s_def_chpl_data;
   th = myth_create_ex((void*(*)(void*)) fn, arg, &opt);
   assert(th);
@@ -373,12 +385,12 @@ void chpl_task_stdModulesInitialized(void) {
 }
 
 void chpl_task_addToTaskList(chpl_fn_int_t fid, void* arg, c_sublocid_t subLoc,
-    chpl_task_list_p *task_list, int32_t task_list_locale,
-    chpl_bool is_begin_stmt, int lineno, chpl_string filename) {
+    void **task_list, int32_t task_list_locale,
+    chpl_bool is_begin_stmt, int lineno, c_string filename) {
   //Create a new task directly
   myth_thread_option opt;
   myth_thread_t th;
-  chpl_bool serial_state = getTaskPrivateData()->serial_state;
+  chpl_bool serial_state = getTaskPrivateData()->prvdata.serial_state;
 
   assert(subLoc == 0 || subLoc == c_sublocid_any);
 
@@ -391,22 +403,14 @@ void chpl_task_addToTaskList(chpl_fn_int_t fid, void* arg, c_sublocid_t subLoc,
   //So this cast is OK unless the definition is changed.
   opt.stack_size = 0;
   opt.switch_immediately = (is_worker_in_cs())?0:1;
-  opt.custom_data_size = sizeof(task_private_data_t);
+  opt.custom_data_size = sizeof(chpl_task_prvDataImpl_t);
   opt.custom_data = getTaskPrivateData();
   th = myth_create_ex((void*(*)(void*)) chpl_ftable[fid], arg, &opt);
   assert(th);
   myth_detach(th);
 }
 
-void chpl_task_processTaskList(chpl_task_list_p task_list) {
-  //Nothing to do because chpl_task_list is actually not used.
-}
-
-void chpl_task_executeTasksInList(chpl_task_list_p task_list) {
-  //Nothing to do because chpl_task_list is actually not used.
-}
-
-void chpl_task_freeTaskList(chpl_task_list_p task_list) {
+void chpl_task_executeTasksInList(void** task_list) {
   //Nothing to do because chpl_task_list is actually not used.
 }
 
@@ -419,6 +423,9 @@ void chpl_task_startMovedTask(chpl_fn_p fp,
   myth_thread_t th;
   myth_thread_option opt;
   moved_task_wrapper_desc_t* pmtwd;
+  chpl_task_prvDataImpl_t private = {
+    .requestedSubloc = subloc,
+    .prvdata = { .serial_state = serial_state } };
 
   assert(subloc == 0 || subloc == c_sublocid_any);
   assert(id == chpl_nullTaskID);
@@ -426,12 +433,13 @@ void chpl_task_startMovedTask(chpl_fn_p fp,
   pmtwd = (moved_task_wrapper_desc_t*) chpl_mem_alloc(sizeof(*pmtwd), 0, 0, 0);
   *pmtwd = (moved_task_wrapper_desc_t)
            { fp, a, canCountRunningTasks,
-             (task_private_data_t) { subloc, serial_state } };
+             private };
+
 
   // Do the same as chpl_task_addToTaskList, except wrap the function
   opt.stack_size = 0;
   opt.switch_immediately = (is_worker_in_cs())?0:1;
-  opt.custom_data_size = sizeof(task_private_data_t);
+  opt.custom_data_size = sizeof(chpl_task_prvDataImpl_t);
   opt.custom_data = (void*)&pmtwd->chpl_data;
   th = myth_create_ex(moved_task_wrapper, pmtwd, &opt);
   assert(th);
@@ -461,29 +469,49 @@ void chpl_task_yield(void) {
   myth_yield(1);
 }
 
-void chpl_task_sleep(int secs) {
+
+void chpl_task_sleep(double secs) {
   //sleep specified seconds
-  sleep(secs);
+  struct timespec delay;
+  delay.tv_sec = (time_t)(secs);
+  delay.tv_nsec = (long)(1e9*(secs - floor(secs)));
+  nanosleep(&delay, NULL);
 }
 
 chpl_bool chpl_task_getSerial(void)
 {
   //get dynamic serial state
-  return getTaskPrivateData()->serial_state;
+  return getTaskPrivateData()->prvdata.serial_state;
 }
 
 void chpl_task_setSerial(chpl_bool new_state) {
   //set dynamic serial state
-  getTaskPrivateData()->serial_state = new_state;
+  getTaskPrivateData()->prvdata.serial_state = new_state;
+}
+
+uint32_t chpl_task_getMaxPar(void) {
+  uint32_t max;
+
+  //
+  // We expect that even if the physical CPU have multiple hardware
+  // threads, cache and pipeline conflicts will typically prevent
+  // applications from gaining by using them.  So, we just return the
+  // lesser of the number of physical CPUs and the number of workers
+  // we have.
+  //
+  max = (uint32_t) chpl_getNumPhysicalCpus(true);
+  if ((uint32_t) s_num_workers < max)
+    max = (uint32_t) s_num_workers;
+  return max;
 }
 
 c_sublocid_t chpl_task_getNumSublocales(void)
 {
-#ifdef CHPL_LOCALE_MODEL_NUM_SUBLOCALES
-  return CHPL_LOCALE_MODEL_NUM_SUBLOCALES;
-#else
   return 0;
-#endif
+}
+
+chpl_task_prvData_t* chpl_task_getPrvData(void) {
+  return & getTaskPrivateData()->prvdata;
 }
 
 size_t chpl_task_getCallStackSize(void) {
