@@ -1,17 +1,26 @@
-/**************************************************************************
-  Copyright (c) 2004-2014, Cray Inc.  (See LICENSE file for more details)
-**************************************************************************/
-
+/*
+ * Copyright 2004-2016 Cray Inc.
+ * Other additional copyright holders may be indicated within.
+ * 
+ * The entirety of this work is licensed under the Apache License,
+ * Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License.
+ * 
+ * You may obtain a copy of the License at
+ * 
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 //
 // Pthread implementation of Chapel threading interface
 //
 
-#ifdef __OPTIMIZE__
-// Turn assert() into a no op if the C compiler defines the macro above.
-#define NDEBUG
-#endif
-
 #include "chplrt.h"
 #include "chpl-comm.h"
 #include "chpl-mem.h"
@@ -101,6 +110,11 @@ void chpl_thread_mutexUnlock(chpl_thread_mutex_p mutex) {
     chpl_internal_error("pthread_mutex_unlock() failed");
 }
 
+void chpl_thread_mutexDestroy(chpl_thread_mutex_p mutex) {
+  int result = pthread_mutex_destroy((pthread_mutex_t*) mutex);
+  if (result)
+    chpl_internal_error("pthread_mutex_destroy() failed");
+}
 
 // Thread management
 
@@ -122,6 +136,17 @@ void chpl_thread_yield(void) {
   //
   sched_yield();
 
+  //
+  // We expect sched_yield to yield the processor and move the thread to the
+  // end of the scheduling queue. However, on cygwin it seems there is no
+  // guarantee that the same thread won't be immediately rescheduled even if
+  // there are others threads waiting to run. Sleeping should yield the rest of
+  // the time slice and allow other threads to actually run.
+  //
+#ifdef __CYGWIN__
+  usleep(1);
+#endif
+
   (void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &last_cancel_state);
   pthread_testcancel();
   (void) pthread_setcancelstate(last_cancel_state, NULL);
@@ -166,10 +191,12 @@ void chpl_thread_init(void(*threadBeginFn)(void*),
   // either the main process or a pthread.
   //
   {
-    size_t        css = chpl_task_getMinCallStackSize();
+    size_t        css;
     size_t        pagesize = (size_t) sysconf(_SC_PAGESIZE);
     struct rlimit rlim;
 
+    if ((css = chpl_task_getEnvCallStackSize()) == 0)
+      css = chpl_task_getDefaultCallStackSize();
     assert(css > 0);
 
     css = (css + pagesize - 1) & ~(pagesize - 1);
@@ -181,7 +208,7 @@ void chpl_thread_init(void(*threadBeginFn)(void*),
       char warning[128];
       sprintf(warning, "call stack size capped at %lu\n", 
               (unsigned long)rlim.rlim_max);
-      chpl_warning(warning, 0, NULL);
+      chpl_warning(warning, 0, 0);
 
       css = rlim.rlim_max;
     }
@@ -240,6 +267,12 @@ static void* initial_pthread_func(void* ignore) {
   while (1) {
     pthread_testcancel();
     sched_yield();
+
+    // see comment in chpl_thread_yield()
+#ifdef __CYGWIN__
+    usleep(1);
+#endif
+
   }
   return NULL;
 }
@@ -337,7 +370,7 @@ static void* pthread_func(void* arg) {
 
   // add us to the list of threads
   tlp = (thread_list_p) chpl_mem_alloc(sizeof(struct thread_list),
-                                       CHPL_RT_MD_THREAD_LIST_DESCRIPTOR, 0, 0);
+                                       CHPL_RT_MD_THREAD_LIST_DESC, 0, 0);
 
   tlp->thread = pthread_self();
   tlp->next   = NULL;
@@ -388,6 +421,11 @@ void chpl_thread_destroy(void) {
   //
   sched_yield();
 
+  // see comment in chpl_thread_yield()
+#ifdef __CYGWIN__
+  usleep(1);
+#endif
+
   (void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &last_cancel_state);
   pthread_testcancel();
   (void) pthread_setcancelstate(last_cancel_state, NULL);
