@@ -1,6 +1,4 @@
-/*    $Source: /var/local/cvs/gasnet/acconfig.h,v $ */
-/*      $Date: 2013/09/15 20:03:49 $ */
-/*  $Revision: 1.196 $ */
+/*    $Source: bitbucket.org:berkeleylab/gasnet.git/acconfig.h $ */
 /*  Description: GASNet acconfig.h (or config.h)                             */
 /*  Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>                  */
 /* Terms of use are as specified in license.txt */
@@ -45,6 +43,9 @@
 #undef GASNETI_HAVE_CC_ATTRIBUTE_NORETURN
 #undef GASNETI_HAVE_CC_ATTRIBUTE_PURE
 #undef GASNETI_HAVE_CC_ATTRIBUTE_CONST
+#undef GASNETI_HAVE_CC_ATTRIBUTE_HOT
+#undef GASNETI_HAVE_CC_ATTRIBUTE_COLD
+#undef GASNETI_HAVE_CC_ATTRIBUTE_DEPRECATED
 #undef GASNETI_HAVE_CC_ATTRIBUTE_FORMAT
 #undef GASNETI_HAVE_CC_ATTRIBUTE_FORMAT_FUNCPTR
 #undef GASNETI_HAVE_CC_ATTRIBUTE_FORMAT_FUNCPTR_ARG
@@ -62,6 +63,9 @@
 #undef GASNETI_HAVE_CXX_ATTRIBUTE_NORETURN
 #undef GASNETI_HAVE_CXX_ATTRIBUTE_PURE
 #undef GASNETI_HAVE_CXX_ATTRIBUTE_CONST
+#undef GASNETI_HAVE_CXX_ATTRIBUTE_HOT
+#undef GASNETI_HAVE_CXX_ATTRIBUTE_COLD
+#undef GASNETI_HAVE_CXX_ATTRIBUTE_DEPRECATED
 #undef GASNETI_HAVE_CXX_ATTRIBUTE_FORMAT
 #undef GASNETI_HAVE_CXX_ATTRIBUTE_FORMAT_FUNCPTR
 #undef GASNETI_HAVE_CXX_ATTRIBUTE_FORMAT_FUNCPTR_ARG
@@ -79,6 +83,9 @@
 #undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_NORETURN
 #undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_PURE
 #undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_CONST
+#undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_HOT
+#undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_COLD
+#undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_DEPRECATED
 #undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_FORMAT
 #undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_FORMAT_FUNCPTR
 #undef GASNETI_HAVE_MPI_CC_ATTRIBUTE_FORMAT_FUNCPTR_ARG
@@ -100,6 +107,7 @@
 #undef GASNETI_PLATFORM_MPI_CC_FAMILYID
 #undef GASNETI_PLATFORM_MPI_CC_ID
 #undef GASNETI_PLATFORM_MPI_CC_VERSION
+#undef GASNETI_MPI_VERSION
 
 /* Defined to be the inline function modifier supported by the C
    compilers (if supported), prefixed by 'static' (if permitted) */
@@ -111,11 +119,31 @@
 #undef GASNETI_CXX_RESTRICT
 #undef GASNETI_MPI_CC_RESTRICT
 
+/* Which inline asm style(s) are supported - these are defined only
+   where we use configure to determine what a compiler supports */
+#undef GASNETI_HAVE_CC_XLC_ASM
+#undef GASNETI_HAVE_CXX_XLC_ASM
+#undef GASNETI_HAVE_MPI_CC_XLC_ASM
+#undef GASNETI_HAVE_CC_GCC_ASM
+#undef GASNETI_HAVE_CXX_GCC_ASM
+#undef GASNETI_HAVE_MPI_CC_GCC_ASM
+
+/* Which non-native atomics are available */
+#undef GASNETI_HAVE_CC_SYNC_ATOMICS_32
+#undef GASNETI_HAVE_CXX_SYNC_ATOMICS_32
+#undef GASNETI_HAVE_MPI_CC_SYNC_ATOMICS_32
+#undef GASNETI_HAVE_CC_SYNC_ATOMICS_64
+#undef GASNETI_HAVE_CXX_SYNC_ATOMICS_64
+#undef GASNETI_HAVE_MPI_CC_SYNC_ATOMICS_64
+
 /* Does CC support C99-type non-constant initializers for structs? */
 #undef HAVE_NONCONST_STRUCT_INIT
 /* Does CC support C99-type constructor expressions? */
 #undef HAVE_CONSTRUCTOR_EXPR
 
+/* Defined if __PIC__ defined at configure time */
+#undef GASNETI_CONFIGURED_PIC
+
 /* true iff GASNETI_RESTRICT may be applied to types which are not pointer types until after typedef expansion */
 #undef GASNETI_CC_RESTRICT_MAY_QUALIFY_TYPEDEFS
 #undef GASNETI_CXX_RESTRICT_MAY_QUALIFY_TYPEDEFS
@@ -177,6 +205,7 @@
 #undef GASNETI_PSHM_SYSV
 #undef GASNETI_PSHM_FILE
 #undef GASNETI_PSHM_XPMEM
+#undef GASNETI_PSHM_GHEAP
 
 /* How many cores/node must we support (255 is default) */
 #undef GASNETI_CONFIG_PSHM_MAX_NODES
@@ -199,6 +228,7 @@
 #undef ADDR2LINE_PATH
 #undef GDB_PATH
 #undef GSTACK_PATH
+#undef PSTACK_PATH
 #undef PGDBG_PATH
 #undef IDB_PATH
 #undef LADEBUG_PATH
@@ -216,6 +246,9 @@
 /* has __thread thread-local-storage support */
 #undef GASNETI_HAVE_TLS_SUPPORT
 
+/* force threadinfo optimization ON or OFF */
+#undef GASNETI_THREADINFO_OPT_CONFIGURE
+
 /* pause instruction, if any */
 #undef GASNETI_PAUSE_INSTRUCTION
 
@@ -238,9 +271,6 @@
 /* has support (toolchain and cpu) for x86_64 cmpxchg16b instruction */
 #undef GASNETI_HAVE_X86_CMPXCHG16B
 
-/* gcc/gas support "LDCW,CO" psuedo-op on PA-RISC */
-#undef GASNETI_HAVE_PARISC_LDCW_CO
-
 /* gcc support for "U" and "h" register classes on SPARC32 */
 #undef GASNETI_HAVE_SPARC32_64BIT_ASM
 
@@ -311,18 +341,9 @@
 /* platform is a Linux cluster running IBM PE software */
 #undef GASNETI_ARCH_IBMPE
 
-/* platform is an IBM BlueGene/P multiprocessor */
-#undef GASNETI_ARCH_BGP
-
 /* platform is an IBM BlueGene/Q multiprocessor */
 #undef GASNETI_ARCH_BGQ
 
-/* platform is a SiCortex multiprocessor */
-#undef GASNETI_ARCH_SICORTEX
-
-/* have (potentially buggy) SiCortex ICE9A CPU */
-#undef GASNETI_ARCH_ICE9A
-
 /* have (potentially buggy) MIPS R10000 multiprocessor */
 #undef GASNETI_ARCH_SGI_IP27
 
@@ -331,8 +352,6 @@
 
 /* Have working PPC64 ISA (lacks an associated builtin preprocessor macro) */
 #undef GASNETI_ARCH_PPC64
-/* Tune for a PPC970 cpu (should not crash other PPCs) */
-#undef GASNETI_TUNE_PPC970
 
 /* Type to use as socklen_t */
 #undef GASNET_SOCKLEN_T
@@ -362,10 +381,10 @@
 /* GASNet {gemini,aries}-conduit settings */
 #undef GASNETC_GNI_MAX_MEDIUM
 #undef GASNETC_GNI_MULTI_DOMAIN
+#undef GASNETC_GNI_FIREHOSE
 
 /* GASNet ibv-conduit features and bug work-arounds */
 #undef HAVE_IBV_SRQ
-#undef HAVE_IBV_XRC
 #undef HAVE_IBV_TRANSPORT_TYPE
 #undef GASNETC_IBV_RCV_THREAD
 #undef GASNETC_IBV_CONN_THREAD
@@ -387,6 +406,8 @@
 
 /* Support for pmi-spawner */
 #undef HAVE_PMI_H
+#undef HAVE_PMI2_H
+#undef GASNETI_PMI2_FENCE_IS_BARRIER
 
 @BOTTOM@
 
