@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/extended-ref/gasnet_coll_internal.h,v $
- *     $Date: 2013/06/19 21:30:02 $
- * $Revision: 1.75 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/extended-ref/gasnet_coll_internal.h $
  * Description: GASNet Collectives conduit header
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -235,6 +233,15 @@ typedef enum {
 } gasnete_coll_barrier_type_t;
 
 
+/* "peers" are sets of nodes at distances +/- powers of two, taken from some parent set */
+typedef struct {
+  unsigned int   num; /* ceil(log_2(ranks)) */
+  gasnet_node_t *fwd; /* fwd[i] is global rank of member (myrank + 2^i) */
+#if 0 /* Not used yet */
+  gasnet_node_t *bwd; /* bwd[i] is global rank of member (myrank - 2^i) */
+#endif
+} gasnete_coll_peer_list_t;
+
 /* Type for collective teams: */
 struct gasnete_coll_team_t_ {
   /* read-only fields: */
@@ -264,6 +271,20 @@ struct gasnete_coll_team_t_ {
   /* ranks of the processes in the team */
   gasnet_node_t *rel2act_map; /* need to be initialized */
 
+  /* nodes in the team at distances +/- powers of two */
+  gasnete_coll_peer_list_t peers;
+
+#if GASNET_PSHM
+  /* Info about the supernode(s) */
+  struct {
+    gasnet_node_t node_count;
+    gasnet_node_t node_rank;
+    gasnet_node_t grp_count;
+    gasnet_node_t grp_rank;
+  } supernode;
+  /* supernode-reps in the team at distances +/- powers of two in supernode space */
+  gasnete_coll_peer_list_t supernode_peers;
+#endif
 
   /* scratch segments allocated on team creation*/
   gasnet_seginfo_t *scratch_segs;
@@ -1370,6 +1391,43 @@ int gasnete_coll_generic_outsync(gasnete_coll_team_t team, gasnete_coll_generic_
 	  (gasnete_coll_consensus_try(team, data->out_barrier) == GASNET_OK));
 }
 
+/* Optional UP half-barrier over the collective tree.
+ * No memory fences. */
+GASNETI_INLINE(gasnete_coll_generic_upsync)
+int gasnete_coll_generic_upsync(gasnete_coll_op_t *op, gasnet_node_t rootnode,
+                                    const int counter, const int count) {
+  gasnete_coll_generic_data_t * const data = op->data;
+  if (gasneti_weakatomic_read(&data->p2p->counter[counter], 0) == count) {
+    if (op->team->myrank != rootnode) {
+      gasnete_coll_tree_data_t * const tree = data->tree_info;
+      gasnete_coll_p2p_advance(op, GASNETE_COLL_REL2ACT(op->team, GASNETE_COLL_TREE_GEOM_PARENT(tree->geom)),0);
+    }
+    return 1;
+  }
+  return 0;
+}
+
+/* Optional UP half-barrier over the collective tree.
+ * Root node will rmb() and non-root will wmb() to ensure that writes
+ * to root's memory will be read by root.  This is appropriate to the
+ * needs of a "push" based broadcast or scatter. */
+GASNETI_INLINE(gasnete_coll_generic_upsync_acq)
+int gasnete_coll_generic_upsync_acq(gasnete_coll_op_t *op, gasnet_node_t rootnode,
+                                    const int counter, const int count) {
+  gasnete_coll_generic_data_t * const data = op->data;
+  if (gasneti_weakatomic_read(&data->p2p->counter[counter], 0) == count) {
+    if (op->team->myrank != rootnode) {
+      gasnete_coll_tree_data_t * const tree = data->tree_info;
+      gasneti_local_wmb();
+      gasnete_coll_p2p_advance(op, GASNETE_COLL_REL2ACT(op->team, GASNETE_COLL_TREE_GEOM_PARENT(tree->geom)),0);
+    } else {
+      gasneti_local_rmb();
+    }
+    return 1;
+  }
+  return 0;
+}
+
 extern int gasnete_coll_generic_coll_sync(gasnet_coll_handle_t *p, size_t count GASNETE_THREAD_FARG);
 
 
