@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/extended-ref/gasnet_coll_rvous.c,v $
- *     $Date: 2010/08/08 06:31:07 $
- * $Revision: 1.68 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/extended-ref/gasnet_coll_rvous.c $
  * Description: Reference implemetation of GASNet Collectives team
  * Copyright 2004, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -114,20 +112,16 @@ static int gasnete_coll_pf_bcast_TreeRVGet(gasnete_coll_op_t *op GASNETE_THREAD_
   int result = 0;
 
   switch (data->state) {
-  case 0:	/* Optional IN barrier and rendezvous */
+  case 0:	/* thread barrier */
     if (!gasnete_coll_generic_all_threads(data)) {
       break;
     }
     data->state = 1;
     
-  case 1:
-    if(op->flags & GASNET_COLL_IN_ALLSYNC) {
-      if (gasneti_weakatomic_read(&(data->p2p->counter[0]), 0) != child_count) {
-        break;
-      }
-      if (op->team->myrank != args->srcnode) {
-        gasnete_coll_p2p_advance(op, GASNETE_COLL_REL2ACT(op->team, GASNETE_COLL_TREE_GEOM_PARENT(tree->geom)),0);
-      }
+  case 1:       /* Optional IN barrier over the SAME tree */
+    if ((op->flags & GASNET_COLL_IN_ALLSYNC) &&
+        !gasnete_coll_generic_upsync(op, args->srcnode, 0, child_count)) {
+      break;
     }
     data->state = 2;
     
@@ -380,14 +374,10 @@ static int gasnete_coll_pf_bcastM_TreeRVGet(gasnete_coll_op_t *op GASNETE_THREAD
     }
     data->state = 1;
     
-  case 1:
-    if(op->flags & GASNET_COLL_IN_ALLSYNC) {
-      if (gasneti_weakatomic_read(&(data->p2p->counter[0]), 0) != child_count) {
-        break;
-      }
-      if (op->team->myrank != args->srcnode) {
-        gasnete_coll_p2p_advance(op, GASNETE_COLL_REL2ACT(op->team, GASNETE_COLL_TREE_GEOM_PARENT(tree->geom)),0);
-      }
+  case 1:       /* Optional IN barrier over the SAME tree */
+    if ((op->flags & GASNET_COLL_IN_ALLSYNC) &&
+        !gasnete_coll_generic_upsync(op, args->srcnode, 0, child_count)) {
+      break;
     }
     data->state = 2;
     
