@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/gasnet_basic.h,v $
- *     $Date: 2012/07/13 23:51:00 $
- * $Revision: 1.118 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/gasnet_basic.h $
  * Description: GASNet basic header utils
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -122,6 +120,12 @@
   #define GASNETI_RESTRICT_MAY_QUALIFY_TYPEDEFS 1
 #endif
 
+#if HAVE_BUILTIN_CONSTANT_P
+  #define gasneti_constant_p(_expr) __builtin_constant_p(_expr)
+#else
+  #define gasneti_constant_p(_expr) (0)
+#endif
+
 #ifndef _STRINGIFY
 #define _STRINGIFY_HELPER(x) #x
 #define _STRINGIFY(x) _STRINGIFY_HELPER(x)
@@ -181,17 +185,6 @@
 #ifndef GASNET_PAGESIZE
   #ifdef GASNETI_PAGESIZE
     #define GASNET_PAGESIZE GASNETI_PAGESIZE
-  #elif PLATFORM_ARCH_CRAYT3E
-    /* on Cray: shmemalign allocates mem aligned across nodes, 
-        but there seems to be no fixed page size (man pagesize)
-        this is probably because they don't support VM
-       actual page size is set separately for each linker section, 
-        ranging from 512KB(default) to 8MB
-       Here we return 8 to reflect the lack of page alignment constraints
-       (for basic sanity, we want page alignment >= MAX(cache line,reqd double alignment))
-   */
-
-    #define GASNET_PAGESIZE GASNETI_CACHE_LINE_BYTES
   #else
     #error GASNET_PAGESIZE unknown and not set by conduit
   #endif
@@ -210,13 +203,7 @@
   #define GASNETT_USE_GCC_ATTRIBUTE_ALWAYSINLINE 0
 #endif
 
-#if PLATFORM_COMPILER_SGI_CXX
-  #define GASNETI_PRAGMA(x) /* despite the docs, not supported in MIPSPro C++ */
-#elif PLATFORM_COMPILER_SGI && _SGI_COMPILER_VERSION < 742
-  #define GASNETI_PRAGMA(x) /* bug1555: broken in older versions (740 fails, 742 works) */
-#elif PLATFORM_COMPILER_COMPAQ_C && __DECC_VER < 60590207
-  #define GASNETI_PRAGMA(x) /* not supported in older versions (60490014) */
-#elif PLATFORM_COMPILER_SUN_C && __SUNPRO_C < 0x570
+#if PLATFORM_COMPILER_SUN_C && __SUNPRO_C < 0x570
   #define GASNETI_PRAGMA(x) /* not supported in older versions (550 fails, 570 works) */
 #else
   #define GASNETI_PRAGMA(x) _Pragma ( #x )
@@ -256,6 +243,15 @@
   #ifndef GASNETT_USE_GCC_ATTRIBUTE_CONST
     #define GASNETT_USE_GCC_ATTRIBUTE_CONST GASNETI_HAVE_CC_ATTRIBUTE_CONST
   #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_HOT
+    #define GASNETT_USE_GCC_ATTRIBUTE_HOT GASNETI_HAVE_CC_ATTRIBUTE_HOT
+  #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_COLD
+    #define GASNETT_USE_GCC_ATTRIBUTE_COLD GASNETI_HAVE_CC_ATTRIBUTE_COLD
+  #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_DEPRECATED
+    #define GASNETT_USE_GCC_ATTRIBUTE_DEPRECATED GASNETI_HAVE_CC_ATTRIBUTE_DEPRECATED
+  #endif
   #ifndef GASNETT_USE_GCC_ATTRIBUTE_FORMAT
     #define GASNETT_USE_GCC_ATTRIBUTE_FORMAT GASNETI_HAVE_CC_ATTRIBUTE_FORMAT
   #endif
@@ -298,6 +294,15 @@
   #ifndef GASNETT_USE_GCC_ATTRIBUTE_CONST
     #define GASNETT_USE_GCC_ATTRIBUTE_CONST GASNETI_HAVE_MPI_CC_ATTRIBUTE_CONST
   #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_HOT
+    #define GASNETT_USE_GCC_ATTRIBUTE_HOT GASNETI_HAVE_MPI_CC_ATTRIBUTE_HOT
+  #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_COLD
+    #define GASNETT_USE_GCC_ATTRIBUTE_COLD GASNETI_HAVE_MPI_CC_ATTRIBUTE_COLD
+  #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_DEPRECATED
+    #define GASNETT_USE_GCC_ATTRIBUTE_DEPRECATED GASNETI_HAVE_MPI_CC_ATTRIBUTE_DEPRECATED
+  #endif
   #ifndef GASNETT_USE_GCC_ATTRIBUTE_FORMAT
     #define GASNETT_USE_GCC_ATTRIBUTE_FORMAT GASNETI_HAVE_MPI_CC_ATTRIBUTE_FORMAT
   #endif
@@ -340,6 +345,15 @@
   #ifndef GASNETT_USE_GCC_ATTRIBUTE_CONST
     #define GASNETT_USE_GCC_ATTRIBUTE_CONST GASNETI_HAVE_CXX_ATTRIBUTE_CONST
   #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_HOT
+    #define GASNETT_USE_GCC_ATTRIBUTE_HOT GASNETI_HAVE_CXX_ATTRIBUTE_HOT
+  #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_COLD
+    #define GASNETT_USE_GCC_ATTRIBUTE_COLD GASNETI_HAVE_CXX_ATTRIBUTE_COLD
+  #endif
+  #ifndef GASNETT_USE_GCC_ATTRIBUTE_DEPRECATED
+    #define GASNETT_USE_GCC_ATTRIBUTE_DEPRECATED GASNETI_HAVE_CXX_ATTRIBUTE_DEPRECATED
+  #endif
   #ifndef GASNETT_USE_GCC_ATTRIBUTE_FORMAT
     #define GASNETT_USE_GCC_ATTRIBUTE_FORMAT GASNETI_HAVE_CXX_ATTRIBUTE_FORMAT
   #endif
@@ -367,8 +381,6 @@
 /* pragma version of GASNETI_MALLOC */
 #if PLATFORM_COMPILER_SUN_C
   #define GASNETI_MALLOCP(fnname) GASNETI_PRAGMA(returns_new_memory(fnname))
-#elif PLATFORM_COMPILER_HP_C && !PLATFORM_ARCH_IA64
-  #define GASNETI_MALLOCP(fnname) GASNETI_PRAGMA(ALLOCS_NEW_MEMORY fnname)
 #else
   #define GASNETI_MALLOCP(fnname)
 #endif
@@ -411,10 +423,6 @@
 /* pragma version of GASNETI_NORETURN */
 #if PLATFORM_COMPILER_SUN_C
   #define GASNETI_NORETURNP(fnname) GASNETI_PRAGMA(does_not_return(fnname))
-#elif PLATFORM_COMPILER_SGI && _SGI_COMPILER_VERSION >= 720 && _MIPS_SIM != _ABIO32
-  #define GASNETI_NORETURNP(fnname) GASNETI_PRAGMA(mips_frequency_hint NEVER fnname)
-#elif PLATFORM_COMPILER_COMPAQ_C /* not Compaq C++ */
-  #define GASNETI_NORETURNP(fnname) GASNETI_PRAGMA(assert func_attrs(fnname) noreturn)
 #elif PLATFORM_COMPILER_XLC && 0
   /* this *should* work but it causes bizarre compile failures, so disable it */
   #define GASNETI_NORETURNP(fnname) GASNETI_PRAGMA(leaves(fnname))
@@ -438,13 +446,6 @@
 #if PLATFORM_COMPILER_XLC && \
    !(PLATFORM_OS_DARWIN && __xlC__ <= 0x0600) /* bug 1542 */
   #define GASNETI_PUREP(fnname) GASNETI_PRAGMA(isolated_call(fnname))
-#elif PLATFORM_COMPILER_SGI && _SGI_COMPILER_VERSION >= 710
-  #define GASNETI_PUREP(fnname) GASNETI_PRAGMA(no side effects (fnname))
-#elif PLATFORM_COMPILER_COMPAQ
-  #define GASNETI_PUREP(fnname) \
-          GASNETI_PRAGMA(assert func_attrs(fnname) noeffects file_scope_vars(nowrites))
-#elif PLATFORM_COMPILER_HP_C && !PLATFORM_ARCH_IA64
-  #define GASNETI_PUREP(fnname) GASNETI_PRAGMA(NO_SIDE_EFFECTS fnname)
 #else
   #define GASNETI_PUREP(fnname) 
 #endif
@@ -462,11 +463,6 @@
 /* pragma version of GASNETI_CONST */
 #if PLATFORM_COMPILER_SUN_C
   #define GASNETI_CONSTP(fnname) GASNETI_PRAGMA(no_side_effect(fnname))
-#elif PLATFORM_COMPILER_SGI && _SGI_COMPILER_VERSION >= 730
-  #define GASNETI_CONSTP(fnname) GASNETI_PRAGMA(pure (fnname))
-#elif PLATFORM_COMPILER_COMPAQ
-  #define GASNETI_CONSTP(fnname) \
-          GASNETI_PRAGMA(assert func_attrs(fnname) nostate noeffects file_scope_vars(none))
 #else
   #define GASNETI_CONSTP(fnname) GASNETI_PUREP(fnname)
 #endif
@@ -481,15 +477,6 @@
   #define _GASNETI_ALWAYS_INLINE(fnname) GASNETI_PRAGMA(_CRI inline fnname)
 #elif PLATFORM_COMPILER_MTA
   #define _GASNETI_ALWAYS_INLINE(fnname) GASNETI_PRAGMA(mta inline)
-#elif PLATFORM_COMPILER_SGI && _SGI_COMPILER_VERSION >= 710
-  #define _GASNETI_ALWAYS_INLINE(fnname) GASNETI_PRAGMA(inline global fnname)
-#elif PLATFORM_COMPILER_COMPAQ_C /* not Compaq C++ */
-  #define _GASNETI_ALWAYS_INLINE(fnname) GASNETI_PRAGMA(inline (fnname))
-#elif PLATFORM_COMPILER_HP_C && GASNET_NDEBUG /* avoid a warning */ \
-   && 0 /* unreliable behavior - Itanium optimizer crashes and 
-           PARISC syntax errors unless it appears on a line by itself */
-  #define _GASNETI_ALWAYS_INLINE(fnname) GASNETI_PRAGMA(INLINE fnname)
-  #undef STATIC_INLINE_WORKS
 #else
   #define _GASNETI_ALWAYS_INLINE(fnname)
 #endif
@@ -525,31 +512,42 @@
 /* GASNETI_NEVER_INLINE: Most forceful demand available to disable inlining for function.
  */
 #if GASNETT_USE_GCC_ATTRIBUTE_NOINLINE
+ #ifdef __noinline__ /* e.g. nvcc's host_defines */
+  #define GASNETI_NEVER_INLINE(fnname,declarator) __attribute__((noinline)) declarator
+ #else
   #define GASNETI_NEVER_INLINE(fnname,declarator) __attribute__((__noinline__)) declarator
+ #endif
 #elif PLATFORM_COMPILER_SUN_C
   #define GASNETI_NEVER_INLINE(fnname,declarator) declarator; GASNETI_PRAGMA(no_inline(fnname)) declarator
 #elif PLATFORM_COMPILER_CRAY
   #define GASNETI_NEVER_INLINE(fnname,declarator) GASNETI_PRAGMA(_CRI inline_never fnname) declarator
-#elif PLATFORM_COMPILER_SGI && _SGI_COMPILER_VERSION >= 710
-  #define GASNETI_NEVER_INLINE(fnname,declarator) GASNETI_PRAGMA(noinline global fnname) declarator
-#elif PLATFORM_COMPILER_COMPAQ
-  #define GASNETI_NEVER_INLINE(fnname,declarator) GASNETI_PRAGMA(noinline (fnname)) declarator
-#elif PLATFORM_COMPILER_HP_C && GASNET_NDEBUG /* avoid a warning */ \
-   && PLATFORM_ARCH_IA64 /* unreliable behavior on PARISC unless it appears on a line by itself */
-  #define GASNETI_NEVER_INLINE(fnname,declarator) GASNETI_PRAGMA(NOINLINE fnname) declarator
 #else
   #define GASNETI_NEVER_INLINE(fnname,declarator) declarator
 #endif
 
+/* GASNETI_HOT (COLD): assert function is frequently (infrequently) called */
+#if GASNETT_USE_GCC_ATTRIBUTE_HOT
+  #define GASNETI_HOT __attribute__((__hot__))
+#else
+  #define GASNETI_HOT
+#endif
+#if GASNETT_USE_GCC_ATTRIBUTE_HOT
+  #define GASNETI_COLD __attribute__((__cold__))
+#else
+  #define GASNETI_COLD
+#endif
+
+/* GASNETI_DEPRECATED: mark a function as deprecated (subject to future removal) */
+#if GASNETT_USE_GCC_ATTRIBUTE_DEPRECATED
+  #define GASNETI_DEPRECATED __attribute__((__deprecated__))
+#else
+  #define GASNETI_DEPRECATED
+#endif
+
 /* GASNETI_FORMAT_PRINTF: enable gcc printf format checking of function args */
 #if GASNETT_USE_GCC_ATTRIBUTE_FORMAT
   #define GASNETI_FORMAT_PRINTF(fnname,fmtarg,firstvararg,declarator) \
           __attribute__((__format__ (__printf__, fmtarg, firstvararg))) declarator
-#elif PLATFORM_COMPILER_COMPAQ_C /* not Compaq C++ */
-  #define GASNETI_FORMAT_PRINTF(fnname,fmtarg,firstvararg,declarator)  \
-          declarator; /* declaration required before pragma */ \
-          GASNETI_PRAGMA(assert func_attrs(fnname) format (printf,fmtarg,firstvararg)) \
-          declarator
 #else
   #define GASNETI_FORMAT_PRINTF(fnname,fmtarg,firstvararg,declarator) declarator
 #endif
@@ -587,10 +585,6 @@
   #define GASNETI_IDENT(identName, identText) \
     GASNETI_PRAGMA(_CRI ident identText) GASNETI_PRAGMA_SEMI     \
     _GASNETI_IDENT(identName, identText)
-#elif PLATFORM_COMPILER_HP_C && PLATFORM_ARCH_IA64 /* bug 1490 */
-  #define GASNETI_IDENT(identName, identText) \
-    GASNETI_PRAGMA(VERSIONID identText);      \
-    _GASNETI_IDENT(identName, identText)
 #elif PLATFORM_COMPILER_XLC
     /* #pragma comment(user,"text...") 
          or
@@ -645,10 +639,6 @@
   #define GASNETT_MTA_PRAGMA_EXPECT_OVERRIDE GASNETT_MTA_PRAGMA_EXPECT_ENABLED
   #define if_pf(cond) GASNETT_MTA_PRAGMA_EXPECT_OVERRIDE("mta expect false") if (cond)
   #define if_pt(cond) GASNETT_MTA_PRAGMA_EXPECT_OVERRIDE("mta expect true")  if (cond)
-#elif PLATFORM_COMPILER_SGI && _SGI_COMPILER_VERSION >= 720 && _MIPS_SIM != _ABIO32
-  /* MIPSPro has a predict-false, but unfortunately no predict-true */
-  #define if_pf(cond) if (cond) GASNETI_PRAGMA(mips_frequency_hint NEVER)
-  #define if_pt(cond) if (GASNETT_PREDICT_TRUE(cond))
 #else
   #define if_pf(cond) if (GASNETT_PREDICT_FALSE(cond))
   #define if_pt(cond) if (GASNETT_PREDICT_TRUE(cond))
