@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/gasnet_help.h,v $
- *     $Date: 2013/08/27 01:23:44 $
- * $Revision: 1.116 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/gasnet_help.h $
  * Description: GASNet Header Helpers (Internal code, not for client use)
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -159,9 +157,9 @@ extern uint64_t gasnet_max_segsize; /* client-overrideable max segment size */
                          gasneti_current_loc);                                 \
     if_pf (_ptr == NULL || !segtest(_node,_ptr,_nbytes))                       \
       gasneti_fatalerror("Remote address out of range "                        \
-         "(node=%lu ptr="GASNETI_LADDRFMT" nbytes=%lu) at %s"                  \
-         "\n  clientsegment=("GASNETI_LADDRFMT"..."GASNETI_LADDRFMT")"         \
-         "\n    fullsegment=("GASNETI_LADDRFMT"..."GASNETI_LADDRFMT")",        \
+         "(node=%lu ptr=" GASNETI_LADDRFMT" nbytes=%lu) at %s"                 \
+         "\n  clientsegment=(" GASNETI_LADDRFMT"..." GASNETI_LADDRFMT")"       \
+         "\n    fullsegment=(" GASNETI_LADDRFMT"..." GASNETI_LADDRFMT")",      \
          (unsigned long)_node, GASNETI_LADDRSTR(_ptr), (unsigned long)_nbytes, \
          gasneti_current_loc,                                                  \
          GASNETI_LADDRSTR(gasneti_seginfo_client[_node].addr),                 \
@@ -364,8 +362,25 @@ extern uint64_t gasnet_max_segsize; /* client-overrideable max segment size */
 /* public threadinfo support */
 
 #if GASNETI_CLIENT_THREADS
+  #if defined GASNETI_THREADINFO_OPT_CONFIGURE
+    /* take user-specified value, if any*/
+    #undef GASNETI_THREADINFO_OPT
+    #define GASNETI_THREADINFO_OPT GASNETI_THREADINFO_OPT_CONFIGURE
+  #elif defined GASNETI_THREADINFO_OPT
+    /* keep conduit-specified value */
+  #elif defined GASNETI_HAVE_TLS_SUPPORT
+    /* Default to OFF on some common ABIs with good TLS support */
+    #if (PLATFORM_ARCH_X86_64 || PLATFORM_ARCH_MIC) && \
+        (PLATFORM_OS_DARWIN || PLATFORM_OS_SOLARIS || PLATFORM_OS_LINUX || PLATFORM_OS_CNL)
+      #define GASNETI_THREADINFO_OPT    0
+    #elif PLATFORM_ARCH_POWERPC && \
+         (PLATFORM_OS_LINUX || PLATFORM_OS_BGQ)
+      #define GASNETI_THREADINFO_OPT    0
+    #endif
+  #endif
   #ifndef GASNETI_THREADINFO_OPT
-  #define GASNETI_THREADINFO_OPT      1
+    /* Default to ON if not set anywhere above */
+    #define GASNETI_THREADINFO_OPT      1
   #endif
   #ifndef GASNETI_LAZY_BEGINFUNCTION
   #define GASNETI_LAZY_BEGINFUNCTION  1
@@ -776,6 +791,7 @@ extern int gasneti_wait_mode; /* current waitmode hint */
   #define gasnet_getNodeInfo(nodeinfo_table, numentries) \
           gasneti_getNodeInfo(nodeinfo_table, numentries)
 #endif
+extern gasnet_nodeinfo_t *gasneti_nodeinfo;
 
 #ifndef _GASNETI_SEGINFO
 #define _GASNETI_SEGINFO
@@ -804,6 +820,85 @@ extern int gasneti_wait_mode; /* current waitmode hint */
         gasneti_pthread_create(&gasneti_pthread_create_system, (thr), (attr), (fn), (arg))
   #endif
 #endif
+
+/* ------------------------------------------------------------------------------------ */
+/* PSHM support */
+#if GASNET_PSHM
+
+/* Max number of processes supported per node */
+#ifndef GASNETI_PSHM_MAX_NODES
+  #ifdef GASNETI_CONFIG_PSHM_MAX_NODES
+    #define GASNETI_PSHM_MAX_NODES GASNETI_CONFIG_PSHM_MAX_NODES
+  #else
+    #define GASNETI_PSHM_MAX_NODES 255
+  #endif
+#endif
+
+#if GASNETI_PSHM_MAX_NODES < 256
+  typedef uint8_t gasneti_pshm_rank_t;
+#elif GASNETI_PSHM_MAX_NODES < 65536
+  typedef uint16_t gasneti_pshm_rank_t;
+#else
+  #error "GASNETI_PSHM_MAX_NODES too large"
+#endif
+
+extern gasneti_pshm_rank_t gasneti_pshm_nodes;  /* # nodes in my supernode */
+extern gasneti_pshm_rank_t gasneti_pshm_mynode; /* my 0-based rank in supernode */
+extern gasnet_node_t gasneti_pshm_firstnode;    /* lowest node # in supernode */
+
+/* # of supernodes */
+#define gasneti_pshm_supernodes (0+gasneti_nodemap_global_count)
+/* my supernode's 0-based rank among supernodes */
+#define gasneti_pshm_mysupernode (0+gasneti_nodemap_global_rank)
+/* vector of first node within each supernode */
+extern gasnet_node_t *gasneti_pshm_firsts;
+
+/* Non-NULL only when supernode members are non-contiguous */
+extern gasneti_pshm_rank_t *gasneti_pshm_rankmap;
+
+/* Returns "local rank" if given node is in the callers supernode.
+ * Otherwise returns an "impossible" value >= gasneti_pshm_nodes.
+ */
+GASNETI_INLINE(gasneti_pshm_local_rank) GASNETI_PURE
+unsigned int gasneti_pshm_local_rank(gasnet_node_t node) {
+#if GASNET_CONDUIT_SMP
+  return node;
+#else
+  if_pt (gasneti_pshm_rankmap == NULL) {
+    /* NOTE: gasnet_node_t is an unsigned type, so in the case of
+     * (node < gasneti_pshm_firstnode), the subtraction will wrap to
+     * a "large" value.
+     */
+    return (node - gasneti_pshm_firstnode);
+  } else {
+    return gasneti_pshm_rankmap[node];
+  }
+#endif
+}
+GASNETI_PUREP(gasneti_pshm_local_rank)
+
+/* Returns 1 if given node is in the caller's supernode, or 0 if it's not.
+ * NOTE: result is false before vnet initialization.
+ */
+GASNETI_INLINE(gasneti_pshm_in_supernode) GASNETI_PURE
+int gasneti_pshm_in_supernode(gasnet_node_t node) {
+#if GASNET_CONDUIT_SMP
+  return 1;
+#else
+  return (gasneti_pshm_local_rank(node) < gasneti_pshm_nodes);
+#endif
+}
+GASNETI_PUREP(gasneti_pshm_in_supernode)
+
+/* Returns local version of remote in-supernode address.
+ */
+GASNETI_INLINE(gasneti_pshm_addr2local) GASNETI_PURE
+void *gasneti_pshm_addr2local(gasnet_node_t node, void *addr) {
+  return  (void*)((uintptr_t)addr
+                   + (uintptr_t)gasneti_nodeinfo[node].offset);
+} 
+GASNETI_PUREP(gasneti_pshm_addr2local)
+#endif /* GASNET_PSHM */
 /* ------------------------------------------------------------------------------------ */
 
 GASNETI_END_EXTERNC
