@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/gasnet_internal.c,v $
- *     $Date: 2013/08/28 02:36:57 $
- * $Revision: 1.239 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/gasnet_internal.c $
  * Description: GASNet implementation of internal helpers
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -72,6 +70,10 @@ GASNETI_IDENT(gasneti_IdentString_SegConfig, "$GASNetSegment: GASNET_SEGMENT_" G
   GASNETI_IDENT(gasneti_IdentString_ConservativeLocalCopy, "$GASNetConservativeLocalCopy: 1 $");
 #endif
 
+#if GASNETI_CLIENT_THREADS
+  GASNETI_IDENT(gasneti_IdentString_ThreadInfoOpt, "$GASNetThreadInfoOpt: " _STRINGIFY(GASNETI_THREADINFO_OPT) " $");
+#endif
+
 /* embed a string with complete configuration info to support versioning checks */
 GASNETI_IDENT(gasneti_IdentString_libraryConfig, "$GASNetConfig: (libgasnet.a) " GASNET_CONFIG_STRING " $");
 /* the canonical conduit name */
@@ -108,6 +110,7 @@ int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_MEMBAR_CONFIG) = 1;
 int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC_CONFIG) = 1;
 int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC32_CONFIG) = 1;
 int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_ATOMIC64_CONFIG) = 1;
+int GASNETI_LINKCONFIG_IDIOTCHECK(GASNETI_TIOPT_CONFIG) = 1;
 int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(CORE_,GASNET_CORE_NAME)) = 1;
 int GASNETI_LINKCONFIG_IDIOTCHECK(_CONCAT(EXTENDED_,GASNET_EXTENDED_NAME)) = 1;
 
@@ -632,7 +635,6 @@ extern void gasneti_decode_args(int *argc, char ***argv) {
       }
     }
   }
-  gasneti_backtrace_init((*argv)[0]);
 }
 
 /* Process environment for exittimeout.
@@ -729,7 +731,7 @@ static void gasneti_check_portable_conduit(void) { /* check for portable conduit
         const char *desc;
         int hwid;
       } known_devs[] = {
-      #if PLATFORM_OS_BLRTS || PLATFORM_OS_BGP || PLATFORM_OS_BGQ
+      #if PLATFORM_OS_BGQ
         { "/dont_probe_an_io_node", S_IFDIR, "", 0 }
       #else
         #if PLATFORM_OS_LINUX && PLATFORM_ARCH_IA64 && GASNET_SEQ
@@ -755,15 +757,9 @@ static void gasneti_check_portable_conduit(void) { /* check for portable conduit
             while (i < lim && hwid == known_devs[i].hwid) i++; /* don't report a network twice */
         }
       }
-      #if PLATFORM_ARCH_CRAYX1
-        if (strlen(natives)) strcat(natives,", ");
-        strcat(natives,"Cray SHMEM (X1)");
-      #elif PLATFORM_OS_CNL
+      #if PLATFORM_OS_CNL
         if (strlen(natives)) strcat(natives,", ");
         strcat(natives,"Cray Gemini (XE and XK) or Aries (XC)");
-      #elif PLATFORM_OS_BGP
-        if (strlen(natives)) strcat(natives,", ");
-        strcat(natives,"IBM DCMF (BG/P)");
       #elif PLATFORM_OS_BGQ
         if (strlen(natives)) strcat(natives,", ");
         strcat(natives,"IBM PAMI (BG/Q)");
@@ -791,11 +787,8 @@ static void gasneti_check_portable_conduit(void) { /* check for portable conduit
  */
 
 gasnet_node_t *gasneti_nodemap = NULL;
-gasnet_node_t *gasneti_nodemap_local = NULL;
-gasnet_node_t gasneti_nodemap_local_count = 0;
-gasnet_node_t gasneti_nodemap_local_rank = (gasnet_node_t)-1;
-gasnet_node_t gasneti_nodemap_global_count = 0;
-gasnet_node_t gasneti_nodemap_global_rank = (gasnet_node_t)-1;
+gasneti_nodegrp_t gasneti_myhost = {NULL,0,(gasnet_node_t)(-1),0,(gasnet_node_t)(-1)};
+gasneti_nodegrp_t gasneti_mysupernode = {NULL,0,(gasnet_node_t)(-1),0,(gasnet_node_t)(-1)};
 gasnet_nodeinfo_t *gasneti_nodeinfo = NULL;
 
 /* This code is "good" for all "sensible" process layouts, where "good"
@@ -916,32 +909,7 @@ void gasneti_nodemap_trivial(void) {
  * Used when no conduit-specific IDs are provided.
  */
 static void gasneti_nodemap_dflt(gasneti_bootstrapExchangefn_t exchangefn) {
-#if PLATFORM_OS_BGP && GASNETI_HAVE_BGP_INLINES
-    /* Build gasneti_nodemap from <X,Y,Z> coords of all ranks. */
-    _BGP_SprgShMem sprg4;
-
-    GASNETI_BGP_SPR(sprg4.shmem, _BGP_SPRGRO_SHMem); /* SPRG4 30:31 = (processes per node) - 1 */
-
-    if ((0 == GASNET_PSHM) ||
-        !sprg4.ShmNumProcs || (gasneti_nodes == 1)) {
-      /* Just build the trivial map if GASNET_PSHM is zero,
-         or are in SMP mode, or we have just a single node */
-      gasneti_nodemap_trivial();
-    } else {
-      uint32_t *allids = gasneti_malloc(gasneti_nodes * sizeof(uint32_t));
-
-      /* Kernel call to get torus coords for all ranks */
-      { int rc;
-        GASNETI_BGP_SYSCALL2(rc, RANKS2COORDS, (uintptr_t)allids, (uint32_t)gasneti_nodes);
-        gasneti_assert(!rc);
-      }
-
-      /* Compare only the upper 3 bytes, discarding the T coordinate */
-      gasneti_nodemap_helper(allids, 3, sizeof(uint32_t));
-
-      gasneti_free(allids);
-    }
-#elif PLATFORM_OS_BGQ && GASNETI_HAVE_BGQ_INLINES
+#if PLATFORM_OS_BGQ && GASNETI_HAVE_BGQ_INLINES
   #if 0 /* "Clean" but not usable in general due to (non)inlined implementation */
     uint64_t count, size = gasneti_nodes * sizeof(BG_CoordinateMapping_t);
     BG_CoordinateMapping_t *allids = gasneti_malloc(size);
@@ -973,7 +941,7 @@ static void gasneti_nodemap_dflt(gasneti_bootstrapExchangefn_t exchangefn) {
 
     gasneti_free(allids);
   #endif
-#elif PLATFORM_OS_BGQ || PLATFORM_OS_BGP || PLATFORM_OS_BLRTS || PLATFORM_OS_CATAMOUNT || !HAVE_GETHOSTID
+#elif PLATFORM_OS_BGQ || !HAVE_GETHOSTID
     /* Nodes are either (at least effectively) single process,
      * or we don't have a usable gethostid().  So, build a trivial nodemap. */
     gasneti_nodemap_trivial();
@@ -1025,27 +993,34 @@ static void gasneti_nodemap_dflt(gasneti_bootstrapExchangefn_t exchangefn) {
  * A conduit which builds a gasneti_nodemap[] w/o calling gasneti_nodemapInit()
  * should still call this function to perform the "common" work.
  *
- * Currently computes some local statistics:
- *   gasneti_nodemap_local_count = number of GASNet nodes collocated w/ gasneti_mynode
- *   gasneti_nodemap_local_rank  = rank of gasneti_mynode among gasneti_nodemap_local_count
- *   gasneti_nodemap_local[]     = array (length gasneti_nodemap_local_count) of local nodes
- *   gasneti_nodemap_global_count = number of unique values in the nodemap
- *   gasneti_nodemap_global_rank  = rank of gasneti_mynode among gasneti_nodemap_global_count
- * and constructs:
+ * Constructs:
  *   gasneti_nodeinfo[] = array of length gasneti_nodes of supernode ids and mmap offsets
+ * and fills in the fields in gasneti_mysupernode:
+ *   gasneti_mysupernode.nodes       array of nodes in my supernode
+ *   gasneti_mysupernode.node_count  count of nodes in my supernode (length of 'nodes' array)
+ *   gasneti_mysupernode.node_rank   my ranks in 'nodes' array
+ *   gasneti_mysupernode.grp_count   number of supernodes in the job
+ *   gasneti_mysupernode.grp_rank    my supernode's rank within all supernodes
+ *   gasneti_myhost.*                same quantities for 'host' rather than for 'supernode'
+ * Those first five quantities are also available via the following respective aliases:
+ *   gasneti_nodemap_local[]
+ *   gasneti_nodemap_local_count
+ *   gasneti_nodemap_local_rank
+ *   gasneti_nodemap_global_count
+ *   gasneti_nodemap_global_rank
  *
  * NOTE: may modify gasneti_nodemap[] if env var GASNET_SUPERNODE_MAXSIZE is set.
  * TODO: splitting by socket or other criteria for/with GASNET_SUPERNODE_MAXSIZE.
  * TODO: keep widths around for conduits to use? (ibv and gemini both use)
  */
 extern void gasneti_nodemapParse(void) {
-  gasnet_node_t host_count = 0;
-  gasnet_node_t i,j,first;
-  gasnet_node_t limit,orig;
+  gasnet_node_t i,j,limit;
+  gasnet_node_t initial,final;
 
   struct { /* TODO: alloca? */
     gasnet_node_t width;
-    gasnet_node_t first;
+    gasnet_node_t h_lead;
+    gasnet_node_t sn_lead;
     gasnet_node_t host;
     gasnet_node_t supernode;
   } *s = gasneti_calloc(gasneti_nodes, sizeof(*s));
@@ -1055,15 +1030,31 @@ extern void gasneti_nodemapParse(void) {
   gasneti_assert(gasneti_nodemap[gasneti_mynode] <= gasneti_mynode);
 
   /* Check for user-imposed limit: 0 (or negative) means no limit */
+#if GASNET_PSHM
   limit = gasneti_getenv_int_withdefault("GASNET_SUPERNODE_MAXSIZE", 0, 0);
-#if GASNET_CONDUIT_SMP && GASNET_PSHM
+ #ifdef GASNETI_PSHM_GHEAP
+  if (limit != 1) {
+    char *envval = getenv("BG_MAPCOMMONHEAP"); /* Yes, plain getenv is intended here */
+    if (!envval || atoi(envval) != 1) {
+      if (!gasneti_mynode) {
+        fprintf(stderr, "WARNING: BG_MAPCOMMONHEAP is not '1' - disabing PSHM-over-gheap.\n");
+        fflush(stderr);
+      }
+      limit = 1;
+    }
+  }
+ #endif
+ #if GASNET_CONDUIT_SMP
   if (limit && !gasneti_mynode) {
     fprintf(stderr, "WARNING: ignoring GASNET_SUPERNODE_MAXSIZE for smp-conduit with PSHM.\n");
     fflush(stderr);
   }
   limit = gasneti_nodes;
-#else
+ #else
   if (limit <= 0) limit = gasneti_nodes;
+ #endif
+#else
+  limit = 1; /* No PSHM */
 #endif
 
   gasneti_assert(!gasneti_nodeinfo);
@@ -1072,41 +1063,49 @@ extern void gasneti_nodemapParse(void) {
 
   /* First pass: 
    * + Apply the supernode size limit, if any.
-   * + Determine global count and rank
-   * + Determine local count and rank
+   * + Determine the counts and ranks
    * + Construct gasneti_nodeinfo[].{host,supernode}
    */
-  gasneti_nodemap_global_count = 0;
-  gasneti_nodemap_local_count = 0;
-  orig = gasneti_nodemap[gasneti_mynode];
+  initial = gasneti_nodemap[gasneti_mynode];
   for (i = 0; i < gasneti_nodes; ++i) {
     const gasnet_node_t n = gasneti_nodemap[i];
     const gasnet_node_t width = s[n].width++;
     const gasnet_node_t lrank = width % limit;
-    if (!lrank) {
-      s[n].first = i;
-      s[n].supernode = gasneti_nodemap_global_count++;
-      if (!width) s[n].host = host_count++;
+    if (!width) { /* First node on host */
+      s[n].host = gasneti_myhost.grp_count++;
+    }
+    if (!lrank) { /* First node on supernode */
+      s[n].sn_lead = i;
+      s[n].supernode = gasneti_mysupernode.grp_count++;
     }
     if (i == gasneti_mynode) {
-      gasneti_nodemap_global_rank = s[n].supernode;
-      gasneti_nodemap_local_rank = lrank;
+      gasneti_mysupernode.node_rank = lrank;
+      gasneti_myhost.node_rank = width;
     }
-    gasneti_nodemap[i] = s[n].first;
+    s[i].h_lead = n;
+    gasneti_nodemap[i] = s[n].sn_lead;
     gasneti_nodeinfo[i].supernode = s[n].supernode;
     gasneti_nodeinfo[i].host = s[n].host;
   }
-  first = gasneti_nodemap[gasneti_mynode];
-  gasneti_nodemap_local_count = (first == s[orig].first) ? (((s[orig].width - 1) % limit) + 1) : limit;
-  gasneti_assert(gasneti_nodeinfo[gasneti_mynode].supernode == gasneti_nodemap_global_rank);
-  gasneti_free(s);
+  final = gasneti_nodemap[gasneti_mynode];
+  gasneti_mysupernode.node_count = (final == s[initial].sn_lead) ? (((s[initial].width - 1) % limit) + 1) : limit;
+  gasneti_myhost.node_count = s[initial].width;
+  gasneti_mysupernode.grp_rank = gasneti_nodeinfo[gasneti_mynode].supernode;
+  gasneti_myhost.grp_rank = gasneti_nodeinfo[gasneti_mynode].host;
 
-  /* Second pass: Construct array of local nodes */
-  gasneti_nodemap_local = gasneti_malloc(gasneti_nodemap_local_count*sizeof(gasnet_node_t));
-  for (i = first, j = 0; j < gasneti_nodemap_local_count; ++i) {
+  /* Second pass: Construct arrays of local nodes */
+  gasneti_assert(gasneti_myhost.node_count >= gasneti_mysupernode.node_count);
+  gasneti_myhost.nodes = gasneti_malloc(gasneti_myhost.node_count*sizeof(gasnet_node_t));
+  for (i = initial, j = 0; j < gasneti_myhost.node_count; ++i) {
     gasneti_assert(i < gasneti_nodes);
-    if (gasneti_nodemap[i] == first) gasneti_nodemap_local[j++] = i;
+    if (s[i].h_lead == initial) {
+      if (i == final) gasneti_mysupernode.nodes = gasneti_myhost.nodes + j;
+      gasneti_myhost.nodes[j++] = i;
+    }
   }
+  gasneti_assert(gasneti_mysupernode.nodes != NULL);
+
+  gasneti_free(s);
 
   #if GASNET_DEBUG_VERBOSE
   if (!gasneti_mynode) {
@@ -1172,11 +1171,12 @@ extern void gasneti_nodemapInit(gasneti_bootstrapExchangefn_t exchangefn,
  */
 extern void gasneti_nodemapFini(void) {
   gasneti_free(gasneti_nodemap);
-  gasneti_free(gasneti_nodemap_local);
+  gasneti_free(gasneti_myhost.nodes);
 #if GASNET_DEBUG
   /* To help catch any use-afer-Fini: */
   gasneti_nodemap = NULL;
-  gasneti_nodemap_local = NULL;
+  gasneti_myhost.nodes = NULL;
+  gasneti_mysupernode.nodes = NULL;
 #endif
 }
 
@@ -1684,7 +1684,9 @@ ssize_t gasneti_getline(char **buf_p, size_t *n_p, FILE *fp) {
       memcpy(ret, ptr, MIN(nbytes, sz));
       _gasneti_free(ptr, curloc);
     }
-    _gasneti_memcheck(ret,curloc,0);
+    if (sz) {
+      _gasneti_memcheck(ret,curloc,0);
+    }
     return ret;
   }
   extern void _gasneti_leak(void *ptr, const char *curloc) {
