@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/gasnet_internal.h,v $
- *     $Date: 2013/08/28 06:04:22 $
- * $Revision: 1.133 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/gasnet_internal.h $
  * Description: GASNet header for internal definitions used in GASNet implementation
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -256,8 +254,8 @@ extern void gasneti_freezeForDebugger(void);
 
 void gasneti_defaultSignalHandler(int sig);
 
-#if defined(HAVE_MMAP) || (GASNET_PSHM && defined(GASNETI_PSHM_SYSV))
-  #define GASNETI_MMAP_OR_SYSV 1
+#if defined(HAVE_MMAP) || GASNET_PSHM
+  #define GASNETI_MMAP_OR_PSHM 1
   extern gasnet_seginfo_t gasneti_mmap_segment_search(uintptr_t maxsz);
  #if defined(HAVE_MMAP)
   extern void gasneti_mmap_fixed(void *segbase, uintptr_t segsize);
@@ -299,11 +297,11 @@ typedef void (*gasneti_bootstrapBroadcastfn_t)(void *src, size_t len, void *dest
 typedef void (*gasneti_bootstrapBarrierfn_t)(void);
 
 #if !GASNET_SEGMENT_EVERYTHING
-#ifdef HAVE_MMAP
+#ifdef GASNETI_MMAP_OR_PSHM
 uintptr_t gasneti_mmapLimit(uintptr_t localLimit, uint64_t sharedLimit,
                             gasneti_bootstrapExchangefn_t exchangefn,
                             gasneti_bootstrapBarrierfn_t barrierfn);
-#endif /* HAVE_MMAP */
+#endif /* GASNETI_MMAP_OR_PSHM */
 void gasneti_segmentInit(uintptr_t localSegmentLimit,
                          gasneti_bootstrapExchangefn_t exchangefn);
 void gasneti_segmentAttach(uintptr_t segsize, uintptr_t minheapoffset,
@@ -667,12 +665,26 @@ extern void gasneti_defaultAMHandler(gasnet_token_t token);
 /* nodemap data and functions */
 
 extern gasnet_node_t *gasneti_nodemap;
-extern gasnet_node_t *gasneti_nodemap_local;
-extern gasnet_node_t gasneti_nodemap_local_count;
-extern gasnet_node_t gasneti_nodemap_local_rank;
-extern gasnet_node_t gasneti_nodemap_global_count;
-extern gasnet_node_t gasneti_nodemap_global_rank;
-extern gasnet_nodeinfo_t *gasneti_nodeinfo;
+
+typedef struct gasneti_nodegrp_s {
+  /* List of member nodes in ascending order */
+  gasnet_node_t *nodes;
+  /* Number of nodes in group and my rank within them */
+  gasnet_node_t node_count;
+  gasnet_node_t node_rank;
+  /* Number of peers (groups of same class) and this group's rank */
+  gasnet_node_t grp_count;
+  gasnet_node_t grp_rank;
+} gasneti_nodegrp_t;
+
+extern gasneti_nodegrp_t gasneti_myhost;
+
+extern gasneti_nodegrp_t gasneti_mysupernode;
+#define gasneti_nodemap_local         gasneti_mysupernode.nodes
+#define gasneti_nodemap_local_count   gasneti_mysupernode.node_count
+#define gasneti_nodemap_local_rank    gasneti_mysupernode.node_rank
+#define gasneti_nodemap_global_count  gasneti_mysupernode.grp_count
+#define gasneti_nodemap_global_rank   gasneti_mysupernode.grp_rank
 
 extern void gasneti_nodemapInit(gasneti_bootstrapExchangefn_t exchangefn,
                                 const void *ids, size_t sz, size_t stride);
