@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/gasnet_mmap.c,v $
- *     $Date: 2013/09/25 02:16:26 $
- * $Revision: 1.130.2.1 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/gasnet_mmap.c $
  * Description: GASNet memory-mapping utilities
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -14,13 +12,13 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#if defined(GASNETI_MMAP_OR_SYSV) && !defined(HAVE_MMAP)
-  /* Ensure PSHM-over-SYSV support never actually calls mmap() */
+#if defined(GASNETI_MMAP_OR_PSHM) && !defined(HAVE_MMAP)
+  /* Ensure PSHM over SYSV or GHEAP never actually call mmap() */
   #define mmap %%%ERROR__GASNet_does_not_support_mmap_in_this_configuration%%%
   #define munmap %%%ERROR__GASNet_does_not_support_munmap_in_this_configuration%%%
 #endif
 
-#ifdef GASNETI_MMAP_OR_SYSV
+#ifdef GASNETI_MMAP_OR_PSHM
  #if GASNET_PSHM && !defined(_POSIX_C_SOURCE) && PLATFORM_OS_SOLARIS
   #define _POSIX_C_SOURCE 200112L /* Required for shm_{open,unlink} decls */
   #include <sys/mman.h>
@@ -49,25 +47,37 @@
   #endif
  #endif
 
+ /* TODO:
+    Below are a couple cases where we want to use the lower portion of the O/S-provided
+    mmap()ed region to create our segment because one must ensure some alignment which
+    is greater than the page size.  Instead of forcing the use of the bottom portion,
+    we *could* change the trimming logic to use a chosen granularity other than that of
+    GASNETI_PAGESIZE (such as the huge page size).
+  */
+
+ #ifdef GASNETI_USE_HUGETLBFS
+  #include <hugetlbfs.h>
+  /* Trim only from top to retain alignment: */
+  #undef GASNETI_USE_HIGHSEGMENT
+  #define GASNETI_USE_HIGHSEGMENT 0
+ #endif
+
+ #if GASNET_PSHM && PLATFORM_ARCH_SPARC
+  /* On SPARC, shared mappings must be L1 data-cache color-aligned.
+     The easiest way to ensure that is to trim only from top to preserve
+     the alignment which the O/S gave to the original (untrimmed) mapping.
+   Possible TODO:
+     Since we only support 64-bit SPARC CPUs the alignment will always be MAX(16K, PAGESIZE).
+     So we could just use that for GASNETI_PAGESIZE.
+   */
+  #undef GASNETI_USE_HIGHSEGMENT
+  #define GASNETI_USE_HIGHSEGMENT 0
+ #endif
+
 #if !HAVE_MMAP
   /* Skip the following platform checks */
-#elif PLATFORM_OS_IRIX
-  #ifdef MAP_SGI_ANYADDR /* allow mmap to use 'reserved' 256MB region on O2k */
-    #define GASNETI_MMAP_FLAGS (MAP_PRIVATE | MAP_SGI_ANYADDR | MAP_AUTORESRV)
-  #else
-    #define GASNETI_MMAP_FLAGS (MAP_PRIVATE | MAP_AUTORESRV)
-  #endif
-  #define GASNETI_MMAP_FILE "/dev/zero"
-#elif PLATFORM_ARCH_CRAYX1
-  #define GASNETI_MMAP_FLAGS (MAP_PRIVATE | MAP_AUTORESRV)
-  #define GASNETI_MMAP_FILE "/dev/zero"
-#elif PLATFORM_ARCH_CRAYT3E
-  #error mmap not supported on Cray-T3E
 #elif PLATFORM_OS_CYGWIN
   #error mmap not supported on Cygwin - it doesnt work properly
-#elif PLATFORM_OS_HPUX
-  #define GASNETI_MMAP_FLAGS (MAP_ANONYMOUS | MAP_NORESERVE | MAP_PRIVATE)
-  #define GASNETI_MMAP_NOTFIXED_FLAG MAP_VARIABLE
 #elif PLATFORM_ARCH_MIC
   #define GASNETI_MMAP_FLAGS (MAP_ANONYMOUS | MAP_PRIVATE | MAP_POPULATE | MAP_HUGETLB)
 #endif
@@ -102,8 +112,8 @@
   #define GASNETI_MMAP_NOTFIXED_FLAG 0
 #endif
 
-#if GASNET_PSHM && (PLATFORM_OS_BGP || PLATFORM_OS_BGQ || PLATFORM_OS_CYGWIN)
-  /* BG/P and /Q: MAP_FIXED is ignored for fd obtained from pshm_open() */
+#if GASNET_PSHM && (PLATFORM_OS_BGQ || PLATFORM_OS_CYGWIN)
+  /* BG/Q: MAP_FIXED is ignored for fd obtained from pshm_open() */
   /* CYGWIN: may not honor the address passed to shmat() */
   #define GASNETI_PSHM_MAP_FIXED_IGNORED 1
 #endif
@@ -159,6 +169,10 @@ static void *gasneti_mmap_internal(void *segbase, uintptr_t segsize) {
                        (unsigned long)segsize, strerror(mmap_errno));
   }
 
+  if ((ptr != (void*)GASNETI_PAGE_ALIGNDOWN(ptr)) && (ptr != MAP_FAILED)) {
+      gasneti_fatalerror("mmap result "GASNETI_LADDRFMT" is not aligned to GASNET_PAGESIZE %lu (0x%lx)",
+              GASNETI_LADDRSTR(ptr), (unsigned long)GASNET_PAGESIZE, (unsigned long)GASNET_PAGESIZE);
+  }
   if (segbase && ptr == MAP_FAILED) {
       gasneti_fatalerror("mmap fixed failed at "GASNETI_LADDRFMT" for size %lu: %s\n",
 	      GASNETI_LADDRSTR(segbase), (unsigned long)segsize, strerror(mmap_errno));
@@ -188,6 +202,10 @@ extern void *gasneti_mmap(uintptr_t segsize) {
 #elif defined(GASNETI_PSHM_XPMEM)
   static gasneti_xpmem_segid_t *gasneti_pshm_segids = NULL;
   static gasneti_xpmem_apid_t *gasneti_pshm_apids = NULL;
+#elif defined(GASNETI_PSHM_GHEAP)
+  static void* gasneti_pshm_valloc(size_t size);
+  static void gasneti_pshm_vfree(void *addr);
+  static void **gasneti_pshm_segaddrs = NULL;
 #endif
 
 static char *gasneti_pshm_tmpfile = NULL;
@@ -250,18 +268,27 @@ static const char *gasneti_pshm_makeunique(const char *unique) {
   static char prefix[] = "/GASNTXXXXXX";
 #if defined(GASNETI_PSHM_FILE) || defined(GASNETI_PSHM_SYSV)
   const char *tmpdir = gasneti_tmpdir();
-  const size_t tmpdir_len = strlen(tmpdir);
-#else
-  const size_t tmpdir_len = 0;
 #endif
+  size_t tmpdir_len = 0;
 #if !defined(GASNETI_PSHM_SYSV)
-  const size_t base_len = tmpdir_len + GASNETI_PSHM_PREFIX_LEN;
+  size_t base_len;
   char *allnames;
 #endif
   int i;
 
   gasneti_assert(strlen(prefix) == GASNETI_PSHM_PREFIX_LEN);
 
+#if defined(GASNETI_PSHM_FILE) && defined(GASNETI_USE_HUGETLBFS)
+  { const char *hugedir = hugetlbfs_find_path();
+    if (hugedir && !access(hugedir, R_OK|W_OK|X_OK)) {
+      tmpdir = hugedir;
+    }
+  }
+#endif
+#if defined(GASNETI_PSHM_FILE) || defined(GASNETI_PSHM_SYSV)
+  tmpdir_len = strlen(tmpdir);
+#endif
+
   if (!unique) { /* We get to pick the unique bits */
 #if defined(GASNETI_PSHM_FILE) || defined(GASNETI_PSHM_SYSV)
     if (gasneti_pshm_mkstemp(prefix, tmpdir)) {
@@ -325,6 +352,7 @@ static const char *gasneti_pshm_makeunique(const char *unique) {
   memcpy(prefix + GASNETI_PSHM_PREFIX_LEN1, unique, GASNETI_PSHM_UNIQUE_LEN);
 
   gasneti_pshmname = (char **)gasneti_malloc((gasneti_pshm_nodes+1)*sizeof(char*));
+  base_len = tmpdir_len + GASNETI_PSHM_PREFIX_LEN;
   allnames = (char *)gasneti_malloc((gasneti_pshm_nodes+1)*(base_len + 4));
 
   for (i = 0; i <= gasneti_pshm_nodes; ++i) {
@@ -354,143 +382,45 @@ static const char *gasneti_pshm_makeunique(const char *unique) {
 #endif /* GASNET_PSHM */
 
 #if defined(GASNETI_USE_HUGETLBFS)
-#include <hugetlbfs.h>
 
-#if defined(HAVE_HUGETLBFS_UNLINKED_FD_FOR_SIZE) && 0 /* Disabled pending further study */
-#define GASNETI_USE_HUGETLBFS_SIZES 1
-#endif
-
-#if defined(GASNETI_USE_HUGETLBFS_SIZES)
-static int compare_long(const void *a_p, const void *b_p) {
-  long a = *(long *)a_p;
-  long b = *(long *)b_p;
-  return (a==b) ? 0 : ((a<b) ? -1 : 1);
-}
-#endif
-
-/* Pick an available hugepage size for mapping of the requested size.
- * With older libhugetlbfs this will always pick the default size.
- * The given addr and size are adjusted for proper alignment.
+/* Apply the default hugepage size for mapping of the requested size.
+ * The given size is adjusted for proper alignment and returned.
  */
-static long pick_pagesz(void **addr_p, uintptr_t *size_p) {
-  uintptr_t size = *size_p;
-
-#if defined(GASNETI_USE_HUGETLBFS_SIZES)
-  /* Currently pick largest size less than the rounded request.
-   * Other possibilities exists, of course.
-   */
-  #define gasneti_unlinked_huge_fd(pgsz) hugetlbfs_unlinked_fd_for_size(pgsz)
-  static long *tbl = NULL;
-  static int count = 0;
-  long pagesz;
-  int i;
-
-  if (!tbl) {
-    long dflt = gethugepagesize();
-    count = gethugepagesizes(NULL, 0);
-    tbl = gasneti_calloc(count+1, sizeof(long)); /* final 0 marks end */
-    gethugepagesizes(tbl, count);
-    qsort(tbl, count, sizeof(long), compare_long);
-    /* remove from consideration any smaller than the default */
-    while (*tbl && (*tbl < dflt)) { ++tbl; --count; }
-  }
-
-  for (i=0; i<count; ++i) {
-    long next = tbl[i+1];
-    pagesz = tbl[i];
-    if (!next || (GASNETI_ALIGNUP(size, pagesz) < next)) break;
-  }
-#elif defined(HAVE_HUGETLBFS_UNLINKED_FD)
-  #define gasneti_unlinked_huge_fd(pgsz) hugetlbfs_unlinked_fd()
+static uintptr_t huge_pagesz(void *addr, uintptr_t size) {
   static long pagesz = 0;
   if (!pagesz) pagesz = gethugepagesize();
-#else
-  #error
-#endif
-
-  if (addr_p) {
-    uintptr_t addr = (uintptr_t)(*addr_p);
-    uintptr_t new_addr = GASNETI_ALIGNDOWN(addr, pagesz);
-    *addr_p = (void*)new_addr;
-    size = GASNETI_ALIGNUP(size + (addr - new_addr), pagesz);
-  } else {
-    size = GASNETI_ALIGNUP(size, pagesz);
-  }
-  *size_p = size;
-
-  return pagesz;
+  gasneti_assert((uintptr_t)addr % pagesz == 0); /* alignment check */
+  return GASNETI_ALIGNUP(size, pagesz);
 }
 
 extern void *gasneti_huge_mmap(void *addr, uintptr_t size) {
-  void *real_addr = addr;
-  GASNETI_UNUSED long pagesz = pick_pagesz(&real_addr, &size);
-  int fd = gasneti_unlinked_huge_fd(pagesz);
+  int fd = hugetlbfs_unlinked_fd();
   const int mmap_flags = MAP_SHARED | (addr ? GASNETI_MMAP_FIXED_FLAG : GASNETI_MMAP_NOTFIXED_FLAG);
-  void *ptr = mmap(real_addr, size, (PROT_READ|PROT_WRITE), mmap_flags, fd, 0);
+  void *ptr = mmap(addr, huge_pagesz(addr, size), (PROT_READ|PROT_WRITE), mmap_flags, fd, 0);
 
+  int save_errno = errno;
   (void) close(fd);
-
-  /* If expanded a fixed mmap then be sure to return the requested fixed addr */
-  if (real_addr != addr) {
-    gasneti_assert(ptr == real_addr);
-    ptr = addr;
-  }
+  errno = save_errno;
 
   return ptr;
 }
 
 extern void gasneti_huge_munmap(void *addr, uintptr_t size) {
-  (void)pick_pagesz(&addr, &size);
-  if (munmap(addr, size) != 0) 
+  if (munmap(addr, huge_pagesz(addr, size)) != 0)
     gasneti_fatalerror("munmap("GASNETI_LADDRFMT",%lu) failed: %s\n",
                        GASNETI_LADDRSTR(addr), (unsigned long)size, strerror(errno));
 }
-#endif
+
+#endif /* defined(GASNETI_USE_HUGETLBFS) */
 
 #if GASNET_PSHM
 
-#if defined(PLATFORM_OS_BGQ) /* must delay close until unmap() */
-  static struct {
-    void *addr;
-    int fd;
-  } *gasneti_addr2fd = NULL;
-  static void gasneti_addr2fd_add(void *addr, int fd) {
-    int i;
-    if (NULL == gasneti_addr2fd) {
-       gasneti_addr2fd = gasneti_calloc(1+gasneti_pshm_nodes, sizeof(*gasneti_addr2fd));
-    }
-    for (i=0; i<=gasneti_pshm_nodes; ++i) {
-      if (gasneti_addr2fd[i].addr == NULL) {
-        gasneti_addr2fd[i].addr = addr;
-        gasneti_addr2fd[i].fd = fd;
-        return;
-      }
-    }
-    gasneti_fatalerror("addr2fd table is full");
-  }
-  static int gasneti_addr2fd_del(void *addr) {
-    int i;
-    for (i=0; i<=gasneti_pshm_nodes; ++i) {
-      if (gasneti_addr2fd[i].addr == addr) {
-        gasneti_addr2fd[i].addr = NULL;
-        return gasneti_addr2fd[i].fd;
-      }
-    }
-    gasneti_fatalerror("address not found in addr2fd table");
-  }
-#endif /* BGQ */
-
 static void gasneti_pshm_unlink(int pshm_rank);
 
 /* create the object/region/segment and return its address */
 static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
-#if defined(PLATFORM_OS_BGP) || defined(PLATFORM_OS_BGQ)
-  /* shm_unlink() is apparently a no-op on BG/P and earlier driver versions on /Q */
-  const int create = ((pshm_rank == gasneti_pshm_nodes) && !gasneti_pshm_mynode);
-#else
   const int create = (pshm_rank == gasneti_pshm_mynode) ||
                      ((pshm_rank == gasneti_pshm_nodes) && !gasneti_pshm_mynode);
-#endif
   void * ptr = MAP_FAILED;
 
 #if defined(GASNETI_PSHM_SYSV)
@@ -518,6 +448,9 @@ static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
 
   /* create or open */
   #if defined(GASNETI_PSHM_FILE)
+   #if defined(GASNETI_USE_HUGETLBFS)
+    segsize = huge_pagesz(segbase, segsize);
+   #endif
     fd = open(filename, flags, S_IRUSR | S_IWUSR);
   #elif defined(GASNETI_PSHM_POSIX)
     fd = shm_open(filename, flags, S_IRUSR | S_IWUSR);
@@ -530,13 +463,6 @@ static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
           fd = shm_open(filename, flags, S_IRUSR | S_IWUSR);
         } while ((fd == -1) && (errno == EEXIST) && retries_remain--);
       }
-    #elif defined(PLATFORM_OS_BGQ)
-      /* XXX: configure or compile time way to know? */
-      if ((fd == -1) && (errno == ENOENT) && (pshm_rank == gasneti_pshm_mynode)) {
-        /* We didn't use O_CREAT+O_EXCL on the first try, because shm_unlink()
-         * does nothing prior to V1R2M1.  So, try again with those flags. */
-        fd = shm_open(filename, flags | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
-      }
     #endif
   #else
     #error
@@ -552,12 +478,6 @@ static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
     ptr = mmap(segbase, segsize, (PROT_READ|PROT_WRITE), mmap_flags, fd, 0);
   }
 
- #if defined(PLATFORM_OS_BGQ)
-  /* must delay close() untile just before munmap() */
-  if (ptr != MAP_FAILED) {
-    gasneti_addr2fd_add(ptr, fd);
-  } else
- #endif
   {
     const int save_errno = errno;
     (void) close(fd);
@@ -565,7 +485,7 @@ static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
   }
 #elif defined(GASNETI_PSHM_XPMEM)
   if (create) {
-  #if HAVE_HUGETLBFS
+  #if GASNETI_USE_HUGETLBFS
     ptr = gasneti_huge_mmap(segbase, segsize);
   #else
     const int mmap_flags = GASNETI_MAP_ANONYMOUS | MAP_SHARED | \
@@ -573,14 +493,29 @@ static void * gasneti_pshm_mmap(int pshm_rank, void *segbase, size_t segsize) {
     ptr = mmap(segbase, segsize, (PROT_READ|PROT_WRITE), mmap_flags, 0, 0);
   #endif
   } else {
-    gasneti_xpmem_addr_t xa;
-    xa.offset = 0;
-    xa.gasneti_xpmem_apid =
-          gasneti_pshm_apids[pshm_rank] = 
+    gasneti_xpmem_apid_t apid =
+  #if HAVE_XPMEM_MAKE_2
+            xpmem_get_2(gasneti_pshm_segids[pshm_rank], XPMEM_RDWR, XPMEM_PERMIT_MODE, NULL);
+  #else
               xpmem_get(gasneti_pshm_segids[pshm_rank], XPMEM_RDWR, XPMEM_PERMIT_MODE, NULL);
-    if (xa.gasneti_xpmem_apid != (gasneti_xpmem_apid_t)-1) {
+  #endif
+    if (apid != (gasneti_xpmem_apid_t)-1) {
+    #if HAVE_XPMEM_MAKE_2
+      ptr = xpmem_attach_2(apid, 0, segsize, segbase);
+    #else
+      gasneti_xpmem_addr_t xa;
+      xa.offset = 0;
+      xa.gasneti_xpmem_apid = apid;
       ptr = xpmem_attach(xa, segsize, segbase);
+    #endif
     }
+    gasneti_pshm_apids[pshm_rank] = apid;
+  }
+#elif defined(GASNETI_PSHM_GHEAP)
+  if (create) {
+    ptr = gasneti_pshm_valloc(segsize);
+  } else {
+    ptr = gasneti_pshm_segaddrs[pshm_rank];
   }
 #else
   #error
@@ -602,14 +537,10 @@ static void gasneti_pshm_munmap(void *segbase, uintptr_t segsize) {
       gasneti_fatalerror("shmdt("GASNETI_LADDRFMT") failed: %s\n",
 	      GASNETI_LADDRSTR(segbase), strerror(errno));
   }
-#elif defined(GASNETI_PSHM_FILE) || defined(GASNETI_PSHM_POSIX)
+#elif defined(GASNETI_PSHM_FILE) || defined(GASNETI_PSHM_POSIX) || defined(GASNETI_PSHM_XPMEM)
   gasneti_munmap(segbase, segsize);
-#elif defined(GASNETI_PSHM_XPMEM)
- #if HAVE_HUGETLBFS
-  gasneti_huge_munmap(segbase, segsize);
- #else
-  gasneti_munmap(segbase, segsize);
- #endif
+#elif defined(GASNETI_PSHM_GHEAP)
+  gasneti_pshm_vfree(segbase);
 #else
   #error
 #endif
@@ -627,8 +558,15 @@ static void gasneti_munmap_remote(gasnet_node_t pshm_rank, void *segbase, uintpt
 #elif defined(GASNETI_PSHM_FILE) || defined(GASNETI_PSHM_POSIX)
   gasneti_munmap(segbase, segsize);
 #elif defined(GASNETI_PSHM_XPMEM)
+ #if HAVE_XPMEM_MAKE_2
+  xpmem_detach_2(segbase, segsize);
+  xpmem_release_2(gasneti_pshm_apids[pshm_rank]);
+ #else
   xpmem_detach(segbase);
   xpmem_release(gasneti_pshm_apids[pshm_rank]);
+ #endif
+#elif defined(GASNETI_PSHM_GHEAP)
+ /* Nothing to do here */
 #else
   #error
 #endif
@@ -639,11 +577,18 @@ GASNETI_INLINE(gasneti_export_segment)
 void gasneti_export_segment(void *segbase, uintptr_t segsize) {
 #if defined(GASNETI_PSHM_XPMEM)
   /* Create and supernode-exchange xpmem segment ids */
-  gasneti_xpmem_segid_t segid = xpmem_make(segbase, segsize, XPMEM_PERMIT_MODE, (void *)(uintptr_t)0600);
+  gasneti_xpmem_segid_t segid =
+  #if HAVE_XPMEM_MAKE_2
+          xpmem_make_2(segbase, segsize, XPMEM_PERMIT_MODE, (void *)(uintptr_t)0600);
+  #else
+            xpmem_make(segbase, segsize, XPMEM_PERMIT_MODE, (void *)(uintptr_t)0600);
+  #endif
   if_pf (segid == (gasneti_xpmem_segid_t)(-1)) {
     fprintf(stderr, "xpmem_make() failed:%s\n", strerror(errno));
   }
   gasneti_pshmnet_bootstrapExchange(gasneti_request_pshmnet, &segid, sizeof(segid), gasneti_pshm_segids);
+#elif defined(GASNETI_PSHM_GHEAP)
+  gasneti_pshmnet_bootstrapExchange(gasneti_request_pshmnet, &segbase, sizeof(segbase), gasneti_pshm_segaddrs);
 #else
   /* empty */
 #endif
@@ -669,7 +614,7 @@ static void gasneti_pshm_unlink(int pshm_rank){
 #elif defined(GASNETI_PSHM_POSIX)
   const char *filename = gasneti_pshmname[pshm_rank];
   (void)shm_unlink(filename);
-#elif defined(GASNETI_PSHM_XPMEM)
+#elif defined(GASNETI_PSHM_XPMEM) || defined(GASNETI_PSHM_GHEAP)
   /* Nothing to do */
 #else
   #error
@@ -682,7 +627,9 @@ static void gasneti_pshm_unlink(int pshm_rank){
  */
 static void gasneti_unlink_segments(void) {
   gasneti_pshmnet_bootstrapBarrier();
-#if !defined(GASNETI_PSHM_XPMEM)
+#if defined(GASNETI_PSHM_XPMEM) || defined(GASNETI_PSHM_GHEAP)
+  /* Nothing to do because gasneti_pshm_unlink() is a no-op */
+#else
   gasneti_pshm_unlink(gasneti_pshm_mynode);
   gasneti_pshmnet_bootstrapBarrier();
 #endif
@@ -710,15 +657,14 @@ static void gasneti_cleanup_shm(void) {
     gasneti_free(gasneti_pshmname);
     gasneti_pshmname = NULL;
   }
-  #if defined(PLATFORM_OS_BGQ)
-    gasneti_free(gasneti_addr2fd);
-    gasneti_addr2fd = NULL;
-  #endif
 #elif defined(GASNETI_PSHM_XPMEM)
   gasneti_free(gasneti_pshm_segids);
   gasneti_pshm_segids = NULL;
   gasneti_free(gasneti_pshm_apids);
   gasneti_pshm_apids = NULL;
+#elif defined(GASNETI_PSHM_GHEAP)
+  gasneti_free(gasneti_pshm_segaddrs);
+  gasneti_pshm_segaddrs = NULL;
 #else
   #error
 #endif
@@ -782,6 +728,11 @@ static void *gasneti_mmap_shared_internal(int pshmnode, void *segbase, uintptr_t
     }
   }
 
+  if ((ptr != (void*)GASNETI_PAGE_ALIGNDOWN(ptr)) && (ptr != MAP_FAILED)) {
+    gasneti_cleanup_shm();
+    gasneti_fatalerror("mmap result "GASNETI_LADDRFMT" is not aligned to GASNET_PAGESIZE %lu (0x%lx)",
+              GASNETI_LADDRSTR(ptr), (unsigned long)GASNET_PAGESIZE, (unsigned long)GASNET_PAGESIZE);
+  }
 #if !GASNETI_PSHM_MAP_FIXED_IGNORED
   if (segbase && (segbase != ptr) && (ptr != MAP_FAILED)) {
     gasneti_cleanup_shm();
@@ -804,19 +755,18 @@ extern void *gasneti_mmap_shared(uintptr_t segsize) {
   return gasneti_mmap_shared_internal(gasneti_pshm_mynode, NULL, segsize, 1);
 }
 
-extern void *gasneti_mmap_vnet(uintptr_t size, gasneti_bootstrapExchangefn_t exchangefn) {
+extern void *gasneti_mmap_vnet(uintptr_t size, gasneti_bootstrapBroadcastfn_t snodebcastfn) {
   void *ptr = MAP_FAILED;
+  int save_errno = 0;
 
   #if defined(GASNETI_PSHM_SYSV) && PLATFORM_OS_CYGWIN
   /* Cygwin may raise SIGSYS when SysV support is absent.
      This will yield more informative error messages. */
   gasneti_sighandlerfn_t prev_handler = gasneti_reghandler(SIGSYS, SIG_IGN);
-  int save_errno;
   #endif
 
   #if defined(GASNETI_PSHM_FILE) || defined(GASNETI_PSHM_SYSV) || defined(GASNETI_PSHM_POSIX)
   {
-    char (*exchg)[GASNETI_PSHM_UNIQUE_LEN];
     char unique[GASNETI_PSHM_UNIQUE_LEN];
 
     /* First in each supernode creates the segment plus the names/keys and returns the unique identifier */
@@ -824,24 +774,21 @@ extern void *gasneti_mmap_vnet(uintptr_t size, gasneti_bootstrapExchangefn_t exc
       const char *tmp = gasneti_pshm_makeunique(NULL);
       memcpy(unique, tmp, GASNETI_PSHM_UNIQUE_LEN);
       ptr = gasneti_mmap_shared_internal(gasneti_pshm_nodes, NULL, size, 1);
+      save_errno = errno;
     }
 
-    /* Conduit's exchangefn is used as a supernode-scoped bcast to
-     * communicate the unique identifier generated by the firsts */
-    exchg = gasneti_malloc(gasneti_nodes * GASNETI_PSHM_UNIQUE_LEN);
-    (*exchangefn)(unique, GASNETI_PSHM_UNIQUE_LEN, exchg);
-
-    /* Non-first nodes attach and generate the same names/keys from the unique identifier */
+    /* Supernode-scoped bcast communicates the names/keys generated by the firsts */
+    (*snodebcastfn)(unique, GASNETI_PSHM_UNIQUE_LEN, unique, gasneti_pshm_firstnode);
+  
+    /* Non-first nodes attach */
     if (gasneti_pshm_mynode != 0) {
-      (void)gasneti_pshm_makeunique((const char *)(exchg + gasneti_pshm_firstnode));
+      (void)gasneti_pshm_makeunique(unique);
       ptr = gasneti_mmap_shared_internal(gasneti_pshm_nodes, NULL, size, 1);
+      save_errno = errno;
     }
-
-    gasneti_free(exchg);
   }
   #elif defined(GASNETI_PSHM_XPMEM)
   {
-    gasneti_xpmem_segid_t *exchg;
     gasneti_xpmem_segid_t segid = (gasneti_xpmem_segid_t)(-1);
 
     /* Initialization */
@@ -851,24 +798,54 @@ extern void *gasneti_mmap_vnet(uintptr_t size, gasneti_bootstrapExchangefn_t exc
     /* First in each supernode creates the segment */
     if (gasneti_pshm_mynode == 0) {
       ptr = gasneti_mmap_shared_internal(gasneti_pshm_nodes, NULL, size, 1);
+      save_errno = errno;
       if (ptr != MAP_FAILED) {
-        segid = xpmem_make(ptr, size, XPMEM_PERMIT_MODE, (void *)(uintptr_t)0600);
+      #if HAVE_XPMEM_MAKE_2
+        segid = xpmem_make_2(ptr, size, XPMEM_PERMIT_MODE, (void *)(uintptr_t)0600);
+      #else
+        segid =   xpmem_make(ptr, size, XPMEM_PERMIT_MODE, (void *)(uintptr_t)0600);
+      #endif
+        save_errno = errno;
         if_pf (segid == (gasneti_xpmem_segid_t)(-1)) {
           fprintf(stderr, "xpmem_make() failed:%s\n", strerror(errno));
         }
       }
     }
     
-    /* Conduit's exchangefn is used as a supernode-scoped bcast to
-     * communicate the segment identifier generated by the firsts */
-    exchg = gasneti_malloc(gasneti_nodes * sizeof(segid));
-    (*exchangefn)(&segid, sizeof(segid), exchg);
-    gasneti_pshm_segids[gasneti_pshm_nodes] = exchg[gasneti_pshm_firstnode];
-    gasneti_free(exchg);
+    /* Supernode-scoped bcast communicates the segment identifier generated by the firsts */
+    (*snodebcastfn)(&segid, sizeof(segid),
+                    &gasneti_pshm_segids[gasneti_pshm_nodes],
+                    gasneti_pshm_firstnode);
 
     /* Non-first nodes attach */
     if (gasneti_pshm_mynode != 0) {
       ptr = gasneti_mmap_shared_internal(gasneti_pshm_nodes, NULL, size, 1);
+      save_errno = errno;
+    }
+  }
+  #elif defined(GASNETI_PSHM_GHEAP)
+  {
+    /* Initialization */
+    gasneti_pshm_segaddrs = gasneti_malloc(sizeof(void*) * (gasneti_pshm_nodes + 1));
+
+    /* First in each supernode creates the segment */
+    if (gasneti_pshm_mynode == 0) {
+      ptr = gasneti_mmap_shared_internal(gasneti_pshm_nodes, NULL, size, 1);
+      save_errno = errno;
+      if (ptr != MAP_FAILED) {
+        memset(ptr, 0, size);
+      }
+    }
+    
+    /* Supernode-scoped bcast communicates the segment addresses allocated by the firsts */
+    (*snodebcastfn)(&ptr, sizeof(ptr),
+                    &gasneti_pshm_segaddrs[gasneti_pshm_nodes],
+                    gasneti_pshm_firstnode);
+
+    /* Non-first nodes attach */
+    if (gasneti_pshm_mynode != 0) {
+      ptr = gasneti_mmap_shared_internal(gasneti_pshm_nodes, NULL, size, 1);
+      save_errno = errno;
     }
   }
   #else
@@ -876,11 +853,12 @@ extern void *gasneti_mmap_vnet(uintptr_t size, gasneti_bootstrapExchangefn_t exc
   #endif
 
   #if defined(GASNETI_PSHM_SYSV) && PLATFORM_OS_CYGWIN
-  save_errno = errno;
   gasneti_reghandler(SIGSYS, prev_handler);
-  errno = save_errno;
   #endif
 
+  /* restore the pertinent errno, if any */
+  errno = save_errno;
+
   return (ptr == MAP_FAILED) ? NULL : ptr;
 }
 extern void gasneti_unlink_vnet(void) {
@@ -894,22 +872,13 @@ extern void gasneti_munmap(void *segbase, uintptr_t segsize) {
   gasneti_tick_t t1, t2;
   gasneti_assert(segsize > 0);
   t1 = gasneti_ticks_now();
-    #if 0 && PLATFORM_OS_TRU64 /* doesn't seem to help */
-      /* invalidate the pages before unmap to avoid write-back penalty */
-      if (madvise(segbase, segsize, MADV_DONTNEED))
-        gasneti_fatalerror("madvise("GASNETI_LADDRFMT",%lu) failed: %s\n",
-	        GASNETI_LADDRSTR(segbase), (unsigned long)segsize, strerror(errno));
-      if (msync(segbase, segsize, MS_INVALIDATE))
-        gasneti_fatalerror("msync("GASNETI_LADDRFMT",%lu) failed: %s\n",
-	        GASNETI_LADDRSTR(segbase), (unsigned long)segsize, strerror(errno));
-    #endif
-    #if GASNET_PSHM && defined(PLATFORM_OS_BGQ)
-      /* we delayed the close() until now */
-      close(gasneti_addr2fd_del(segbase));
-    #endif
+  #if defined(GASNETI_USE_HUGETLBFS)
+    gasneti_huge_munmap(segbase, segsize);
+  #else
     if (munmap(segbase, segsize) != 0) 
       gasneti_fatalerror("munmap("GASNETI_LADDRFMT",%lu) failed: %s\n",
 	      GASNETI_LADDRSTR(segbase), (unsigned long)segsize, strerror(errno));
+  #endif
   t2 = gasneti_ticks_now();
 
   GASNETI_TRACE_PRINTF(D,("munmap("GASNETI_LADDRFMT", %lu): %.3fus\n", 
@@ -1005,7 +974,7 @@ static gasnet_seginfo_t _gasneti_mmap_segment_search_inner(uintptr_t maxsz) {
     si.size = maxsz;
     mmaped = 1;
   } else { /* use a search to find largest possible */
-    #if PLATFORM_OS_TRU64
+    #if 0
       /* linear descending search best on systems with 
          fast mmap-failed and very slow unmap and/or mmap-succeed */
       si = gasneti_mmap_lineardesc_segsrch(maxsz);
@@ -1072,7 +1041,7 @@ extern gasnet_seginfo_t gasneti_mmap_segment_search(uintptr_t maxsz) {
 }
 
 /* ------------------------------------------------------------------------------------ */
-#endif /* GASNETI_MMAP_OR_SYSV */
+#endif /* GASNETI_MMAP_OR_PSHM */
 
 #if defined(GASNETI_MMAP_MAX_SIZE)
   GASNETI_IDENT(gasneti_IdentString_DefaultMaxSegsize, 
@@ -1113,14 +1082,14 @@ uintptr_t _gasneti_max_segsize(uint64_t configure_val) {
 #if !GASNET_SEGMENT_EVERYTHING
 /* mmap-based segment init/attach */
 static gasnet_seginfo_t gasneti_segment = {0,0}; /* local segment info */
-#ifdef GASNETI_MMAP_OR_SYSV
+#ifdef GASNETI_MMAP_OR_PSHM
 static uintptr_t gasneti_myheapend = 0; /* top of my malloc heap */
 static uintptr_t gasneti_maxheapend = 0; /* top of max malloc heap */
 static uintptr_t gasneti_maxbase = 0; /* start of segment overlap region */
 #if GASNET_PSHM
 static gasnet_seginfo_t *gasneti_remote_segments;
 #endif /* GASNET_PSHM */
-#endif /* GASNETI_MMAP_OR_SYSV */
+#endif /* GASNETI_MMAP_OR_PSHM */
 
 typedef struct {
   gasnet_seginfo_t seginfo;
@@ -1129,7 +1098,7 @@ typedef struct {
 } gasneti_segexch_t;
 static gasneti_segexch_t *gasneti_segexch = NULL; /* exchanged segment information */
 
-#ifdef GASNETI_MMAP_OR_SYSV
+#ifdef GASNETI_MMAP_OR_PSHM
 /* perform a coordinated mmap probe to determine the max memory
     that can be mmap()ed while considering multiple GASNet nodes
     per shared memory node
@@ -1150,6 +1119,7 @@ uintptr_t gasneti_mmapLimit(uintptr_t localLimit, uint64_t sharedLimit,
                             gasneti_bootstrapBarrierfn_t barrierfn) {
   int i;
   uintptr_t maxsz;
+  const gasnet_node_t local_count = gasneti_myhost.node_count;
 
 #if GASNET_PSHM
   gasneti_pshm_cs_enter();
@@ -1160,9 +1130,7 @@ uintptr_t gasneti_mmapLimit(uintptr_t localLimit, uint64_t sharedLimit,
   gasneti_assert(gasneti_nodemap);
 
   /* Apply system-dependent defaults, if any */
-#if defined(GASNETI_HAVE_BGP_INLINES) && 0 /* Not implemented */
-    /* XXX: should be able to do something like done for BG/Q, below */
-#elif defined(GASNETI_HAVE_BGQ_INLINES)
+#if defined(GASNETI_HAVE_BGQ_INLINES)
   if ((localLimit == (uintptr_t)-1) || (sharedLimit == (uint64_t)-1)) {
     const uint64_t nodemem = gasneti_getPhysMemSz(1); /* sysconf() reports phys mem for full node */
     const uint64_t safemem = (nodemem * 4) / 5; /* 80% as a safety margin (but just a guess) */
@@ -1190,25 +1158,25 @@ uintptr_t gasneti_mmapLimit(uintptr_t localLimit, uint64_t sharedLimit,
   maxsz = MIN(maxsz, localLimit);
 
   /* Coordinate the search IFF there are any shared nodes. */
-  if (gasneti_nodemap_global_count != gasneti_nodes) {
+  if (gasneti_myhost.grp_count != gasneti_nodes) {
     uintptr_t *sz_exchg = gasneti_malloc(gasneti_nodes * sizeof(uintptr_t));
     gasnet_seginfo_t se = {0,0};
 
     /* Ensure our probe will not collectively exceed the shareLimit, if any. */
-    if ((sharedLimit != (uint64_t)-1) && (gasneti_nodemap_local_count > 1)) {
+    if ((sharedLimit != (uint64_t)-1) && (local_count > 1)) {
 #if SIZEOF_VOID_P != 8
        /* Skip MIN() on overflow */
-       if ((sharedLimit / gasneti_nodemap_local_count) < (uint64_t)(uintptr_t)(-1))
+       if ((sharedLimit / local_count) < (uint64_t)(uintptr_t)(-1))
 #endif
-       { uintptr_t tmp = sharedLimit / gasneti_nodemap_local_count;
+       { uintptr_t tmp = sharedLimit / local_count;
          maxsz = MIN(maxsz, tmp);
        }
     }
 
-    /* Allow each node to probe SEQUENTIALLY, and then collect the results */
+    /* Allow each node in a given host to probe SEQUENTIALLY, and then collect the results */
     maxsz = GASNETI_PAGE_ALIGNDOWN(maxsz);
 #if GASNET_PSHM
-    if (maxsz) {
+    if (maxsz && (gasneti_myhost.grp_count == gasneti_mysupernode.grp_count)) { /* host==supernode */
       for (i = 0; i < gasneti_nodemap_local_count; ++i) {
         if (i == gasneti_nodemap_local_rank) {
           se = _gasneti_mmap_segment_search_inner(maxsz);
@@ -1218,49 +1186,46 @@ uintptr_t gasneti_mmapLimit(uintptr_t localLimit, uint64_t sharedLimit,
         gasneti_pshmnet_bootstrapBroadcast(gasneti_request_pshmnet, &maxsz, sizeof(uintptr_t), &maxsz, i);
         sz_exchg[gasneti_nodemap_local[i]] = maxsz;
       }
-    }
-#else
+    } else
+#endif
     if (maxsz) {
-      /* Find widest supernode */
+      /* Find widest host */
       gasnet_node_t rounds = 0;
       {
-        gasnet_node_t *tmp = gasneti_calloc(gasneti_nodemap_global_count, sizeof(gasnet_node_t));
+        const gasnet_node_t num_hosts = gasneti_myhost.grp_count;
+        gasnet_node_t *tmp = gasneti_calloc(num_hosts, sizeof(gasnet_node_t));
         for (i = 0; i < gasneti_nodes; ++i) {
-          const gasnet_node_t supernode = gasneti_nodeinfo[i].supernode;
-          tmp[supernode] += 1;
-          rounds = MAX(rounds, tmp[supernode]);
+          const gasnet_node_t host = gasneti_nodeinfo[i].host;
+          gasneti_assert(host < num_hosts);
+          tmp[host] += 1;
+          rounds = MAX(rounds, tmp[host]);
         }
         gasneti_free(tmp);
       }
 
       for (i = 0; i < rounds; ++i) {
-        if (i == gasneti_nodemap_local_rank) {
+        if (i == gasneti_myhost.node_rank) {
           se = _gasneti_mmap_segment_search_inner(maxsz);
         }
         (*barrierfn)();
       }
     }
     (*exchangefn)(&se.size, sizeof(uintptr_t), sz_exchg);
-#endif
 
-    /* Compute the supernode-local mean */
-    { uint64_t sum;
-      gasnet_node_t first, j;
-
-      first = gasneti_nodemap[gasneti_mynode];
-      sum = sz_exchg[first];
-      j = 1;
-
-      for (i = (first + 1); j < gasneti_nodemap_local_count; ++i) {
-        if (gasneti_nodemap[i] == first) {
-          sum += sz_exchg[i];
-          j += 1;
-        }
+    /* Compute the host-local mean */
+    { uint64_t sum = 0;
+      for (i = 0; i < local_count; ++i) {
+        sum += sz_exchg[gasneti_myhost.nodes[i]];
       }
-      maxsz = sum / gasneti_nodemap_local_count;
+      maxsz = sum / local_count;
       maxsz = GASNETI_PAGE_ALIGNDOWN(maxsz);
 
 #if GASNET_PSHM
+    #if defined(GASNETI_PSHM_GHEAP)
+      /* We don't consue any additional virtual address space for each segment.
+       * So, the coordnation below is not required.
+       */
+    #else
       /* The probe completed has determined how much memory we can map.
        * However, we now need to be sure that multiple segments will fit.
        * The limit could be, for instance, due to
@@ -1309,6 +1274,7 @@ uintptr_t gasneti_mmapLimit(uintptr_t localLimit, uint64_t sharedLimit,
       /* Unlink the shared segments to prevent leaks (they are recreated in segmentInit) */
       /* XXX: redundant? */
       gasneti_unlink_segments();
+    #endif
 #endif
     }
 
@@ -1323,7 +1289,7 @@ uintptr_t gasneti_mmapLimit(uintptr_t localLimit, uint64_t sharedLimit,
 #endif
   return maxsz;
 }
-#endif /* GASNETI_MMAP_OR_SYSV */
+#endif /* GASNETI_MMAP_OR_PSHM */
 
 /* do the work necessary for initing a standard segment map in arbitrary memory 
      uses mmap if available, or malloc otherwise
@@ -1352,7 +1318,7 @@ void gasneti_segmentInit(uintptr_t localSegmentLimit,
   if (localSegmentLimit != (uintptr_t)-1) 
     localSegmentLimit = GASNETI_PAGE_ALIGNDOWN(localSegmentLimit);
 
-  #ifdef GASNETI_MMAP_OR_SYSV
+  #ifdef GASNETI_MMAP_OR_PSHM
   { gasneti_segexch_t se;
     int i;
 
@@ -1430,7 +1396,7 @@ void gasneti_segmentInit(uintptr_t localSegmentLimit,
       gasneti_maxbase = maxbase;
       #if GASNET_ALIGNED_SEGMENTS
        /* BG/[PQ] would incorrectly probe the I/O node */
-       #if !defined(PLATFORM_OS_BGP) && !defined(PLATFORM_OS_BGQ)
+       #if !defined(PLATFORM_OS_BGQ)
         if (gasneti_nodes > 1) { 
           /* bug 2067 - detect if the compute nodes are using Linux's 'intentional VM space randomization'
            * security feature, which is known to break GASNET_ALIGNED_SEGMENTS, esp at large scale
@@ -1482,7 +1448,7 @@ void gasneti_segmentInit(uintptr_t localSegmentLimit,
       #endif
     }
   }
-  #else /* !GASNETI_MMAP_OR_SYSV */
+  #else /* !GASNETI_MMAP_OR_PSHM */
     #if GASNET_ALIGNED_SEGMENTS && !GASNET_CONDUIT_SMP
       #error bad config: dont know how to provide GASNET_ALIGNED_SEGMENTS when !HAVE_MMAP
     #endif
@@ -1519,7 +1485,7 @@ void gasneti_segmentAttachLocal(uintptr_t segsize, uintptr_t minheapoffset,
   gasneti_memcheck(gasneti_segexch);
 
   #ifndef GASNETI_SEGMENT_DISALIGN_BIAS
-    #if GASNET_DEBUG && !GASNET_ALIGNED_SEGMENTS && (GASNET_PAGESIZE < 1024*1024)
+    #if GASNET_DEBUG && !GASNET_ALIGNED_SEGMENTS && !GASNET_PSHM && !GASNETI_USE_HUGETLBFS && (GASNET_PAGESIZE < 1024*1024)
       /* force segment disalignment for debugging purposes */
       #define GASNETI_SEGMENT_DISALIGN_BIAS GASNET_PAGESIZE
     #else
@@ -1529,7 +1495,7 @@ void gasneti_segmentAttachLocal(uintptr_t segsize, uintptr_t minheapoffset,
 
   bias = GASNETI_SEGMENT_DISALIGN_BIAS * (gasneti_mynode%2);
 
-  #ifdef GASNETI_MMAP_OR_SYSV
+  #ifdef GASNETI_MMAP_OR_PSHM
   { /* TODO: this assumes heap grows up */
     uintptr_t topofheap;
     #if GASNET_ALIGNED_SEGMENTS
@@ -1620,7 +1586,7 @@ void gasneti_segmentAttachLocal(uintptr_t segsize, uintptr_t minheapoffset,
       }
     }
   }
-  #else /* !GASNETI_MMAP_OR_SYSV */
+  #else /* !GASNETI_MMAP_OR_PSHM */
     /* for the T3E, and other platforms which don't support mmap */
     segbase = gasneti_malloc_allowfail(segsize + GASNET_PAGESIZE + bias);
     while (!segbase) {
@@ -1632,7 +1598,7 @@ void gasneti_segmentAttachLocal(uintptr_t segsize, uintptr_t minheapoffset,
       segbase = (void *)GASNETI_PAGE_ALIGNUP(segbase);
       segbase = (void *)(((uintptr_t)segbase)+bias);
     }
-  #endif /* GASNETI_MMAP_OR_SYSV */
+  #endif /* GASNETI_MMAP_OR_PSHM */
   gasneti_assert(((uintptr_t)segbase) % GASNET_PAGESIZE == 0);
   gasneti_assert(segsize % GASNET_PAGESIZE == 0);
   GASNETI_TRACE_PRINTF(C, ("Final segment: segbase="GASNETI_LADDRFMT"  segsize=%lu",
@@ -2202,3 +2168,21 @@ void gasneti_auxseg_attach(void) {
   gasneti_leak(gasneti_auxseg_alignedsz);
 }
 /* ------------------------------------------------------------------------------------ */
+#if defined(GASNETI_PSHM_GHEAP)
+  /* Access to unwrapped malloc/free */
+  #undef malloc
+  #undef free
+  static void *gasneti_pshm_valloc(size_t size) {
+    void *result;
+    /* TODO: configure probe for posix_memalign() and alternatives when missing? */
+    if (0 != posix_memalign(&result, GASNETI_PSHMNET_PAGESIZE, size)) {
+      result = MAP_FAILED;
+    }
+    return result;
+  }
+  static void gasneti_pshm_vfree(void *addr) {
+    /* TODO: alternatives as above? */
+    free(addr);
+  }
+#endif
+/* ------------------------------------------------------------------------------------ */
