@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/gemini-conduit/gasnet_core.h,v $
- *     $Date: 2013/09/03 04:28:19 $
- * $Revision: 1.5 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/gemini-conduit/gasnet_core.h $
  * Description: GASNet header for gemini conduit core
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -33,6 +31,10 @@ extern int gasnetc_attach(gasnet_handlerentry_t *table, int numentries,
 extern void gasnetc_exit(int exitcode) GASNETI_NORETURN;
 GASNETI_NORETURNP(gasnetc_exit)
 #define gasnet_exit gasnetc_exit
+
+/* Some conduits permit gasnet_init(NULL,NULL).
+   Define to 1 if this conduit supports this extension, or to 0 otherwise.  */
+#define GASNET_NULL_ARGV_OK 1
 /* ------------------------------------------------------------------------------------ */
 /*
   No-interrupt sections
