@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/other/amudp/amudp_spmd.cpp,v $
- *     $Date: 2013/07/31 19:01:11 $
- * $Revision: 1.51 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/other/amudp/amudp_spmd.cpp $
  * Description: AMUDP Implementations of SPMD operations (bootstrapping and parallel job control)
  * Copyright 2000, Dan Bonachea <bonachea@cs.berkeley.edu>
  */
@@ -75,8 +73,7 @@ static void freezeForDebugger() {
   #define DEBUG_MASTER(msg) do {} while(0)
 #endif
 
-#define AMUDP_SPMDSLAVE_FLAG "__AMUDP_SLAVE_PROCESS__"
-#define AMUDP_SPMDSLAVE_FLAG_VERBOSE "__AMUDP_SLAVE_PROCESS_VERBOSE__"
+#define AMUDP_SPMDSLAVE_ARGS "AMUDP_SLAVE_ARGS"
 
 static int AMUDP_SPMDShutdown(int exitcode);
 
@@ -180,11 +177,21 @@ static void flushStreams(const char *context) {
   }
   fsync(STDOUT_FILENO); /* ignore errors for output is a console */
   fsync(STDERR_FILENO); /* ignore errors for output is a console */
+
+  static int do_sync = -1;
+  if (do_sync < 0) {
+    /* Approximate match to GASNet's acceptance of 'Y|YES|y|yes|1' */
+    char *envval = AMUDP_getenv_prefixed_withdefault("FS_SYNC", "NO");
+    char c = envval[0];
+    do_sync = ((c == '1') || (c == 'y') || (c == 'Y'));
+  }
+  if (do_sync) {
   #if PLATFORM_OS_MTA
     mta_sync();
   #elif !PLATFORM_OS_CATAMOUNT
     sync();
   #endif
+  }
   sched_yield();
 }
 //------------------------------------------------------------------------------------
@@ -289,9 +296,8 @@ extern int AMUDP_SPMDMyProc() {
 extern int AMUDP_SPMDIsWorker(char **argv) {
   if (AMUDP_SPMDStartupCalled) return 1; 
   else {
-    AMUDP_assert(argv != NULL);
-    return (argv[1] && 
-       (!strcmp(argv[1], AMUDP_SPMDSLAVE_FLAG) || !strcmp(argv[1], AMUDP_SPMDSLAVE_FLAG_VERBOSE) ));
+    const char *env_val = getenv(AMUDP_SPMDSLAVE_ARGS);
+    return (env_val && (0 != atoi(env_val)));
   }
 }
 /* ------------------------------------------------------------------------------------ */
@@ -302,20 +308,23 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
                              eb_t *eb, ep_t *ep) {
   
   if (AMUDP_SPMDStartupCalled) AMUDP_RETURN_ERR(RESOURCE);
-  if (!argc || !argv) AMUDP_RETURN_ERR(BAD_ARG);
   /* we need a separate socklibinit for master 
      and to prevent AM_Terminate from murdering all our control sockets */
   if (!socklibinit()) AMUDP_RETURN_ERRFR(RESOURCE, AMUDP_SPMDStartup, "socklibinit() failed");
 
+  const char *env_var = getenv(AMUDP_SPMDSLAVE_ARGS);
+  const int slave_flag = env_var ? atoi(env_var) : 0;
+
   /* ------------------------------------------------------------------------------------ 
    *  I'm a master 
    * ------------------------------------------------------------------------------------ */
-  if ((*argc) < 2 || 
-    (strcmp((*argv)[1], AMUDP_SPMDSLAVE_FLAG) && strcmp((*argv)[1], AMUDP_SPMDSLAVE_FLAG_VERBOSE))) { 
+  if (! slave_flag) {
     int usingdefaultdegree = 0;
     uint64_t npid;
     if (nproc < 0 || nproc > AMUDP_MAX_SPMDPROCS) AMUDP_RETURN_ERR(BAD_ARG);
 
+    if (!argc || !argv) AMUDP_RETURN_ERR(BAD_ARG);
+
     #if AMUDP_DEBUG_VERBOSE
       AMUDP_SilentMode = 0;
     #else
@@ -365,7 +374,7 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
       }
     }
 
-    if (!spawnfn && *argc > 1 && strlen((*argv)[1]) == 1) {
+    if (argv && !spawnfn && *argc > 1 && strlen((*argv)[1]) == 1) {
       for (int i=0; AMUDP_Spawnfn_Desc[i].abbrev; i++) {
         if (toupper((*argv)[1][0]) == toupper(AMUDP_Spawnfn_Desc[i].abbrev)) {
           spawnfn = AMUDP_Spawnfn_Desc[i].fnptr;
@@ -483,64 +492,51 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
         SockAddr dnsAddr = DNSLookup(getMyHostName());
         masterAddr = SockAddr(dnsAddr.IP(), masterAddr.port());
       } catch (xBase &exn) {
-        AMUDP_RETURN_ERRFR(RESOURCE, AMUDP_SPMDStartup, exn.why());
         AMUDP_Warn("Master %s failed to resolve its own hostname: %s%s",
           getMyHostName(),exn.why(),
           (USE_NUMERIC_MASTER_ADDR?"\nTry setting AMUDP_MASTERIP":"")); 
+        AMUDP_RETURN_ERRFR(RESOURCE, AMUDP_SPMDStartup, exn.why());
       }
     }
 
-    //
-    // For ease-of-debugging with Chapel, hacked this so that GASNet
-    // will launch each task in an xterm running gdb if
-    // CHPL_COMM_USE_GDB is set.
-    //
-    int gdbargs = 0;
-    if (getenv("CHPL_COMM_USE_GDB")) {
-      gdbargs = 4;
+    // setup NULL-terminated array of extra environment vars for slave
+    // Currently have only one such variable, "AMUDP_SLAVE_ARGS":
+    //          flag[,master,[network]]
+    //     flag: zero = this is not a slave
+    //           non-zero = this *is* a slave and value is verbosity (1 = not verbose)
+    //   master: IP or hostname of the master node (require if flag != 0)
+    //  network: value of [PREFIX]_WORKERIP if given
+    char slave_env[1024] = AMUDP_SPMDSLAVE_ARGS "=";
+    strncat(slave_env, (AMUDP_SilentMode ? "1," : "2,"), sizeof(slave_env) - 1);
+    ssize_t remain = sizeof(slave_env) - (strlen(slave_env) + 1);
+    if (*masterIPstr) {
+      strncat(slave_env, masterAddr.FTPStr(), remain);
     }
-
-    // setup a slave argv
-    const char **slaveargv = (const char**)AMUDP_malloc(sizeof(const char*)*((*argc)+3+gdbargs));
-    int slaveargc = (*argc)+2+gdbargs;
-    if (gdbargs) {
-      slaveargv[0] = "/usr/bin/xterm";
-      slaveargv[1] = "-e";
-      slaveargv[2] = "gdb";
-      slaveargv[3] = "--args";
-    }
-
-    slaveargv[0+gdbargs] = (*argv)[0];
-    slaveargv[1+gdbargs] = (AMUDP_SilentMode?AMUDP_SPMDSLAVE_FLAG:AMUDP_SPMDSLAVE_FLAG_VERBOSE);
-    if (*masterIPstr) slaveargv[2+gdbargs] = masterAddr.FTPStr();
     else {
       #if USE_NUMERIC_MASTER_ADDR
-        slaveargv[2+gdbargs] = masterAddr.FTPStr();
+        strncat(slave_env, masterAddr.FTPStr(), remain);
       #else
-        char masteraddrstr[1024];
-        sprintf(masteraddrstr, "%s:%i", masterHostname, masterAddr.port());
-        slaveargv[2+gdbargs] = masteraddrstr;
+        char *tmp = slave_env + strlen(slave_env);
+        snprintf(tmp, remain, "%s:%i", masterHostname, masterAddr.port());
       #endif
     }
+    remain = sizeof(slave_env) - (strlen(slave_env) + 1);
+    strncat(slave_env, ",", remain);
     // append WORKERIP which it is needed before the master env is sent
     { char *network = AMUDP_getenv_prefixed_withdefault("WORKERIP","");
       if (network && network[0]) {
         #if HAVE_GETIFADDRS
-          char *tmp = (char*)AMUDP_malloc(2+strlen(network)+strlen(slaveargv[2+gdbargs]));
-          strcpy(tmp, slaveargv[2+gdbargs]);
-          strcat(tmp,"@");
-          strcat(tmp,network);
-          slaveargv[2+gdbargs] = tmp;
+          strncat(slave_env, network, remain-1);
         #else
           fprintf(stderr,"AMUDP: Warning: WORKERIP set in the environment, but your platform "
                          "lacks the required getifaddrs() support.  Ignoring WORKERIP.\n");
         #endif
       }
     }
-    for (int k = 1; k < (*argc); k++) {
-      slaveargv[k+2+gdbargs] = (*argv)[k];
+    if (!remain) { // ran out of space!
+      AMUDP_FatalErr("Error assembling arguments to SPMD worker threads. Exiting...");
     }
-    slaveargv[slaveargc] = NULL;
+    char *extra_env[2] = { slave_env, NULL };
 
     { int masterpid = getpid();
       uint32_t masterIP = masterAddr.IP();
@@ -560,9 +556,64 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
 
     AMUDP_SPMDRedirectStdsockets = strcmp(AMUDP_getenv_prefixed_withdefault("ROUTE_OUTPUT",(DISABLE_STDSOCKET_REDIRECT?"0":"1")),"0");
 
+    //
+    // For ease-of-debugging with Chapel, hacked this so that GASNet
+    // will launch each task in an xterm running a debugger if
+    // CHPL_COMM_USE_GDB or CHPL_COMM_USE_LLDB is set.
+    //
+    if (getenv("CHPL_COMM_USE_GDB") != NULL
+        || getenv("CHPL_COMM_USE_LLDB") != NULL) {
+      char xtermPath[1 << 12];
+      char *pp, *ppn;
+      int new_argc = 4 + *argc;
+      char** new_argv
+             = (char**) AMUDP_malloc((new_argc + 1) * sizeof((*new_argv)));
+
+      if ((pp = getenv("PATH")) == NULL || *pp == '\0')
+        AMUDP_FatalErr("CHPL_COMM_USE_(G|LL)DB: no PATH.  Exiting...");
+      else {
+        do {
+          ppn = strchr(pp, ':');
+          if (ppn == NULL)
+            (void) snprintf(xtermPath, sizeof(xtermPath), "%s/xterm",
+                            (*pp == '\0') ? "." : pp);
+          else if (ppn == pp)
+            (void) snprintf(xtermPath, sizeof(xtermPath), "./xterm");
+          else
+            (void) snprintf(xtermPath, sizeof(xtermPath), "%.*s/xterm",
+                            (int) (ppn - pp), pp);
+          if (access(xtermPath, X_OK) == 0)
+            break;
+          xtermPath[0] = '\0';
+          pp = ppn + 1;
+        } while (ppn != NULL);
+
+        if (xtermPath[0] == '\0')
+          AMUDP_FatalErr("CHPL_COMM_USE_(G|LL)DB: no xterm in PATH.  "
+                         "Exiting...");
+        else {
+          new_argv[0] = (char*) AMUDP_malloc(strlen(xtermPath) + 1);
+          strcpy(new_argv[0], xtermPath);
+          new_argv[1] = (char *) "-e";
+
+          if (getenv("CHPL_COMM_USE_GDB") != NULL) {
+            new_argv[2] = (char *) "gdb";
+            new_argv[3] = (char *) "--args";
+          } else {
+            new_argv[2] = (char *) "lldb";
+            new_argv[3] = (char *) "--";
+          }
+
+          memcpy(&new_argv[4], *argv, (*argc + 1) * sizeof(*new_argv));
+          *argc = new_argc;
+          *argv = new_argv;
+        }
+      }
+    }
+
     // call system-specific spawning routine
     AMUDP_SPMDSpawnRunning = TRUE;
-    if (!spawnfn(AMUDP_SPMDNUMPROCS, slaveargc, (char **)slaveargv))
+    if (!spawnfn(AMUDP_SPMDNUMPROCS, *argc, *argv, extra_env))
       AMUDP_FatalErr("Error spawning SPMD worker threads. Exiting...");
     AMUDP_SPMDSpawnRunning = FALSE;
 
@@ -844,6 +895,7 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
                 exitCode = ntoh32(exitCode_nb);
                 // tell all other slaves to terminate
                 // TODO: perhaps use an active message for this? for now, just rely on coord socket dying
+                exitCode_nb = hton32(0);
                 for (int i=0; i < (int)coordList.getCount(); i++) {
                   sendAll(coordList[i], "E");
                   sendAll(coordList[i], &exitCode_nb, sizeof(int32_t));
@@ -899,7 +951,7 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
     int temp;
 
     /* propagate verbosity setting from master */
-    AMUDP_SilentMode = !strcmp((*argv)[1], AMUDP_SPMDSLAVE_FLAG);
+    AMUDP_SilentMode = (1 == slave_flag); // TODO: values >2 for more verbose
 
     #if FREEZE_SLAVE
       freezeForDebugger();
@@ -914,34 +966,39 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
       AMUDP_RETURN_ERRFR(RESOURCE, AMUDP_SPMDStartup, "AM_Init() failed");
     }
 
-    // parse special args 
+    // parse special env var with our arguments
+    char * slave_args = strdup(env_var);
     SockAddr masterAddr;
-    #if HAVE_GETIFADDRS
-      const char *network = "";
-    #endif
-    if ((*argc) < 3) AMUDP_Err("Missing arguments to slave process");
-    {
-      #if HAVE_GETIFADDRS
-        // extract appended WORKERIP which it is needed before the master env is sent
-        char *delimiter = strchr((*argv)[2],'@');
-        if (delimiter != NULL) {
-          network = delimiter+1;
-          *delimiter = '\0';
-        }
-      #endif
-      if (strchr((*argv)[2],',')) {
-        masterAddr = SockAddr((*argv)[2]);
+    { // Strip required "flag," off beginning
+      char *endptr;
+      (void) strtol(slave_args, &endptr, 0);
+      if (! endptr || (',' != endptr[0])) AMUDP_Err("Malformed arguments '%s' to slave process", env_var);
+      slave_args = endptr + 1;
+    }
+  #if HAVE_GETIFADDRS
+    // extract appended WORKERIP which it is needed before the master env is sent
+    const char *network = "";
+    { char *delimiter = strrchr(slave_args,',');
+      if (delimiter != NULL) {
+        network = delimiter+1;
+        *delimiter = '\0';
+      }
+    }
+  #endif
+    { // extract master's address
+      if (strchr(slave_args,',')) {
+        masterAddr = SockAddr(slave_args);
       } else {
-        char *IPStr = (char *)AMUDP_malloc(strlen((*argv)[2])+10);
-        strcpy(IPStr, (*argv)[2]);
+        char *IPStr = (char *)AMUDP_malloc(strlen(slave_args)+10);
+        strcpy(IPStr, slave_args);
         char *portStr = strchr(IPStr, ':');
         if (!portStr) {
-          AMUDP_Err("Malformed address argument passed to slave:'%s' (missing port)", (*argv)[2]);
+          AMUDP_Err("Malformed address argument passed to slave:'%s' (missing port)", slave_args);
           AMUDP_RETURN_ERR(BAD_ARG);
         }
         int masterPort = atoi(portStr+1);
         if (masterPort < 1 || masterPort > 65535) {
-          AMUDP_Err("Malformed address argument passed to slave:'%s' (bad port=%i)", (*argv)[2], masterPort);
+          AMUDP_Err("Malformed address argument passed to slave:'%s' (bad port=%i)", slave_args, masterPort);
           AMUDP_RETURN_ERR(BAD_ARG);
         }
         (*portStr) = '\0';
@@ -952,9 +1009,6 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
         }
         AMUDP_free(IPStr);
       }
-      (*argv)[2] = (*argv)[0]; // strip off our special args
-      (*argv) += 2;
-      (*argc) -= 2;
     }
 
     try {
@@ -1026,7 +1080,6 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
       // sanity checking on bootstrap info
       AMUDP_assert(AMUDP_SPMDNUMPROCS > 0 && AMUDP_SPMDNUMPROCS < AMUDP_MAX_SPMDPROCS);
       AMUDP_assert(AMUDP_SPMDMYPROC >= 0 && AMUDP_SPMDMYPROC < AMUDP_SPMDNUMPROCS);
-      AMUDP_assert(AMUDP_SPMDMYPROC >= 0 && AMUDP_SPMDMYPROC < AMUDP_SPMDNUMPROCS);
 
       en_t *tempTranslation_name = (en_t *)AMUDP_malloc(AMUDP_SPMDNUMPROCS*sizeof(en_t));
       tag_t *tempTranslation_tag = (tag_t *)AMUDP_malloc(AMUDP_SPMDNUMPROCS*sizeof(tag_t));
@@ -1139,6 +1192,13 @@ extern int AMUDP_SPMDStartup(int *argc, char ***argv,
     *ep = AMUDP_SPMDEndpoint;
     AMUDP_SPMDStartupCalled = 1;
 
+    /* Ensure that any children we fork() won't appear to be slaves */
+    #if 1
+      unsetenv(AMUDP_SPMDSLAVE_ARGS);
+    #else
+      putenv((char*)AMUDP_SPMDSLAVE_ARGS "=0");
+    #endif
+
     #if USE_ASYNC_TCP_CONTROL
       // enable async notification
       reghandler(AMUDP_SIGIO, AMUDP_SPMDControlSocketCallback);
@@ -1215,7 +1275,7 @@ extern int AMUDP_SPMDHandleControlTraffic(int *controlMessagesServiced) {
         }
 
         case 'G': { // gather complete
-          AMUDP_assert(!AMUDP_SPMDBarrierDone && AMUDP_SPMDGatherLen > 0 && AMUDP_SPMDGatherData != NULL);
+          AMUDP_assert(!AMUDP_SPMDGatherDone && AMUDP_SPMDGatherLen > 0 && AMUDP_SPMDGatherData != NULL);
           try {
             int32_t len_nb = -1;
             recvAll(s, &len_nb, sizeof(int32_t));
@@ -1561,8 +1621,8 @@ extern int AMUDP_SPMDExit(int exitcode) {
 
   AMUDP_SPMDStartupCalled = 0;
   DEBUG_SLAVE("AMUDP_SPMDShutdown..");
-  /* exit this proc */
-  AMUDP_SPMDShutdown(exitcode);
+  /* exit this proc gracefully */
+  AMUDP_SPMDShutdown(0);
   AMUDP_FatalErr("AMUDP_SPMDShutdown failed");
   return AM_OK;
 }
