@@ -1,6 +1,4 @@
-/*   $Source: /var/local/cvs/gasnet/tests/test.h,v $
- *     $Date: 2013/01/12 00:36:01 $
- * $Revision: 1.151 $
+/*   $Source: bitbucket.org:berkeleylab/gasnet.git/tests/test.h $
  * Description: helpers for GASNet tests
  * Copyright 2002, Dan Bonachea <bonachea@cs.berkeley.edu>
  * Terms of use are as specified in license.txt
@@ -50,6 +48,18 @@
 #include <assert.h>
 #undef assert
 
+/* Grumble... On Solaris-x86 /usr/include/sys/regset.h polutes the namespace */
+#if PLATFORM_OS_SOLARIS && (PLATFORM_ARCH_X86 || PLATFORM_ARCH_X86_64)
+  #if defined(_SYS_REGSET_H) && defined(ERR)
+    #undef ERR
+  #endif
+#endif
+
+#if HAVE_SRAND_DETERMINISTIC
+  /* OpenBSD "breaks" rand() by design */
+  #define srand(seed) srand_deterministic(seed)
+#endif
+
 GASNETT_BEGIN_EXTERNC
 
 #define assert_always(expr) \
@@ -233,7 +243,7 @@ static void *_test_malloc(size_t sz, const char *curloc) {
   test_hold_interrupts();
   ptr = malloc(sz);
   test_resume_interrupts();
-  if (ptr == NULL) FATALERR("Failed to malloc(%i) bytes at %s\n",(int)sz,curloc);
+  if (ptr == NULL) FATALERR("Failed to malloc(%lu) bytes at %s\n",(unsigned long)sz,curloc);
   return ptr;
 }
 static void *_test_calloc(size_t sz, const char *curloc) {
@@ -351,14 +361,14 @@ static int64_t test_calibrate_delay(int iters, int pollcnt, int64_t *time_p)
   #endif
   #define TEST_TITANIUM_BACKEND "sequential"
   #define TEST_CONFIG_STRING \
-    "RELEASE=x,SPEC=x,CONDUIT="_STRINGIFY(GASNETI_TOOLS_CONDUIT)"("_STRINGIFY(GASNETI_TOOLS_CONDUIT)"-x/REFERENCE-x),THREADMODEL=PAR,SEGMENT=FAST,PTR=x,align,nodebug,notrace,nostats"
+    "RELEASE=x,SPEC=x,CONDUIT=" _STRINGIFY(GASNETI_TOOLS_CONDUIT) "(" _STRINGIFY(GASNETI_TOOLS_CONDUIT) "-x/REFERENCE-x),THREADMODEL=PAR,SEGMENT=FAST,PTR=x,align,nodebug,notrace,nostats"
 #endif
 /* mimic Berkeley UPC build config strings, to allow running GASNet tests using upcrun */
 GASNETT_IDENT(GASNetT_IdentString_link_GASNetConfig, 
  "$GASNetConfig: (<link>) " TEST_CONFIG_STRING " $");
 #ifndef HAVE_PTHREAD_H
   /* for systems lacking pthread support - ensure upcrun never tries to use it */
-  #if GASNET_PSHM
+  #if GASNET_PSHM || (defined(GASNETI_PSHM_ENABLED) && defined(TEST_GASNET_TOOLS_ONLY))
     #define TEST_SHMEM_CONFIG "pshm"
   #else
     #define TEST_SHMEM_CONFIG "none"
@@ -369,7 +379,7 @@ GASNETT_IDENT(GASNetT_IdentString_link_GASNetConfig,
      such a setup will only run the gasnet test on N/T nodes (as opposed to N as one might like)
      but the alternative is not to run at all. harness -nopthreads does not have this problem.
    */
-  #if GASNET_PSHM
+  #if GASNET_PSHM || (defined(GASNETI_PSHM_ENABLED) && defined(TEST_GASNET_TOOLS_ONLY))
     #define TEST_SHMEM_CONFIG "pthreads/pshm"
   #else
     #define TEST_SHMEM_CONFIG "pthreads"
@@ -377,7 +387,7 @@ GASNETT_IDENT(GASNetT_IdentString_link_GASNetConfig,
   GASNETT_IDENT(GASNetT_IdentString_PthCnt, "$UPCRDefaultPthreadCount: 1 $");
 #endif
 GASNETT_IDENT(GASNetT_IdentString_link_UPCRConfig,
-   "$UPCRConfig: (<link>) " TEST_CONFIG_STRING ",SHMEM="TEST_SHMEM_CONFIG",SHAREDPTRREP=packed,dynamicthreads $");
+   "$UPCRConfig: (<link>) " TEST_CONFIG_STRING ",SHMEM=" TEST_SHMEM_CONFIG ",SHAREDPTRREP=packed,dynamicthreads $");
 GASNETT_IDENT(GASNetT_IdentString_link_upcver, 
  "$UPCVersion: (<link>) *** GASNet test *** $");
 GASNETT_IDENT(GASNetT_IdentString_link_compileline, 
@@ -408,9 +418,12 @@ GASNETT_IDENT(GASNetT_TiCompiler_IdentString,
    TEST_USE_PRIMORDIAL_THREAD can be defined to spawn only numthreads-1 new pthreads and run the last on this thread
  */
 #ifndef TEST_USE_PRIMORDIAL_THREAD
-  #if PLATFORM_OS_BGP || PLATFORM_OS_BGQ
+  #if PLATFORM_OS_BGQ
     /* some systems have strict limits on how many threads can exist */
     #define TEST_USE_PRIMORDIAL_THREAD 1
+  #elif PLATFORM_OS_CNL
+    /* some do default thread pinning that can mess with our results */
+    #define TEST_USE_PRIMORDIAL_THREAD 1
   #else
     #define TEST_USE_PRIMORDIAL_THREAD 0
   #endif
@@ -430,7 +443,7 @@ GASNETT_IDENT(GASNetT_TiCompiler_IdentString,
 static int test_thread_limit(int numthreads) {
     int limit = gasnett_getenv_int_withdefault("GASNET_TEST_THREAD_LIMIT", TEST_MAXTHREADS, 0);
     limit = MIN(limit, TEST_MAXTHREADS); /* Ignore attempt to raise above TEST_MAXTHREADS */
-  #if PLATFORM_OS_BGP || PLATFORM_OS_BGQ
+  #if PLATFORM_OS_BGQ
     { int cores = gasnett_cpu_count();
       int depth = gasnett_getenv_int_withdefault("BG_APPTHREADDEPTH", 1, 0); /* V1R4M0 and later */
       limit = MIN(limit, cores*depth);
@@ -877,7 +890,7 @@ static void _test_set_waitmode(int threads) {
     }
   }
 #endif
-#if PLATFORM_OS_BGP || PLATFORM_OS_BGQ
+#if PLATFORM_OS_BGQ
   /* assume no overcommit, since gasnett_cpu_count() reports cores per proc, NOT per node */
 #else
   threads *= local_procs;
