@@ -15,7 +15,8 @@
 #include <stdio.h>
 #include <pthread.h>
 
-#include "chpltypes.h" // for chpl_bool
+#include "chpltypes.h"
+#include "chpl-tasks-prvdata.h"
 
 #define CHPL_COMM_YIELD_TASK_WHILE_POLLING
 void chpl_task_yield(void);
@@ -92,6 +93,7 @@ typedef syncvar_t chpl_mutex_t;
 typedef struct {
     aligned_t lockers_in;
     aligned_t lockers_out;
+    uint_fast32_t uncontested_locks;
     int       is_full;
     syncvar_t signal_full;
     syncvar_t signal_empty;
@@ -259,40 +261,51 @@ void threadlayer_exit(void);
 
 
 #ifndef QTHREAD_MULTINODE
+extern
+#ifdef __cplusplus
+"C"
+#endif
 volatile int chpl_qthread_done_initializing;
 #endif
 
-
 typedef struct {
-#if !(defined(CHPL_LOCALE_MODEL_NUM_SUBLOCALES) &&  \
-  CHPL_LOCALE_MODEL_NUM_SUBLOCALES == 0)
-  c_sublocid_t requestedSubloc;  // requested sublocal for task
-#endif
-  chpl_bool    serial_state;     // true: serialize execution
-} chpl_qthread_private_data_t;
+    c_string task_filename;
+    int task_lineno;
+    chpl_taskID_t id;
+    chpl_bool is_executeOn;
+    c_sublocid_t requestedSubloc;  // requested sublocal for task
+    chpl_task_prvData_t prvdata;
+} chpl_task_prvDataImpl_t;
+
+// Define PRV_DATA_IMPL_VAL to set up a chpl_task_prvData_t.
+#define PRV_DATA_IMPL_VAL(_fn, _ln, _id, _is_execOn, _subloc, _serial) \
+        { .task_filename = _fn, \
+          .task_lineno = _ln, \
+          .id = _id, \
+          .is_executeOn = _is_execOn, \
+          .requestedSubloc = _subloc, \
+          .prvdata = { .serial_state = _serial } }
 
 typedef struct {
     void                     *fn;
     void                     *args;
-    chpl_string              task_filename;
-    int                      lineno;
     chpl_bool                countRunning;
-    chpl_qthread_private_data_t      chpl_data;
+    chpl_task_prvDataImpl_t  chpl_data;
 } chpl_qthread_wrapper_args_t;
 
 // Structure of task-local storage
 typedef struct chpl_qthread_tls_s {
     /* Task private data: serial state, etc. */
-    chpl_qthread_private_data_t chpl_data;
+    chpl_task_prvDataImpl_t chpl_data;
     /* Reports */
-    chpl_string lock_filename;
+    c_string    lock_filename;
     size_t      lock_lineno;
-    const char *task_filename;
-    size_t      task_lineno;
 } chpl_qthread_tls_t;
 
-pthread_t chpl_qthread_comm_pthread;
+extern pthread_t chpl_qthread_process_pthread;
+extern pthread_t chpl_qthread_comm_pthread;
 
+extern chpl_qthread_tls_t chpl_qthread_process_tls;
 extern chpl_qthread_tls_t chpl_qthread_comm_task_tls;
 
 #define CHPL_TASK_STD_MODULES_INITIALIZED chpl_task_stdModulesInitialized
@@ -301,14 +314,18 @@ void chpl_task_stdModulesInitialized(void);
 // Wrap qthread_get_tasklocal() and assert that it is always available.
 static inline chpl_qthread_tls_t * chpl_qthread_get_tasklocal(void)
 {
-    chpl_qthread_tls_t * tls;
+    chpl_qthread_tls_t* tls;
 
     if (chpl_qthread_done_initializing) {
         tls = (chpl_qthread_tls_t *)
               qthread_get_tasklocal(sizeof(chpl_qthread_tls_t));
-        if (tls == NULL &&
-            pthread_equal(pthread_self(), chpl_qthread_comm_pthread))
-          tls = &chpl_qthread_comm_task_tls;
+        if (tls == NULL) {
+            pthread_t me = pthread_self();
+            if (pthread_equal(me, chpl_qthread_comm_pthread))
+                tls = &chpl_qthread_comm_task_tls;
+            else if (pthread_equal(me, chpl_qthread_process_pthread))
+                tls = &chpl_qthread_process_tls;
+        }
         assert(tls);
     }
     else
@@ -317,6 +334,21 @@ static inline chpl_qthread_tls_t * chpl_qthread_get_tasklocal(void)
     return tls;
 }
 
+#ifdef CHPL_TASK_GET_PRVDATA_IMPL_DECL
+#error "CHPL_TASK_GET_PRVDATA_IMPL_DECL is already defined!"
+#else
+#define CHPL_TASK_GET_PRVDATA_IMPL_DECL 1
+#endif
+static inline chpl_task_prvData_t* chpl_task_getPrvData(void)
+{
+    chpl_qthread_tls_t * data = chpl_qthread_get_tasklocal();
+    if (data) {
+        return &data->chpl_data.prvdata;
+    }
+    assert(data);
+    return NULL;
+}
+
 #ifdef CHPL_TASK_GETSUBLOC_IMPL_DECL
 #error "CHPL_TASK_GETSUBLOC_IMPL_DECL is already defined!"
 #else
@@ -325,12 +357,7 @@ static inline chpl_qthread_tls_t * chpl_qthread_get_tasklocal(void)
 static inline
 c_sublocid_t chpl_task_getSubloc(void)
 {
-#if defined(CHPL_LOCALE_MODEL_NUM_SUBLOCALES) &&  \
-  CHPL_LOCALE_MODEL_NUM_SUBLOCALES == 0
-    return 0;
-#else
     return (c_sublocid_t) qthread_shep();
-#endif
 }
 
 #ifdef CHPL_TASK_SETSUBLOC_IMPL_DECL
@@ -341,8 +368,6 @@ c_sublocid_t chpl_task_getSubloc(void)
 static inline
 void chpl_task_setSubloc(c_sublocid_t subloc)
 {
-#if !(defined(CHPL_LOCALE_MODEL_NUM_SUBLOCALES) &&  \
-  CHPL_LOCALE_MODEL_NUM_SUBLOCALES == 0)
     qthread_shepherd_id_t curr_shep;
 
     assert(subloc != c_sublocid_none);
@@ -369,7 +394,6 @@ void chpl_task_setSubloc(c_sublocid_t subloc)
             qthread_migrate_to((qthread_shepherd_id_t) subloc);
         }
     }
-#endif
 }
 
 #ifdef CHPL_TASK_GETREQUESTEDSUBLOC_IMPL_DECL
@@ -380,18 +404,22 @@ void chpl_task_setSubloc(c_sublocid_t subloc)
 static inline
 c_sublocid_t chpl_task_getRequestedSubloc(void)
 {
-#if defined(CHPL_LOCALE_MODEL_NUM_SUBLOCALES) &&  \
-  CHPL_LOCALE_MODEL_NUM_SUBLOCALES == 0
-    return 0;
-#else
     chpl_qthread_tls_t * data = chpl_qthread_get_tasklocal();
     if (data) {
         return data->chpl_data.requestedSubloc;
     }
-    else {
-        return c_sublocid_any;
-    }
+    return c_sublocid_any;
+}
+
+#ifdef CHPL_TASK_SUPPORTS_REMOTE_CACHE_IMPL_DECL
+#error "CHPL_TASK_SUPPORTS_REMOTE_CACHE_IMPL_DECL is already defined!"
+#else
+#define CHPL_TASK_SUPPORTS_REMOTE_CACHE_IMPL_DECL 1
 #endif
+extern int chpl_qthread_supports_remote_cache;
+static inline
+int chpl_task_supportsRemoteCache(void) {
+  return chpl_qthread_supports_remote_cache;
 }
 
 #endif // ifndef _tasks_qthreads_h_
